/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_adc.h
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#ifndef __rx32s12xx_ADC_H__
#define __rx32s12xx_ADC_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s12xx.h"

/******************************************************************************/
/*                                                                            */
/*                      Analog to Digital Converter (ADC)                     */
/*                                                                            */
/******************************************************************************/
 
/********************  Bit definition for ADC_SR register  ********************/
#define ADC_SR_AWD                          ((uint32_t)0x00000001)                     /*!< ADC analog watchdog 1 flag */
#define ADC_SR_EOC                          ((uint32_t)0x00000002)                     /*!< ADC group regular end of sequence conversions flag */
#define ADC_SR_JEOC                         ((uint32_t)0x00000004)                     /*!< ADC group injected end of sequence conversions flag */
#define ADC_SR_JSTRT                        ((uint32_t)0x00000008)                     /*!< ADC group injected conversion start flag */
#define ADC_SR_STRT                         ((uint32_t)0x00000010)                     /*!< ADC group regular conversion start flag */

/*******************  Bit definition for ADC_CR1 register  ********************/
#define ADC_CR1_AWDCH                       ((uint32_t)0x0000001F)                     /*!< ADC analog watchdog 1 monitored channel selection */
#define ADC_CR1_AWDCH_0                     ((uint32_t)0x00000001)
#define ADC_CR1_AWDCH_1                     ((uint32_t)0x00000002)
#define ADC_CR1_AWDCH_2                     ((uint32_t)0x00000004)
#define ADC_CR1_AWDCH_3                     ((uint32_t)0x00000008)
#define ADC_CR1_AWDCH_4                     ((uint32_t)0x00000010)

#define ADC_CR1_EOCIE                       ((uint32_t)0x00000020)                     /*!< ADC group regular end of sequence conversions interrupt */
#define ADC_CR1_AWDIE                       ((uint32_t)0x00000040)                     /*!< ADC analog watchdog 1 interrupt */
#define ADC_CR1_JEOCIE                      ((uint32_t)0x00000080)                     /*!< ADC group injected end of sequence conversions interrupt */
#define ADC_CR1_SCAN                        ((uint32_t)0x00000100)                     /*!< ADC scan mode */
#define ADC_CR1_AWDSGL                      ((uint32_t)0x00000200)                     /*!< ADC analog watchdog 1 monitoring a single channel or all channels */
#define ADC_CR1_JAUTO                       ((uint32_t)0x00000400)                     /*!< ADC group injected automatic trigger mode */
#define ADC_CR1_DISCEN                      ((uint32_t)0x00000800)                     /*!< ADC group regular sequencer discontinuous mode */
#define ADC_CR1_JDISCEN                     ((uint32_t)0x00001000)                     /*!< ADC group injected sequencer discontinuous mode */

#define ADC_CR1_DISCNUM                     ((uint32_t)0x0000E000)                     /*!< ADC group regular sequencer discontinuous number of ranks */
#define ADC_CR1_DISCNUM_0                   ((uint32_t)0x00002000)
#define ADC_CR1_DISCNUM_1                   ((uint32_t)0x00004000)
#define ADC_CR1_DISCNUM_2                   ((uint32_t)0x00008000)

#define ADC_CR1_JAWDEN                      ((uint32_t)0x00400000)                     /*!< ADC analog watchdog 1 enable on scope ADC group injected */
#define ADC_CR1_AWDEN                       ((uint32_t)0x00800000)                     /*!< ADC analog watchdog 1 enable on scope ADC group regular */

/*******************  Bit definition for ADC_CR2 register  ********************/
#define ADC_CR2_ADON                        ((uint32_t)0x00000001)                     /*!< ADC enable */
#define ADC_CR2_CONT                        ((uint32_t)0x00000002)                     /*!< ADC group regular continuous conversion mode */
#define ADC_CR2_CAL                         ((uint32_t)0x00000004)                     /*!< ADC calibration start */
#define ADC_CR2_RSTCAL                      ((uint32_t)0x00000008)                     /*!< ADC calibration reset */
#define ADC_CR2_DMA                         ((uint32_t)0x00000100)                     /*!< ADC DMA transfer enable */
#define ADC_CR2_ALIGN                       ((uint32_t)0x00000400)                     /*!< ADC data alignement */

#define ADC_CR2_JEXTSEL                     ((uint32_t)0x00007800)                     /*!< ADC group injected external trigger source */
#define ADC_CR2_JEXTSEL_0                   ((uint32_t)0x00000800)
#define ADC_CR2_JEXTSEL_1                   ((uint32_t)0x00001000)
#define ADC_CR2_JEXTSEL_2                   ((uint32_t)0x00002000)
#define ADC_CR2_JEXTSEL_3                   ((uint32_t)0x00004000)

#define ADC_CR2_JEXTTRIG                    ((uint32_t)0x00008000)                     /*!< ADC group injected external trigger enable */

#define ADC_CR2_EXTSEL                      ((uint32_t)0x000E0000)                     /*!< ADC group regular external trigger source */
#define ADC_CR2_EXTSEL_0                    ((uint32_t)0x00020000) 
#define ADC_CR2_EXTSEL_1                    ((uint32_t)0x00040000) 
#define ADC_CR2_EXTSEL_2                    ((uint32_t)0x00080000) 

#define ADC_CR2_EXTTRIG                     ((uint32_t)0x00100000)                     /*!< ADC group regular external trigger enable */

#define ADC_CR2_JSWSTART                    ((uint32_t)0x00200000)                     /*!< ADC group injected conversion start */
#define ADC_CR2_SWSTART                     ((uint32_t)0x00400000)                     /*!< ADC group regular conversion start */
#define ADC_CR2_TSVREFE                     ((uint32_t)0x00800000)                     /*!< ADC internal path to VrefInt and temperature sensor enable */

/******************  Bit definition for ADC_SMPR1 register  *******************/
#define ADC_SMPR1_SMP10                     ((uint32_t)0x00000007)                     /*!< ADC channel 0 sampling time selection  */
#define ADC_SMPR1_SMP10_0                   ((uint32_t)0x00000001) 
#define ADC_SMPR1_SMP10_1                   ((uint32_t)0x00000002) 
#define ADC_SMPR1_SMP10_2                   ((uint32_t)0x00000004) 

#define ADC_SMPR1_SMP11                     ((uint32_t)0x00000038)                     /*!< ADC channel 1 sampling time selection  */
#define ADC_SMPR1_SMP11_0                   ((uint32_t)0x00000008) 
#define ADC_SMPR1_SMP11_1                   ((uint32_t)0x00000010) 
#define ADC_SMPR1_SMP11_2                   ((uint32_t)0x00000020) 

#define ADC_SMPR1_SMP12                     ((uint32_t)0x000001C0)                     /*!< ADC channel 2 sampling time selection  */
#define ADC_SMPR1_SMP12_0                   ((uint32_t)0x00000040) 
#define ADC_SMPR1_SMP12_1                   ((uint32_t)0x00000080) 
#define ADC_SMPR1_SMP12_2                   ((uint32_t)0x00000100) 

#define ADC_SMPR1_SMP13                     ((uint32_t)0x00000E00)                     /*!< ADC channel 3 sampling time selection  */
#define ADC_SMPR1_SMP13_0                   ((uint32_t)0x00000200) 
#define ADC_SMPR1_SMP13_1                   ((uint32_t)0x00000400) 
#define ADC_SMPR1_SMP13_2                   ((uint32_t)0x00000800) 

#define ADC_SMPR1_SMP14                     ((uint32_t)0x00007000)                     /*!< ADC channel 4 sampling time selection  */
#define ADC_SMPR1_SMP14_0                   ((uint32_t)0x00001000) 
#define ADC_SMPR1_SMP14_1                   ((uint32_t)0x00002000) 
#define ADC_SMPR1_SMP14_2                   ((uint32_t)0x00004000) 

#define ADC_SMPR1_SMP15                     ((uint32_t)0x00038000)                     /*!< ADC channel 5 sampling time selection  */
#define ADC_SMPR1_SMP15_0                   ((uint32_t)0x00008000) 
#define ADC_SMPR1_SMP15_1                   ((uint32_t)0x00010000) 
#define ADC_SMPR1_SMP15_2                   ((uint32_t)0x00020000) 

#define ADC_SMPR1_SMP16                     ((uint32_t)0x001C0000)                     /*!< ADC channel 6 sampling time selection  */
#define ADC_SMPR1_SMP16_0                   ((uint32_t)0x00040000) 
#define ADC_SMPR1_SMP16_1                   ((uint32_t)0x00080000) 
#define ADC_SMPR1_SMP16_2                   ((uint32_t)0x00100000) 

#define ADC_SMPR1_SMP17                     ((uint32_t)0x00E00000)                     /*!< ADC channel 7 sampling time selection  */
#define ADC_SMPR1_SMP17_0                   ((uint32_t)0x00200000) 
#define ADC_SMPR1_SMP17_1                   ((uint32_t)0x00400000) 
#define ADC_SMPR1_SMP17_2                   ((uint32_t)0x00800000) 

#define ADC_SMPR1_SMP18                     ((uint32_t)0x07000000)                     /*!< ADC channel 8 sampling time selection  */
#define ADC_SMPR1_SMP18_0                   ((uint32_t)0x01000000) 
#define ADC_SMPR1_SMP18_1                   ((uint32_t)0x02000000) 
#define ADC_SMPR1_SMP18_2                   ((uint32_t)0x04000000) 


/******************  Bit definition for ADC_SMPR2 register  *******************/
#define ADC_SMPR2_SMP0                      ((uint32_t)0x00000007)                     /*!< ADC channel 0 sampling time selection  */
#define ADC_SMPR2_SMP0_0                    ((uint32_t)0x00000001) 
#define ADC_SMPR2_SMP0_1                    ((uint32_t)0x00000002) 
#define ADC_SMPR2_SMP0_2                    ((uint32_t)0x00000004) 

#define ADC_SMPR2_SMP1                      ((uint32_t)0x00000038)                     /*!< ADC channel 1 sampling time selection  */
#define ADC_SMPR2_SMP1_0                    ((uint32_t)0x00000008) 
#define ADC_SMPR2_SMP1_1                    ((uint32_t)0x00000010) 
#define ADC_SMPR2_SMP1_2                    ((uint32_t)0x00000020) 

#define ADC_SMPR2_SMP2                      ((uint32_t)0x000001C0)                     /*!< ADC channel 2 sampling time selection  */
#define ADC_SMPR2_SMP2_0                    ((uint32_t)0x00000040) 
#define ADC_SMPR2_SMP2_1                    ((uint32_t)0x00000080) 
#define ADC_SMPR2_SMP2_2                    ((uint32_t)0x00000100) 

#define ADC_SMPR2_SMP3                      ((uint32_t)0x00000E00)                     /*!< ADC channel 3 sampling time selection  */
#define ADC_SMPR2_SMP3_0                    ((uint32_t)0x00000200) 
#define ADC_SMPR2_SMP3_1                    ((uint32_t)0x00000400) 
#define ADC_SMPR2_SMP3_2                    ((uint32_t)0x00000800) 

#define ADC_SMPR2_SMP4                      ((uint32_t)0x00007000)                     /*!< ADC channel 4 sampling time selection  */
#define ADC_SMPR2_SMP4_0                    ((uint32_t)0x00001000) 
#define ADC_SMPR2_SMP4_1                    ((uint32_t)0x00002000) 
#define ADC_SMPR2_SMP4_2                    ((uint32_t)0x00004000) 

#define ADC_SMPR2_SMP5                      ((uint32_t)0x00038000)                     /*!< ADC channel 5 sampling time selection  */
#define ADC_SMPR2_SMP5_0                    ((uint32_t)0x00008000) 
#define ADC_SMPR2_SMP5_1                    ((uint32_t)0x00010000) 
#define ADC_SMPR2_SMP5_2                    ((uint32_t)0x00020000) 

#define ADC_SMPR2_SMP6                      ((uint32_t)0x001C0000)                     /*!< ADC channel 6 sampling time selection  */
#define ADC_SMPR2_SMP6_0                    ((uint32_t)0x00040000) 
#define ADC_SMPR2_SMP6_1                    ((uint32_t)0x00080000) 
#define ADC_SMPR2_SMP6_2                    ((uint32_t)0x00100000) 

#define ADC_SMPR2_SMP7                      ((uint32_t)0x00E00000)                     /*!< ADC channel 7 sampling time selection  */
#define ADC_SMPR2_SMP7_0                    ((uint32_t)0x00200000) 
#define ADC_SMPR2_SMP7_1                    ((uint32_t)0x00400000) 
#define ADC_SMPR2_SMP7_2                    ((uint32_t)0x00800000) 

#define ADC_SMPR2_SMP8                      ((uint32_t)0x07000000)                     /*!< ADC channel 8 sampling time selection  */
#define ADC_SMPR2_SMP8_0                    ((uint32_t)0x01000000) 
#define ADC_SMPR2_SMP8_1                    ((uint32_t)0x02000000) 
#define ADC_SMPR2_SMP8_2                    ((uint32_t)0x04000000) 

#define ADC_SMPR2_SMP9                      ((uint32_t)0x38000000)                     /*!< ADC channel 8 sampling time selection  */
#define ADC_SMPR2_SMP9_0                    ((uint32_t)0x01000000) 
#define ADC_SMPR2_SMP9_1                    ((uint32_t)0x10000000) 
#define ADC_SMPR2_SMP9_2                    ((uint32_t)0x20000000) 

/******************  Bit definition for ADC_JOFR1 register  *******************/
#define ADC_JOFR1_JOFFSET1                  ((uint32_t)0x00000FFF)                     /*!< ADC group injected sequencer rank 1 offset value */

/******************  Bit definition for ADC_JOFR2 register  *******************/
#define ADC_JOFR2_JOFFSET2                  ((uint32_t)0x00000FFF)                     /*!< ADC group injected sequencer rank 2 offset value */

/******************  Bit definition for ADC_JOFR3 register  *******************/
#define ADC_JOFR3_JOFFSET3                  ((uint32_t)0x00000FFF)                     /*!< ADC group injected sequencer rank 3 offset value */

/******************  Bit definition for ADC_JOFR4 register  *******************/
#define ADC_JOFR4_JOFFSET4                  ((uint32_t)0x00000FFF)                     /*!< ADC group injected sequencer rank 4 offset value */

/*******************  Bit definition for ADC_HTR register  ********************/
#define ADC_HTR_HT                          ((uint32_t)0x00000FFF)                     /*!< ADC analog watchdog 1 threshold high */

/*******************  Bit definition for ADC_LTR register  ********************/
#define ADC_LTR_LT                          ((uint32_t)0x00000FFF)                     /*!< ADC analog watchdog 1 threshold low */

/*******************  Bit definition for ADC_SQR1 register  *******************/
#define ADC_SQR1_L                          ((uint32_t)0x00F00000)                     /*!< ADC group regular sequencer scan length */
#define ADC_SQR1_L_0                        ((uint32_t)0x00100000)            
#define ADC_SQR1_L_1                        ((uint32_t)0x00200000)         
#define ADC_SQR1_L_2                        ((uint32_t)0x00400000)          
#define ADC_SQR1_L_3                        ((uint32_t)0x00800000)

#define ADC_SQR1_SQ13                       ((uint32_t)0x0000001F)                     /*!< ADC group regular sequencer rank 7 */
#define ADC_SQR1_SQ13_0                     ((uint32_t)0x00000001) 
#define ADC_SQR1_SQ13_1                     ((uint32_t)0x00000002) 
#define ADC_SQR1_SQ13_2                     ((uint32_t)0x00000004) 
#define ADC_SQR1_SQ13_3                     ((uint32_t)0x00000008) 
#define ADC_SQR1_SQ13_4                     ((uint32_t)0x00000010)

#define ADC_SQR1_SQ14                       ((uint32_t)0x000003E0)                     /*!< ADC group regular sequencer rank 8 */
#define ADC_SQR1_SQ14_0                     ((uint32_t)0x00000020) 
#define ADC_SQR1_SQ14_1                     ((uint32_t)0x00000040) 
#define ADC_SQR1_SQ14_2                     ((uint32_t)0x00000080) 
#define ADC_SQR1_SQ14_3                     ((uint32_t)0x00000100) 
#define ADC_SQR1_SQ14_4                     ((uint32_t)0x00000200) 

#define ADC_SQR1_SQ15                       ((uint32_t)0x00007C00)                     /*!< ADC group regular sequencer rank 3 */
#define ADC_SQR1_SQ15_0                     ((uint32_t)0x00000400) 
#define ADC_SQR1_SQ15_1                     ((uint32_t)0x00000800) 
#define ADC_SQR1_SQ15_2                     ((uint32_t)0x00001000) 
#define ADC_SQR1_SQ15_3                     ((uint32_t)0x00002000) 
#define ADC_SQR1_SQ15_4                     ((uint32_t)0x00004000) 

#define ADC_SQR1_SQ16                       ((uint32_t)0x000F8000)                     /*!< ADC group regular sequencer rank 4 */
#define ADC_SQR1_SQ16_0                     ((uint32_t)0x00008000) 
#define ADC_SQR1_SQ16_1                     ((uint32_t)0x00010000) 
#define ADC_SQR1_SQ16_2                     ((uint32_t)0x00020000) 
#define ADC_SQR1_SQ16_3                     ((uint32_t)0x00040000) 
#define ADC_SQR1_SQ16_4                     ((uint32_t)0x00080000) 

/*******************  Bit definition for ADC_SQR2 register  *******************/
#define ADC_SQR2_SQ7                        ((uint32_t)0x0000001F)                     /*!< ADC group regular sequencer rank 7 */
#define ADC_SQR2_SQ7_0                      ((uint32_t)0x00000001) 
#define ADC_SQR2_SQ7_1                      ((uint32_t)0x00000002) 
#define ADC_SQR2_SQ7_2                      ((uint32_t)0x00000004) 
#define ADC_SQR2_SQ7_3                      ((uint32_t)0x00000008) 
#define ADC_SQR2_SQ7_4                      ((uint32_t)0x00000010)

#define ADC_SQR2_SQ8                        ((uint32_t)0x000003E0)                     /*!< ADC group regular sequencer rank 8 */
#define ADC_SQR2_SQ8_0                      ((uint32_t)0x00000020) 
#define ADC_SQR2_SQ8_1                      ((uint32_t)0x00000040) 
#define ADC_SQR2_SQ8_2                      ((uint32_t)0x00000080) 
#define ADC_SQR2_SQ8_3                      ((uint32_t)0x00000100) 
#define ADC_SQR2_SQ8_4                      ((uint32_t)0x00000200) 

#define ADC_SQR2_SQ9                        ((uint32_t)0x00007C00)                     /*!< ADC group regular sequencer rank 3 */
#define ADC_SQR2_SQ9_0                      ((uint32_t)0x00000400) 
#define ADC_SQR2_SQ9_1                      ((uint32_t)0x00000800) 
#define ADC_SQR2_SQ9_2                      ((uint32_t)0x00001000) 
#define ADC_SQR2_SQ9_3                      ((uint32_t)0x00002000) 
#define ADC_SQR2_SQ9_4                      ((uint32_t)0x00004000) 

#define ADC_SQR2_SQ10                       ((uint32_t)0x000F8000)                     /*!< ADC group regular sequencer rank 4 */
#define ADC_SQR2_SQ10_0                     ((uint32_t)0x00008000) 
#define ADC_SQR2_SQ10_1                     ((uint32_t)0x00010000) 
#define ADC_SQR2_SQ10_2                     ((uint32_t)0x00020000) 
#define ADC_SQR2_SQ10_3                     ((uint32_t)0x00040000) 
#define ADC_SQR2_SQ10_4                     ((uint32_t)0x00080000) 

#define ADC_SQR2_SQ11                       ((uint32_t)0x01F00000)                     /*!< ADC group regular sequencer rank 5 */
#define ADC_SQR2_SQ11_0                     ((uint32_t)0x00100000) 
#define ADC_SQR2_SQ11_1                     ((uint32_t)0x00200000) 
#define ADC_SQR2_SQ11_2                     ((uint32_t)0x00400000) 
#define ADC_SQR2_SQ11_3                     ((uint32_t)0x00800000) 
#define ADC_SQR2_SQ11_4                     ((uint32_t)0x01000000) 

#define ADC_SQR2_SQ12                       ((uint32_t)0x3E000000)                     /*!< ADC group regular sequencer rank 6 */
#define ADC_SQR2_SQ12_0                     ((uint32_t)0x02000000)
#define ADC_SQR2_SQ12_1                     ((uint32_t)0x04000000) 
#define ADC_SQR2_SQ12_2                     ((uint32_t)0x08000000) 
#define ADC_SQR2_SQ12_3                     ((uint32_t)0x10000000) 
#define ADC_SQR2_SQ12_4                     ((uint32_t)0x20000000) 

/*******************  Bit definition for ADC_SQR3 register  *******************/
#define ADC_SQR3_SQ1                        ((uint32_t)0x0000001F)                     /*!< ADC group regular sequencer rank 1 */
#define ADC_SQR3_SQ1_0                      ((uint32_t)0x00000001)
#define ADC_SQR3_SQ1_1                      ((uint32_t)0x00000002)
#define ADC_SQR3_SQ1_2                      ((uint32_t)0x00000004)
#define ADC_SQR3_SQ1_3                      ((uint32_t)0x00000008)
#define ADC_SQR3_SQ1_4                      ((uint32_t)0x00000010)

#define ADC_SQR3_SQ2                        ((uint32_t)0x000003E0)                     /*!< ADC group regular sequencer rank 2 */
#define ADC_SQR3_SQ2_0                      ((uint32_t)0x00000020)
#define ADC_SQR3_SQ2_1                      ((uint32_t)0x00000040)
#define ADC_SQR3_SQ2_2                      ((uint32_t)0x00000080) 
#define ADC_SQR3_SQ2_3                      ((uint32_t)0x00000100) 
#define ADC_SQR3_SQ2_4                      ((uint32_t)0x00000200) 

#define ADC_SQR3_SQ3                        ((uint32_t)0x00007C00)                     /*!< ADC group regular sequencer rank 3 */
#define ADC_SQR3_SQ3_0                      ((uint32_t)0x00000400) 
#define ADC_SQR3_SQ3_1                      ((uint32_t)0x00000800) 
#define ADC_SQR3_SQ3_2                      ((uint32_t)0x00001000) 
#define ADC_SQR3_SQ3_3                      ((uint32_t)0x00002000) 
#define ADC_SQR3_SQ3_4                      ((uint32_t)0x00004000) 

#define ADC_SQR3_SQ4                        ((uint32_t)0x000F8000)                     /*!< ADC group regular sequencer rank 4 */
#define ADC_SQR3_SQ4_0                      ((uint32_t)0x00008000) 
#define ADC_SQR3_SQ4_1                      ((uint32_t)0x00010000) 
#define ADC_SQR3_SQ4_2                      ((uint32_t)0x00020000) 
#define ADC_SQR3_SQ4_3                      ((uint32_t)0x00040000) 
#define ADC_SQR3_SQ4_4                      ((uint32_t)0x00080000) 

#define ADC_SQR3_SQ5                        ((uint32_t)0x01F00000)                     /*!< ADC group regular sequencer rank 5 */
#define ADC_SQR3_SQ5_0                      ((uint32_t)0x00100000) 
#define ADC_SQR3_SQ5_1                      ((uint32_t)0x00200000) 
#define ADC_SQR3_SQ5_2                      ((uint32_t)0x00400000) 
#define ADC_SQR3_SQ5_3                      ((uint32_t)0x00800000) 
#define ADC_SQR3_SQ5_4                      ((uint32_t)0x01000000) 

#define ADC_SQR3_SQ6                        ((uint32_t)0x3E000000)                     /*!< ADC group regular sequencer rank 6 */
#define ADC_SQR3_SQ6_0                      ((uint32_t)0x02000000)
#define ADC_SQR3_SQ6_1                      ((uint32_t)0x04000000) 
#define ADC_SQR3_SQ6_2                      ((uint32_t)0x08000000) 
#define ADC_SQR3_SQ6_3                      ((uint32_t)0x10000000) 
#define ADC_SQR3_SQ6_4                      ((uint32_t)0x20000000) 

/*******************  Bit definition for ADC_JSQR register  *******************/
#define ADC_JSQR_JSQ1                       ((uint32_t)0x0000001F)                     /*!< ADC group injected sequencer rank 1 */
#define ADC_JSQR_JSQ1_0                     ((uint32_t)0x00000001) 
#define ADC_JSQR_JSQ1_1                     ((uint32_t)0x00000002) 
#define ADC_JSQR_JSQ1_2                     ((uint32_t)0x00000004) 
#define ADC_JSQR_JSQ1_3                     ((uint32_t)0x00000008) 
#define ADC_JSQR_JSQ1_4                     ((uint32_t)0x00000010) 

#define ADC_JSQR_JSQ2                       ((uint32_t)0x000003E0)                     /*!< ADC group injected sequencer rank 2 */
#define ADC_JSQR_JSQ2_0                     ((uint32_t)0x00000020)
#define ADC_JSQR_JSQ2_1                     ((uint32_t)0x00000040) 
#define ADC_JSQR_JSQ2_2                     ((uint32_t)0x00000080)
#define ADC_JSQR_JSQ2_3                     ((uint32_t)0x00000100) 
#define ADC_JSQR_JSQ2_4                     ((uint32_t)0x00000200)

#define ADC_JSQR_JSQ3                       ((uint32_t)0x00007C00)                     /*!< ADC group injected sequencer rank 3 */
#define ADC_JSQR_JSQ3_0                     ((uint32_t)0x00000400) 
#define ADC_JSQR_JSQ3_1                     ((uint32_t)0x00000800) 
#define ADC_JSQR_JSQ3_2                     ((uint32_t)0x00001000) 
#define ADC_JSQR_JSQ3_3                     ((uint32_t)0x00002000) 
#define ADC_JSQR_JSQ3_4                     ((uint32_t)0x00004000) 

#define ADC_JSQR_JSQ4                       ((uint32_t)0x000F8000)                     /*!< ADC group injected sequencer rank 4 */
#define ADC_JSQR_JSQ4_0                     ((uint32_t)0x00008000) 
#define ADC_JSQR_JSQ4_1                     ((uint32_t)0x00010000)
#define ADC_JSQR_JSQ4_2                     ((uint32_t)0x00020000)
#define ADC_JSQR_JSQ4_3                     ((uint32_t)0x00040000)
#define ADC_JSQR_JSQ4_4                     ((uint32_t)0x00080000)

#define ADC_JSQR_JL                         ((uint32_t)0x00300000)                      /*!< ADC group injected sequencer scan length */
#define ADC_JSQR_JL_0                       ((uint32_t)0x00100000)  
#define ADC_JSQR_JL_1                       ((uint32_t)0x00200000)  

/*******************  Bit definition for ADC_JDR1 register  *******************/
#define ADC_JDR1_JDATA                      ((uint32_t)0x0000FFFF)                      /*!< ADC group injected sequencer rank 1 conversion data */

/*******************  Bit definition for ADC_JDR2 register  *******************/
#define ADC_JDR2_JDATA                      ((uint32_t)0x0000FFFF)                      /*!< ADC group injected sequencer rank 2 conversion data */

/*******************  Bit definition for ADC_JDR3 register  *******************/
#define ADC_JDR3_JDATA                      ((uint32_t)0x0000FFFF)                      /*!< ADC group injected sequencer rank 3 conversion data */

/*******************  Bit definition for ADC_JDR4 register  *******************/
#define ADC_JDR4_JDATA                      ((uint32_t)0x0000FFFF)                      /*!< ADC group injected sequencer rank 4 conversion data */

/********************  Bit definition for ADC_DR register  ********************/
#define ADC_DR_DATA                         ((uint32_t)0x0000FFFF)                      /*!< ADC group regular conversion data */                     /*!< ADC group regular conversion data for ADC slave, in multimode */

/*******************  Bit definition for ADC_CAL register  *******************/
#define ADC_CAL_DATA                        ((uint32_t)0x0000007F)                      /*!< ADC calibration data */

/*******************  Bit definition for ADC_SMPS register  *******************/
#define ADC_SMPS_IDX0                       ((uint32_t)0x00000100)     
#define ADC_SMPS_IDX1                       ((uint32_t)0x00000200)     
#define ADC_SMPS_IDX2                       ((uint32_t)0x00000400)     
#define ADC_SMPS_IDX3                       ((uint32_t)0x00000800)     

/*******************  Bit definition for ADC_SMPSV register  *******************/
#define ADC_SMPV_IDX0                       ((uint32_t)0x000000FF)     
#define ADC_SMPV_IDX1                       ((uint32_t)0x0000FF00)     
#define ADC_SMPV_IDX2                       ((uint32_t)0x00FF0000)     
#define ADC_SMPV_IDX3                       ((uint32_t)0xFF000000)  


/*******************  Bit definition for ADC_SMPR3 register  *******************/
#define ADC_SMPR3_SMP19                     ((uint32_t)0x00000007)    
#define ADC_SMPR3_SMP19_0                   ((uint32_t)0x00000001)
#define ADC_SMPR3_SMP19_1                   ((uint32_t)0x00000002)
#define ADC_SMPR3_SMP19_2                   ((uint32_t)0x00000004)

#define ADC_SMPR3_SMP20                     ((uint32_t)0x00000038)    
#define ADC_SMPR3_SMP20_0                   ((uint32_t)0x00000008)
#define ADC_SMPR3_SMP20_1                   ((uint32_t)0x00000010)
#define ADC_SMPR3_SMP20_2                   ((uint32_t)0x00000020)

#define ADC_SMPR3_SMP21                     ((uint32_t)0x000001C0)    
#define ADC_SMPR3_SMP21_0                   ((uint32_t)0x00000040)
#define ADC_SMPR3_SMP21_1                   ((uint32_t)0x00000080)
#define ADC_SMPR3_SMP21_2                   ((uint32_t)0x00000100)

#define ADC_SMPR3_SMP22                     ((uint32_t)0x00000E00)    
#define ADC_SMPR3_SMP22_0                   ((uint32_t)0x00000200)
#define ADC_SMPR3_SMP22_1                   ((uint32_t)0x00000400)
#define ADC_SMPR3_SMP22_2                   ((uint32_t)0x00000800)

#define ADC_SMPR3_SMP23                     ((uint32_t)0x00007000)    
#define ADC_SMPR3_SMP23_0                   ((uint32_t)0x00001000)
#define ADC_SMPR3_SMP23_1                   ((uint32_t)0x00002000)
#define ADC_SMPR3_SMP23_2                   ((uint32_t)0x00004000)


/*
  * @brief  Driver macro reserved for internal use: isolate bits with the
  *         selected mask and shift them to the register LSB
  *         (shift mask on register position bit 0).
  * @param  __BITS__ Bits in register 32 bits
  * @param  __MASK__ Mask in register 32 bits
  * @retval Bits in register 32 bits
 */
#define __ADC_MASK_SHIFT(__BITS__, __MASK__)                                   \
  (((__BITS__) & (__MASK__)) >> POSITION_VAL((__MASK__)))

/*
  * @brief  Driver macro reserved for internal use: set a pointer to
  *         a register from a register basis from which an offset
  *         is applied.
  * @param  __REG__ Register basis from which the offset is applied.
  * @param  __REG_OFFFSET__ Offset to be applied (unit: number of registers).
  * @retval Pointer to register address
  */
#define __ADC_PTR_REG_OFFSET(__REG__, __REG_OFFFSET__)                         \
 ((uint32_t *)((uint32_t) ((uint32_t)(&(__REG__)) + ((__REG_OFFFSET__) << 2U))))
   
/**********************  Bit definition for option ***********************/ 
#define ADC_CHANNEL_0_NUMBER            ((uint32_t)0x00000000)
#define ADC_CHANNEL_1_NUMBER            (                                                                        ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_2_NUMBER            (                                                      ADC_CR1_AWDCH_1                  )
#define ADC_CHANNEL_3_NUMBER            (                                                      ADC_CR1_AWDCH_1 | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_4_NUMBER            (                                    ADC_CR1_AWDCH_2                                    )
#define ADC_CHANNEL_5_NUMBER            (                                    ADC_CR1_AWDCH_2                   | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_6_NUMBER            (                                    ADC_CR1_AWDCH_2 | ADC_CR1_AWDCH_1                  )
#define ADC_CHANNEL_7_NUMBER            (                                    ADC_CR1_AWDCH_2 | ADC_CR1_AWDCH_1 | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_8_NUMBER            (                  ADC_CR1_AWDCH_3                                                      )
#define ADC_CHANNEL_9_NUMBER            (                  ADC_CR1_AWDCH_3                                     | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_10_NUMBER           (                  ADC_CR1_AWDCH_3 |                   ADC_CR1_AWDCH_1                  )
#define ADC_CHANNEL_11_NUMBER           (                  ADC_CR1_AWDCH_3 |                   ADC_CR1_AWDCH_1 | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_12_NUMBER           (                  ADC_CR1_AWDCH_3 | ADC_CR1_AWDCH_2                                    )
#define ADC_CHANNEL_13_NUMBER           (                  ADC_CR1_AWDCH_3 | ADC_CR1_AWDCH_2                   | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_14_NUMBER           (                  ADC_CR1_AWDCH_3 | ADC_CR1_AWDCH_2 | ADC_CR1_AWDCH_1                  )
#define ADC_CHANNEL_15_NUMBER           (                  ADC_CR1_AWDCH_3 | ADC_CR1_AWDCH_2 | ADC_CR1_AWDCH_1 | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_16_NUMBER           (ADC_CR1_AWDCH_4                                                                        )
#define ADC_CHANNEL_17_NUMBER           (ADC_CR1_AWDCH_4 |                                                       ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_18_NUMBER           (ADC_CR1_AWDCH_4 |                                     ADC_CR1_AWDCH_1                  )
#define ADC_CHANNEL_19_NUMBER           (ADC_CR1_AWDCH_4 |                                     ADC_CR1_AWDCH_1 | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_20_NUMBER           (ADC_CR1_AWDCH_4 |                   ADC_CR1_AWDCH_2                                    )
#define ADC_CHANNEL_21_NUMBER           (ADC_CR1_AWDCH_4 |                   ADC_CR1_AWDCH_2                   | ADC_CR1_AWDCH_0)
#define ADC_CHANNEL_22_NUMBER           (ADC_CR1_AWDCH_4 |                   ADC_CR1_AWDCH_2 | ADC_CR1_AWDCH_1                  )
#define ADC_CHANNEL_23_NUMBER           (ADC_CR1_AWDCH_4 |                   ADC_CR1_AWDCH_2 | ADC_CR1_AWDCH_1 | ADC_CR1_AWDCH_0)

#define ADC_CR1_SCAN_DISABLE            ((uint32_t)0x00000000) 
#define ADC_CR1_SCAN_ENABLE             (ADC_CR1_SCAN)

#define ADC_CR1_JAUTO_INDEPENDENT       ((uint32_t)0x00000000) 
#define ADC_CR1_JAUTO_FROM_REG          (ADC_CR1_JAUTO)

#define ADC_AWD_DISABLE                 ((uint32_t)0x00000000)                                                                                 
#define ADC_AWD_ALL_CHANNELS_REG        (                                        ADC_CR1_AWDEN                 ) 
#define ADC_AWD_ALL_CHANNELS_INJ        (                       ADC_CR1_JAWDEN                                 ) 
#define ADC_AWD_ALL_CHANNELS_REG_INJ    (                       ADC_CR1_JAWDEN | ADC_CR1_AWDEN                 ) 
#define ADC_AWD_CHANNEL_0_REG           (ADC_CHANNEL_0_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_0_INJ           (ADC_CHANNEL_0_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_0_REG_INJ       (ADC_CHANNEL_0_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_1_REG           (ADC_CHANNEL_1_NUMBER | ADC_CR1_AWDEN                  | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_1_INJ           (ADC_CHANNEL_1_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_1_REG_INJ       (ADC_CHANNEL_1_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_2_REG           (ADC_CHANNEL_2_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_2_INJ           (ADC_CHANNEL_2_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_2_REG_INJ       (ADC_CHANNEL_2_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_3_REG           (ADC_CHANNEL_3_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_3_INJ           (ADC_CHANNEL_3_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_3_REG_INJ       (ADC_CHANNEL_3_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_4_REG           (ADC_CHANNEL_4_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_4_INJ           (ADC_CHANNEL_4_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_4_REG_INJ       (ADC_CHANNEL_4_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_5_REG           (ADC_CHANNEL_5_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_5_INJ           (ADC_CHANNEL_5_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_5_REG_INJ       (ADC_CHANNEL_5_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_6_REG           (ADC_CHANNEL_6_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_6_INJ           (ADC_CHANNEL_6_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_6_REG_INJ       (ADC_CHANNEL_6_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_7_REG           (ADC_CHANNEL_7_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_7_INJ           (ADC_CHANNEL_7_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_7_REG_INJ       (ADC_CHANNEL_7_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_8_REG           (ADC_CHANNEL_8_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_8_INJ           (ADC_CHANNEL_8_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_8_REG_INJ       (ADC_CHANNEL_8_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL)  
#define ADC_AWD_CHANNEL_9_REG           (ADC_CHANNEL_9_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_9_INJ           (ADC_CHANNEL_9_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_9_REG_INJ       (ADC_CHANNEL_9_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL)  
#define ADC_AWD_CHANNEL_10_REG          (ADC_CHANNEL_10_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_10_INJ          (ADC_CHANNEL_10_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_10_REG_INJ      (ADC_CHANNEL_10_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_11_REG          (ADC_CHANNEL_11_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_11_INJ          (ADC_CHANNEL_11_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_11_REG_INJ      (ADC_CHANNEL_11_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_12_REG          (ADC_CHANNEL_12_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_12_INJ          (ADC_CHANNEL_12_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_12_REG_INJ      (ADC_CHANNEL_12_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_13_REG          (ADC_CHANNEL_13_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_13_INJ          (ADC_CHANNEL_13_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_13_REG_INJ      (ADC_CHANNEL_13_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_14_REG          (ADC_CHANNEL_14_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_14_INJ          (ADC_CHANNEL_14_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_14_REG_INJ      (ADC_CHANNEL_14_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_15_REG          (ADC_CHANNEL_15_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_15_INJ          (ADC_CHANNEL_15_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_15_REG_INJ      (ADC_CHANNEL_15_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_16_REG          (ADC_CHANNEL_16_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_16_INJ          (ADC_CHANNEL_16_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_16_REG_INJ      (ADC_CHANNEL_16_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_17_REG          (ADC_CHANNEL_17_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_17_INJ          (ADC_CHANNEL_17_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_17_REG_INJ      (ADC_CHANNEL_17_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_18_REG          (ADC_CHANNEL_18_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_18_INJ          (ADC_CHANNEL_18_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_18_REG_INJ      (ADC_CHANNEL_18_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL)
#define ADC_AWD_CHANNEL_19_REG          (ADC_CHANNEL_19_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_19_INJ          (ADC_CHANNEL_19_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_19_REG_INJ      (ADC_CHANNEL_19_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL)
#define ADC_AWD_CHANNEL_20_REG          (ADC_CHANNEL_20_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_20_INJ          (ADC_CHANNEL_20_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_20_REG_INJ      (ADC_CHANNEL_20_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL)
#define ADC_AWD_CHANNEL_21_REG          (ADC_CHANNEL_21_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_21_INJ          (ADC_CHANNEL_21_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_21_REG_INJ      (ADC_CHANNEL_21_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL)
#define ADC_AWD_CHANNEL_22_REG          (ADC_CHANNEL_22_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_22_INJ          (ADC_CHANNEL_22_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_22_REG_INJ      (ADC_CHANNEL_22_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL)
#define ADC_AWD_CHANNEL_23_REG          (ADC_CHANNEL_23_NUMBER                  | ADC_CR1_AWDEN | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_23_INJ          (ADC_CHANNEL_23_NUMBER | ADC_CR1_JAWDEN                 | ADC_CR1_AWDSGL) 
#define ADC_AWD_CHANNEL_23_REG_INJ      (ADC_CHANNEL_23_NUMBER | ADC_CR1_JAWDEN | ADC_CR1_AWDEN | ADC_CR1_AWDSGL)

#define ADC_CR1_DISCONT_DISABLE         ((uint32_t)0x00000000)      
#define ADC_CR1_DISCONT_1RANK           (                                                            ADC_CR1_DISCEN)
#define ADC_CR1_DISCONT_2RANKS          (                                        ADC_CR1_DISCNUM_0 | ADC_CR1_DISCEN)
#define ADC_CR1_DISCONT_3RANKS          (                    ADC_CR1_DISCNUM_1                     | ADC_CR1_DISCEN)
#define ADC_CR1_DISCONT_4RANKS          (                    ADC_CR1_DISCNUM_1 | ADC_CR1_DISCNUM_0 | ADC_CR1_DISCEN) 
#define ADC_CR1_DISCONT_5RANKS          (ADC_CR1_DISCNUM_2                                         | ADC_CR1_DISCEN)
#define ADC_CR1_DISCONT_6RANKS          (ADC_CR1_DISCNUM_2                     | ADC_CR1_DISCNUM_0 | ADC_CR1_DISCEN)
#define ADC_CR1_DISCONT_7RANKS          (ADC_CR1_DISCNUM_2 | ADC_CR1_DISCNUM_1                     | ADC_CR1_DISCEN)
#define ADC_CR1_DISCONT_8RANKS          (ADC_CR1_DISCNUM_2 | ADC_CR1_DISCNUM_1 | ADC_CR1_DISCNUM_0 | ADC_CR1_DISCEN)

#define ADC_CR2_CONT_SINGLE             ((uint32_t)0x00000000) 
#define ADC_CR2_CONT_CONTINUOUS         (ADC_CR2_CONT)

#define ADC_CR2_DMA_NONE                ((uint32_t)0x00000000) 
#define ADC_CR2_DMA_UNLIMITED           (ADC_CR2_DMA)

#define ADC_CR2_ALIGN_RIGHT             ((uint32_t)0x00000000) 
#define ADC_CR2_ALIGN_LEFT              (ADC_CR2_ALIGN)

#define ADC_CR2_JEXTSEL_INSOURCE        ((uint32_t)0x00000000)   
#define ADC_CR2_JEXTSEL_TIM1_TRGO       (                                                                                ADC_CR2_JEXTTRIG)         
#define ADC_CR2_JEXTSEL_TIM1_CC4        (                                                            ADC_CR2_JEXTSEL_0 | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM2_TRGO       (                                        ADC_CR2_JEXTSEL_1                     | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM2_CC1        (                                        ADC_CR2_JEXTSEL_1 | ADC_CR2_JEXTSEL_0 | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM3_CC4        (                    ADC_CR2_JEXTSEL_2                                         | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM4_TRGO       (                    ADC_CR2_JEXTSEL_2                     | ADC_CR2_JEXTSEL_0 | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_EXTI15          (                    ADC_CR2_JEXTSEL_2 | ADC_CR2_JEXTSEL_1                     | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_JSWSTART        (                    ADC_CR2_JEXTSEL_2 | ADC_CR2_JEXTSEL_1 | ADC_CR2_JEXTSEL_0 | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM1_CC1        (ADC_CR2_JEXTSEL_3                                                             | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM1_CC2        (ADC_CR2_JEXTSEL_3 |                                          ADC_CR2_JEXTSEL_0| ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM1_CC3        (ADC_CR2_JEXTSEL_3 |                      ADC_CR2_JEXTSEL_1                    | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM1_CC5        (ADC_CR2_JEXTSEL_3 |                      ADC_CR2_JEXTSEL_1 | ADC_CR2_JEXTSEL_0| ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM1_CC4_CC5    (ADC_CR2_JEXTSEL_3 | ADC_CR2_JEXTSEL_2                                         | ADC_CR2_JEXTTRIG) 
#define ADC_CR2_JEXTSEL_TIM1_CC6        (ADC_CR2_JEXTSEL_3 | ADC_CR2_JEXTSEL_2                      | ADC_CR2_JEXTSEL_0| ADC_CR2_JEXTTRIG) 
                                                           
#define ADC_CR2_EXTSEL_INSOURCE         ((uint32_t)0x00000000)   
#define ADC_CR2_EXTSEL_TIM1_CC1         (                                                         ADC_CR2_EXTTRIG)         
#define ADC_CR2_EXTSEL_TIM1_CC2         (                                      ADC_CR2_EXTSEL_0 | ADC_CR2_EXTTRIG) 
#define ADC_CR2_EXTSEL_TIM1_CC3         (                   ADC_CR2_EXTSEL_1                    | ADC_CR2_EXTTRIG) 
#define ADC_CR2_EXTSEL_TIM2_CC2         (                   ADC_CR2_EXTSEL_1 | ADC_CR2_EXTSEL_0 | ADC_CR2_EXTTRIG) 
#define ADC_CR2_EXTSEL_TIM3_TRGO        (ADC_CR2_EXTSEL_2                                       | ADC_CR2_EXTTRIG) 
#define ADC_CR2_EXTSEL_TIM4_CC4         (ADC_CR2_EXTSEL_2                    | ADC_CR2_EXTSEL_0 | ADC_CR2_EXTTRIG) 
#define ADC_CR2_EXTSEL_EXTI11           (ADC_CR2_EXTSEL_2 | ADC_CR2_EXTSEL_1                    | ADC_CR2_EXTTRIG) 
#define ADC_CR2_EXTSEL_SWSTART          (ADC_CR2_EXTSEL_2 | ADC_CR2_EXTSEL_1 | ADC_CR2_EXTSEL_0 | ADC_CR2_EXTTRIG) 

#define ADC_SAMPLINGTIME_1p5CYCLES      ((uint32_t)0x00000000)
#define ADC_SAMPLINGTIME_2p5CYCLES      (                                      ADC_SMPR2_SMP0_0)
#define ADC_SAMPLINGTIME_3p5CYCLES      (                   ADC_SMPR2_SMP0_1                   )
#define ADC_SAMPLINGTIME_4p5CYCLES      (                   ADC_SMPR2_SMP0_1 | ADC_SMPR2_SMP0_0)
#define ADC_SAMPLINGTIME_7p5CYCLES      (ADC_SMPR2_SMP0_2                                      )
#define ADC_SAMPLINGTIME_13p5CYCLES     (ADC_SMPR2_SMP0_2                    | ADC_SMPR2_SMP0_0)
#define ADC_SAMPLINGTIME_28p5CYCLES     (ADC_SMPR2_SMP0_2 | ADC_SMPR2_SMP0_1                   )
#define ADC_SAMPLINGTIME_764p5CYCLES    (ADC_SMPR2_SMP0_2 | ADC_SMPR2_SMP0_1 | ADC_SMPR2_SMP0_0)

#define ADC_SMPR1_REGOFFSET              ((uint32_t)0x00000000)
#define ADC_SMPR2_REGOFFSET              ((uint32_t)0x00000020)
#define ADC_SMPR3_REGOFFSET              ((uint32_t)0x000002E0)
#define ADC_CHANNEL_SMPRX_REGOFFSET_MASK (ADC_SMPR1_REGOFFSET | ADC_SMPR2_REGOFFSET | ADC_SMPR3_REGOFFSET)

#define ADC_SMPR_BITOFFSET_MASK      ((uint32_t)0x01F00000)
#define ADC_SMPR_BITOFFSET_POS       (20U)   

#define ADC_SMP_0_BITOFFSET  ( 0U)
#define ADC_SMP_1_BITOFFSET  ( 3U)
#define ADC_SMP_2_BITOFFSET  ( 6U)
#define ADC_SMP_3_BITOFFSET  ( 9U) 
#define ADC_SMP_4_BITOFFSET  (12U) 
#define ADC_SMP_5_BITOFFSET  (15U) 
#define ADC_SMP_6_BITOFFSET  (18U)
#define ADC_SMP_7_BITOFFSET  (21U) 
#define ADC_SMP_8_BITOFFSET  (24U) 
#define ADC_SMP_9_BITOFFSET  (27U) 
#define ADC_SMP_10_BITOFFSET ( 0U) 
#define ADC_SMP_11_BITOFFSET ( 3U)
#define ADC_SMP_12_BITOFFSET ( 6U)
#define ADC_SMP_13_BITOFFSET ( 9U) 
#define ADC_SMP_14_BITOFFSET (12U) 
#define ADC_SMP_15_BITOFFSET (15U) 
#define ADC_SMP_16_BITOFFSET (18U)
#define ADC_SMP_17_BITOFFSET (21U) 
#define ADC_SMP_18_BITOFFSET (24U) 
#define ADC_SMP_19_BITOFFSET ( 0U) 
#define ADC_SMP_20_BITOFFSET ( 3U)
#define ADC_SMP_21_BITOFFSET ( 6U)
#define ADC_SMP_22_BITOFFSET ( 9U) 
#define ADC_SMP_23_BITOFFSET (12U) 

#define ADC_CHANNEL_0_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_0_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_1_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_1_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_2_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_2_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_3_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_3_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_4_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_4_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_5_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_5_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_6_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_6_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_7_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_7_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_8_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_8_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_9_SMP        (ADC_SMPR2_REGOFFSET | (ADC_SMP_9_BITOFFSET  << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_10_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_10_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_11_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_11_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_12_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_12_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_13_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_13_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_14_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_14_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_15_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_15_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_16_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_16_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_17_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_17_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_18_SMP       (ADC_SMPR1_REGOFFSET | (ADC_SMP_18_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_19_SMP       (ADC_SMPR3_REGOFFSET | (ADC_SMP_19_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_20_SMP       (ADC_SMPR3_REGOFFSET | (ADC_SMP_20_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_21_SMP       (ADC_SMPR3_REGOFFSET | (ADC_SMP_21_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_22_SMP       (ADC_SMPR3_REGOFFSET | (ADC_SMP_22_BITOFFSET << ADC_SMPR_BITOFFSET_POS))
#define ADC_CHANNEL_23_SMP       (ADC_SMPR3_REGOFFSET | (ADC_SMP_23_BITOFFSET << ADC_SMPR_BITOFFSET_POS))

#define ADC_CHANNEL_0            (ADC_CHANNEL_0_NUMBER  | ADC_CHANNEL_0_SMP)
#define ADC_CHANNEL_1            (ADC_CHANNEL_1_NUMBER  | ADC_CHANNEL_1_SMP)
#define ADC_CHANNEL_2            (ADC_CHANNEL_2_NUMBER  | ADC_CHANNEL_2_SMP)
#define ADC_CHANNEL_3            (ADC_CHANNEL_3_NUMBER  | ADC_CHANNEL_3_SMP)
#define ADC_CHANNEL_4            (ADC_CHANNEL_4_NUMBER  | ADC_CHANNEL_4_SMP)
#define ADC_CHANNEL_5            (ADC_CHANNEL_5_NUMBER  | ADC_CHANNEL_5_SMP)
#define ADC_CHANNEL_6            (ADC_CHANNEL_6_NUMBER  | ADC_CHANNEL_6_SMP)
#define ADC_CHANNEL_7            (ADC_CHANNEL_7_NUMBER  | ADC_CHANNEL_7_SMP)
#define ADC_CHANNEL_8            (ADC_CHANNEL_8_NUMBER  | ADC_CHANNEL_8_SMP)
#define ADC_CHANNEL_9            (ADC_CHANNEL_9_NUMBER  | ADC_CHANNEL_9_SMP)
#define ADC_CHANNEL_10           (ADC_CHANNEL_10_NUMBER | ADC_CHANNEL_10_SMP)
#define ADC_CHANNEL_11           (ADC_CHANNEL_11_NUMBER | ADC_CHANNEL_11_SMP)
#define ADC_CHANNEL_12           (ADC_CHANNEL_12_NUMBER | ADC_CHANNEL_12_SMP)
#define ADC_CHANNEL_13           (ADC_CHANNEL_13_NUMBER | ADC_CHANNEL_13_SMP)
#define ADC_CHANNEL_14           (ADC_CHANNEL_14_NUMBER | ADC_CHANNEL_14_SMP)
#define ADC_CHANNEL_15           (ADC_CHANNEL_15_NUMBER | ADC_CHANNEL_15_SMP)
#define ADC_CHANNEL_16           (ADC_CHANNEL_16_NUMBER | ADC_CHANNEL_16_SMP)
#define ADC_CHANNEL_17           (ADC_CHANNEL_17_NUMBER | ADC_CHANNEL_17_SMP)
#define ADC_CHANNEL_18           (ADC_CHANNEL_18_NUMBER | ADC_CHANNEL_18_SMP)
#define ADC_CHANNEL_19           (ADC_CHANNEL_19_NUMBER | ADC_CHANNEL_19_SMP)
#define ADC_CHANNEL_20           (ADC_CHANNEL_20_NUMBER | ADC_CHANNEL_20_SMP)
#define ADC_CHANNEL_21           (ADC_CHANNEL_21_NUMBER | ADC_CHANNEL_21_SMP)
#define ADC_CHANNEL_22           (ADC_CHANNEL_22_NUMBER | ADC_CHANNEL_22_SMP)
#define ADC_CHANNEL_23           (ADC_CHANNEL_23_NUMBER | ADC_CHANNEL_23_SMP)
#define ADC_CHANNEL_TPS          (ADC_CHANNEL_16)
#define ADC_CHANNEL_VREF         (ADC_CHANNEL_17)
#define ADC_CHANNEL_SH           (ADC_CHANNEL_18)
#define ADC_CHANNEL_OPAMP1       (ADC_CHANNEL_19)
#define ADC_CHANNEL_OPAMP2       (ADC_CHANNEL_20)
#define ADC_CHANNEL_OPAMP3       (ADC_CHANNEL_21)
#define ADC_CHANNEL_OPAMP4       (ADC_CHANNEL_22)
#define ADC_CHANNEL_LDO          (ADC_CHANNEL_23)

#define ADC_AWD_THRESHOLD_HIGH   ((uint32_t)0x00000000)
#define ADC_AWD_THRESHOLD_LOW    ((uint32_t)0x00000001) 

#define ADC_SQRL_SCAN_DISABLE    ((uint32_t)0x00000000)
#define ADC_SQRL_SCAN_2RANKS     (                                             ADC_SQR1_L_0)
#define ADC_SQRL_SCAN_3RANKS     (                              ADC_SQR1_L_1               )
#define ADC_SQRL_SCAN_4RANKS     (                              ADC_SQR1_L_1 | ADC_SQR1_L_0)
#define ADC_SQRL_SCAN_5RANKS     (               ADC_SQR1_L_2                              )
#define ADC_SQRL_SCAN_6RANKS     (               ADC_SQR1_L_2                | ADC_SQR1_L_0)
#define ADC_SQRL_SCAN_7RANKS     (               ADC_SQR1_L_2 | ADC_SQR1_L_1               )
#define ADC_SQRL_SCAN_8RANKS     (               ADC_SQR1_L_2 | ADC_SQR1_L_1 | ADC_SQR1_L_0) 
#define ADC_SQRL_SCAN_9RANKS     (ADC_SQR1_L_3                                             )
#define ADC_SQRL_SCAN_10RANKS    (ADC_SQR1_L_3 |                               ADC_SQR1_L_0)
#define ADC_SQRL_SCAN_11RANKS    (ADC_SQR1_L_3 |                ADC_SQR1_L_1               )
#define ADC_SQRL_SCAN_12RANKS    (ADC_SQR1_L_3 |                ADC_SQR1_L_1 | ADC_SQR1_L_0)
#define ADC_SQRL_SCAN_13RANKS    (ADC_SQR1_L_3 | ADC_SQR1_L_2                              )
#define ADC_SQRL_SCAN_14RANKS    (ADC_SQR1_L_3 | ADC_SQR1_L_2                | ADC_SQR1_L_0)
#define ADC_SQRL_SCAN_15RANKS    (ADC_SQR1_L_3 | ADC_SQR1_L_2 | ADC_SQR1_L_1               )
#define ADC_SQRL_SCAN_16RANKS    (ADC_SQR1_L_3 | ADC_SQR1_L_2 | ADC_SQR1_L_1 | ADC_SQR1_L_0) 

#define ADC_SQR1_REGOFFSET       ((uint32_t)0x00000000)
#define ADC_SQR2_REGOFFSET       ((uint32_t)0x00001000) 
#define ADC_SQR3_REGOFFSET       ((uint32_t)0x00002000)
#define ADC_SQRX_REGOFFSET_MASK  (ADC_SQR1_REGOFFSET | ADC_SQR2_REGOFFSET | ADC_SQR3_REGOFFSET)

#define ADC_REG_RANK_1_BITOFFSET ( 0U)
#define ADC_REG_RANK_2_BITOFFSET ( 5U)
#define ADC_REG_RANK_3_BITOFFSET (10U)
#define ADC_REG_RANK_4_BITOFFSET (15U) 
#define ADC_REG_RANK_5_BITOFFSET (20U) 
#define ADC_REG_RANK_6_BITOFFSET (25U) 
#define ADC_REG_RANK_7_BITOFFSET ( 0U)
#define ADC_REG_RANK_8_BITOFFSET ( 5U) 
#define ADC_REG_RANK_9_BITOFFSET (10U)
#define ADC_REG_RANK_10_BITOFFSET (15U) 
#define ADC_REG_RANK_11_BITOFFSET (20U) 
#define ADC_REG_RANK_12_BITOFFSET (25U)
#define ADC_REG_RANK_13_BITOFFSET ( 0U)
#define ADC_REG_RANK_14_BITOFFSET ( 5U) 
#define ADC_REG_RANK_15_BITOFFSET (10U)
#define ADC_REG_RANK_16_BITOFFSET (15U) 
#define ADC_REG_RANK_MASK        ((uint32_t)0x0000001F)

#define ADC_REG_RANK_1           (ADC_SQR3_REGOFFSET | ADC_REG_RANK_1_BITOFFSET)
#define ADC_REG_RANK_2           (ADC_SQR3_REGOFFSET | ADC_REG_RANK_2_BITOFFSET)
#define ADC_REG_RANK_3           (ADC_SQR3_REGOFFSET | ADC_REG_RANK_3_BITOFFSET)
#define ADC_REG_RANK_4           (ADC_SQR3_REGOFFSET | ADC_REG_RANK_4_BITOFFSET)
#define ADC_REG_RANK_5           (ADC_SQR3_REGOFFSET | ADC_REG_RANK_5_BITOFFSET)
#define ADC_REG_RANK_6           (ADC_SQR3_REGOFFSET | ADC_REG_RANK_6_BITOFFSET)
#define ADC_REG_RANK_7           (ADC_SQR2_REGOFFSET | ADC_REG_RANK_7_BITOFFSET)
#define ADC_REG_RANK_8           (ADC_SQR2_REGOFFSET | ADC_REG_RANK_8_BITOFFSET)
#define ADC_REG_RANK_9           (ADC_SQR2_REGOFFSET | ADC_REG_RANK_9_BITOFFSET)
#define ADC_REG_RANK_10          (ADC_SQR2_REGOFFSET | ADC_REG_RANK_10_BITOFFSET)
#define ADC_REG_RANK_11          (ADC_SQR2_REGOFFSET | ADC_REG_RANK_11_BITOFFSET)
#define ADC_REG_RANK_12          (ADC_SQR2_REGOFFSET | ADC_REG_RANK_12_BITOFFSET)
#define ADC_REG_RANK_13          (ADC_SQR1_REGOFFSET | ADC_REG_RANK_13_BITOFFSET)
#define ADC_REG_RANK_14          (ADC_SQR1_REGOFFSET | ADC_REG_RANK_14_BITOFFSET)
#define ADC_REG_RANK_15          (ADC_SQR1_REGOFFSET | ADC_REG_RANK_15_BITOFFSET)
#define ADC_REG_RANK_16          (ADC_SQR1_REGOFFSET | ADC_REG_RANK_16_BITOFFSET)

#define ADC_JDR1_REGOFFSET       ((uint32_t)0x00000000)
#define ADC_JDR2_REGOFFSET       ((uint32_t)0x00000100)
#define ADC_JDR3_REGOFFSET       ((uint32_t)0x00000200)
#define ADC_JDR4_REGOFFSET       ((uint32_t)0x00000300)

#define ADC_JOFR1_REGOFFSET      ((uint32_t)0x00000000)
#define ADC_JOFR2_REGOFFSET      ((uint32_t)0x00001000)
#define ADC_JOFR3_REGOFFSET      ((uint32_t)0x00002000)
#define ADC_JOFR4_REGOFFSET      ((uint32_t)0x00003000)

#define ADC_JDRX_REGOFFSET_MASK  (ADC_JDR1_REGOFFSET | ADC_JDR2_REGOFFSET | ADC_JDR3_REGOFFSET | ADC_JDR4_REGOFFSET)
#define ADC_JOFRX_REGOFFSET_MASK (ADC_JOFR1_REGOFFSET | ADC_JOFR2_REGOFFSET | ADC_JOFR3_REGOFFSET | ADC_JOFR4_REGOFFSET)

#define ADC_INJ_RANK_1           (ADC_JDR1_REGOFFSET | ADC_JOFR1_REGOFFSET | 0x00000001)
#define ADC_INJ_RANK_2           (ADC_JDR2_REGOFFSET | ADC_JOFR2_REGOFFSET | 0x00000002)      
#define ADC_INJ_RANK_3           (ADC_JDR3_REGOFFSET | ADC_JOFR3_REGOFFSET | 0x00000003)       
#define ADC_INJ_RANK_4           (ADC_JDR4_REGOFFSET | ADC_JOFR4_REGOFFSET | 0x00000004)
#define ADC_INJ_RANK_MASK        ((uint32_t)0x0000001F)

#define ADC_JSQRL_SCAN_DISABLE   ((uint32_t)0x00000000)
#define ADC_JSQRL_SCAN_2RANKS    (                ADC_JSQR_JL_0)
#define ADC_JSQRL_SCAN_3RANKS    (ADC_JSQR_JL_1                )
#define ADC_JSQRL_SCAN_4RANKS    (ADC_JSQR_JL_1 | ADC_JSQR_JL_0)

#define ADC_IDX_0                ((uint32_t)0x00000000)
#define ADC_IDX_1                ((uint32_t)0x00000001)
#define ADC_IDX_2                ((uint32_t)0x00000002)
#define ADC_IDX_3                ((uint32_t)0x00000003)

#define ADC_SMPS_IDX_SMPR        ((uint32_t)0x00000000)  
#define ADC_SMPS_IDX_SMPV         ADC_SMPS_IDX0     


/*******************  Bit definition for SH register  *******************/
#define SH_TRIG_EN               ((uint32_t)0x00000001)
#define SH_BUSY                  ((uint32_t)0x00000002)
#define SH_EN                    ((uint32_t)0x00000004)

#define SH_TRIG_SEL              ((uint32_t)0x00000070)
#define SH_TRIG_SEL_0            ((uint32_t)0x00000010)
#define SH_TRIG_SEL_1            ((uint32_t)0x00000020)
#define SH_TRIG_SEL_2            ((uint32_t)0x00000040)

#define SH_CH_SEL                ((uint32_t)0x00000380)
#define SH_CH_SEL_0              ((uint32_t)0x00000080)
#define SH_CH_SEL_1              ((uint32_t)0x00000100)
#define SH_CH_SEL_2              ((uint32_t)0x00000200)

#define SH_DELAY_SEL              ((uint32_t)0x00000C00)
#define SH_DELAY_SEL_0            ((uint32_t)0x00000400)
#define SH_DELAY_SEL_1            ((uint32_t)0x00000800)

#define SH_SMP                    ((uint32_t)0x00007000)
#define SH_SMP_0                  ((uint32_t)0x00001000)
#define SH_SMP_1                  ((uint32_t)0x00002000)
#define SH_SMP_2                  ((uint32_t)0x00004000)

/**********************  Bit definition for option ***********************/ 
#define SH_TRIG_DISABLE          ((uint32_t)0x00000000)
#define SH_TRIG_ENABLE           ((uint32_t)0x00000001)

#define SH_EN_DISABLE            ((uint32_t)0x00000000)
#define SH_EN_ENABLE             ((uint32_t)0x00000004)

#define SH_TRIG_TIM1_CC4         ((uint32_t)0x00000000)
#define SH_TRIG_TIM1_CC5         (                                SH_TRIG_SEL_0)
#define SH_TRIG_TIM1_CC4_CC5     (                SH_TRIG_SEL_1                )
#define SH_TRIG_TIM2_CC1         (                SH_TRIG_SEL_1 | SH_TRIG_SEL_0)
#define SH_TRIG_TIM3_CC1         (SH_TRIG_SEL_2                                )
#define SH_TRIG_TIM4_CC1         (SH_TRIG_SEL_2                 | SH_TRIG_SEL_0)

#define SH_CH_OPAMP1               ((uint32_t)0x00000000)
#define SH_CH_OPAMP2               (                            SH_CH_SEL_0)
#define SH_CH_OPAMP3               (              SH_CH_SEL_1              )
#define SH_CH_OPAMP4               (              SH_CH_SEL_1 | SH_CH_SEL_0)
#define SH_CH_ADC_IN12           (SH_CH_SEL_2                            )
#define SH_CH_ADC_IN8            (SH_CH_SEL_2               | SH_CH_SEL_0)
#define SH_CH_ADC_IN6            (SH_CH_SEL_2 | SH_CH_SEL_1              )

#define SH_DELAY_10ns             ((uint32_t)0x00000000)
#define SH_DELAY_20ns             (                 SH_DELAY_SEL_0)
#define SH_DELAY_30ns             (SH_DELAY_SEL_1                 )
#define SH_DELAY_40ns             (SH_DELAY_SEL_1 | SH_DELAY_SEL_0)

#define SH_SMP_3p5CYCLES         ((uint32_t)0x00000000)
#define SH_SMP_4p5CYCLES         (                      SH_SMP_0)
#define SH_SMP_5p5CYCLES         (           SH_SMP_1           )
#define SH_SMP_6p5CYCLES         (           SH_SMP_1 | SH_SMP_0)
#define SH_SMP_8p5CYCLES         (SH_SMP_2                      )
#define SH_SMP_10p5CYCLES        (SH_SMP_2            | SH_SMP_0)
#define SH_SMP_25p5CYCLES        (SH_SMP_2 | SH_SMP_1           )
#define SH_SMP_40p5CYCLES        (SH_SMP_2 | SH_SMP_1 | SH_SMP_0)


typedef struct
{
	uint32_t TriggerSource;
	uint32_t Continuous;
	uint32_t ScanMode;
	uint32_t DataAlignment;
  uint32_t SequencerNbRanks;
	uint32_t DMATransfer;
} ADC_Reg_InitTypeDef;

typedef struct
{
	uint32_t TriggerSource;
	uint32_t ScanMode;
	uint32_t DataAlignment;
  uint32_t SequencerNbRanks;
  uint32_t TrigAuto;
} ADC_Inj_InitTypeDef;

typedef struct
{
	uint32_t REGTriggerSource;
  uint32_t INJTriggerSource;
	uint32_t Continuous;
	uint32_t ScanMode;
	uint32_t DataAlignment;
  uint32_t REGSequencerNbRanks;
  uint32_t INJSequencerNbRanks;
	uint32_t DMATransfer;
  uint32_t TrigAuto;
  uint32_t AWDChannelGroup;
} ADC_InitTypeDef;


typedef struct
{
	uint32_t Channel;
	uint32_t SamplingTime;
}RankType;
typedef struct
{
  uint32_t Length;
  RankType Rank[16];
} ADC_REG_RankInitTypeDef;

typedef struct
{
  uint32_t Length;
  RankType Rank[4];
} ADC_INJ_RankInitTypeDef;


void ADC_Reg_StructInit(ADC_Reg_InitTypeDef* ADC_Reg_InitStruct);
void ADC_Reg_Init(ADC_TypeDef *ADCx,ADC_Reg_InitTypeDef* ADC_Reg_InitStruct);
void ADC_Inj_StructInit(ADC_Inj_InitTypeDef* ADC_Inj_InitStruct);
void ADC_Inj_Init(ADC_TypeDef *ADCx,ADC_Inj_InitTypeDef* ADC_Inj_InitStruct);
void ADC_REG_RankInit(ADC_TypeDef* ADCx,ADC_REG_RankInitTypeDef* ADC_REG_RankInitStruct);
void ADC_INJ_RankInit(ADC_TypeDef* ADCx,ADC_INJ_RankInitTypeDef* ADC_INJ_RankInitStruct);
uint32_t ADC_Get_SR(ADC_TypeDef* ADCx, uint32_t IF);
void ADC_Clear_SR(ADC_TypeDef *ADCx, uint32_t IF);
void ADC_Set_AWDChannels(ADC_TypeDef *ADCx, uint32_t AWDChannelGroup);
void ADC_Enable_IT(ADC_TypeDef *ADCx,uint16_t IT);
void ADC_Disable_IT(ADC_TypeDef *ADCx,uint16_t IT);
uint32_t ADC_Get_IT(ADC_TypeDef *ADCx,uint16_t IT);
void ADC_Set_SCAN(ADC_TypeDef *ADCx, uint32_t ScanMode);
void ADC_Set_JAUTO(ADC_TypeDef *ADCx, uint32_t TrigAuto);
void ADC_Set_DISCNUM(ADC_TypeDef *ADCx, uint32_t SeqDiscont);
void ADC_Enable_DISCEN(ADC_TypeDef *ADCx);
void ADC_Disable_DISCEN(ADC_TypeDef *ADCx);
void ADC_Enable_JDISCEN(ADC_TypeDef *ADCx);
void ADC_Disable_JDISCEN(ADC_TypeDef *ADCx);
void ADC_Enable_ADON(ADC_TypeDef *ADCx);
void ADC_Disable_ADON(ADC_TypeDef *ADCx);
uint32_t ADC_Get_ADON(ADC_TypeDef *ADCx);
void ADC_Set_CONT(ADC_TypeDef *ADCx, uint32_t Continuous);
void ADC_CAL(ADC_TypeDef *ADCx);
uint32_t ADC_Get_CAL(ADC_TypeDef *ADCx);
void ADC_RSTCAL(ADC_TypeDef *ADCx);
uint32_t ADC_Get_RSTCAL(ADC_TypeDef *ADCx);
void ADC_Set_DMA(ADC_TypeDef *ADCx, uint32_t DMATransfer);
void ADC_Set_ALIGN(ADC_TypeDef *ADCx, uint32_t DataAlignment);
void ADC_Set_JEXTSEL(ADC_TypeDef *ADCx, uint32_t TriggerSource);
void ADC_Set_EXTSEL(ADC_TypeDef *ADCx, uint32_t TriggerSource);
void ADC_Start_SW(ADC_TypeDef *ADCx);
void ADC_Start_JSW(ADC_TypeDef *ADCx);
void ADC_Enable_TSVREFE(ADC_TypeDef *ADCx);
void ADC_Disable_TSVREFE(ADC_TypeDef *ADCx);
void ADC_Set_SMPR(ADC_TypeDef *ADCx, uint32_t Channel, uint32_t SamplingTime);
void ADC_Set_JOFR(ADC_TypeDef *ADCx, uint32_t Rank, uint32_t offest);
uint32_t ADC_Read_JOFR(ADC_TypeDef *ADCx, uint32_t Rank);
void ADC_Set_AWDThresholds(ADC_TypeDef *ADCx, uint32_t AWDThresholdsHighLow, uint32_t AWDThresholdValue);
uint32_t ADC_Read_AWDThresholds(ADC_TypeDef *ADCx, uint32_t AWDThresholdsHighLow);
void ADC_Set_SQRL(ADC_TypeDef *ADCx, uint32_t SequencerNbRanks);
void ADC_Set_SQ(ADC_TypeDef *ADCx, uint32_t Rank, uint32_t Channel);
void ADC_Set_JSQRL(ADC_TypeDef *ADCx, uint32_t SequencerNbRanks);
void ADC_Set_JSQ(ADC_TypeDef *ADCx, uint32_t Rank, uint32_t Channel);
uint32_t ADC_Read_JDR(ADC_TypeDef *ADCx, uint32_t Rank);
uint32_t ADC_Read_DR(ADC_TypeDef *ADCx);
uint16_t ADC_Read_DR_DATA(ADC_TypeDef *ADCx);
void ADC_Set_CAL_DATA(ADC_TypeDef *ADCx, uint32_t offest);
uint32_t ADC_Read_CAL_DATA(ADC_TypeDef *ADCx);
void ADC_Set_SMPS(ADC_TypeDef *ADCx, uint32_t Idx, uint32_t Sel);
void ADC_Set_SMPV(ADC_TypeDef *ADCx, uint32_t Idx, uint32_t Value);



#ifdef __cplusplus
}
#endif

#endif /* __rx32s12xx_ADC_H__ */
