/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2019, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_gpio.h
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#ifndef _rx32s12xx_GPIO_H
#define _rx32s12xx_GPIO_H

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s12xx.h"
#include "main.h"

typedef struct
{
  uint32_t Pin;        /*!< Specifies the GPIO pins to be configured.
                           This parameter can be any value of @ref GPIO_pins */

  uint32_t Mode;       /*!< Specifies the operating mode for the selected pins.
                           This parameter can be a value of @ref GPIO_mode */

  uint32_t Pull;       /*!< Specifies the Pull-up or Pull-Down activation for the selected pins.
                           This parameter can be a value of @ref GPIO_pull */

  uint32_t Speed;      /*!< Specifies the speed for the selected pins.
                           This parameter can be a value of @ref GPIO_speed */

  uint32_t Alternate;  /*!< Peripheral to be connected to the selected pins 
                            This parameter can be a value of @ref GPIOx_Alternate_function_selection */
}GPIO_InitTypeDef;


/* Exported constants --------------------------------------------------------*/


/*
* @brief  GPIO_PIN
*/
#define GPIO_PIN_0                 (BIT0)     /*!< Pin 0 selected    */
#define GPIO_PIN_1                 (BIT1)     /*!< Pin 1 selected    */
#define GPIO_PIN_2                 (BIT2)     /*!< Pin 2 selected    */
#define GPIO_PIN_3                 (BIT3)     /*!< Pin 3 selected    */
#define GPIO_PIN_4                 (BIT4)     /*!< Pin 4 selected    */
#define GPIO_PIN_5                 (BIT5)     /*!< Pin 5 selected    */
#define GPIO_PIN_6                 (BIT6)     /*!< Pin 6 selected    */
#define GPIO_PIN_7                 (BIT7)     /*!< Pin 7 selected    */                                   /*!< Select all pins */
#define GPIO_PIN_ALL               (0xFF) 		/*!< Pin ALL selected   */                                      /*!< Select all pins */

/******************************************************************************/
/*                                                                            */
/*                General Purpose and Alternate Function I/O                  */
/*                                                                            */
/******************************************************************************/

/*******************  Bit definition for GPIO_MODER register  *******************/
#define GPIO_MODER                             0x00000003
#define GPIO_MODER_MODER_Input                 0x00000000
#define GPIO_MODER_MODER_Output                0x00000001
#define GPIO_MODER_MODER_AF                    0x00000002
#define GPIO_MODER_MODER_Analog                0x00000003

#define  GPIO_MODE_INPUT                       (0x00000000U)   /*!< Input Floating Mode                   */
#define  GPIO_MODE_OUTPUT_PP                   (0x00000001U)   /*!< Output Push Pull Mode                 */
#define  GPIO_MODE_OUTPUT_OD                   (0x00000011U)   /*!< Output Open Drain Mode                */
#define  GPIO_MODE_AF_PP                       (0x00000002U)   /*!< Alternate Function Push Pull Mode     */
#define  GPIO_MODE_AF_OD                       (0x00000012U)   /*!< Alternate Function Open Drain Mode    */
#define  GPIO_MODE_ANALOG                      (0x00000003U)   /*!< Analog Mode  */

/*******************  Bit definition for GPIO_OTYPER register  *******************/
#define GPIO_OTYPER                            0x00000001
#define GPIO_OTYPER_OUT_Pushpull               0x00000000
#define GPIO_OTYPER_OUT_Opendrain              0x00000001

/*!<******************  Bit definition for GPIO_OSPEEDR register  *******************/
#define GPIO_OSPEEDR                           0x00000003
#define GPIO_OSPEEDR_Low                       0x00000000
#define GPIO_OSPEEDR_Medium                    0x00000001
#define GPIO_OSPEEDR_High                   	 0x00000002
#define GPIO_OSPEEDR_VeryHigh                  0x00000003

/*!<******************  Bit definition for GPIO_PUPDR register  *******************/
#define GPIO_PUPDR                             0x00000003
#define GPIO_PUPDR_Floating 			             0x00000000
#define GPIO_PUPDR_Pullup             			   0x00000001
#define GPIO_PUPDR_Pulldown          			     0x00000002
#define GPIO_PUPDR_reserved           			   0x00000003

/*!<******************  Bit definition for GPIO_IDR register  *******************/
#define GPIO_IDR                               0x0000FFFF
#define GPIO_IDR_IDR0                          0x00000001
#define GPIO_IDR_IDR1                          0x00000002
#define GPIO_IDR_IDR2                          0x00000004
#define GPIO_IDR_IDR3                          0x00000008
#define GPIO_IDR_IDR4                          0x00000010
#define GPIO_IDR_IDR5                          0x00000020
#define GPIO_IDR_IDR6                          0x00000040
#define GPIO_IDR_IDR7                          0x00000080

/*!<******************  Bit definition for GPIO_ODR register  *******************/
#define GPIO_ODR                               0x0000FFFF
#define GPIO_ODR_ODR0                          0x00000001
#define GPIO_ODR_ODR1                          0x00000002
#define GPIO_ODR_ODR2                          0x00000004
#define GPIO_ODR_ODR3                          0x00000008
#define GPIO_ODR_ODR4                          0x00000010
#define GPIO_ODR_ODR5                          0x00000020
#define GPIO_ODR_ODR6                          0x00000040
#define GPIO_ODR_ODR7                          0x00000080

/*!<******************  Bit definition for GPIO_BSRR register  *******************/
#define GPIO_BSRR                              0xFFFFFFFF

#define GPIO_BSRR_BR0                          0x00010000
#define GPIO_BSRR_BR1                          0x00020000
#define GPIO_BSRR_BR2                          0x00040000
#define GPIO_BSRR_BR3                          0x00080000
#define GPIO_BSRR_BR4                          0x00100000
#define GPIO_BSRR_BR5                          0x00200000
#define GPIO_BSRR_BR6                          0x00400000
#define GPIO_BSRR_BR7                          0x00800000

#define GPIO_BSRR_BS0                          0x00000001
#define GPIO_BSRR_BS1                          0x00000002
#define GPIO_BSRR_BS2                          0x00000004
#define GPIO_BSRR_BS3                          0x00000008
#define GPIO_BSRR_BS4                          0x00000010
#define GPIO_BSRR_BS5                          0x00000020
#define GPIO_BSRR_BS6                          0x00000040
#define GPIO_BSRR_BS7                          0x00000080

/*!<******************  Bit definition for GPIO_LCKR register  *******************/
#define GPIO_LCKR                              0xFFFFFFFF

#define GPIO_LCKR_LCKK                         0x00010000

#define GPIO_LCKR_LCK0                         0x00000001
#define GPIO_LCKR_LCK1                         0x00000002
#define GPIO_LCKR_LCK2                         0x00000004
#define GPIO_LCKR_LCK3                         0x00000008
#define GPIO_LCKR_LCK4                         0x00000010
#define GPIO_LCKR_LCK5                         0x00000020
#define GPIO_LCKR_LCK6                         0x00000040
#define GPIO_LCKR_LCK7                         0x00000080

/*!<******************  Bit definition for GPIO_AFRL register  *******************/
#define GPIO_AFRL                              0x0000000F
#define GPIO_AF0                        			 0x00000000
#define GPIO_AF1                       				 0x00000001
#define GPIO_AF2                        			 0x00000002
#define GPIO_AF3                          		 0x00000003
#define GPIO_AF4                        			 0x00000004
#define GPIO_AF5                         			 0x00000005
#define GPIO_AF6                      				 0x00000006
#define GPIO_AF7                       				 0x00000007

/*!<******************  Bit definition for AFIO register  *******************/
/** @defgroup GPIO_EC_EXTI_PORT GPIO EXTI PORT
  * @{
  */
#define GPIO_AF_EXTI_PORTA          					 0U   /*!< EXTI PORT A */
#define GPIO_AF_EXTI_PORTB           					 1U   /*!< EXTI PORT B */
#define GPIO_AF_EXTI_PORTC          				   2U   /*!< EXTI PORT C */
#define GPIO_AF_EXTI_PORTD           				   3U   /*!< EXTI PORT D */


/** @defgroup GPIO_EC_EXTI_LINE GPIO EXTI LINE
  * @{
  */
#define GPIO_AF_EXTI_LINE0        				    (0x000FU << 16U | 0U)  /*!< EXTI_POSITION_0  | EXTICR[0] */
#define GPIO_AF_EXTI_LINE1         				    (0x00F0U << 16U | 0U)  /*!< EXTI_POSITION_4  | EXTICR[0] */
#define GPIO_AF_EXTI_LINE2          				  (0x0F00U << 16U | 0U)  /*!< EXTI_POSITION_8  | EXTICR[0] */
#define GPIO_AF_EXTI_LINE3          				  (0xF000U << 16U | 0U)  /*!< EXTI_POSITION_12 | EXTICR[0] */
#define GPIO_AF_EXTI_LINE4          				  (0x000FU << 16U | 1U)  /*!< EXTI_POSITION_0  | EXTICR[1] */
#define GPIO_AF_EXTI_LINE5           				  (0x00F0U << 16U | 1U)  /*!< EXTI_POSITION_4  | EXTICR[1] */
#define GPIO_AF_EXTI_LINE6           				  (0x0F00U << 16U | 1U)  /*!< EXTI_POSITION_8  | EXTICR[1] */
#define GPIO_AF_EXTI_LINE7          				  (0xF000U << 16U | 1U)  /*!< EXTI_POSITION_12 | EXTICR[1] */


void GPIO_DeInit(GPIO_TypeDef *GPIOx);
void GPIO_Init(GPIO_TypeDef  *GPIOx, GPIO_InitTypeDef *GPIO_Init);
void GPIO_Set_PinMode(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t Mode);
uint32_t GPIO_Get_PinMode(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Set_PinOutputType(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t OutputType);
uint32_t GPIO_Get_PinOutputType(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Set_PinOutSpeed(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t  Speed);
uint32_t GPIO_Get_PinOutSpeed(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Set_PinPull(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t Pull);
uint32_t GPIO_Get_PinPull(GPIO_TypeDef *GPIOx, uint32_t Pin);
uint32_t GPIO_Read_InputPort(GPIO_TypeDef *GPIOx);
uint32_t GPIO_Read_PinIDR(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Write_ODR(GPIO_TypeDef *GPIOx, uint32_t Port);
uint32_t GPIO_Read_ODR(GPIO_TypeDef *GPIOx);
uint32_t GPIO_Read_PinODR(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Set_OutputPin(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Reset_OutputPin(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Toggle_Pin(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Lock_Pin(GPIO_TypeDef *GPIOx, uint32_t Pin);
uint32_t GPIO_Get_PinLocked(GPIO_TypeDef *GPIOx, uint32_t Pin);
uint32_t GPIO_Get_AnyPinLocked(GPIO_TypeDef *GPIOx);
void GPIO_Set_AF(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t Alternate);
uint32_t GPIO_Get_AF(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Set_EXTISource(uint32_t Port, uint32_t Line);
uint32_t GPIO_Get_EXTISource(uint32_t Line);


#ifdef __cplusplus
}
#endif

#endif /* __rx32s12xx_GPIO_H__ */

