/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2019, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_opamp.h
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#ifndef _rx32s12xx_OPAMP_H
#define _rx32s12xx_OPAMP_H

#ifdef __cplusplus
extern "C" {
#endif
	
#include "rx32s12xx.h"
#include "main.h"


/*
**********************************************************************************************************
*                         Peripheral Registers_Bits_Definition of OPAMP Module
**********************************************************************************************************
*/

/**********************  Bit definition for OPAMPx register of OPAMP_TypeDef ***********************/
#define  OPAMP_CSR                                      ((uint32_t)0xDFFFF9F3)	
#define  OPAMP_CSR_OPAEN                                ((uint32_t)0x00000001)
#define  OPAMP_CSR_FORCEVP                              ((uint32_t)0x00000002)
#define  OPAMP_CSR_USERTRIM                             ((uint32_t)0x00000010)
#define  OPAMP_CSR_VMSEL                                ((uint32_t)0x00000060)
#define  OPAMP_CSR_VMSEL_0                              ((uint32_t)0x00000020)
#define  OPAMP_CSR_VMSEL_1                              ((uint32_t)0x00000040)
#define  OPAMP_CSR_OPAHSM                               ((uint32_t)0x00000080)
#define  OPAMP_CSR_OPAINTOEN                            ((uint32_t)0x00000100)
#define  OPAMP_CSR_CALON                                ((uint32_t)0x00000800)
#define  OPAMP_CSR_CALSEL                               ((uint32_t)0x00003000)
#define  OPAMP_CSR_CALSEL_0                             ((uint32_t)0x00001000)
#define  OPAMP_CSR_CALSEL_1                             ((uint32_t)0x00002000)
#define  OPAMP_CSR_GAIN                                 ((uint32_t)0x0007C000)
#define  OPAMP_CSR_GAIN_0                               ((uint32_t)0x00004000)
#define  OPAMP_CSR_GAIN_1                               ((uint32_t)0x00008000)
#define  OPAMP_CSR_GAIN_2                               ((uint32_t)0x00010000)
#define  OPAMP_CSR_GAIN_3                               ((uint32_t)0x00020000)
#define  OPAMP_CSR_GAIN_4                               ((uint32_t)0x00040000)
#define  OPAMP_CSR_TRIMOFFSETP_Pos                       (19U)
#define  OPAMP_CSR_TRIMOFFSETP                          ((uint32_t)0x00F80000)
#define  OPAMP_CSR_TRIMOFFSETN_Pos                       (24U)
#define  OPAMP_CSR_TRIMOFFSETN                          ((uint32_t)0x1F000000)
#define  OPAMP_CSR_CALOUT                               ((uint32_t)0x40000000)
#define  OPAMP_CSR_LOCK                                 ((uint32_t)0x80000000)



/* Internal mask for OPAMP trimming of transistors differential pair NMOS     */
/* or PMOS.                                                                   */
/* To select into literal OPAMP_TRIMMING_x the relevant bits for:          */
/* - OPAMP trimming selection of transistors differential pair                */
/* - OPAMP trimming values of transistors differential pair                   */
#define OPAMP_TRIMMING_SELECT_MASK          (OPAMP_CSR_CALSEL)
#define OPAMP_TRIMMING_VALUE_MASK           (OPAMP_CSR_TRIMOFFSETN | OPAMP_CSR_TRIMOFFSETP)


/** @defgroup OPAMP_EC_MODE OPAMP mode calibration or functional.
  * @{
  */
#define OPAMP_MODE_FUNCTIONAL        (0x00000000UL)                              /*!< OPAMP functional mode */
#define OPAMP_MODE_CALIBRATION       (OPAMP_CSR_CALON)                           /*!< OPAMP calibration mode */


/** @defgroup OPAMP_EC_FUNCTIONAL_MODE OPAMP functional mode
  * @{
  */
#define OPAMP_MODE_N1                (0x00000000UL)                                                /*!< OPA1_N1 (PB1) pin connected to OPAMP VINM input */
#define OPAMP_MODE_N2                (OPAMP_CSR_VMSEL_0)                                           /*!< OPA1_N2(PA5) pin connected to OPAMP VINM input */
#define OPAMP_MODE_PGA               (OPAMP_CSR_VMSEL_1)                                           /*!< OPAMP functional mode, OPAMP operation in PGA */
#define OPAMP_MODE_FOLLOWER          (OPAMP_CSR_VMSEL_1 | OPAMP_CSR_VMSEL_0)                       /*!< OPAMP functional mode, OPAMP operation in follower */


/** @defgroup OPAMP_EC_MODE_PGA_GAIN OPAMP gain (relevant when OPAMP is in functional mode PGA)
  * @{
  */
#define OPAMP_GAIN_2            (0x00000000UL)                                           /*!< OPAMP gain 2  */
#define OPAMP_GAIN_4            (                                      OPAMP_CSR_GAIN_0) /*!< OPAMP gain 4  */
#define OPAMP_GAIN_8            (                   OPAMP_CSR_GAIN_1                   ) /*!< OPAMP gain 8  */
#define OPAMP_GAIN_16           (                   OPAMP_CSR_GAIN_1 | OPAMP_CSR_GAIN_0) /*!< OPAMP gain 16 */
#define OPAMP_GAIN_32           (OPAMP_CSR_GAIN_2                                      ) /*!< OPAMP gain 32 */
#define OPAMP_GAIN_64           (OPAMP_CSR_GAIN_2 |                    OPAMP_CSR_GAIN_0) /*!< OPAMP gain 64 */

/** @defgroup OPAMP_EC_INPUT_INVERTING OPAMP input inverting
  * @note OPAMP inverting input is used with OPAMP in mode standalone with negative gain or bias.
  *       Otherwise (OPAMP in mode follower), OPAMP inverting input is not used (not connected to GPIO pin).
  * @{
  */
#define OPAMP_INPUT_INVERT_IO0         (0x00000000UL)           /*!< OPAMP inverting input connected to I/O VINM0
                                                                     (PB1  for OPAMP1, PA5  for OPAMP2, PA1  for OPAMP3, PC13 for OPAMP4)
                                                                     Note: On this STM32 series, all OPAMPx are not available on all devices. Refer to device datasheet for more details */
#define OPAMP_INPUT_INVERT_IO1         OPAMP_CSR_VMSEL_0        /*!< OPAMP inverting input connected to I/0 VINM1
                                                                     (PA5  for OPAMP1, PA7  for OPAMP2, PA5 for OPAMP3, PC15  for OPAMP4)
                                                                     Note: On this STM32 series, all OPAMPx are not available on all devices. Refer to device datasheet for more details */
#define OPAMP_INPUT_INVERT_CONNECT_NO  OPAMP_CSR_VMSEL_1        /*!< OPAMP inverting input not externally connected (intended for OPAMP in mode follower with positive gain without bias).
                                                                        Note: On this STM32 series, this literal include cases of value 0x11 for mode follower and value 0x10 for mode PGA. */

/** @defgroup OPAMP_EC_INTERNAL_OUPUT_MODE OPAMP internal output mode
  * @{
  */
#define OPAMP_INTERNAL_OUPUT_DISABLED       (0x00000000UL)         /*!< OPAMP internal output to ADC disabled. */
#define OPAMP_INTERNAL_OUPUT_ENABLED        OPAMP_CSR_OPAINTOEN    /*!< OPAMP internal output to ADC enabled.
                                                                             - OPAMP1 internal output is connected to ADC1/Channel13
                                                                             - OPAMP2 internal output is connected to ADC2/Channel16
                                                                             - OPAMP3 internal output is connected to ADC2/Channel18 & ADC3/Channel13
                                                                             - OPAMP4 internal output is connected to ADC5/Channel5
                                                                             - OPAMP5 internal output is connected to ADC5/Channel3
                                                                             - OPAMP6 internal output is connected to ADC4/Channel17
                                                                           Note: On this STM32 series, all OPAMPx are not available on all devices. Refer to device datasheet for more details */


/** @defgroup OPAMP_EC_TRIMMING_MODE OPAMP trimming mode
  * @{
  */
#define OPAMP_TRIMMING_FACTORY       (0x00000000UL)           /*!< OPAMP trimming factors set to factory values */
#define OPAMP_TRIMMING_USER          OPAMP_CSR_USERTRIM      /*!< OPAMP trimming factors set to user values */

/** @defgroup OPAMP_EC_TRIMMING_TRANSISTORS_DIFF_PAIR OPAMP trimming of transistors differential pair NMOS or PMOS
  * @{
  */
#define OPAMP_TRIMMING_NMOS_VREF_90PC_VDDA  (OPAMP_CSR_TRIMOFFSETN | OPAMP_CSR_CALSEL_1 | OPAMP_CSR_CALSEL_0) /*!< OPAMP trimming of transistors differential pair NMOS (internal reference voltage set to 0.9*Vdda). Default parameters to be used for calibration using two trimming steps (one with each transistors differential pair NMOS and PMOS). */
#define OPAMP_TRIMMING_NMOS_VREF_50PC_VDDA  (OPAMP_CSR_TRIMOFFSETN | OPAMP_CSR_CALSEL_1                     ) /*!< OPAMP trimming of transistors differential pair NMOS (internal reference voltage set to 0.5*Vdda). */
#define OPAMP_TRIMMING_PMOS_VREF_10PC_VDDA  (OPAMP_CSR_TRIMOFFSETP                      | OPAMP_CSR_CALSEL_0) /*!< OPAMP trimming of transistors differential pair PMOS (internal reference voltage set to 0.1*Vdda). Default parameters to be used for calibration using two trimming steps (one with each transistors differential pair NMOS and PMOS). */
#define OPAMP_TRIMMING_PMOS_VREF_3p3PC_VDDA (OPAMP_CSR_TRIMOFFSETP                                          ) /*!< OPAMP trimming of transistors differential pair PMOS (internal reference voltage set to 0.33*Vdda). */
#define OPAMP_TRIMMING_NMOS                 (OPAMP_TRIMMING_NMOS_VREF_90PC_VDDA) /*!< OPAMP trimming of transistors differential pair NMOS (internal reference voltage set to 0.9*Vdda). Default parameters to be used for calibration using two trimming steps (one with each transistors differential pair NMOS and PMOS). */
#define OPAMP_TRIMMING_PMOS                 (OPAMP_TRIMMING_PMOS_VREF_10PC_VDDA) /*!< OPAMP trimming of transistors differential pair PMOS (internal reference voltage set to 0.1*Vdda). Default parameters to be used for calibration using two trimming steps (one with each transistors differential pair NMOS and PMOS). */





/** @defgroup OPAMP_EC_HW_DELAYS  Definitions of OPAMP hardware constraints delays
  * @note   Only OPAMP peripheral HW delays are defined in OPAMP driver driver,
  *         not timeout values.
  *         For details on delays values, refer to descriptions in source code
  *         above each literal definition.
  * @{
  */

/* Delay for OPAMP startup time (transition from state disable to enable).    */
/* Note: OPAMP startup time depends on board application environment:         */
/*       impedance connected to OPAMP output.                                 */
/*       The delay below is specified under conditions:                       */
/*        - OPAMP in functional mode follower                                 */
/*        - load impedance of 4kOhm (min), 50pF (max)                         */
/* Literal set to maximum value (refer to device datasheet,                   */
/* parameter "tWAKEUP").                                                      */
/* Unit: us                                                                   */
#define OPAMP_DELAY_STARTUP_US         (6)  /*!< Delay for OPAMP startup time */


/**
  * @brief  Driver macro reserved for internal use: set a pointer to
  *         a register from a register basis from which an offset
  *         is applied.
  * @param  __REG__ Register basis from which the offset is applied.
  * @param  __REG_OFFSET__ Offset to be applied (unit: number of registers).
  * @retval Register address
  */
#define __OPAMP_PTR_REG_OFFSET(__REG__, __REG_OFFSET__)                        \
  ((uint32_t *)((uint32_t) ((uint32_t)(&(__REG__)) + ((__REG_OFFSET__) << 2))))
  
  
/**
  * @brief  Write a value in OPAMP register
  * @param  __INSTANCE__ OPAMP Instance
  * @param  __REG__ Register to be written
  * @param  __VALUE__ Value to be written in the register
  * @retval None
  */
#define OPAMP_WriteReg(__INSTANCE__, __REG__, __VALUE__) WRITE_REG((__INSTANCE__)->__REG__, (__VALUE__))

/**
  * @brief  Read a value in OPAMP register
  * @param  __INSTANCE__ OPAMP Instance
  * @param  __REG__ Register to be read
  * @retval Register value
  */
#define OPAMP_ReadReg(__INSTANCE__, __REG__) READ_REG((__INSTANCE__)->__REG__)



typedef struct
{
  uint32_t PGAGain;
  uint32_t FunctionalMode;              /*!< Set OPAMP functional mode by setting internal connections: OPAMP operation in standalone, follower, ...
                                             This parameter can be a value of @ref OPAMP_EC_FUNCTIONAL_MODE
                                             @note If OPAMP is configured in mode PGA, the gain can be configured using function @ref OPAMP1_Set_PGAGain().

                                             This feature can be modified afterwards using unitary function @ref OPAMP1_Set_FunctionalMode(). */

  uint32_t InputInverting;              /*!< Set OPAMP inverting input connection.
                                             This parameter can be a value of @ref OPAMP_EC_INPUT_INVERTING
                                             @note OPAMP inverting input is used with OPAMP in mode standalone or PGA with external capacitors for filtering circuit. Otherwise (OPAMP in mode follower), OPAMP inverting input is not used (not connected to GPIO pin), this parameter is discarded.

                                             This feature can be modified afterwards using unitary function @ref OPAMP1_Set_InputInverting(). */

} OPAMP_InitTypeDef;



void OPAMP_DeInit(OPAMP_TypeDef *OPAMPx);
void OPAMP_Init(OPAMP_TypeDef *OPAMPx, OPAMP_InitTypeDef *OPAMP_InitStruct);
void OPAMP_StructInit(OPAMP_InitTypeDef *OPAMP_InitStruct);
void OPAMP_Set_Mode(OPAMP_TypeDef *OPAMPx, uint32_t Mode);
uint32_t OPAMP_Get_Mode(OPAMP_TypeDef *OPAMPx);
void OPAMP_Set_FunctionalMode(OPAMP_TypeDef *OPAMPx, uint32_t FunctionalMode);
uint32_t OPAMP_Get_FunctionalMode(OPAMP_TypeDef *OPAMPx);
void OPAMP_Set_PGAGain(OPAMP_TypeDef *OPAMPx, uint32_t OPAGain);
uint32_t OPAMP_Get_PGAGain(OPAMP_TypeDef *OPAMPx);
void OPAMP_Set_InputInverting(OPAMP_TypeDef *OPAMPx, uint32_t InputInverting);
uint32_t OPAMP_Get_InputInverting(OPAMP_TypeDef *OPAMPx);
void OPAMP_Set_TrimmingMode(OPAMP_TypeDef *OPAMPx, uint32_t TrimmingMode);
uint32_t OPAMP_Get_TrimmingMode(OPAMP_TypeDef *OPAMPx);
void OPAMP_Set_CalibrationSelection(OPAMP_TypeDef *OPAMPx, uint32_t TransistorsDiffPair);
uint32_t OPAMP_Get_CalibrationSelection(OPAMP_TypeDef *OPAMPx);
void OPAMP_Set_TrimmingValue(OPAMP_TypeDef *OPAMPx, uint32_t TransistorsDiffPair, uint32_t TrimmingValue);
uint32_t OPAMP_Get_TrimmingValue(OPAMP_TypeDef *OPAMPx, uint32_t TransistorsDiffPair);
void OPAMP_Enable(OPAMP_TypeDef *OPAMPx);
void OPAMP_Disable(OPAMP_TypeDef *OPAMPx);
uint32_t OPAMP_Is_Enabled(OPAMP_TypeDef *OPAMPx);
void OPAMP_Lock(OPAMP_TypeDef *OPAMPx);
uint32_t OPAMP_Is_Locked(OPAMP_TypeDef *OPAMPx);
  
  

#ifdef __cplusplus
}
#endif

#endif /* __rx32s12xx_OPAMP_H__ */
