/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_spi.h
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/

#ifndef __rx32s12xx_SPI_H__
#define __rx32s12xx_SPI_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s12xx.h"

/*********************************************************************************/
/*                                                                               */
/*                     Serial Peripheral Interface(SPI)                          */
/*                                                                               */
/*********************************************************************************/

/*******************  Bit definition for SPI_CR1 register  ********************/
#define SPI_CR1_CPHA                        (0x00000001)  /*!< Clock Phase */
#define SPI_CR1_CPOL                        (0x00000002)	/*!< Clock Polarity */
#define SPI_CR1_MSTR                        (0x00000004)	/*!< Master Selection */
#define SPI_CR1_BR                          (0x00000038)	/*!< BR[2:0] bits (Baud Rate Control) */
#define SPI_CR1_BR_0                        (0x00000008)
#define SPI_CR1_BR_1                        (0x00000010)
#define SPI_CR1_BR_2                        (0x00000020)
#define SPI_CR1_SPE                         (0x00000040)	/*!< SPI Enable */
#define SPI_CR1_LSBFIRST                    (0x00000080)	/*!< Frame Format */
#define SPI_CR1_SSI                         (0x00000100)	/*!< Internal slave select */
#define SPI_CR1_SSM                         (0x00000200)	/*!< Software slave management */
#define SPI_CR1_RXONLY                      (0x00000400)	/*!< Receive only */
#define SPI_CR1_DFF                         (0x00000800)	/*!< Data Frame Format */
#define SPI_CR1_CRCNEXT                     (0x00001000)	/*!< Transmit CRC next */
#define SPI_CR1_CRCEN                       (0x00002000)	/*!< Hardware CRC calculation enable */
#define SPI_CR1_BIDIOE                      (0x00004000)	/*!< Output enable in bidirectional mode */
#define SPI_CR1_BIDIMODE                    (0x00008000)	/*!< Bidirectional data mode enable */

/*******************  Bit definition for SPI_CR2 register  ********************/
#define SPI_CR2_RXDMAEN                     (0x00000001)  /*!< Rx Buffer DMA Enable */
#define SPI_CR2_TXDMAEN                     (0x00000002)  /*!< Tx Buffer DMA Enable */
#define SPI_CR2_SSOE                        (0x00000004)	/*!< SS Output Enable */
#define SPI_CR2_ERRIE                       (0x00000020)	/*!< Error Interrupt Enable */
#define SPI_CR2_RXNEIE                      (0x00000040)	/*!< RX buffer Not Empty Interrupt Enable */
#define SPI_CR2_TXEIE                       (0x00000080)	/*!< Tx buffer Empty Interrupt Enable */

/********************  Bit definition for SPI_SR register  ********************/
#define SPI_SR_RXNE                         (0x00000001)	/*!< Receive buffer Not Empty */
#define SPI_SR_TXE                          (0x00000002)	/*!< Transmit buffer Empty */
#define SPI_SR_CRCERR                       (0x00000010)  /*!< CRC Error flag */
#define SPI_SR_MODF                         (0x00000020)	/*!< Mode fault */
#define SPI_SR_OVR                          (0x00000040)	/*!< Overrun flag */
#define SPI_SR_BSY                          (0x00000080)	/*!< Busy flag */

/********************  Bit definition for SPI_DR register  ********************/
#define SPI_DR_DR                           (0x0000FFFF)  /*!< Data Register */

/*******************  Bit definition for SPI_CRCPR register  ******************/
#define SPI_CRCPR_CRCPOLY                   (0x0000FFFF)  /*!< CRC polynomial register */

/******************  Bit definition for SPI_RXCRCR register  ******************/
#define SPI_RXCRCR_RXCRC                    (0x0000FFFF)  /*!< Rx CRC Register */

/******************  Bit definition for SPI_TXCRCR register  ******************/
#define SPI_TXCRCR_TXCRC                    (0x0000FFFF)  /*!< Tx CRC Register */

/******************  Bit definition for SPI_I2SCFGR register  *****************/
#define SPI_I2SCFGR_CHLEN                   (0x00000001)  /*!< Channel length (number of bits per audio channel) */
#define SPI_I2SCFGR_DATLEN                  (0x00000006)	/*!< DATLEN[1:0] bits (Data length to be transferred) */
#define SPI_I2SCFGR_DATLEN_0                (0x00000002)
#define SPI_I2SCFGR_DATLEN_1                (0x00000004)
#define SPI_I2SCFGR_CKPOL                   (0x00000008)  /*!< steady state clock polarity */
#define SPI_I2SCFGR_I2SSTD                  (0x00000030)	/*!< I2SSTD[1:0] bits (I2S standard selection) */
#define SPI_I2SCFGR_I2SSTD_0                (0x00000010)
#define SPI_I2SCFGR_I2SSTD_1                (0x00000020)
#define SPI_I2SCFGR_PCMSYNC                 (0x00000080)  /*!< PCM frame synchronization */
#define SPI_I2SCFGR_I2SCFG                  (0x00000300)	/*!< I2SCFG[1:0] bits (I2S configuration mode) */
#define SPI_I2SCFGR_I2SCFG_0                (0x00000100)
#define SPI_I2SCFGR_I2SCFG_1                (0x00000200)
#define SPI_I2SCFGR_I2SE                    (0x00000400)  /*!< I2S Enable */
#define SPI_I2SCFGR_I2SMOD                  (0x00000800)  /*!< I2S mode selection */

/******************  Bit definition for SPI_I2SPR register  *******************/
#define SPI_I2SPR_I2SDIV                    (0x000000FF)  /*!< I2S Linear prescaler */
#define SPI_I2SPR_ODD                       (0x00000100)  /*!< Odd factor for the prescaler */
#define SPI_I2SPR_MCKOE                     (0x00000200)  /*!< Master Clock Output Enable */

/** @defgroup SPI_EC_MODE Operation Mode
  */
#define SPI_MODE_MASTER                     (SPI_CR1_MSTR | SPI_CR1_SSI)    /*!< Master configuration  */
#define SPI_MODE_SLAVE                      0x00000000U                     /*!< Slave configuration   */


/** @defgroup SPI_EC_PHASE Clock Phase
  */
#define SPI_PHASE_1EDGE                 		0x00000000U          /*!< First clock transition is the first data capture edge  */
#define SPI_PHASE_2EDGE                 		(SPI_CR1_CPHA)       /*!< Second clock transition is the first data capture edge */

/** @defgroup SPI_EC_POLARITY Clock Polarity
	*/
#define SPI_POLARITY_LOW               		  0x00000000U          /*!< Clock to 0 when idle */
#define SPI_POLARITY_HIGH               	 (SPI_CR1_CPOL)        /*!< Clock to 1 when idle */

/** @defgroup SPI_EC_BAUDRATEPRESCALER Baud Rate Prescaler
  */
#define SPI_BAUDRATEPRESCALER_DIV2          0x00000000U                                    /*!< BaudRate control equal to fPCLK/2   */
#define SPI_BAUDRATEPRESCALER_DIV4          (SPI_CR1_BR_0)                                 /*!< BaudRate control equal to fPCLK/4   */
#define SPI_BAUDRATEPRESCALER_DIV8          (SPI_CR1_BR_1)                                 /*!< BaudRate control equal to fPCLK/8   */
#define SPI_BAUDRATEPRESCALER_DIV16         (SPI_CR1_BR_1 | SPI_CR1_BR_0)                  /*!< BaudRate control equal to fPCLK/16  */
#define SPI_BAUDRATEPRESCALER_DIV32         (SPI_CR1_BR_2)                                 /*!< BaudRate control equal to fPCLK/32  */
#define SPI_BAUDRATEPRESCALER_DIV64         (SPI_CR1_BR_2 | SPI_CR1_BR_0)                  /*!< BaudRate control equal to fPCLK/64  */
#define SPI_BAUDRATEPRESCALER_DIV128        (SPI_CR1_BR_2 | SPI_CR1_BR_1)                  /*!< BaudRate control equal to fPCLK/128 */
#define SPI_BAUDRATEPRESCALER_DIV256        (SPI_CR1_BR_2 | SPI_CR1_BR_1 | SPI_CR1_BR_0)   /*!< BaudRate control equal to fPCLK/256 */

/** @defgroup SPI_EC_BIT_ORDER Transmission Bit Order
  */
#define SPI_LSB_FIRST                       (SPI_CR1_LSBFIRST)        /*!< Data is transmitted/received with the LSB first */
#define SPI_MSB_FIRST                       0x00000000U               /*!< Data is transmitted/received with the MSB first */

/** @defgroup SPI_EC_TRANSFER_MODE Transfer Mode
  */
#define SPI_FULL_DUPLEX                      0x00000000U                         /*!< Full-Duplex mode. Rx and Tx transfer on 2 lines */
#define SPI_SIMPLEX_RX                      (SPI_CR1_RXONLY)                     /*!< Simplex Rx mode.  Rx transfer only on 1 line    */
#define SPI_HALF_DUPLEX_RX                  (SPI_CR1_BIDIMODE)                   /*!< Half-Duplex Rx mode. Rx transfer on 1 line      */
#define SPI_HALF_DUPLEX_TX                  (SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE)  /*!< Half-Duplex Tx mode. Tx transfer on 1 line      */


/** @defgroup SPI_EC_NSS_MODE Slave Select Pin Mode
  */
#define SPI_NSS_SOFT                        (SPI_CR1_SSM)                        /*!< NSS managed internally. NSS pin not used and free              */
#define SPI_NSS_HARD_INPUT                  0x00000000U                          /*!< NSS pin used in Input. Only used in Master mode                */
#define SPI_NSS_HARD_OUTPUT                 (((uint32_t)SPI_CR2_SSOE << 16U))    /*!< NSS pin used in Output. Only used in Slave mode as chip select */

/** @defgroup SPI_EC_DATAWIDTH Datawidth
  */
#define SPI_DATAWIDTH_8BIT                  0x00000000U         /*!< Data length for SPI transfer:  8 bits */
#define SPI_DATAWIDTH_16BIT                 (SPI_CR1_DFF)       /*!< Data length for SPI transfer:  16 bits */
																															  
/** @defgroup SPI_EC_CRC_CALCULATION CRC Calculation            
  */                                                            
#define SPI_CRCCALCULATION_DISABLE          0x00000000U         /*!< CRC calculation disabled */
#define SPI_CRCCALCULATION_ENABLE           (SPI_CR1_CRCEN)     /*!< CRC calculation enabled  */

/* SPI registers Masks */
#define SPI_CR1_CLEAR_MASK                 (SPI_CR1_CPHA    | SPI_CR1_CPOL     | SPI_CR1_MSTR   | \
                                            SPI_CR1_BR      | SPI_CR1_LSBFIRST | SPI_CR1_SSI    | \
                                            SPI_CR1_SSM     | SPI_CR1_RXONLY   | SPI_CR1_DFF    | \
                                            SPI_CR1_CRCNEXT | SPI_CR1_CRCEN    | SPI_CR1_BIDIOE | \
                                            SPI_CR1_BIDIMODE)

/**
  * @brief  SPI Init structures definition
  */
typedef struct
{
  uint32_t TransferDirection;       /*!< Specifies the SPI unidirectional or bidirectional data mode.
                                         This parameter can be a value of @ref SPI_EC_TRANSFER_MODE.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetTransferDirection().*/

  uint32_t Mode;                    /*!< Specifies the SPI mode (Master/Slave).
                                         This parameter can be a value of @ref SPI_EC_MODE.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetMode().*/

  uint32_t DataWidth;               /*!< Specifies the SPI data width.
                                         This parameter can be a value of @ref SPI_EC_DATAWIDTH.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetDataWidth().*/

  uint32_t ClockPolarity;           /*!< Specifies the serial clock steady state.
                                         This parameter can be a value of @ref SPI_EC_POLARITY.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetClockPolarity().*/

  uint32_t ClockPhase;              /*!< Specifies the clock active edge for the bit capture.
                                         This parameter can be a value of @ref SPI_EC_PHASE.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetClockPhase().*/

  uint32_t NSS;                     /*!< Specifies whether the NSS signal is managed by hardware (NSS pin) or by software using the SSI bit.
                                         This parameter can be a value of @ref SPI_EC_NSS_MODE.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetNSSMode().*/

  uint32_t BaudRate;                /*!< Specifies the BaudRate prescaler value which will be used to configure the transmit and receive SCK clock.
                                         This parameter can be a value of @ref SPI_EC_BAUDRATEPRESCALER.
                                         @note The communication clock is derived from the master clock. The slave clock does not need to be set.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetBaudRatePrescaler().*/

  uint32_t BitOrder;                /*!< Specifies whether data transfers start from MSB or LSB bit.
                                         This parameter can be a value of @ref SPI_EC_BIT_ORDER.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetTransferBitOrder().*/

  uint32_t CRCCalculation;          /*!< Specifies if the CRC calculation is enabled or not.
                                         This parameter can be a value of @ref SPI_EC_CRC_CALCULATION.

                                         This feature can be modified afterwards using unitary functions @ref SPI_Enable_CRC() and @ref SPI_Disable_CRC().*/

  uint32_t CRCPoly;                 /*!< Specifies the polynomial used for the CRC calculation.
                                         This parameter must be a number between Min_Data = 0x00 and Max_Data = 0xFFFF.

                                         This feature can be modified afterwards using unitary function @ref SPI_SetCRCPolynomial().*/

} SPI_InitTypeDef;

void     SPI_StructInit(SPI_InitTypeDef *SPI_InitStruct);
void     SPI_Init(SPI_TypeDef *SPIx, SPI_InitTypeDef *SPI_InitStruct);
void     SPI_Enable(SPI_TypeDef *SPIx);
void     SPI_Disable(SPI_TypeDef *SPIx);
uint32_t SPI_Is_Enabled(SPI_TypeDef *SPIx);
void     SPI_Set_Mode(SPI_TypeDef *SPIx, uint32_t Mode);
uint32_t SPI_Get_Mode(SPI_TypeDef *SPIx);
void     SPI_Set_ClockPhase(SPI_TypeDef *SPIx, uint32_t ClockPhase);
uint32_t SPI_Get_ClockPhase(SPI_TypeDef *SPIx);
void     SPI_Set_ClockPolarity(SPI_TypeDef *SPIx, uint32_t ClockPolarity);
uint32_t SPI_Get_ClockPolarity(SPI_TypeDef *SPIx);
void     SPI_Set_BaudRatePrescaler(SPI_TypeDef *SPIx, uint32_t BaudRate);
uint32_t SPI_Get_BaudRatePrescaler(SPI_TypeDef *SPIx);
void     SPI_Set_TransferBitOrder(SPI_TypeDef *SPIx, uint32_t BitOrder);
uint32_t SPI_Get_TransferBitOrder(SPI_TypeDef *SPIx);
void     SPI_Set_TransferDirection(SPI_TypeDef *SPIx, uint32_t TransferDirection);
uint32_t SPI_Get_TransferDirection(SPI_TypeDef *SPIx);
void     SPI_Set_DataWidth(SPI_TypeDef *SPIx, uint32_t DataWidth);
uint32_t SPI_Get_DataWidth(SPI_TypeDef *SPIx);
void     SPI_Enable_CRC(SPI_TypeDef *SPIx);
void     SPI_Disable_CRC(SPI_TypeDef *SPIx);
uint32_t SPI_Get_CRC(SPI_TypeDef *SPIx);
void     SPI_Set_CRCNext(SPI_TypeDef *SPIx);
void     SPI_Set_CRCPolynomial(SPI_TypeDef *SPIx, uint32_t CRCPoly);
uint32_t SPI_Get_CRCPolynomial(SPI_TypeDef *SPIx);
uint32_t SPI_Get_RxCRC(SPI_TypeDef *SPIx);
uint32_t SPI_Get_TxCRC(SPI_TypeDef *SPIx);
void     SPI_Set_NSSMode(SPI_TypeDef *SPIx, uint32_t NSS);
uint32_t SPI_Get_NSSMode(SPI_TypeDef *SPIx);
uint32_t SPI_Get_Flag(SPI_TypeDef *SPIx, uint32_t Flag);
void SPI_Clear_Flag(SPI_TypeDef *SPIx, uint32_t Flag);
void SPI_Clear_FlagFRE(SPI_TypeDef *SPIx);
void SPI_Enable_IT(SPI_TypeDef *SPIx, uint32_t IT);
void SPI_Disable_IT(SPI_TypeDef *SPIx, uint32_t IT);
uint32_t SPI_Get_IT(SPI_TypeDef *SPIx, uint32_t IT);
void     SPI_Enable_DMAReqRX(SPI_TypeDef *SPIx);
void     SPI_Disable_DMAReqRX(SPI_TypeDef *SPIx);
uint32_t SPI_Get_DMAReqRX(SPI_TypeDef *SPIx);
void     SPI_Enable_DMAReqTX(SPI_TypeDef *SPIx);
void     SPI_Disable_DMAReqTX(SPI_TypeDef *SPIx);
uint32_t SPI_Get_DMAReqTX(SPI_TypeDef *SPIx);
uint32_t SPI_Get_DMARegAddr(SPI_TypeDef *SPIx);
uint8_t  SPI_Receive_Data8(SPI_TypeDef *SPIx);
uint16_t SPI_Receive_Data16(SPI_TypeDef *SPIx);
void     SPI_Transmit_Data8(SPI_TypeDef *SPIx, uint8_t TxData);
void     SPI_Transmit_Data16(SPI_TypeDef *SPIx, uint16_t TxData);
void     SPI_DeInit(SPI_TypeDef *SPIx);

#ifdef __cplusplus
}
#endif

#endif /* __rx32s12xx_SPI_H__ */

