/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_usart.h
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/

#ifndef __rx32s12xx_USART_H__
#define __rx32s12xx_USART_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s12xx.h"

/*********************************************************************************/
/*                                                                               */
/*       Universal synchronous asynchronous receiver transmitter (USART)         */
/*                                                                               */
/*********************************************************************************/

/*******************  Bit definition for USART_SR register  *******************/      
#define USART_SR_PE                         (0x00000001)  /*!< Parity Error */
#define USART_SR_FE                         (0x00000002)  /*!< Framing Error */
#define USART_SR_NE                         (0x00000004)  /*!< Noise Error Flag */
#define USART_SR_ORE                        (0x00000008)  /*!< OverRun Error */
#define USART_SR_IDLE                       (0x00000010)  /*!< IDLE line detected */
#define USART_SR_RXNE                       (0x00000020)  /*!< Read Data Register Not Empty */
#define USART_SR_TC                         (0x00000040)  /*!< Transmission Complete */
#define USART_SR_TXE                        (0x00000080)  /*!< Transmit Data Register Empty */
#define USART_SR_LBD                        (0x00000100)  /*!< LIN Break Detection Flag */
#define USART_SR_CTS                        (0x00000200)  /*!< CTS Flag */

/*******************  Bit definition for USART_DR register  *******************/
#define USART_DR_DR                         (0x000001FF)  /*!< Data value */

/******************  Bit definition for USART_BRR register  *******************/
#define USART_BRR_DIV_Fraction              (0x0000000F)  /*!< Fraction of USARTDIV */
#define USART_BRR_DIV_Mantissa              (0x0000FFF0)  /*!< Mantissa of USARTDIV */

/******************  Bit definition for USART_CR1 register  *******************/
#define USART_CR1_SBK                       (0x00000001)  /*!< Send Break */
#define USART_CR1_RWU                       (0x00000002)  /*!< Receiver wakeup */
#define USART_CR1_RE                        (0x00000004)  /*!< Receiver Enable */
#define USART_CR1_TE                        (0x00000008)  /*!< Transmitter Enable */
#define USART_CR1_IDLEIE                    (0x00000010)  /*!< IDLE Interrupt Enable */
#define USART_CR1_RXNEIE                    (0x00000020)  /*!< RXNE Interrupt Enable */
#define USART_CR1_TCIE                      (0x00000040)  /*!< Transmission Complete Interrupt Enable */
#define USART_CR1_TXEIE                     (0x00000080)  /*!< TXE Interrupt Enable */
#define USART_CR1_PEIE                      (0x00000100)  /*!< PE Interrupt Enable */
#define USART_CR1_PS                        (0x00000200)  /*!< Parity Selection */
#define USART_CR1_PCE                       (0x00000400)  /*!< Parity Control Enable */
#define USART_CR1_WAKE                      (0x00000800)  /*!< Wakeup method */
#define USART_CR1_M                         (0x00001000)  /*!< Word length */
#define USART_CR1_UE                        (0x00002000)  /*!< USART Enable */


/******************  Bit definition for USART_FLAG register  *******************/
#define USART_FLAG_PE                    (0x10000001)  /*!< Parity Error */
#define USART_FLAG_FE                    (0x10000002)  /*!< Framing Error */
#define USART_FLAG_NE                    (0x10000004)  /*!< Noise Error Flag */
#define USART_FLAG_ORE                   (0x10000008)  /*!< OverRun Error */
#define USART_FLAG_IDLE                  (0x10000010)  /*!< IDLE line detected */
#define USART_FLAG_RXNE                  (0x10000020)  /*!< Read Data Register Not Empty */
#define USART_FLAG_TC                    (0x10000040)  /*!< Transmission Complete */
#define USART_FLAG_TXE                   (0x10000080)  /*!< Transmit Data Register Empty */
#define USART_FLAG_LBD                   (0x10000100)  /*!< LIN Break Detection Flag */
#define USART_FLAG_CTS                   (0x10000200)  /*!< CTS Flag */
#define USART_FLAG_SBK                   (0x20000001)  /*!< Send Break */
#define USART_FLAG_RWU                   (0x20000002)  /*!< Receiver wakeup */

/******************  Bit definition for USART_CR2 register  *******************/
#define USART_CR2_ADD                       (0x0000000F)  /*!< Address of the USART node */
#define USART_CR2_LBDL                      (0x00000020)  /*!< LIN Break Detection Length */
#define USART_CR2_LBDIE                     (0x00000040)  /*!< LIN Break Detection Interrupt Enable */
#define USART_CR2_STOP                      (0x00003000)  /*!< STOP[1:0] bits (STOP bits) */
#define USART_CR2_STOP_0                    (0x00001000)  
#define USART_CR2_STOP_1                    (0x00002000)
#define USART_CR2_LINEN                     (0x00004000)  /*!< LIN mode enable */

/******************  Bit definition for USART_CR3 register  *******************/
#define USART_CR3_EIE                       (0x00000001)  /*!< Error Interrupt Enable */
#define USART_CR3_HDSEL                     (0x00000008)  /*!< Half-Duplex Selection */
#define USART_CR3_DMAR                      (0x00000040)  /*!< DMA Enable Receiver */
#define USART_CR3_DMAT                      (0x00000080)  /*!< DMA Enable Transmitter */
#define USART_CR3_RTSE                      (0x00000100)  /*!< RTS Enable */
#define USART_CR3_CTSE                      (0x00000200)  /*!< CTS Enable */
#define USART_CR3_CTSIE                     (0x00000400)  /*!< CTS Interrupt Enable */


/******************  Bit definition for USART_IT register  *******************/
#define USART_IT_IDLEIE                 (0x10000010)  /*!< IDLE Interrupt Enable */
#define USART_IT_RXNEIE                 (0x10000020)  /*!< RXNE Interrupt Enable */
#define USART_IT_TCIE                   (0x10000040)  /*!< Transmission Complete Interrupt Enable */
#define USART_IT_TXEIE                  (0x10000080)  /*!< TXE Interrupt Enable */
#define USART_IT_PEIE                   (0x10000100)  /*!< PE Interrupt Enable */
#define USART_IT_LBDIE                  (0x20000040)  /*!< LIN Break Detection Interrupt Enable */
#define USART_IT_EIE                    (0x40000001)  /*!< Error Interrupt Enable */
#define USART_IT_CTSIE                  (0x40000400)  /*!< CTS Interrupt Enable */


/** @defgroup USART_EC_DATAWIDTH Datawidth
  */
#define USART_DATAWIDTH_8B                   0x00000000U             /*!< 8 bits word length : Start bit, 8 data bits, n stop bits */
#define USART_DATAWIDTH_9B                   USART_CR1_M             /*!< 9 bits word length : Start bit, 9 data bits, n stop bits */

/** @defgroup USART_EC_STOPBITS Stop Bits
  */
#define USART_STOPBITS_0_5                   USART_CR2_STOP_0                           /*!< 0.5 stop bit */
#define USART_STOPBITS_1                     0x00000000U                                /*!< 1 stop bit */
#define USART_STOPBITS_1_5                   (USART_CR2_STOP_0 | USART_CR2_STOP_1)      /*!< 1.5 stop bits */
#define USART_STOPBITS_2                     USART_CR2_STOP_1                           /*!< 2 stop bits */

/** @defgroup USART_EC_PARITY Parity Control
  */
#define USART_PARITY_NONE                    0x00000000U                          /*!< Parity control disabled */
#define USART_PARITY_EVEN                    USART_CR1_PCE                        /*!< Parity control enabled and Even Parity is selected */
#define USART_PARITY_ODD                     (USART_CR1_PCE | USART_CR1_PS)       /*!< Parity control enabled and Odd Parity is selected */

/** @defgroup USART_EC_DIRECTION Communication Direction
  */
#define USART_DIRECTION_NONE                 0x00000000U                        /*!< Transmitter and Receiver are disabled */
#define USART_DIRECTION_RX                   USART_CR1_RE                       /*!< Transmitter is disabled and Receiver is enabled */
#define USART_DIRECTION_TX                   USART_CR1_TE                       /*!< Transmitter is enabled and Receiver is disabled */
#define USART_DIRECTION_TX_RX                (USART_CR1_TE |USART_CR1_RE)       /*!< Transmitter and Receiver are enabled */

/** @defgroup USART_EC_HWCONTROL Hardware Control
  */
#define USART_HWCONTROL_NONE                 0x00000000U                          /*!< CTS and RTS hardware flow control disabled */
#define USART_HWCONTROL_RTS                  USART_CR3_RTSE                       /*!< RTS output enabled, data is only requested when there is space in the receive buffer */
#define USART_HWCONTROL_CTS                  USART_CR3_CTSE                       /*!< CTS mode enabled, data is only transmitted when the nCTS input is asserted (tied to 0) */
#define USART_HWCONTROL_RTS_CTS              (USART_CR3_RTSE | USART_CR3_CTSE)    /*!< CTS and RTS hardware flow control enabled */

/** @defgroup USART_WAKEUP Wakeup
  */
#define USART_WAKEUP_IDLELINE                0x00000000U           /*!<  USART wake up from Mute mode on Idle Line */
#define USART_WAKEUP_ADDRESSMARK             USART_CR1_WAKE        /*!<  USART wake up from Mute mode on Address Mark */

/** @defgroup USART_EC_LINBREAK_DETECT LIN Break Detection Length
  */
#define USART_LINBREAK_DETECT_10B            0x00000000U           /*!< 10-bit break detection method selected */
#define USART_LINBREAK_DETECT_11B            USART_CR2_LBDL        /*!< 11-bit break detection method selected */

/**
  * @brief USART Init Structure definition
  */
typedef struct
{
  uint32_t BaudRate;                  /*!< This field defines expected Usart communication baud rate.

                                           This feature can be modified afterwards using unitary function @ref USART_Set_BaudRate().*/

  uint32_t DataWidth;                 /*!< Specifies the number of data bits transmitted or received in a frame.
                                           This parameter can be a value of @ref USART_EC_DATAWIDTH.

                                           This feature can be modified afterwards using unitary function @ref USART_Set_DataWidth().*/

  uint32_t StopBits;                  /*!< Specifies the number of stop bits transmitted.
                                           This parameter can be a value of @ref USART_EC_STOPBITS.

                                           This feature can be modified afterwards using unitary function @ref USART_Set_StopBitsLength().*/

  uint32_t Parity;                    /*!< Specifies the parity mode.
                                           This parameter can be a value of @ref USART_EC_PARITY.

                                           This feature can be modified afterwards using unitary function @ref USART_Set_Parity().*/

  uint32_t TransferDirection;         /*!< Specifies whether the Receive and/or Transmit mode is enabled or disabled.
                                           This parameter can be a value of @ref USART_EC_DIRECTION.

                                           This feature can be modified afterwards using unitary function @ref USART_Set_TransferDirection().*/

  uint32_t HardwareFlowControl;       /*!< Specifies whether the hardware flow control mode is enabled or disabled.
                                           This parameter can be a value of @ref USART_EC_HWCONTROL.

                                           This feature can be modified afterwards using unitary function @ref USART_Set_HWFlowCtrl().*/

} USART_InitTypeDef;


/**
  * @brief  Compute USARTDIV value according to Peripheral Clock and
  *         expected Baud Rate in 16 bits sampling mode (32 bits value of USARTDIV is returned)
  * @param  __PERIPHCLK__ Peripheral Clock frequency used for USART instance
  * @param  __BAUDRATE__ Baud rate value to achieve
  * @retval USARTDIV value to be used for BRR register filling in OverSampling_16 case
  */
#define __USART_DIV_SAMPLING16_100(__PERIPHCLK__, __BAUDRATE__)     (((__PERIPHCLK__)*25)/(4*(__BAUDRATE__)))
#define __USART_DIVMANT_SAMPLING16(__PERIPHCLK__, __BAUDRATE__)     (__USART_DIV_SAMPLING16_100((__PERIPHCLK__), (__BAUDRATE__))/100)
#define __USART_DIVFRAQ_SAMPLING16(__PERIPHCLK__, __BAUDRATE__)     (((__USART_DIV_SAMPLING16_100((__PERIPHCLK__), (__BAUDRATE__)) - (__USART_DIVMANT_SAMPLING16((__PERIPHCLK__), (__BAUDRATE__)) * 100)) * 16 + 50) / 100)
/* USART BRR = mantissa + overflow + fraction
            = (USART DIVMANT << 4) + (USART DIVFRAQ & 0xF0) + (USART DIVFRAQ & 0x0F) */
#define __USART_DIV_SAMPLING16(__PERIPHCLK__, __BAUDRATE__)            (((__USART_DIVMANT_SAMPLING16((__PERIPHCLK__), (__BAUDRATE__)) << 4) + \
                                                                           (__USART_DIVFRAQ_SAMPLING16((__PERIPHCLK__), (__BAUDRATE__)) & 0xF0)) + \
                                                                           (__USART_DIVFRAQ_SAMPLING16((__PERIPHCLK__), (__BAUDRATE__)) & 0x0F))

void     USART_DeInit(USART_TypeDef *USARTx);
void	   USART_StructInit(USART_InitTypeDef *USART_InitStruct);
void     USART_Init(USART_TypeDef *USARTx, USART_InitTypeDef *USART_InitStruct);
void     USART_Enable(USART_TypeDef *USARTx);
void     USART_Disable(USART_TypeDef *USARTx);
uint32_t USART_Is_Enable(USART_TypeDef *USARTx);
void     USART_Enable_Rx(USART_TypeDef *USARTx);
void     USART_Disable_Rx(USART_TypeDef *USARTx);
void     USART_Enable_Tx(USART_TypeDef *USARTx);
void     USART_Disable_Tx(USART_TypeDef *USARTx);
void     USART_Set_TransferDirection(USART_TypeDef *USARTx, uint32_t TransferDirection);
uint32_t USART_Get_TransferDirection(USART_TypeDef *USARTx);
void     USART_Set_Parity(USART_TypeDef *USARTx, uint32_t Parity);
uint32_t USART_Get_Parity(USART_TypeDef *USARTx);
void     USART_Set_WakeUpMethod(USART_TypeDef *USARTx, uint32_t Method);
uint32_t USART_Get_WakeUpMethod(USART_TypeDef *USARTx);
void     USART_Set_DataWidth(USART_TypeDef *USARTx, uint32_t DataWidth);
uint32_t USART_Get_DataWidth(USART_TypeDef *USARTx);
void     USART_Set_StopBitsLength(USART_TypeDef *USARTx, uint32_t StopBits);
uint32_t USART_Get_StopBitsLength(USART_TypeDef *USARTx);
void     USART_Set_Character(USART_TypeDef *USARTx, uint32_t DataWidth, uint32_t Parity, uint32_t StopBits);
void     USART_Set_NodeAddress(USART_TypeDef *USARTx, uint32_t NodeAddress);
uint32_t USART_Get_NodeAddress(USART_TypeDef *USARTx);
void     USART_Enable_RTSHWFlowCtrl(USART_TypeDef *USARTx);
void     USART_Disable_RTSHWFlowCtrl(USART_TypeDef *USARTx);
void     USART_Enable_CTSHWFlowCtrl(USART_TypeDef *USARTx);
void     USART_Disable_CTSHWFlowCtrl(USART_TypeDef *USARTx);
void     USART_Set_HWFlowCtrl(USART_TypeDef *USARTx, uint32_t HardwareFlowControl);
uint32_t USART_Get_HWFlowCtrl(USART_TypeDef *USARTx);
void     USART_Set_BaudRate(USART_TypeDef *USARTx, uint32_t PeriphClk, uint32_t BaudRate);
uint32_t USART_Get_BaudRate(USART_TypeDef *USARTx, uint32_t PeriphClk);
void     USART_Enable_HalfDuplex(USART_TypeDef *USARTx);
void     USART_Disable_HalfDuplex(USART_TypeDef *USARTx);
uint32_t USART_Get_HalfDuplex(USART_TypeDef *USARTx);
void     USART_Set_LINBrkDetectionLen(USART_TypeDef *USARTx, uint32_t LINBDLength);
uint32_t USART_Get_LINBrkDetectionLen(USART_TypeDef *USARTx);
void     USART_Enable_LIN(USART_TypeDef *USARTx);
void     USART_Disable_LIN(USART_TypeDef *USARTx);
uint32_t USART_Get_LIN(USART_TypeDef *USARTx);
uint32_t USART_Get_Flag(USART_TypeDef *USARTx, uint32_t Flag);
void USART_Clear_Flag(USART_TypeDef *USARTx, uint32_t Flag);
void USART_Enable_IT(USART_TypeDef *USARTx, uint32_t IT);
void USART_Disable_IT(USART_TypeDef *USARTx, uint32_t IT);
uint32_t USART_Get_IT(USART_TypeDef *USARTx, uint32_t IT);
void     USART_Enable_DMAReq_RX(USART_TypeDef *USARTx);
void     USART_Disable_DMAReq_RX(USART_TypeDef *USARTx);
uint32_t USART_Get_DMAReq_RX(USART_TypeDef *USARTx);
void     USART_Enable_DMAReq_TX(USART_TypeDef *USARTx);
void     USART_Disable_DMAReq_TX(USART_TypeDef *USARTx);
uint32_t USART_Get_DMAReq_TX(USART_TypeDef *USARTx);
uint32_t USART_Get_DMARegAddr(USART_TypeDef *USARTx);
uint8_t  USART_Receive_Data8(USART_TypeDef *USARTx);
uint16_t USART_Receive_Data9(USART_TypeDef *USARTx);
void     USART_Transmit_Data8(USART_TypeDef *USARTx, uint8_t Value);
void     USART_Transmit_Data9(USART_TypeDef *USARTx, uint16_t Value);
void     USART_Request_BreakSending(USART_TypeDef *USARTx);
void     USART_Enter_MuteMode(USART_TypeDef *USARTx);
void     USART_Exit_MuteMode(USART_TypeDef *USARTx);

#ifdef __cplusplus
}
#endif

#endif /* __rx32s12xx_USART_H__ */
