/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_adc.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/

#define  __rx32s12xx_ADC_C

#include "rx32s12xx_adc.h"

/* Private macros ------------------------------------------------------------*/

/**
  * @function  ADC_Reg_StructInit
  * @brief  Set each  ADC_Reg_InitStruct field to default value
  * @param  ADCx :
            @arg ADC Instance
  * @retval 
  */
void ADC_Reg_StructInit(ADC_Reg_InitTypeDef* ADC_Reg_InitStruct)
{
	ADC_Reg_InitStruct->TriggerSource = ADC_CR2_EXTSEL_INSOURCE;                       
	ADC_Reg_InitStruct->Continuous = ADC_CR2_CONT_SINGLE;                              
	ADC_Reg_InitStruct->ScanMode = ADC_CR1_SCAN_DISABLE;                                  
	ADC_Reg_InitStruct->SequencerNbRanks = ADC_SQRL_SCAN_DISABLE;                                
	ADC_Reg_InitStruct->DataAlignment = ADC_CR2_ALIGN_RIGHT;                          
  ADC_Reg_InitStruct->DMATransfer = ADC_CR2_DMA_NONE;                            
}

/**
  * @function  ADC_Reg_Init
  * @brief  read complete interrupt Flag status of the selected ADC
  * @param  ADCx :
            @arg ADC Instance
  * @retval 
  */
void ADC_Reg_Init(ADC_TypeDef *ADCx,ADC_Reg_InitTypeDef* ADC_Reg_InitStruct)
{
	ADC_Set_EXTSEL(ADCx, ADC_Reg_InitStruct->TriggerSource);                           /* Set ADC group regular trigger source */
	ADC_Set_CONT(ADCx, ADC_Reg_InitStruct->Continuous);                                /* Set ADC group regular continuous mode */
	ADC_Set_SCAN(ADCx, ADC_Reg_InitStruct->ScanMode);                                  /* Set ADC sacn regular group*/
	ADC_Set_SQRL(ADCx, ADC_Reg_InitStruct->SequencerNbRanks);                          /* Set ADC group regular sequencer length and scan direction */
	ADC_Set_ALIGN(ADCx, ADC_Reg_InitStruct->DataAlignment);                            /* Set ADC conversion data alignment */
  ADC_Set_DMA(ADCx, ADC_Reg_InitStruct->DMATransfer);                                /* Set ADC group regular conversion data transfer */
}

/**
  * @function  ADC_Inj_StructInit
  * @brief  Set each  ADC_Reg_InitStruct field to default value
  * @param  ADCx :
            @arg ADC Instance
  * @retval 
  */
void ADC_Inj_StructInit(ADC_Inj_InitTypeDef* ADC_Inj_InitStruct)
{
	ADC_Inj_InitStruct->TriggerSource = ADC_CR2_JEXTSEL_INSOURCE;                                  
	ADC_Inj_InitStruct->ScanMode = ADC_CR1_SCAN_DISABLE;                                  
	ADC_Inj_InitStruct->SequencerNbRanks = ADC_JSQRL_SCAN_DISABLE;                                
	ADC_Inj_InitStruct->DataAlignment = ADC_CR2_ALIGN_RIGHT;                          
  ADC_Inj_InitStruct->TrigAuto = ADC_CR1_JAUTO_INDEPENDENT;                            
}

/**
  * @function  ADC_Inj_Init
  * @brief  read complete interrupt Flag status of the selected ADC
  * @param  ADCx :
            @arg ADC Instance
  * @retval 
  */
void ADC_Inj_Init(ADC_TypeDef *ADCx,ADC_Inj_InitTypeDef* ADC_Inj_InitStruct)
{
	ADC_Set_JEXTSEL(ADCx, ADC_Inj_InitStruct->TriggerSource);                          /* Set ADC group Inj trigger source */
	ADC_Set_SCAN(ADCx, ADC_Inj_InitStruct->ScanMode);                                  /* Set ADC sacn Inj group*/
	ADC_Set_JSQRL(ADCx, ADC_Inj_InitStruct->SequencerNbRanks);                         /* Set ADC Inj sequencer length and scan direction */
	ADC_Set_ALIGN(ADCx, ADC_Inj_InitStruct->DataAlignment);                            /* Set ADC conversion data alignment */
  ADC_Set_JAUTO(ADCx, ADC_Inj_InitStruct->TrigAuto);                                 /* Set ADC JAUTO mode*/
}

/**
  * @function  ADC_REG_RankInit
  * @brief  Initial REG Rank of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Length:
  *         @ref ADC_SQRL_SCAN_DISABLE
  *         @ref ADC_SQRL_SCAN_2RANKS
  *         @ref ADC_SQRL_SCAN_3RANKS
  *         @ref ADC_SQRL_SCAN_4RANKS
  *         @ref ADC_SQRL_SCAN_5RANKS
  *         @ref ADC_SQRL_SCAN_6RANKS
  *         @ref ADC_SQRL_SCAN_7RANKS
  *         @ref ADC_SQRL_SCAN_8RANKS
  *         @ref ADC_SQRL_SCAN_9RANKS 
  *         @ref ADC_SQRL_SCAN_10RANKS
  *         @ref ADC_SQRL_SCAN_11RANKS
  *         @ref ADC_SQRL_SCAN_12RANKS
  *         @ref ADC_SQRL_SCAN_13RANKS
  *         @ref ADC_SQRL_SCAN_14RANKS
  *         @ref ADC_SQRL_SCAN_15RANKS
  *         @ref ADC_SQRL_SCAN_16RANKS
  * @param  Channel:
  *         @ref ADC_CHANNEL_0    
  *         @ref ADC_CHANNEL_1   
  *         @ref ADC_CHANNEL_2   
  *         @ref ADC_CHANNEL_3   
  *         @ref ADC_CHANNEL_4   
  *         @ref ADC_CHANNEL_5   
  *         @ref ADC_CHANNEL_6   
  *         @ref ADC_CHANNEL_7   
  *         @ref ADC_CHANNEL_8   
  *         @ref ADC_CHANNEL_9   
  *         @ref ADC_CHANNEL_10  
  *         @ref ADC_CHANNEL_11  
  *         @ref ADC_CHANNEL_12  
  *         @ref ADC_CHANNEL_13  
  *         @ref ADC_CHANNEL_14  
  *         @ref ADC_CHANNEL_15  
  *         @ref ADC_CHANNEL_16  
  *         @ref ADC_CHANNEL_17  
  *         @ref ADC_CHANNEL_18  
  *         @ref ADC_CHANNEL_19  
  *         @ref ADC_CHANNEL_20  
  *         @ref ADC_CHANNEL_21  
  *         @ref ADC_CHANNEL_22  
  *         @ref ADC_CHANNEL_23  
  *         @ref ADC_CHANNEL_TPS 
  *         @ref ADC_CHANNEL_VREF
  *         @ref ADC_CHANNEL_SH  
  *         @ref ADC_CHANNEL_OPAMP1
  *         @ref ADC_CHANNEL_OPAMP2
  *         @ref ADC_CHANNEL_OPAMP3
  *         @ref ADC_CHANNEL_OPAMP4
  *         @ref ADC_CHANNEL_LDO 
  * @param  SamplingTime:
  *         @ref ADC_SAMPLINGTIME_1p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_2p5CYCLES        
  *         @ref ADC_SAMPLINGTIME_3p5CYCLES         
  *         @ref ADC_SAMPLINGTIME_4p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_7p5CYCLES        
  *         @ref ADC_SAMPLINGTIME_13p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_28p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_764p5CYCLES   
  * @retval None
  */
void ADC_REG_RankInit(ADC_TypeDef* ADCx,ADC_REG_RankInitTypeDef* ADC_REG_RankInitStruct)
{
  int i,rank;
  rank  = ADC_REG_RankInitStruct->Length >> 20 ;
  ADC_Set_SQRL(ADCx, ADC_REG_RankInitStruct->Length);                                                           /* Set ADC reg scan Length */
  for(i=0; i<=rank; i++)
  {
    if( i < 6 )
    {
      ADC_Set_SQ(ADCx,(ADC_SQR3_REGOFFSET|(i*5)), ADC_REG_RankInitStruct->Rank[i].Channel);                     /* Set reg sequence rank channel */        
    }
    else if( i < 12 )
    {		
      ADC_Set_SQ(ADCx,(ADC_SQR2_REGOFFSET|((i-6)*5)), ADC_REG_RankInitStruct->Rank[i].Channel);                 /* Set reg sequence rank channel */ 
    }
		else
		{
			ADC_Set_SQ(ADCx,(ADC_SQR1_REGOFFSET|((i-12)*5)), ADC_REG_RankInitStruct->Rank[i].Channel);                 /* Set reg sequence rank channel */ 
		}
    ADC_Set_SMPR(ADCx,ADC_REG_RankInitStruct->Rank[i].Channel, ADC_REG_RankInitStruct->Rank[i].SamplingTime);          /* Set channel sampling time */
  }
}

/**
  * @function  ADC_INJ_RankInit
  * @brief  Initial INJ Rank of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Length:
  *         @ref ADC_JSQRL_SCAN_DISABLE
  *         @ref ADC_JSQRL_SCAN_2RANKS
  *         @ref ADC_JSQRL_SCAN_3RANKS
  *         @ref ADC_JSQRL_SCAN_4RANKS
  * @param  Channel:
  *         @ref ADC_CHANNEL_0    
  *         @ref ADC_CHANNEL_1   
  *         @ref ADC_CHANNEL_2   
  *         @ref ADC_CHANNEL_3   
  *         @ref ADC_CHANNEL_4   
  *         @ref ADC_CHANNEL_5   
  *         @ref ADC_CHANNEL_6   
  *         @ref ADC_CHANNEL_7   
  *         @ref ADC_CHANNEL_8   
  *         @ref ADC_CHANNEL_9   
  *         @ref ADC_CHANNEL_10  
  *         @ref ADC_CHANNEL_11  
  *         @ref ADC_CHANNEL_12  
  *         @ref ADC_CHANNEL_13  
  *         @ref ADC_CHANNEL_14  
  *         @ref ADC_CHANNEL_15  
  *         @ref ADC_CHANNEL_16  
  *         @ref ADC_CHANNEL_17  
  *         @ref ADC_CHANNEL_18  
  *         @ref ADC_CHANNEL_19  
  *         @ref ADC_CHANNEL_20  
  *         @ref ADC_CHANNEL_21  
  *         @ref ADC_CHANNEL_22  
  *         @ref ADC_CHANNEL_23  
  *         @ref ADC_CHANNEL_TPS 
  *         @ref ADC_CHANNEL_VREF
  *         @ref ADC_CHANNEL_SH  
  *         @ref ADC_CHANNEL_OPAMP1
  *         @ref ADC_CHANNEL_OPAMP2
  *         @ref ADC_CHANNEL_OPAMP3
  *         @ref ADC_CHANNEL_OPAMP4
  *         @ref ADC_CHANNEL_LDO 
  * @param  SamplingTime:
  *         @ref ADC_SAMPLINGTIME_1p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_2p5CYCLES        
  *         @ref ADC_SAMPLINGTIME_3p5CYCLES         
  *         @ref ADC_SAMPLINGTIME_4p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_7p5CYCLES        
  *         @ref ADC_SAMPLINGTIME_13p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_28p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_764p5CYCLES   
  * @retval None
  */
void ADC_INJ_RankInit(ADC_TypeDef* ADCx,ADC_INJ_RankInitTypeDef* ADC_INJ_RankInitStruct)
{
  int i,rank;
  rank  = ADC_INJ_RankInitStruct->Length >> 20 ;
  ADC_Set_JSQRL(ADCx, ADC_INJ_RankInitStruct->Length);                                                            /* Set ADC inj scan Length */
  for(i=0; i<=rank; i++)
  {
    switch(i)
    {
      case 0:
        ADC_Set_JSQ(ADCx,ADC_INJ_RANK_1, ADC_INJ_RankInitStruct->Rank[i].Channel);                                 /* Set inj sequence rank channel */
        break;
      case 1:
        ADC_Set_JSQ(ADCx,ADC_INJ_RANK_2, ADC_INJ_RankInitStruct->Rank[i].Channel);                                 /* Set inj sequence rank channel */
        break;
      case 2:
        ADC_Set_JSQ(ADCx,ADC_INJ_RANK_3, ADC_INJ_RankInitStruct->Rank[i].Channel);                                 /* Set inj sequence rank channel */
        break;
      case 3:
        ADC_Set_JSQ(ADCx,ADC_INJ_RANK_4, ADC_INJ_RankInitStruct->Rank[i].Channel);                                  /* Set inj sequence rank channel */
        break;
      default:break;
    }                                           
    ADC_Set_SMPR(ADCx,ADC_INJ_RankInitStruct->Rank[i].Channel, ADC_INJ_RankInitStruct->Rank[i].SamplingTime);   /* Set channel sampling time */
  }    
}

/**
  * @function  ADC_Get_SR
  * @brief  read complete interrupt Flag status of the selected ADC
  * @param  ADCx :
            @arg ADC Instance
  * @param  IF:
  *         @arg  ADC_SR_AWD  
  *         @arg  ADC_SR_EOC  
  *         @arg  ADC_SR_JEOC 
  *         @arg  ADC_SR_JSTRT 
  *         @arg  ADC_SR_STRT 
  * @retval State of bit (1 or 0)
  */
uint32_t ADC_Get_SR(ADC_TypeDef* ADCx, uint32_t IF)
{
  return (READ_BIT(ADCx->SR, IF) == (IF));
}

/**
  * @function  ADC_Clear_SR
  * @brief  Clear complete interrupt Flag status of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  IF:
  *         @arg  ADC_SR_AWD 
  *         @arg  ADC_SR_EOC 
  *         @arg  ADC_SR_JEOC
  *         @arg  ADC_SR_JSTRT 
  *         @arg  ADC_SR_STRT
  */
void ADC_Clear_SR(ADC_TypeDef *ADCx, uint32_t IF)
{
  CLEAR_BIT(ADCx->SR, IF);
}

/**
  * @function  ADC_Set_AWDChannels
  * @brief  Set AWDChannels of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  AWDChannelGroup:
  *         @arg ADC_AWD_DISABLE
  *         @arg ADC_AWD_ALL_CHANNELS_REG
  *         @arg ADC_AWD_ALL_CHANNELS_INJ
  *         @arg ADC_AWD_ALL_CHANNELS_REG_INJ
  *         @arg ADC_AWD_CHANNEL_0_REG 
  *         @arg ADC_AWD_CHANNEL_0_INJ 
  *         @arg ADC_AWD_CHANNEL_0_REG_INJ
  *         @arg ADC_AWD_CHANNEL_1_REG 
  *         @arg ADC_AWD_CHANNEL_1_INJ 
  *         @arg ADC_AWD_CHANNEL_1_REG_INJ
  *         @arg ADC_AWD_CHANNEL_2_REG 
  *         @arg ADC_AWD_CHANNEL_2_INJ 
  *         @arg ADC_AWD_CHANNEL_2_REG_INJ
  *         @arg ADC_AWD_CHANNEL_3_REG 
  *         @arg ADC_AWD_CHANNEL_3_INJ 
  *         @arg ADC_AWD_CHANNEL_3_REG_INJ
  *         @arg ADC_AWD_CHANNEL_4_REG 
  *         @arg ADC_AWD_CHANNEL_4_INJ 
  *         @arg ADC_AWD_CHANNEL_4_REG_INJ
  *         @arg ADC_AWD_CHANNEL_5_REG 
  *         @arg ADC_AWD_CHANNEL_5_INJ 
  *         @arg ADC_AWD_CHANNEL_5_REG_INJ
  *         @arg ADC_AWD_CHANNEL_6_REG 
  *         @arg ADC_AWD_CHANNEL_6_INJ 
  *         @arg ADC_AWD_CHANNEL_6_REG_INJ
  *         @arg ADC_AWD_CHANNEL_7_REG       
  *         @arg ADC_AWD_CHANNEL_7_INJ      
  *         @arg ADC_AWD_CHANNEL_7_REG_INJ  
  *         @arg ADC_AWD_CHANNEL_8_REG       
  *         @arg ADC_AWD_CHANNEL_8_INJ      
  *         @arg ADC_AWD_CHANNEL_8_REG_INJ 
  *         @arg ADC_AWD_CHANNEL_9_REG      
  *         @arg ADC_AWD_CHANNEL_9_INJ       
  *         @arg ADC_AWD_CHANNEL_9_REG_INJ  
  *         @arg ADC_AWD_CHANNEL_10_REG      
  *         @arg ADC_AWD_CHANNEL_10_INJ     
  *         @arg ADC_AWD_CHANNEL_10_REG_INJ       
  *         @arg ADC_AWD_CHANNEL_11_REG      
  *         @arg ADC_AWD_CHANNEL_11_INJ     
  *         @arg ADC_AWD_CHANNEL_11_REG_INJ   
  *         @arg ADC_AWD_CHANNEL_12_REG      
  *         @arg ADC_AWD_CHANNEL_12_INJ     
  *         @arg ADC_AWD_CHANNEL_12_REG_INJ   
  *         @arg ADC_AWD_CHANNEL_13_REG      
  *         @arg ADC_AWD_CHANNEL_13_INJ     
  *         @arg ADC_AWD_CHANNEL_13_REG_INJ   
  *         @arg ADC_AWD_CHANNEL_14_REG      
  *         @arg ADC_AWD_CHANNEL_14_INJ     
  *         @arg ADC_AWD_CHANNEL_14_REG_INJ   
  *         @arg ADC_AWD_CHANNEL_15_REG      
  *         @arg ADC_AWD_CHANNEL_15_INJ     
  *         @arg ADC_AWD_CHANNEL_15_REG_INJ   
  *         @arg ADC_AWD_CHANNEL_16_REG      
  *         @arg ADC_AWD_CHANNEL_16_INJ     
  *         @arg ADC_AWD_CHANNEL_16_REG_INJ   
  *         @arg ADC_AWD_CHANNEL_17_REG      
  *         @arg ADC_AWD_CHANNEL_17_INJ     
  *         @arg ADC_AWD_CHANNEL_17_REG_INJ 
  *         @arg ADC_AWD_CHANNEL_18_REG    
  *         @arg ADC_AWD_CHANNEL_18_INJ    
  *         @arg ADC_AWD_CHANNEL_18_REG_INJ
  *         @arg ADC_AWD_CHANNEL_19_REG    
  *         @arg ADC_AWD_CHANNEL_19_INJ    
  *         @arg ADC_AWD_CHANNEL_19_REG_INJ
  *         @arg ADC_AWD_CHANNEL_20_REG    
  *         @arg ADC_AWD_CHANNEL_20_INJ    
  *         @arg ADC_AWD_CHANNEL_20_REG_INJ
  *         @arg ADC_AWD_CHANNEL_21_REG    
  *         @arg ADC_AWD_CHANNEL_21_INJ    
  *         @arg ADC_AWD_CHANNEL_21_REG_INJ
  *         @arg ADC_AWD_CHANNEL_22_REG    
  *         @arg ADC_AWD_CHANNEL_22_INJ    
  *         @arg ADC_AWD_CHANNEL_22_REG_INJ
  *         @arg ADC_AWD_CHANNEL_23_REG    
  *         @arg ADC_AWD_CHANNEL_23_INJ    
  *         @arg ADC_AWD_CHANNEL_23_REG_INJ
  * @retval None
  */
void ADC_Set_AWDChannels(ADC_TypeDef *ADCx, uint32_t AWDChannelGroup)
{
  MODIFY_REG(ADCx->CR1,
             (ADC_CR1_AWDEN | ADC_CR1_JAWDEN | ADC_CR1_AWDSGL | ADC_CR1_AWDCH),
             AWDChannelGroup);
}

/**
  * @function  ADC_Enable_IT
  * @brief  Enable ADC interrupt of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  IT:
            @arg ADC_CR1_EOCIE
            @arg ADC_CR1_AWDIE
            @arg ADC_CR1_JEOCIE
  * @retval None
  */
void ADC_Enable_IT(ADC_TypeDef *ADCx,uint16_t IT)
{
	SET_BIT(ADCx->CR1,IT);  //Enable
}

/**
  * @function  ADC_Disable_IT
  * @brief  Disable ADC interrupt of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  IT:
            @arg ADC_CR1_EOCIE
            @arg ADC_CR1_AWDIE
            @arg ADC_CR1_JEOCIE
  * @retval None
  */
void ADC_Disable_IT(ADC_TypeDef *ADCx,uint16_t IT)
{
	CLEAR_BIT(ADCx->CR1,IT); //Disable
}

/**
  * @function  ADC_Get_IT
  * @brief  read complete interrupt status of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  IT:
            @arg ADC_CR1_EOCIE
            @arg ADC_CR1_AWDIE
            @arg ADC_CR1_JEOCIE
  * @retval 0 is disable, 1 is enable
  */
uint32_t ADC_Get_IT(ADC_TypeDef *ADCx,uint16_t IT)
{
  return (READ_BIT(ADCx->CR1, IT) == (IT));
}

/**
  * @function  ADC_Set_SCAN
  * @brief  Set SCAN of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  ScanMode:
  *         @arg ADC_CR1_SCAN_DISABLE
  *         @arg ADC_CR1_SCAN_ENABLE
  * @retval None
  */
void ADC_Set_SCAN(ADC_TypeDef *ADCx, uint32_t ScanMode)
{
  MODIFY_REG(ADCx->CR1, ADC_CR1_SCAN, ScanMode);
}

/**
  * @function  ADC_Set_JAUTO
  * @brief  Set JAUTO of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  TrigAuto:
  *         @arg ADC_CR1_JAUTO_INDEPENDENT
  *         @arg ADC_CR1_JAUTO_FROM_REG
  * @retval None
  */
void ADC_Set_JAUTO(ADC_TypeDef *ADCx, uint32_t TrigAuto)
{
  MODIFY_REG(ADCx->CR1, ADC_CR1_JAUTO, TrigAuto);
}

/**
  * @function  ADC_Set_DISCNUM
  * @brief  Set AWDChannels of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  SeqDiscont:
  *         @arg ADC_CR1_DISCONT_DISABLE
  *         @arg ADC_CR1_DISCONT_1RANK
  *         @arg ADC_CR1_DISCONT_2RANKS
  *         @arg ADC_CR1_DISCONT_3RANKS
  *         @arg ADC_CR1_DISCONT_4RANKS
  *         @arg ADC_CR1_DISCONT_5RANKS
  *         @arg ADC_CR1_DISCONT_6RANKS
  *         @arg ADC_CR1_DISCONT_7RANKS
  *         @arg ADC_CR1_DISCONT_8RANKS
  * @retval None
  */
void ADC_Set_DISCNUM(ADC_TypeDef *ADCx, uint32_t SeqDiscont)
{
  MODIFY_REG(ADCx->CR1,
             (ADC_CR1_DISCEN | ADC_CR1_DISCNUM),
             SeqDiscont);
}

/**
  * @function  ADC_Enable_DISCEN
  * @brief  Enable DISCEN of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_Enable_DISCEN(ADC_TypeDef *ADCx)
{
	SET_BIT(ADCx->CR1,ADC_CR1_DISCEN);  //Enable
}

/**
  * @function  ADC_Disable_DISCEN
  * @brief  Disable DISCEN of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_Disable_DISCEN(ADC_TypeDef *ADCx)
{
  CLEAR_BIT(ADCx->CR1,ADC_CR1_DISCEN); //Disable
}

/**
  * @function  ADC_Enable_JDISCEN
  * @brief  Enable JDISCEN of the selected ADC
  * @param  ADCx:
            @arg ADC Instance 
  * @retval None
  */
void ADC_Enable_JDISCEN(ADC_TypeDef *ADCx)
{
	SET_BIT(ADCx->CR1,ADC_CR1_JDISCEN);  //Enable
}

/**
  * @function  ADC_Disable_JDISCEN
  * @brief  Disable JDISCEN of the selected ADC
  * @param  ADCx:
            @arg ADC Instance 
  * @retval None
  */
void ADC_Disable_JDISCEN(ADC_TypeDef *ADCx)
{
	CLEAR_BIT(ADCx->CR1,ADC_CR1_JDISCEN); //Disable
}

/**
  * @function  ADC_Enable_ADON
  * @brief  Enable ADON of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_Enable_ADON(ADC_TypeDef *ADCx)
{
  SET_BIT(ADCx->CR2, ADC_CR2_ADON);     //Enable
}

/**
  * @function  ADC_Disable_ADON
  * @brief  Disable ADON of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_Disable_ADON(ADC_TypeDef *ADCx)
{
  CLEAR_BIT(ADCx->CR2, ADC_CR2_ADON);  //Disable
}

/**
  * @function  ADC_Get_ADON
  * @brief  Get ADCON state of the selected ADC 
  * @param  ADCx:
            @arg ADC Instance
  * @retval 0 is disable, 1 is enable
  */
uint32_t ADC_Get_ADON(ADC_TypeDef *ADCx)
{
  return (READ_BIT(ADCx->CR2, ADC_CR2_ADON) == (ADC_CR2_ADON));
}

/**
  * @function  ADC_Set_CONT
  * @brief  Set ADC group regular continuous mode of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Continuous:
  *         @arg ADC_CR2_CONT_SINGLE
  *         @arg ADC_CR2_CONT_CONTINUOUS
  * @retval None
  */
void ADC_Set_CONT(ADC_TypeDef *ADCx, uint32_t Continuous)
{
  MODIFY_REG(ADCx->CR2, ADC_CR2_CONT, Continuous);
}

/**
  * @function  ADC_CAL
  * @brief  Trig CAL of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_CAL(ADC_TypeDef *ADCx)
{
  SET_BIT(ADCx->CR2, ADC_CR2_CAL);
}

/**
  * @function  ADC_Get_CAL
  * @brief  Get CAL state of the selected ADC 
  * @param  ADCx:
            @arg ADC Instance
  * @retval 0:CAL is finish, 1:CAL is going
  */
uint32_t ADC_Get_CAL(ADC_TypeDef *ADCx)
{
  return (READ_BIT(ADCx->CR2, ADC_CR2_CAL) == (ADC_CR2_CAL));
}

/**
  * @function  ADC_RSTCAL
  * @brief  Trig RSTCAL of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_RSTCAL(ADC_TypeDef *ADCx)
{
  SET_BIT(ADCx->CR2, ADC_CR2_RSTCAL);
}

/**
  * @function  ADC_Get_RSTCAL
  * @brief  Get RSTCAL state of the selected ADC 
  * @param  ADCx:
            @arg ADC Instance
  * @retval 0:RSTCAL is finish, 1:RSTCAL is going
  */
uint32_t ADC_Get_RSTCAL(ADC_TypeDef *ADCx)
{
  return (READ_BIT(ADCx->CR2, ADC_CR2_RSTCAL) == (ADC_CR2_RSTCAL));
}

/**
  * @function  ADC_Set_DMA
  * @brief  Set ADC group regular conversion data transfer of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  DMATransfer:
  *         @arg ADC_CR2_DMA_NONE
  *         @arg ADC_CR2_DMA_UNLIMITED
  * @retval None
  */
void ADC_Set_DMA(ADC_TypeDef *ADCx, uint32_t DMATransfer)
{
  MODIFY_REG(ADCx->CR2, ADC_CR2_DMA, DMATransfer);
}

/**
  * @function  ADC_Set_ALIGN
  * @brief  Set ADC conversion data alignment of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  DataAlignment:
  *         @arg ADC_CR2_ALIGN_RIGHT
  *         @arg ADC_CR2_ALIGN_LEFT
  * @retval None
  */
void ADC_Set_ALIGN(ADC_TypeDef *ADCx, uint32_t DataAlignment)
{
  MODIFY_REG(ADCx->CR2, ADC_CR2_ALIGN, DataAlignment);
}

/**
  * @function  ADC_Set_JEXTSEL
  * @brief  Set group injected trigger source of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  TriggerSource:
  *         @arg ADC_CR2_JEXTSEL_INSOURCE          
  *         @arg ADC_CR2_JEXTSEL_TIM1_TRGO            
  *         @arg ADC_CR2_JEXTSEL_TIM1_CC4            
  *         @arg ADC_CR2_JEXTSEL_TIM2_TRGO         
  *         @arg ADC_CR2_JEXTSEL_TIM2_CC1        
  *         @arg ADC_CR2_JEXTSEL_TIM3_CC4           
  *         @arg ADC_CR2_JEXTSEL_TIM4_TRGO       
  *         @arg ADC_CR2_JEXTSEL_EXTI15       
  *         @arg ADC_CR2_JEXTSEL_JSWSTART       
  *         @arg ADC_CR2_JEXTSEL_TIM1_CC1         
  *         @arg ADC_CR2_JEXTSEL_TIM1_CC2          
  *         @arg ADC_CR2_JEXTSEL_TIM1_CC3        
  *         @arg ADC_CR2_JEXTSEL_TIM1_CC5       
  *         @arg ADC_CR2_JEXTSEL_TIM1_CC4_CC5 
  *         @arg ADC_CR2_JEXTSEL_TIM1_CC6      
  * @retval None
  */
void ADC_Set_JEXTSEL(ADC_TypeDef *ADCx, uint32_t TriggerSource)
{
  MODIFY_REG(ADCx->CR2, ADC_CR2_JEXTSEL | ADC_CR2_JEXTTRIG , TriggerSource);
}

/**
  * @function  ADC_Set_EXTSEL
  * @brief  Set group regular trigger source of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  TriggerSource:
  *         @arg ADC_CR2_EXTSEL_INSOURCE  
  *         @arg ADC_CR2_EXTSEL_TIM1_CC1       
  *         @arg ADC_CR2_EXTSEL_TIM1_CC2   
  *         @arg ADC_CR2_EXTSEL_TIM1_CC3 
  *         @arg ADC_CR2_EXTSEL_TIM2_CC2    
  *         @arg ADC_CR2_EXTSEL_TIM3_TRGO  
  *         @arg ADC_CR2_EXTSEL_TIM4_CC4    
  *         @arg ADC_CR2_EXTSEL_EXTI11   
  *         @arg ADC_CR2_EXTSEL_SWSTART    
  * @retval None
  */
void ADC_Set_EXTSEL(ADC_TypeDef *ADCx, uint32_t TriggerSource)
{
  MODIFY_REG(ADCx->CR2, ADC_CR2_EXTSEL | ADC_CR2_EXTTRIG , TriggerSource);
}

/**
  * @function  ADC_Start_SW
  * @brief  Start ADC group regular conversion of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_Start_SW(ADC_TypeDef *ADCx)
{
  SET_BIT(ADCx->CR2, ADC_CR2_SWSTART);
}

/**
  * @function  ADC_Start_JSW
  * @brief  Start ADC group injected conversion of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_Start_JSW(ADC_TypeDef *ADCx)
{
  SET_BIT(ADCx->CR2, ADC_CR2_JSWSTART);
}

/**
  * @function  ADC_Enable_TSVREFE
  * @brief  Enable TSVREFE of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_Enable_TSVREFE(ADC_TypeDef *ADCx)
{
  SET_BIT(ADCx->CR2, ADC_CR2_TSVREFE);     //Enable
}


/**
  * @function  ADC_Disable_TSVREFE
  * @brief  Disable TSVREFE of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval None
  */
void ADC_Disable_TSVREFE(ADC_TypeDef *ADCx)
{
  CLEAR_BIT(ADCx->CR2, ADC_CR2_TSVREFE);   //Disable
}

/**
  * @function  ADC_Set_SMPR
  * @brief  Set selected channel sampling time of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Channel:
  *         @ref ADC_CHANNEL_0   
  *         @ref ADC_CHANNEL_1   
  *         @ref ADC_CHANNEL_2   
  *         @ref ADC_CHANNEL_3   
  *         @ref ADC_CHANNEL_4   
  *         @ref ADC_CHANNEL_5   
  *         @ref ADC_CHANNEL_6   
  *         @ref ADC_CHANNEL_7   
  *         @ref ADC_CHANNEL_8   
  *         @ref ADC_CHANNEL_9   
  *         @ref ADC_CHANNEL_10  
  *         @ref ADC_CHANNEL_11  
  *         @ref ADC_CHANNEL_12  
  *         @ref ADC_CHANNEL_13  
  *         @ref ADC_CHANNEL_14  
  *         @ref ADC_CHANNEL_15  
  *         @ref ADC_CHANNEL_16  
  *         @ref ADC_CHANNEL_17  
  *         @ref ADC_CHANNEL_18         
  *         @ref ADC_CHANNEL_19   
  *         @ref ADC_CHANNEL_20  
  *         @ref ADC_CHANNEL_21  
  *         @ref ADC_CHANNEL_22  
  *         @ref ADC_CHANNEL_23  
  *         @ref ADC_CHANNEL_TPS 
  *         @ref ADC_CHANNEL_VREF
  *         @ref ADC_CHANNEL_SH  
  *         @ref ADC_CHANNEL_OPAMP1
  *         @ref ADC_CHANNEL_OPAMP2
  *         @ref ADC_CHANNEL_OPAMP3
  *         @ref ADC_CHANNEL_OPAMP4
  *         @ref ADC_CHANNEL_LDO 
  * @param  SamplingTime:
  *         @ref ADC_SAMPLINGTIME_1p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_2p5CYCLES        
  *         @ref ADC_SAMPLINGTIME_3p5CYCLES         
  *         @ref ADC_SAMPLINGTIME_4p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_7p5CYCLES        
  *         @ref ADC_SAMPLINGTIME_13p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_28p5CYCLES       
  *         @ref ADC_SAMPLINGTIME_764p5CYCLES            
  * @retval None           
  */
void ADC_Set_SMPR(ADC_TypeDef *ADCx, uint32_t Channel, uint32_t SamplingTime)
{
  register uint32_t *preg = __ADC_PTR_REG_OFFSET(ADCx->SMPR1, __ADC_MASK_SHIFT(Channel, ADC_CHANNEL_SMPRX_REGOFFSET_MASK));
      MODIFY_REG(*preg, 
             ADC_SMPR2_SMP0 << __ADC_MASK_SHIFT(Channel, ADC_SMPR_BITOFFSET_MASK),
             (SamplingTime & ADC_SMPR2_SMP0)   << __ADC_MASK_SHIFT(Channel, ADC_SMPR_BITOFFSET_MASK));
}

/**
  * @function  ADC_Set_JOFR
  * @brief  Set ADC JOFRx of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Rank:
  *         @ref ADC_INJ_RANK_1       
  *         @ref ADC_INJ_RANK_2        
  *         @ref ADC_INJ_RANK_3       
  *         @ref ADC_INJ_RANK_4 
  * @param  offest:
  *         @ref offest
  * @retval None
  */
void ADC_Set_JOFR(ADC_TypeDef *ADCx, uint32_t Rank, uint32_t offest)
{
  register uint32_t *preg = __ADC_PTR_REG_OFFSET(ADCx->JOFR1,  __ADC_MASK_SHIFT(Rank, ADC_JOFRX_REGOFFSET_MASK));
  MODIFY_REG(*preg, ADC_JOFR1_JOFFSET1, offest);
}

/**
  * @function  ADC_Read_JOFR
  * @brief  Get JOFRx offest of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Rank:
  *         @ref ADC_INJ_RANK_1       
  *         @ref ADC_INJ_RANK_2        
  *         @ref ADC_INJ_RANK_3       
  *         @ref ADC_INJ_RANK_4 
  * @retval offest of JOFRx.
  */
uint32_t ADC_Read_JOFR(ADC_TypeDef *ADCx, uint32_t Rank)
{
  register uint32_t *preg = __ADC_PTR_REG_OFFSET(ADCx->JOFR1,  __ADC_MASK_SHIFT(Rank, ADC_JOFRX_REGOFFSET_MASK));
  return (uint32_t)(READ_BIT(*preg, ADC_JOFR1_JOFFSET1));
}

/**
  * @function  ADC_Set_AWDThresholds
  * @brief  Set ADC AWDThresholds of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  AWDThresholdsHighLow:
  *         @ref ADC_AWD_THRESHOLD_HIGH    
  *         @ref ADC_AWD_THRESHOLD_LOW
  * @param  AWDThresholdValue:
  *         @ref offest
  * @retval None
  */
void ADC_Set_AWDThresholds(ADC_TypeDef *ADCx, uint32_t AWDThresholdsHighLow, uint32_t AWDThresholdValue)
{
  register uint32_t *preg = __ADC_PTR_REG_OFFSET(ADCx->HTR, AWDThresholdsHighLow);
  MODIFY_REG(*preg, ADC_HTR_HT, AWDThresholdValue);
}

/**
  * @function  ADC_Read_AWDThresholds
  * @brief  Get AWDThresholds of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  AWDThresholdsHighLow:
  *         @ref ADC_AWD_THRESHOLD_HIGH    
  *         @ref ADC_AWD_THRESHOLD_LOW
  * @retval offest of HTR/LTR.
  */
uint32_t ADC_Read_AWDThresholds(ADC_TypeDef *ADCx, uint32_t AWDThresholdsHighLow)
{
  register uint32_t *preg = __ADC_PTR_REG_OFFSET(ADCx->HTR, AWDThresholdsHighLow);
  return (uint32_t)(READ_BIT(*preg, ADC_HTR_HT));
}

/**
  * @function  ADC_Set_SQRL
  * @brief  Set ADC group regular sequencer length and scan direction of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  SEQLENG:
  *         @ref ADC_SQRL_SCAN_DISABLE
  *         @ref ADC_SQRL_SCAN_2RANKS
  *         @ref ADC_SQRL_SCAN_3RANKS
  *         @ref ADC_SQRL_SCAN_4RANKS
  *         @ref ADC_SQRL_SCAN_5RANKS
  *         @ref ADC_SQRL_SCAN_6RANKS
  *         @ref ADC_SQRL_SCAN_7RANKS
  *         @ref ADC_SQRL_SCAN_8RANKS
  *         @ref ADC_SQRL_SCAN_9RANKS
  *         @ref ADC_SQRL_SCAN_10RANKS
  *         @ref ADC_SQRL_SCAN_11RANKS
  *         @ref ADC_SQRL_SCAN_12RANKS
  *         @ref ADC_SQRL_SCAN_13RANKS
  *         @ref ADC_SQRL_SCAN_14RANKS
  *         @ref ADC_SQRL_SCAN_15RANKS
  *         @ref ADC_SQRL_SCAN_16RANKS
  * @retval None
  */
void ADC_Set_SQRL(ADC_TypeDef *ADCx, uint32_t SequencerNbRanks)
{
  MODIFY_REG(ADCx->SQR1, ADC_SQR1_L, SequencerNbRanks);
}

/**
  * @function  ADC_Set_SQ
  * @brief  Set ADC group regular sequence: channel on the selected sequence rank of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Rank:
  *         @ref ADC_REG_RANK_1
  *         @ref ADC_REG_RANK_2
  *         @ref ADC_REG_RANK_3
  *         @ref ADC_REG_RANK_4 
  *         @ref ADC_REG_RANK_5
  *         @ref ADC_REG_RANK_6
  *         @ref ADC_REG_RANK_7
  *         @ref ADC_REG_RANK_8
  *         @ref ADC_REG_RANK_9
  *         @ref ADC_REG_RANK_10
  *         @ref ADC_REG_RANK_11
  *         @ref ADC_REG_RANK_12
  *         @ref ADC_REG_RANK_13
  *         @ref ADC_REG_RANK_14
  *         @ref ADC_REG_RANK_15
  *         @ref ADC_REG_RANK_16
  * @param  Channel:
  *         @ref ADC_CHANNEL_0   
  *         @ref ADC_CHANNEL_1   
  *         @ref ADC_CHANNEL_2   
  *         @ref ADC_CHANNEL_3   
  *         @ref ADC_CHANNEL_4   
  *         @ref ADC_CHANNEL_5   
  *         @ref ADC_CHANNEL_6   
  *         @ref ADC_CHANNEL_7   
  *         @ref ADC_CHANNEL_8   
  *         @ref ADC_CHANNEL_9   
  *         @ref ADC_CHANNEL_10  
  *         @ref ADC_CHANNEL_11  
  *         @ref ADC_CHANNEL_12  
  *         @ref ADC_CHANNEL_13  
  *         @ref ADC_CHANNEL_14  
  *         @ref ADC_CHANNEL_15  
  *         @ref ADC_CHANNEL_16  
  *         @ref ADC_CHANNEL_17  
  *         @ref ADC_CHANNEL_18         
  *         @ref ADC_CHANNEL_19        
  *         @ref ADC_CHANNEL_20    
  *         @ref ADC_CHANNEL_21  
  *         @ref ADC_CHANNEL_22  
  *         @ref ADC_CHANNEL_23  
  *         @ref ADC_CHANNEL_TPS 
  *         @ref ADC_CHANNEL_VREF
  *         @ref ADC_CHANNEL_SH  
  *         @ref ADC_CHANNEL_OPAMP1
  *         @ref ADC_CHANNEL_OPAMP2
  *         @ref ADC_CHANNEL_OPAMP3
  *         @ref ADC_CHANNEL_OPAMP4
  *         @ref ADC_CHANNEL_LDO 
  * @retval None
  */
void ADC_Set_SQ(ADC_TypeDef *ADCx, uint32_t Rank, uint32_t Channel)
{
  register uint32_t *preg = __ADC_PTR_REG_OFFSET(ADCx->SQR1, __ADC_MASK_SHIFT(Rank, ADC_SQRX_REGOFFSET_MASK));
  MODIFY_REG(*preg,
             ADC_SQR3_SQ1 << (Rank & ADC_REG_RANK_MASK),
             (Channel & ADC_SQR3_SQ1) << (Rank & ADC_REG_RANK_MASK));
}

/**
  * @function  ADC_Set_JSQRL
  * @brief  Set ADC group injected sequencer length and scan direction of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  SEQLENG:
  *         @ref ADC_JSQRL_SCAN_DISABLE
  *         @ref ADC_JSQRL_SCAN_2RANKS
  *         @ref ADC_JSQRL_SCAN_3RANKS
  *         @ref ADC_JSQRL_SCAN_4RANKS
  * @retval None
  */
void ADC_Set_JSQRL(ADC_TypeDef *ADCx, uint32_t SequencerNbRanks)
{
  MODIFY_REG(ADCx->JSQR, ADC_JSQR_JL, SequencerNbRanks);
}

/**
  * @function  ADC_Set_JSQ
  * @brief  Set ADC group injected sequence: channel on the selected sequence rank of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Rank:
  *         @ref ADC_INJ_RANK_1
  *         @ref ADC_INJ_RANK_2
  *         @ref ADC_INJ_RANK_3
  *         @ref ADC_INJ_RANK_4 
  * @param  Channel:
  *         @ref ADC_CHANNEL_0   
  *         @ref ADC_CHANNEL_1   
  *         @ref ADC_CHANNEL_2   
  *         @ref ADC_CHANNEL_3   
  *         @ref ADC_CHANNEL_4   
  *         @ref ADC_CHANNEL_5   
  *         @ref ADC_CHANNEL_6   
  *         @ref ADC_CHANNEL_7   
  *         @ref ADC_CHANNEL_8   
  *         @ref ADC_CHANNEL_9   
  *         @ref ADC_CHANNEL_10  
  *         @ref ADC_CHANNEL_11  
  *         @ref ADC_CHANNEL_12  
  *         @ref ADC_CHANNEL_13  
  *         @ref ADC_CHANNEL_14  
  *         @ref ADC_CHANNEL_15  
  *         @ref ADC_CHANNEL_16  
  *         @ref ADC_CHANNEL_17  
  *         @ref ADC_CHANNEL_18         
  *         @ref ADC_CHANNEL_19        
  *         @ref ADC_CHANNEL_20    
  *         @ref ADC_CHANNEL_21  
  *         @ref ADC_CHANNEL_22  
  *         @ref ADC_CHANNEL_23  
  *         @ref ADC_CHANNEL_TPS 
  *         @ref ADC_CHANNEL_VREF
  *         @ref ADC_CHANNEL_SH  
  *         @ref ADC_CHANNEL_OPAMP1
  *         @ref ADC_CHANNEL_OPAMP2
  *         @ref ADC_CHANNEL_OPAMP3
  *         @ref ADC_CHANNEL_OPAMP4
  *         @ref ADC_CHANNEL_LDO    
  * @retval None
  */
void ADC_Set_JSQ(ADC_TypeDef *ADCx, uint32_t Rank, uint32_t Channel)
{
  register uint32_t tmpreg1 = (READ_BIT(ADCx->JSQR, ADC_JSQR_JL) >> POSITION_VAL(ADC_JSQR_JL)) + 1U;
  MODIFY_REG(ADCx->JSQR,
             ADC_JSQR_JSQ1 << (5U * (uint8_t)(((Rank & ADC_INJ_RANK_MASK) + 3U) - (tmpreg1))),
             (Channel & ADC_JSQR_JSQ1) << (5U * (uint8_t)(((Rank & ADC_INJ_RANK_MASK) + 3U) - (tmpreg1))));
}

/**
  * @function  ADC_Read_JDR
  * @brief  Get value of JDRx of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Rank:
  *         @ref ADC_INJ_RANK_1
  *         @ref ADC_INJ_RANK_2
  *         @ref ADC_INJ_RANK_3
  *         @ref ADC_INJ_RANK_4 
  * @retval value of JDRx
  */
uint32_t ADC_Read_JDR(ADC_TypeDef *ADCx, uint32_t Rank)
{
  register uint32_t *preg = __ADC_PTR_REG_OFFSET(ADCx->JDR1, __ADC_MASK_SHIFT(Rank, ADC_JDRX_REGOFFSET_MASK));
  
  return (uint32_t)(READ_BIT(*preg, ADC_JDR1_JDATA));

}

/**
  * @function  ADC_Read_DR
  * @brief  Get value of DR of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval value of DR
  */
uint32_t ADC_Read_DR(ADC_TypeDef *ADCx)
{
  return (uint32_t)(READ_REG(ADCx->DR));
}

/**
  * @function  ADC_Read_DR_DATA
  * @brief  Get value of DR of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval value of DR_DATA
  */
uint16_t ADC_Read_DR_DATA(ADC_TypeDef *ADCx)
{
  return (uint16_t)(READ_BIT(ADCx->DR, ADC_DR_DATA));
}

/**
  * @function  ADC_Set_CAL_DATA
  * @brief  Set  CAL_DATA of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  offest:
  *         @ref offest
  * @retval None
  */
void ADC_Set_CAL_DATA(ADC_TypeDef *ADCx, uint32_t offest)
{
  MODIFY_REG(ADCx->CAL, ADC_CAL_DATA, offest);
}

/**
  * @function  ADC_Read_CAL_DATA
  * @brief  Get value of CAL_DATA of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @retval offest of CAL_DATA
  */
uint32_t ADC_Read_CAL_DATA(ADC_TypeDef *ADCx)
{
  return (uint32_t)(READ_BIT(ADCx->CAL, ADC_CAL_DATA));
}

/**
  * @function  ADC_Set_SMPS
  * @brief  Set selected channel sampling time sel of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Idx:
  *         @ref ADC_IDX_0       
  *         @ref ADC_IDX_1        
  *         @ref ADC_IDX_2        
  *         @ref ADC_IDX_3              
  * @param  Sel:
  *         @ref ADC_SMPS_IDX_SMPR       
  *         @ref ADC_SMPS_IDX_SMPV            
  * @retval None           
  */
//void ADC_Set_SMPS(ADC_TypeDef *ADCx, uint32_t Idx, uint32_t Sel)
//{
//  MODIFY_REG(ADCx->SMPS, (ADC_SMPS_IDX0 << Idx), (Sel << Idx));
//}


/**
  * @function  ADC_Set_SMPV
  * @brief  Set selected channel sampling time value of the selected ADC
  * @param  ADCx:
            @arg ADC Instance
  * @param  Idx:
  *         @ref ADC_IDX_0       
  *         @ref ADC_IDX_1        
  *         @ref ADC_IDX_2        
  *         @ref ADC_IDX_3              
  * @param  Value:  
  * @retval None           
  */
//void ADC_Set_SMPV(ADC_TypeDef *ADCx, uint32_t Idx, uint32_t Value)
//{
//  MODIFY_REG(ADCx->SMPV, (ADC_SMPV_IDX0 <<  (Idx*8)), (Value << (Idx*8)));
//}



