/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_cmp.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#define  __rx32s12xx_CMP_C

#include "rx32s12xx_cmp.h"
#include "rx32s12xx_rcc.h"
#include "rx32s12xx_adc.h"


/**
***********************************************************************************
*@Function CMP_Init
*@brief    Initial of COMPx
*@param1   COMPx
*@param2  CMP_InitStruct
*@param Pol:
*	        @arg CMP_CR1_POL_Positive
*	        @arg CMP_CR1_POL_Negative
*@param OFLT:
*	         @arg CMP_CxCR_OFLT_DIV1    
*	         @arg CMP_CxCR_OFLT_DIV32   
*	         @arg CMP_CxCR_OFLT_DIV64   
*	         @arg CMP_CxCR_OFLT_DIV128  
*	         @arg CMP_CxCR_OFLT_DIV256  
*	         @arg CMP_CxCR_OFLT_DIV512  
*	         @arg CMP_CxCR_OFLT_DIV1024 
*	         @arg CMP_CxCR_OFLT_DIV2048 
*@param Rhyst:
*	        @arg CMP_CR1_RHYST_0mV
*	        @arg CMP_CR1_RHYST_20mV
*         @arg CMP_CR1_RHYST_40mV
*	        @arg CMP_CR1_RHYST_60mV
*@param Fhyst:
*	        @arg CMP_CR1_FHYST_0mV
*	        @arg CMP_CR1_FHYST_20mV
*	        @arg CMP_CR1_FHYST_40mV
*	        @arg CMP_CR1_FHYST_60mV
*@param INP_Sel:
*	        @arg CMP_INP_SEL_1P
*	        @arg CMP_INP_SEL_2P
*          @arg CMP_INP_SEL_3P
*         @arg CMP_INP_SEL_OPA_out
*@param INN_Sel:
*	        @arg CMP_INN_SEL_1N 
*	        @arg CMP_INN_SEL_2N 
*	        @arg CMP_INN_SEL_3N 
*	        @arg CMP_INN_SEL_SRN
*	        @arg CMP_INN_SEL_CRV
***********************************************************************************
  */
  
void CMP_Init(COMP_TypeDef *COMPx, CMP_InitTypeDef *CMP_InitStruct)
{ 
	MODIFY_REG(COMPx->CR, (CMP_CxCR_POL | CMP_CxCR_RHYST | CMP_CxCR_FHYST),\
							(CMP_InitStruct->Pol | CMP_InitStruct->Rhyst | CMP_InitStruct->Fhyst));
	
	MODIFY_REG(COMPx->CR, CMP_CxCR_INM_SEL | CMP_CxCR_INP_SEL,\
							( CMP_InitStruct->INM_Sel | CMP_InitStruct->INP_Sel ));
	
	MODIFY_REG(COMPx->CR, CMP_CxCR_OFLT ,\
							( CMP_InitStruct->OFLT ));
  
  	MODIFY_REG(COMP->CR2, CMP_CR2_CRV_EN | CMP_CR2_CRV_SRC ,\
  ( CMP_InitStruct->CRV_En<<1 | CMP_InitStruct->CRV_Src ));
  
  
  	MODIFY_REG(COMP->CR2, CMP_CR2_CRV1_SEL ,CMP_InitStruct->CRV1SEL);
  	MODIFY_REG(COMP->CR2, CMP_CR2_CRV2_SEL ,CMP_InitStruct->CRV2SEL);
}

/**
  * @function  CMP_Enable
  * @brief  Enable EN of the selected CMP
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Enable(COMP_TypeDef *COMPx)
{
	SET_BIT(COMPx->CR,CMP_CxCR_EN);     
}

/**
  * @function  CMP_Disable
  * @brief  Disable EN of the selected CMP
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Disable(COMP_TypeDef *COMPx)
{
	CLEAR_BIT(COMPx->CR,CMP_CxCR_EN);  
}

/**
  * @function  CMP_Set_POL
  * @brief  Set POL of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @param  Continuous:
  *         @arg CMP_CR1_POL_Positive
  *         @arg CMP_CR1_POL_Negative
  * @retval None
  */
void CMP_Set_POL(COMP_TypeDef *COMPx, uint32_t Pol)
{
  MODIFY_REG(COMPx->CR, CMP_CxCR_POL, Pol);
}

/**
  * @function  CMP_Read_OUT
  * @brief  Get OUT Status of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @retval 0 is low output, 1 is high output
  */
uint32_t CMP_Read_OUT(COMP_TypeDef *COMPx)
{
  return (READ_BIT(COMPx->CR, CMP_CxCR_OUT) == (CMP_CxCR_OUT));
}

/**
  * @function  CMP_Enable_Lock
  * @brief  CR LOCK of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @retval None
  */
void CMP_Enable_Lock(COMP_TypeDef *COMPx)
{
  SET_BIT(COMPx->CR, CMP_CxCR_LOCK);
}

/**
  * @function  CMP_Enable_CR1Lock
  * @brief  CR1 LOCK of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @retval None
  */
void CMP_Enable_CR1Lock(void)
{
  SET_BIT(COMP->CR1, CMP_CR1_CRV_LOCK);
}

/**
  * @function  CMP_Enable_CR2Lock
  * @brief  CR LOCK of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @retval None
  */
void CMP_Enable_CR2Lock(void)
{
  SET_BIT(COMP->CR2, CMP_CR2_CRV_LOCK);
}
/**
***********************************************************************************
*@Function CMP_Enable_IT
*@brief    Enable Interrupt of the selected CMP
*@param1   COMPx:
*           @arg CMP Instance
*@param1   Flag:
*          @arg  CMP_CxCR_RIE
*          @arg  CMP_CxCR_FIE
*          @arg  CMP_CxCR_HIE
*          @arg  CMP_CxCR_LIE
*@retval State of bit (1 or 0).
***********************************************************************************
  */
void CMP_Enable_IT(COMP_TypeDef *COMPx, uint32_t IT)
{
	SET_BIT(COMPx->CR, IT );
}

/**
***********************************************************************************
*@Function CMP_Disable_IT
*@brief    Disable Interrupt of the selected CMP
*@param1   COMPx:
*           @arg CMP Instance
*@param1   Flag:
*          @arg  CMP_CxCR_RIE
*          @arg  CMP_CxCR_FIE
*          @arg  CMP_CxCR_HIE
*          @arg  CMP_CxCR_LIE
*@retval State of bit (1 or 0).
***********************************************************************************
  */
void CMP_Disable_IT(COMP_TypeDef *COMPx, uint32_t IT)
{
	CLEAR_BIT(COMPx->CR, IT );
}

/**
***********************************************************************************
*@Function CMP_Clear_Flag
*@brief    Clear CMP Flag.
*@param1   COMPx:
*           @arg CMP Instance
*@param1   Flag:
*          @arg  CMP_CxCR_RIF
*          @arg  CMP_CxCR_FIF
*          @arg  CMP_CxCR_HIF
*          @arg  CMP_CxCR_LIF
*@retval State of bit (1 or 0).
***********************************************************************************
  */
void CMP_Clear_Flag(COMP_TypeDef *COMPx, uint32_t Flag)
{
	CLEAR_BIT(COMPx->CR, Flag);
}

/**
***********************************************************************************
*@Function CMP_Get_Flag
*@brief    Get CMP Flag.
*@param1   COMPx:
*           @arg CMP Instance
*@param1   Flag:
*          @arg  CMP_CxCR_RIF
*          @arg  CMP_CxCR_FIF
*          @arg  CMP_CxCR_HIF
*          @arg  CMP_CxCR_LIF
*@retval State of bit (1 or 0).
***********************************************************************************
  */
uint32_t CMP_Get_Flag(COMP_TypeDef *COMPx, uint32_t Flag)
{
    return (READ_BIT(COMPx->CR, Flag) == (Flag));
}

/**
  * @function  CMP_Set_RHYST
  * @brief  Set RHYST of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @param  Continuous:
  *         @arg CMP_CxCR_RHYST_0mV
  *         @arg CMP_CxCR_RHYST_20mV
  *         @arg CMP_CxCR_RHYST_40mV
  *         @arg CMP_CxCR_RHYST_60mV
  * @retval None
  */
void CMP_Set_RHYST(COMP_TypeDef *COMPx, uint32_t Rhyst)
{
  MODIFY_REG(COMPx->CR, CMP_CxCR_RHYST, Rhyst);
}

/**
  * @function  CMP_Set_FHYST
  * @brief  Set FHYST of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @param  Continuous:
  *         @arg CMP_CxCR_FHYST_0mV 
  *         @arg CMP_CxCR_FHYST_20mV
  *         @arg CMP_CxCR_FHYST_40mV
  *         @arg CMP_CxCR_FHYST_60mV
  * @retval None
  */
void CMP_Set_FHYST(COMP_TypeDef *COMPx, uint32_t Fhyst)
{
  MODIFY_REG(COMPx->CR, CMP_CxCR_FHYST, Fhyst);
}

/**
  * @function  CMP_Set_INNSEL
  * @brief  Set INNSEL of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @param  Innsel:
  *         @arg CMP_INN_SEL_1N         
  *         @arg CMP_INN_SEL_2N         
  *         @arg CMP_INN_SEL_SRN (CMP1 only)       
  *         @arg CMP_INN_SEL_CRV       
  * @retval None
  */
void CMP_Set_INNSEL(COMP_TypeDef *COMPx, uint32_t Innsel)
{
  MODIFY_REG(COMPx->CR, CMP_CxCR_INM_SEL, Innsel);
}

/**
  * @function  CMP_Set_INPSEL
  * @brief  Set INNSEL of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @param  Continuous:
  *         @arg CMP_INP_SEL_1P
  *         @arg CMP_INP_SEL_2P
  *         @arg CMP_INP_SEL_3P(CMP1),OPA2_out(CMP2)
  *         @arg CMP_INP_SEL_OPA_out  (OPA1(CMP1),OPA3(CMP2))
  * @retval None
  */
void CMP_Set_INPSEL(COMP_TypeDef *COMPx, uint32_t Inpsel)
{
  MODIFY_REG(COMPx->CR, CMP_CxCR_INP_SEL, Inpsel);
}

/**
  * @function  CMP_Set_BLANKING
  * @brief  Set Blanking of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @param  Continuous:
  *         @arg CMP_CxCR_BLANKING_SEL_NoBlanking
  *         @arg CMP_CxCR_BLANKING_SEL_TIM2_OC4  
  *         @arg CMP_CxCR_BLANKING_SEL_TIM2_OC3  
  *         @arg CMP_CxCR_BLANKING_SEL_TIM3_OC3  
  *         @arg CMP_CxCR_BLANKING_SEL_TIM8_OC5  
  *         @arg CMP_CxCR_BLANKING_SEL_TIM8_OC4  
  *         @arg CMP_CxCR_BLANKING_SEL_TIM15_OC1  
  *         @arg CMP_CxCR_BLANKING_SEL_TIM15_OC2

  * @retval None
  */
void CMP_Set_BLANKING(COMP_TypeDef *COMPx, uint32_t Blanking)
{
  MODIFY_REG(COMPx->CR, CMP_CxCR_BLANKING_SEL, Blanking);
}

/**
  * @function  CMP_Set_PADJ_DATA
  * @brief  Set  PADJ_DATA of the selected CMP
  * @param  ADCx:
            @arg CMP Instance
  * @param  offest:
  *         @ref offest
  * @retval None
  */
void CMP_Set_PADJ_DATA(COMP_TypeDef *COMPx, uint32_t offest)
{
  MODIFY_REG(COMPx->CAL, CMP_RGxCAL_PADJ, offest);
}

/**
  * @function  CMP_Read_PADJ_DATA
  * @brief  Get value of PADJ_DATA of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @retval offest of PADJ_DATA
  */
uint32_t CMP_Read_PADJ_DATA(COMP_TypeDef *COMPx)
{
  return (uint32_t)(READ_BIT(COMPx->CAL, CMP_RGxCAL_PADJ));
}


/**
  * @function  CMP_Set_NADJ_DATA
  * @brief  Set  NADJ_DATA of the selected CMP
  * @param  ADCx:
            @arg CMP Instance
  * @param  offest:
  *         @ref offest
  * @retval None
  */
void CMP_Set_NADJ_DATA(COMP_TypeDef *COMPx, uint32_t offest)
{
  MODIFY_REG(COMPx->CAL, CMP_RGxCAL_NADJ, offest);
}

/**
  * @function  CMP_Get_NADJ_DATA
  * @brief  Get value of NADJ_DATA of the selected CMP
  * @param  COMPx:
            @arg CMP Instance
  * @retval offest of NADJ_DATA
  */
uint32_t CMP_Get_NADJ_DATA(COMP_TypeDef *COMPx)
{
  return (uint32_t)(READ_BIT(COMPx->CAL, CMP_RGxCAL_NADJ));
}

/**
  * @function  CMP_Enable_CALP
  * @brief  Enable PEN of the selected CMP
  * @param  COMPx:
  *          @arg CMP Instance
  * @retval None
  */
void CMP_Enable_CALP(COMP_TypeDef *COMPx)
{
	SET_BIT(COMPx->CAL,CMP_RGxCAL_PEN);      
}

/**
  * @function  CMP_Disable_PEN
  * @brief  Disable PEN of the selected CMP
  * @param  COMPx:
  *         @arg CMP Instance 
  * @retval None
  */
void CMP_Disable_CALP(COMP_TypeDef *COMPx)
{
	CLEAR_BIT(COMPx->CAL,CMP_RGxCAL_PEN); 
}

/**
  * @function  CMP_Enable_CALN
  * @brief  Enable NEN of the selected CMP
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Enable_CALN(COMP_TypeDef *COMPx)
{
	SET_BIT(COMPx->CAL,CMP_RGxCAL_NEN);      
}

/**
  * @function  CMP_Disable_CALN
  * @brief  Disable NEN of the selected CMP
  * @param  COMPx:
  *         @arg CMP Instance 
  * @retval None
  */
void CMP_Disable_CALN(COMP_TypeDef *COMPx)
{
	CLEAR_BIT(COMPx->CAL,CMP_RGxCAL_NEN); 
}

/**
  * @function  CMP_CH_BLANK_Enable
  * @brief  Enable the CMP change chanel blank
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Enable_CHBLANK(COMP_TypeDef *COMPx)
{
	SET_BIT(COMPx->CR,CMP_CxCR_CH_EN);     
}

/**
  * @function  CMP_CH_BLANK_Disable
  * @brief  Disable the CMP change chanel blank
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Disable_CHBLANK(COMP_TypeDef *COMPx)
{
	CLEAR_BIT(COMPx->CR,CMP_CxCR_CH_EN);  
}

/**
  * @function  CMP_BAS_Enable
  * @brief  Enable the ADC/OPA/SRN/CRV sampling blank
  * @param  COMPx:
  *         @arg CMP Instance
  * @param  Continuous:
  *         @arg CMP_CxCR_BASEN
  *         @arg CMP_CxCR_CMP_BAS_EN_OPA
  * @retval None
  */
void CMP_Enable_BAS(COMP_TypeDef *COMPx,uint32_t bas_select)
{
	SET_BIT(COMPx->CR,bas_select);     
}

void CMP_Enable_SRNBAS(void)
{
	SET_BIT(COMP->CR1,CMP_CR1_SRN_BAS_EN);     
}

void CMP_Enable_CRVBAS(void )
{
	SET_BIT(COMP->CR2,CMP_CRV_BAS_EN);     
}

/**
  * @function  CMP_BAS_Disable
  * @brief  Disable the ADC/OPA/SRN/CRV sampling blank
  * @param  COMPx:
  *         @arg CMP Instance
  * @param  Continuous:
  *         @arg CMP_CxCR_BASEN
  *         @arg CMP_CxCR_CMP_BAS_EN_OPA
  * @retval None
  */
void CMP_Disable_BAS(COMP_TypeDef *COMPx,uint32_t bas_select)
{
	CLEAR_BIT(COMPx->CR,bas_select);  
}

void CMP_Disable_SRNBAS(void)
{
	CLEAR_BIT(COMP->CR1,CMP_CR1_SRN_BAS_EN);     
}

void CMP_Disable_CRVBAS(void )
{
	CLEAR_BIT(COMP->CR2,CMP_CRV_BAS_EN);     
}

/**
  * @function  CMP_CH_DIS_Enable
  * @brief  Enable the p end of the CMP  switches over to actively close chanel
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Enable_CHDIS(COMP_TypeDef *COMPx)
{
	SET_BIT(COMPx->CR,CMP_CxCR_CH_DIS);     
}

/**
  * @function  CMP_CH_DIS_Disable
  * @brief  Disable the p end of the CMP  switches over to actively close chanel
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Disable_CHDIS(COMP_TypeDef *COMPx)
{
	CLEAR_BIT(COMPx->CR,CMP_CxCR_CH_DIS);  
}

/**
  * @function  CMP_CxCR_CHDIS_BLK_Enable
  * @brief  Enable EN of the CMP change chanel blank
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Enable_CHDISBLK(COMP_TypeDef *COMPx)
{
	SET_BIT(COMPx->CR,CMP_CxCR_CH_DIS_BLANKING);     
}

/**
  * @function  CMP_CxCR_CHDIS_BLK_Disable
  * @brief  Disable EN of the CMP change chanel blank
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Disable_CHDISBLK(COMP_TypeDef *COMPx)
{
	CLEAR_BIT(COMPx->CR,CMP_CxCR_CH_DIS_BLANKING);  
}

/**
  * @function  CMP_SRN_Enable
  * @brief  Enable the CMP SRN
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Enable_SRN(void)
{
	SET_BIT(COMP->CR1,CMP_CR1_SRN_EN);     
}

/**
  * @function  CMP_SRN_Disable
  * @brief  Disable the CMP SRN
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Disable_SRN(void)
{
	CLEAR_BIT(COMP->CR1,CMP_CR1_SRN_EN);     
}

/**
  * @function  CMP_SRNdout_Enable
  * @brief  Enable the CMP SRN_dout
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Enable_SRNdout(void)
{
	SET_BIT(COMP->CR1,CMP_CR1_SRN_Dout);     
}

/**
  * @function  CMP_SRNdout_Disable
  * @brief  Disable the CMP SRN_dout
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Disable_SRNdout(void)
{
	CLEAR_BIT(COMP->CR1,CMP_CR1_SRN_Dout);     
}

/**
  * @function  CMP_CRV_Enable
  * @brief  Enable the CMP CRV
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Enable_CRV(void)
{
	SET_BIT(COMP->CR2,CMP_CR2_CRV_EN);     
}

/**
  * @function  CMP_CRV_Disable
  * @brief  Disable the CMP CRV
  * @param  COMPx:
  *         @arg CMP Instance
  * @retval None
  */
void CMP_Disable_CRV(void)
{
	CLEAR_BIT(COMP->CR2,CMP_CR2_CRV_EN);     
}

/**
***********************************************************************************
*@Function CMP_Set_CH1VSEL
*@brief    SET CMP_CR2_CRV_SEL.
*@param   val:
*         @arg   CMP_CRV1_SEL_1_32 
*         @arg   CMP_CRV1_SEL_2_32 
*         @arg   CMP_CRV1_SEL_3_32 
*         @arg   CMP_CRV1_SEL_4_32 
*         @arg   CMP_CRV1_SEL_5_32 
*         @arg   CMP_CRV1_SEL_6_32 
*         @arg   CMP_CRV1_SEL_7_32 
*         @arg   CMP_CRV1_SEL_8_32 
*         @arg   CMP_CRV1_SEL_9_32 
*         @arg   CMP_CRV1_SEL_10_32
*         @arg   CMP_CRV1_SEL_11_32
*         @arg   CMP_CRV1_SEL_12_32
*         @arg   CMP_CRV1_SEL_13_32
*         @arg   CMP_CRV1_SEL_14_32
*         @arg   CMP_CRV1_SEL_15_32
*         @arg   CMP_CRV1_SEL_16_32
*         @arg   CMP_CRV1_SEL_17_32
*         @arg   CMP_CRV1_SEL_18_32
*         @arg   CMP_CRV1_SEL_19_32
*         @arg   CMP_CRV1_SEL_20_32
*         @arg   CMP_CRV1_SEL_21_32
*         @arg   CMP_CRV1_SEL_22_32
*         @arg   CMP_CRV1_SEL_23_32
*         @arg   CMP_CRV1_SEL_24_32
*         @arg   CMP_CRV1_SEL_25_32
*         @arg   CMP_CRV1_SEL_26_32
*         @arg   CMP_CRV1_SEL_27_32
*         @arg   CMP_CRV1_SEL_28_32
*         @arg   CMP_CRV1_SEL_29_32
*         @arg   CMP_CRV1_SEL_30_32
*         @arg   CMP_CRV1_SEL_31_32
*         @arg   CMP_CRV1_SEL_32_32
***********************************************************************************
  */
void CMP_Set_CRV1SEL(uint32_t val)
{
	MODIFY_REG(COMP->CR2,  CMP_CR2_CRV1_SEL , val );
}

/**
***********************************************************************************
*@Function CMP_Set_CH2VSEL
*@brief    SET CMP_CR2_CRV_SEL.
*@param   val:
*         @arg   CMP_CRV2_SEL_1_32 
*         @arg   CMP_CRV2_SEL_2_32 
*         @arg   CMP_CRV2_SEL_3_32 
*         @arg   CMP_CRV2_SEL_4_32 
*         @arg   CMP_CRV2_SEL_5_32 
*         @arg   CMP_CRV2_SEL_6_32 
*         @arg   CMP_CRV2_SEL_7_32 
*         @arg   CMP_CRV2_SEL_8_32 
*         @arg   CMP_CRV2_SEL_9_32 
*         @arg   CMP_CRV2_SEL_10_32
*         @arg   CMP_CRV2_SEL_11_32
*         @arg   CMP_CRV2_SEL_12_32
*         @arg   CMP_CRV2_SEL_13_32
*         @arg   CMP_CRV2_SEL_14_32
*         @arg   CMP_CRV2_SEL_15_32
*         @arg   CMP_CRV2_SEL_16_32
*         @arg   CMP_CRV2_SEL_17_32
*         @arg   CMP_CRV2_SEL_18_32
*         @arg   CMP_CRV2_SEL_19_32
*         @arg   CMP_CRV2_SEL_20_32
*         @arg   CMP_CRV2_SEL_21_32
*         @arg   CMP_CRV2_SEL_22_32
*         @arg   CMP_CRV2_SEL_23_32
*         @arg   CMP_CRV2_SEL_24_32
*         @arg   CMP_CRV2_SEL_25_32
*         @arg   CMP_CRV2_SEL_26_32
*         @arg   CMP_CRV2_SEL_27_32
*         @arg   CMP_CRV2_SEL_28_32
*         @arg   CMP_CRV2_SEL_29_32
*         @arg   CMP_CRV2_SEL_30_32
*         @arg   CMP_CRV2_SEL_31_32
*         @arg   CMP_CRV2_SEL_32_32
***********************************************************************************
  */
void CMP_Set_CRV2SEL(uint32_t val)
{
	MODIFY_REG(COMP->CR2,  CMP_CR2_CRV2_SEL , val );
}

