/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2019, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_flash.c
* By           : RX_SD_Team
* Description  : Only rx32s12xx
*********************************************************************************************************
*/

#define  __rx32s12xx_FLASH_C
#include "rx32s12xx_flash.h"



#define FLASH_PAGE_SIZE                 0x400U  /* 1 KB */



uint16_t FLASH_Read_HalfWord(uint32_t ADDR)
{
  return *(__IO uint16_t*)ADDR ; 
}
/**
  * @brief  Sets the code latency value.
  * @param  FLASH_Latency: specifies the FLASH Latency value.
  *     @arg FLASH_Latency_0
  *     @arg FLASH_Latency_1
  *     @arg FLASH_Latency_2
  *     @arg FLASH_Latency_3
  * @retval None
  */
void FLASH_Set_Latency(uint32_t FLASH_Latency)
{
  MODIFY_REG(FLASH->ACR, FLASH_ACR_LATENCY, FLASH_Latency);
}

/**
  * @brief  Sets the code latency value.
  * @param  none
  * @retval FLASH_Latency: specifies the FLASH Latency value.
  *     @arg FLASH_Latency_0
  *     @arg FLASH_Latency_1
  *     @arg FLASH_Latency_2
  */
uint32_t FLASH_Get_Latency( void )
{
  return (uint32_t)(READ_BIT(FLASH->ACR, FLASH_ACR_LATENCY));
}


/**
  * @brief  Enables or disables the Half cycle flash access.
  * @param  FLASH_HalfCycleAccess: specifies the FLASH Half cycle Access mode.
  *     @arg FLASH_HalfCycleAccess_Enable
  *     @arg FLASH_HalfCycleAccess_Disable
  * @retval None
  */
void FLASH_Set_HalfCycleAccessCmd(uint32_t FLASH_HalfCycleAccess)
{
  /* Enable or disable the Half cycle access */
  FLASH->ACR &= FLASH_ACR_HLFCYA_Mask;
  FLASH->ACR |= FLASH_HalfCycleAccess;
}

/**
  * @brief  Enables or disables the Prefetch Buffer.
  * @param  FLASH_PrefetchBuffer: specifies the Prefetch buffer status.
  *     @arg FLASH_PrefetchBuffer_Enable
  *     @arg FLASH_PrefetchBuffer_Disable
  * @retval None
  */
void FLASH_Set_PrefetchBufferCmd(uint32_t FLASH_PrefetchBuffer)
{
  /* Enable or disable the Prefetch Buffer */
  FLASH->ACR &= FLASH_ACR_PRFTBE_Mask;
  FLASH->ACR |= FLASH_PrefetchBuffer;
}

/**
  * @brief  Unlocks the FLASH Program Erase Controller.
  * @note 
  * @param  None
  * @retval None
  */
void FLASH_Unlock(void)
{
  /* Authorize the FPEC of Bank1 Access */
  FLASH->KEYR = FLASH_CRKEY1;
  FLASH->KEYR = FLASH_CRKEY2;
}



void FLASH_OB_Unlock(void)
{
  /* Authorize the FPEC of Bank1 Access */
  FLASH->OPTKEYR = FLASH_OPTKEY1;
  FLASH->OPTKEYR = FLASH_OPTKEY2;
}
/**
  * @brief  Locks the FLASH Program Erase Controller.
  * @note   
  * @param  None
  * @retval None
  */
void FLASH_Lock(void)
{
  /* Set the Lock Bit to lock the FPEC and the CR of  Bank1 */
  FLASH->CR |= FLASH_CR_LOCK;
}

void FLASH_OB_Lock(void)
{
  /* Set the Lock Bit to lock the FPEC and the CR of  Bank1 */
  FLASH->CR |= FLASH_CR_OPTLOCK;
}

/**
  * @brief  Erases a specified FLASH page.
  * @param  Page_Address: The page address to be erased.
  * @retval None.
  */
void FLASH_Erase_Page(uint32_t Page_Num)
{
    /* if the previous operation is completed, proceed to erase the page */
  /* Disable the PER Bit */
    FLASH_Clear_Flag(FLASH_SR_WRPRTERR|FLASH_SR_EOP); 
    CLEAR_BIT(FLASH->CR, FLASH_CR_PG);
    CLEAR_BIT(FLASH->CR, FLASH_CR_PER);


    /* Proceed to erase the page 256-512 */
    MODIFY_REG(FLASH->CR, FLASH_CR_PNB, ((Page_Num & 0xFFU) << 3U));
 
//    /* Proceed to erase the page 0-255 */
//    MODIFY_REG(FLASH->CR, FLASH_CR_PNB, ((Page_Address & 0xFFU) << 3U));
 
	
    SET_BIT(FLASH->CR, FLASH_CR_PER);
    SET_BIT(FLASH->CR, FLASH_CR_STRT);
	
//    for(uint32_t m =EraseTimeout;m>0;m--){};
    while((FLASH->SR & FLASH_SR_BSY) == FLASH_SR_BSY)
    {
//       IWDG_Reload_Counter(IWDG);
    }; 

    /* Disable the PER Bit */
    CLEAR_BIT(FLASH->CR, (FLASH_CR_PER | FLASH_CR_PNB));
}

/**
  * @brief  Erases all FLASH pages.
  * @param  None
  * @retval None.
  */
void FLASH_Erase_AllPages(void)
{
  FLASH_Clear_Flag(FLASH_SR_WRPRTERR|FLASH_SR_EOP); 
    CLEAR_BIT(FLASH->CR, FLASH_CR_PG);
    CLEAR_BIT(FLASH->CR, FLASH_CR_PER);
	
	    SET_BIT(FLASH->CR, FLASH_CR_MER);
    SET_BIT(FLASH->CR, FLASH_CR_STRT);
	
  while((FLASH->SR & FLASH_SR_BSY) == FLASH_SR_BSY)
    {
//       IWDG_Reload_Counter(IWDG);
    }; 

    /* Disable the PER Bit */
    CLEAR_BIT(FLASH->CR, (FLASH_CR_MER | FLASH_CR_PNB));
}


void FLASH_Program_DoubleWord(uint32_t Address, uint64_t Data)
{
  /* Set PG bit */
  CLEAR_BIT(FLASH->CR, FLASH_CR_PER);
  if(FLASH_Get_Flag(FLASH_SR_EOP))
  {
    SET_BIT(FLASH->SR, FLASH_SR_EOP);
  };
  
    /* clear prften bit */
   uint32_t temp =0x0;
   temp = FLASH->ACR;
  CLEAR_BIT(FLASH->ACR,FLASH_ACR_PRFTEN);
  
  SET_BIT(FLASH->CR, FLASH_CR_PG);

  /* Program first word */
  *(uint32_t *)Address = (uint32_t)Data;

  /* Barrier to ensure programming is performed in 2 steps, in right order
    (independently of compiler optimization behavior) */
  __ISB();

  /* Program second word */
  *(uint32_t *)(Address + 4U) = (uint32_t)(Data >> 32U) ;
	/* Wait for last operation to be completed */
//  FLASH_WaitForLastOperation(ProgramTimeout);
	 while((FLASH->SR & FLASH_SR_BSY) == FLASH_SR_BSY)
    {
//			   IWDG_Reload_Counter(IWDG);
    }; 
  CLEAR_BIT(FLASH->CR, FLASH_CR_PG);
    
      FLASH->ACR = temp;
}

void FLASH_Program_DoubleDoubleWord(uint32_t Address, uint64_t DataL, uint64_t DataH)
{
  /* Set PG bit */
  CLEAR_BIT(FLASH->CR, FLASH_CR_PER);
  
  FLASH->SR = FLASH_PGERR;
  
  if(FLASH_Get_Flag(FLASH_SR_EOP))
  {
    SET_BIT(FLASH->SR, FLASH_SR_EOP);
  };
  SET_BIT(FLASH->CR, FLASH_CR_PG);
  
  /* clear prften bit */
   uint32_t temp =0x0;
   temp = FLASH->ACR;
  CLEAR_BIT(FLASH->ACR,FLASH_ACR_PRFTEN);
  
  /* Program first word */
  *(uint32_t *)Address = (uint32_t)DataL;
  __ISB();
  /* Program second word */
  *(uint32_t *)(Address + 4U) = (uint32_t)(DataL >> 32U);
  __ISB();
  /* Program third word */
  *(uint32_t *)(Address + 8U) = (uint32_t)(DataH);
  __ISB();
  /* Program four word */
  *(uint32_t *)(Address + 0xCU) = (uint32_t)(DataH >> 32U);
	/* Wait for last operation to be completed */
//  FLASH_WaitForLastOperation(ProgramTimeout);
	 while((FLASH->SR & FLASH_SR_BSY) == FLASH_SR_BSY)
    {
//			   IWDG_Reload_Counter(IWDG);
    }; 
	
  CLEAR_BIT(FLASH->CR, FLASH_CR_PG);
    
     FLASH->ACR = temp;
}
 



uint32_t FLASH_Read_Word(uint32_t ADDR)
{
  return *(__IO uint32_t*)ADDR ; 
}

/**
  * @brief  Enables or disables the read out protection.
  * @param  Newstate: new state of the ReadOut Protection.
  *     @arg ENABLE
  *     @arg DISABLE.
  * @retval None.
  */
void FLASH_Set_ReadOutProtection(FunctionalState NewState)
{

    /* unlock CR and OPTR key  */
    FLASH_Unlock();
    FLASH_OB_Unlock();

      if(NewState != DISABLE)
      {
        MODIFY_REG(FLASH->OPTR, FLASH_OPTR_RDP, 0xBB);
      }
      else
      {
        MODIFY_REG(FLASH->OPTR, FLASH_OPTR_RDP, 0xAA);   
      }  
      
	SET_BIT(FLASH->CR,FLASH_CR_OPTSTRT);
  
   while((FLASH->SR & FLASH_SR_BSY) == FLASH_SR_BSY)
    {
//		  IWDG_Reload_Counter(IWDG);
    }; 
	 FLASH_Lock();
	FLASH_OB_Lock();      
      
}

/**
  * @brief  Write protects the desired pages
  * @param  FLASH_Pages: specifies the address of the pages to be write protected.
  *   This parameter can be:
  *     @arg For @b Medium-density_devices: value between FLASH_WRProt_Pages0to63
  *     @arg FLASH_WRProt_AllPages
  * @retval None.
  */
void FLASH_Enable_WriteProtection(uint32_t START_Pages,uint32_t END_Pages)
{
 FLASH_Unlock();
 FLASH_OB_Unlock();
  
	MODIFY_REG(FLASH->WRP1AR, FLASH_WRP1AR_WRP1A_STRT, START_Pages );  
	MODIFY_REG(FLASH->WRP1AR, FLASH_WRP1AR_WRP1A_END, END_Pages << 16 );        
  
  
	SET_BIT(FLASH->CR,FLASH_CR_OPTSTRT);
  
   while((FLASH->SR & FLASH_SR_BSY) == FLASH_SR_BSY)
    {
//		  IWDG_Reload_Counter(IWDG);
    }; 
	 FLASH_Lock();
	FLASH_OB_Lock();
}
/**
  * @brief  Programs the FLASH User Option Byte: IWDG_SW / RST_STOP / RST_STDBY / IWDG_STDBY / IWDG_STOP / WWDG_SW / ECC / Nboot1 / nSWBOOT0 / nBOOT0.
  * @param  OB_IWDG_SW: Selects the IWDG mode
  *    @arg FLASH_OPTR_IWDG_SW
  * @param  OB_IWDG_STOP: Independent watchdog counter freeze in Stop mode.
  *    @arg FLASH_OPTR_IWDG_STOP
  * @param  OB_IWDG_STDBY: Independent watchdog counter freeze in Standby mode.
  *    @arg FLASH_OPTR_IWDG_STDBY
  * @param  OB_nRST_STOP: Reset event when entering STOP mode.
  *    @arg FLASH_OPTR_nRST_STOP
  * @param  FLASH_OPTR_WWDG_SW: Window watchdog selection.
  *    @arg FLASH_OPTR_WWDG_SW
  * @param  FLASH_OPTR_ECC:for ECC Mode.
  *    @arg FLASH_OPTR_ECC
  * @param  FLASH_OPTR_Nboot1: Boot configuration.
  *    @arg FLASH_OPTR_Nboot1
  * @param  FLASH_OPTR_nSWBOOT0: Software BOOT0.
  *    @arg FLASH_OPTR_nSWBOOT0
  * @param  FLASH_OPTR_nBOOT0: Boot configuration.
  *    @arg FLASH_OPTR_nBOOT0
  * @retval None.
  */
void FLASH_Set_UserOptionByte(uint32_t OB_USER)
{
  if(OB_USER != 0)
	{
   FLASH_Unlock();
   FLASH_OB_Unlock();
    
   SET_BIT(FLASH->OPTR,OB_USER);      

	 SET_BIT(FLASH->CR,FLASH_CR_OPTSTRT);
   
   while((FLASH->SR & FLASH_SR_BSY) == FLASH_SR_BSY)
   {
   }; 
	 FLASH_Lock();
	 FLASH_OB_Lock();    
  }
}
/**
  * @brief  Clear the FLASH User Option Byte:IWDG_SW / RST_STOP / RST_STDBY / IWDG_STDBY / IWDG_STOP / WWDG_SW / ECC / Nboot1 / nSWBOOT0 / nBOOT0.
  * @param  OB_IWDG_SW: Selects the IWDG mode
  *    @arg FLASH_OPTR_IWDG_SW
  * @param  OB_IWDG_STOP: Independent watchdog counter freeze in Stop mode.
  *    @arg FLASH_OPTR_IWDG_STOP
  * @param  OB_IWDG_STDBY: Independent watchdog counter freeze in Standby mode.
  *    @arg FLASH_OPTR_IWDG_STDBY
  * @param  OB_nRST_STOP: Reset event when entering STOP mode.
  *    @arg FLASH_OPTR_nRST_STOP
  * @param  FLASH_OPTR_WWDG_SW: Window watchdog selection.
  *    @arg FLASH_OPTR_WWDG_SW
  * @param  FLASH_OPTR_ECC:for ECC Mode.
  *    @arg FLASH_OPTR_ECC
  * @param  FLASH_OPTR_Nboot1: Boot configuration.
  *    @arg FLASH_OPTR_Nboot1
  * @param  FLASH_OPTR_nSWBOOT0: Software BOOT0.
  *    @arg FLASH_OPTR_nSWBOOT0
  * @param  FLASH_OPTR_nBOOT0: Boot configuration.
  *    @arg FLASH_OPTR_nBOOT0
  * @retval None.
    */
void FLASH_Clear_UserOptionByte(uint32_t OB_USER)
{ 
  if(OB_USER != 0)
	{
    FLASH_Unlock();
    FLASH_OB_Unlock();
    
    CLEAR_BIT(FLASH->OPTR,OB_USER);        
    
    SET_BIT(FLASH->CR,FLASH_CR_OPTSTRT);
  
    while((FLASH->SR & FLASH_SR_BSY) == FLASH_SR_BSY)
    {
    }; 
    
//    SET_BIT(FLASH->CR,FLASH_CR_OBL_LAUNCH); 
    
	  FLASH_Lock();
	  FLASH_OB_Lock(); 
  }
}

/**
  * @brief  Returns the FLASH Write Protection Option Bytes Register value.
  * @param  None
  * @retval The FLASH Write Protection  Option Bytes Register value
  */
uint32_t FLASH_Get_WriteProtectionOptionByte(void)
{
  /* Return the Flash write protection Register value */
  return (uint32_t)(FLASH->WRP1AR);
}


/**
  * @brief  Checks whether the FLASH Prefetch Buffer status is set or not.
  * @param  None
  * @retval FLASH Prefetch Buffer Status (1 or 0).
  */
uint32_t FLASH_Get_PrefetchBufferStatus(void)
{
  return (uint32_t)( (READ_BIT(FLASH->ACR, FLASH_ACR_PRFTEN)) != 0); 
}

/**
  * @brief  Enables the specified FLASH interrupts.
  * @param  FLASH_IT: specifies the FLASH interrupt sources to be enabled or disabled.
  *     @arg FLASH_CR_ERRIE
  *     @arg FLASH_CR_EOPIE
  *     @arg FLASH_CR_RDERRIE
  * @retval None 
  */
void FLASH_Enable_IT(uint32_t FLASH_IT)
{
  /* Enable the interrupt sources */
  SET_BIT(FLASH->CR, FLASH_IT);
}

/**
  * @brief  Disables the specified FLASH interrupts.
  * @param  FLASH_IT: specifies the FLASH interrupt sources to be enabled or disabled.
  *     @arg FLASH_CR_ERRIE
  *     @arg FLASH_CR_EOPIE   
  *     @arg FLASH_CR_RDERRIE   
  * @retval None 
  */
void FLASH_Disable_IT(uint32_t FLASH_IT)
{
  /* Disable the interrupt sources */
  CLEAR_BIT(FLASH->CR, FLASH_IT);
}

/**
  * @brief  Get the specified FLASH interrupts.
  * @param  FLASH_IT: specifies the FLASH interrupt sources to be enabled or disabled.
  *     @arg FLASH_CR_ERRIE
  *     @arg FLASH_CR_EOPIE   
  *     @arg FLASH_CR_RDERRIE   
  * @retval (0 or 1) 
  */
uint32_t FLASH_Get_IT(uint32_t FLASH_IT)
{
   return (uint32_t)(READ_BIT(FLASH->CR, FLASH_IT) == (FLASH_IT));
}

/**
  * @brief  Checks whether the specified FLASH flag is set or not.
  * @param  FLASH_FLAG: specifies the FLASH flag to check.
  *     @arg FLASH_SR_BSY      
  *     @arg FLASH_SR_PGSERR   
  *     @arg FLASH_SR_WRPRTERR    
  *     @arg FLASH_SR_EOP         
  *     @arg FLASH_SR_OPTVERR
  * @retval The new state of FLASH_FLAG (1 or 0).
  */
uint32_t FLASH_Get_Flag(uint32_t FLASH_FLAG)
{
  if(FLASH_FLAG == FLASH_SR_OPTVERR) 
  {
    return (uint32_t)(READ_BIT(FLASH->SR, FLASH_SR_OPTVERR) != 0);
  }
  else
  {
    return (uint32_t)(READ_BIT(FLASH->SR, FLASH_FLAG) != 0);
  }
}

/**
  * @brief  Clears the FLASH's pending flags.
  * @param  FLASH_FLAG: specifies the FLASH flags to clear.      
  *     @arg FLASH_SR_PGSERR  
  *     @arg FLASH_SR_WRPRTERR 
  *     @arg FLASH_SR_RDERR        
  * @retval None
  */
void FLASH_Clear_Flag(uint32_t FLASH_FLAG)
{
  /* Clear the flags */
  FLASH->SR = FLASH_FLAG;
}
