/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_iwdg.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#define  __rx32s12xx_IWDG_C


/* Includes ------------------------------------------------------------------*/
#include "rx32s12xx.h"
#include "rx32s12xx_iwdg.h"


/* Exported functions --------------------------------------------------------*/
/** @defgroup IWDG_Exported_Functions IWDG Exported Functions
  * @{
  */
/** @defgroup IWDG_EF_Configuration Configuration
  * @{
  */
  
void IWDG_Initial(IWDG_InitTypeDef *IWDG_InitStruct)
{                         
  IWDG_Enable_WriteAccess(IWDG);                  
  IWDG_Set_Prescaler(IWDG, IWDG_InitStruct->Prescaler);  
  IWDG_Set_ReloadCounter(IWDG, IWDG_InitStruct->Counter);            
  while (IWDG_Is_Ready(IWDG) != 1)                
  {
  }
  IWDG_Reload_Counter(IWDG);                      
}

/**
  * @function  IWDG_Enable
  * @brief  Start the Independent Watchdog
  * @param  IWDGx:
            @arg IWDG Instance
  * @retval None
  */
 void IWDG_Enable(IWDG_TypeDef *IWDGx)
{
  WRITE_REG(IWDG->KR, IWDG_KEY_ENABLE);
}


/**
  * @function  IWDG_Reload_Counter
  * @brief Reloads IWDG counter with value defined in the reload register
  * @param  IWDGx:
            @arg IWDG Instance
  * @retval None
  */
 void IWDG_Reload_Counter(IWDG_TypeDef *IWDGx)
{
  WRITE_REG(IWDG->KR, IWDG_KEY_RELOAD);
}


/**
  * @function  IWDG_Enable_WriteAccess
  * @brief  Enable write access to IWDG_PR, IWDG_RLR and IWDG_WINR registers
  * @param  IWDGx:
            @arg IWDG Instance
  * @retval None
  */
 void IWDG_Enable_WriteAccess(IWDG_TypeDef *IWDGx)
{
  WRITE_REG(IWDG->KR, IWDG_KEY_WR_ACCESS_ENABLE);
}


/**
  * @function  IWDG_Disable_WriteAccess
  * @brief  Disable write access to IWDG_PR, IWDG_RLR and IWDG_WINR registers
  * @param  IWDGx:
            @arg IWDG Instance
  * @retval None
  */
 void IWDG_Disable_WriteAccess(IWDG_TypeDef *IWDGx)
{
  WRITE_REG(IWDG->KR, IWDG_KEY_WR_ACCESS_DISABLE);
}


/**
  * @function  IWDG_Set_Prescaler
  * @brief  Select the prescaler of the IWDG
  * @param  IWDGx:
            @arg IWDG Instance
  * @param  Prescaler This parameter can be one of the following values:
  *         @arg @ref IWDG_PRESCALER_4
  *         @arg @ref IWDG_PRESCALER_8
  *         @arg @ref IWDG_PRESCALER_16
  *         @arg @ref IWDG_PRESCALER_32
  *         @arg @ref IWDG_PRESCALER_64
  *         @arg @ref IWDG_PRESCALER_128
  *         @arg @ref IWDG_PRESCALER_256
  * @retval None
  */
 void IWDG_Set_Prescaler(IWDG_TypeDef *IWDGx, uint32_t Prescaler)
{
  WRITE_REG(IWDGx->PR, Prescaler);
}


/**
  * @function  IWDG_Get_Prescaler
  * @brief  Get the selected prescaler of the IWDG
  * @param  IWDGx:
            @arg IWDG Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref IWDG_PRESCALER_4
  *         @arg @ref IWDG_PRESCALER_8
  *         @arg @ref IWDG_PRESCALER_16
  *         @arg @ref IWDG_PRESCALER_32
  *         @arg @ref IWDG_PRESCALER_64
  *         @arg @ref IWDG_PRESCALER_128
  *         @arg @ref IWDG_PRESCALER_256
  */
 uint32_t IWDG_Get_Prescaler(IWDG_TypeDef *IWDGx)
{
  return (uint32_t)(READ_REG(IWDGx->PR));
}


/**
  * @function  IWDG_Set_ReloadCounter
  * @brief  Specify the IWDG down-counter reload value
  * @param  IWDGx:
            @arg IWDG Instance
  * @param  Counter Value between Min_Data=0 and Max_Data=0x0FFF
  * @retval None
  */
 void IWDG_Set_ReloadCounter(IWDG_TypeDef *IWDGx, uint32_t Counter)
{
  WRITE_REG(IWDGx->RLR, Counter);
}



/**
  * @function  IWDG_Get_ReloadCounter
  * @brief  Get the specified IWDG down-counter reload value
  * @param  IWDGx:
            @arg IWDG Instance
  * @retval Value between Min_Data=0 and Max_Data=0x0FFF
  */
 uint32_t IWDG_Get_ReloadCounter(IWDG_TypeDef *IWDGx)
{
  return (uint32_t)(READ_REG(IWDGx->RLR));
}


/**
***********************************************************************************
*@Function IWDG_Get_Flag
*@brief    Get IWDG Flag.
*@param  IWDGx IWDG Instance
*@param1   Flag:
*          @arg  IWDG_SR_PVU
*          @arg  IWDG_SR_RVU
*@retval State of bit (1 or 0).
***********************************************************************************
  */
uint32_t IWDG_Get_Flag(IWDG_TypeDef *IWDGx, uint32_t Flag)
{	
	return (READ_BIT(IWDGx->SR, Flag) == (Flag));
}



/**
  * @brief  Check if all flags Prescaler, Reload & Window Value Update are reset or not
  * @rmtoll SR           PVU           IWDG_Is_Ready\n
  *         SR           RVU           IWDG_Is_Ready
  * @param  IWDGx IWDG Instance
  * @retval State of bits (1 or 0).
  */
uint32_t IWDG_Is_Ready(IWDG_TypeDef *IWDGx)
{
  return (READ_BIT(IWDGx->SR, IWDG_SR_PVU | IWDG_SR_RVU) == 0U);
}

