/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2019, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_opamp.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#define  __rx32s12xx_OPAMP_C

#include "rx32s12xx_opamp.h"


 
/**
  * @function  OPAMP_DeInit
  * @brief  De-initialize registers of the selected OPAMP instance
  *         to their default reset values.
  * @param  OPAMPx:
  *         @arg OPAMP Instance
  * @retval None.
  */
void OPAMP_DeInit(OPAMP_TypeDef *OPAMPx)
{
  OPAMPx->CSR = 0x00000000;
  OPAMPx->CSR = 0x00000000;
  OPAMPx->CSR = 0x00000000;
  OPAMPx->CSR = 0x00000000;
}

/**
  * @function  OPAMP_StructInit
  * @brief  Set each @ref OPAMP_InitTypeDef field to default value.
  * @param  OPAMP_InitStruct pointer to a @ref OPAMP_InitTypeDef structure
  *                         whose fields will be set to default values.
  * @retval None
  */
void OPAMP_StructInit(OPAMP_InitTypeDef *OPAMP_InitStruct)
{
  /* Set OPAMP_InitStruct fields to default values */
  OPAMP_InitStruct->FunctionalMode    = OPAMP_MODE_FOLLOWER;
  /* Note: Parameter discarded if OPAMP in functional mode follower,          */
  /*       set anyway to its default value.                                   */
  OPAMP_InitStruct->InputInverting    = OPAMP_INPUT_INVERT_CONNECT_NO;
}

/**
  * @function  OPAMP_Init
  * @brief  Initialize some features of OPAMP instance.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  OPAMP_InitStruct Pointer to a @ref OPAMP_InitTypeDef structure
  * @retval None.
  */
void OPAMP_Init(OPAMP_TypeDef *OPAMPx, OPAMP_InitTypeDef *OPAMP_InitStruct)
{
  if( OPAMP_Is_Locked(OPAMPx) == 0U)
  {
    if (OPAMP_InitStruct->FunctionalMode != OPAMP_MODE_FOLLOWER)
    { 
      MODIFY_REG(OPAMPx->CSR,
                   OPAMP_CSR_GAIN
                   | OPAMP_CSR_CALON
                   | OPAMP_CSR_VMSEL
                   | OPAMP_CSR_GAIN_4 | OPAMP_CSR_GAIN_3
                   ,
                    OPAMP_InitStruct->PGAGain
                   | OPAMP_InitStruct->FunctionalMode
                   | OPAMP_InitStruct->InputInverting
                  );
    }
    else
    {
      MODIFY_REG(OPAMPx->CSR,
                 OPAMP_CSR_CALON
                 | OPAMP_CSR_VMSEL
                 | OPAMP_CSR_GAIN_4 | OPAMP_CSR_GAIN_3
                 ,
                 OPAMP_MODE_FOLLOWER
                );
    }
  }
}

/**
  * @function  OPAMP_Set_Mode
  * @brief  Set OPAMP mode calibration or functional.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  Mode:
  *         @arg OPAMP_MODE_FUNCTIONAL
  *         @arg OPAMP_MODE_CALIBRATION 
  * @retval None
  */
void OPAMP_Set_Mode(OPAMP_TypeDef *OPAMPx, uint32_t Mode)
{
  MODIFY_REG( OPAMPx->CSR, OPAMP_CSR_CALON, Mode);
}

/**
  * @function  OPAMP_Get_Mode
  * @brief  Get OPAMP mode calibration or functional 
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval Returned value can be one of the following values:
  *         @arg @ref OPAMP_MODE_FUNCTIONAL
  *         @arg @ref OPAMP_MODE_CALIBRATION
  */
uint32_t OPAMP_Get_Mode(OPAMP_TypeDef *OPAMPx)
{
  return (uint32_t)(READ_BIT(OPAMPx->CSR, OPAMP_CSR_CALON));
}

/**
  * @function  OPAMP_Set_FunctionalMode
  * @brief  Set OPAMP functional mode by setting internal connections.
  *         OPAMP operation in standalone, follower, ...
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  FunctionalMode:
  *         @arg OPAMP_MODE_N1      
  *         @arg OPAMP_MODE_N2       
  *         @arg OPAMP_MODE_PGA     
  *         @arg OPAMP_MODE_FOLLOWER    
  * @retval None
  */
void OPAMP_Set_FunctionalMode(OPAMP_TypeDef *OPAMPx, uint32_t FunctionalMode)
{
  /* Note: Bit OPAMP_CSR_CALON reset to ensure to be in functional mode */
  MODIFY_REG(OPAMPx->CSR, OPAMP_CSR_VMSEL | OPAMP_CSR_CALON, FunctionalMode);
}

/**
  * @function  OPAMP_Get_FunctionalMode
  * @brief  Get OPAMP functional mode from setting of internal connections.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval Returned value can be one of the following values:
  *         @arg OPAMP_MODE_N1      
  *         @arg OPAMP_MODE_N2       
  *         @arg OPAMP_MODE_PGA     
  *         @arg OPAMP_MODE_FOLLOWER 
  */
uint32_t OPAMP_Get_FunctionalMode(OPAMP_TypeDef *OPAMPx)
{
  return (uint32_t)(READ_BIT(OPAMPx->CSR, OPAMP_CSR_VMSEL));
}

/**
  * @function  OPAMP_Set_PGAGain
  * @brief  Set OPAMP gain.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  OPAGain:
  *         @arg OPAMP_GAIN_2
  *         @arg OPAMP_GAIN_4 
  *         @arg OPAMP_GAIN_8
  *         @arg OPAMP_GAIN_16    
  *         @arg OPAMP_GAIN_32
  *         @arg OPAMP_GAIN_64 
  * @retval None
  */
void OPAMP_Set_PGAGain(OPAMP_TypeDef *OPAMPx, uint32_t OPAGain)
{
  MODIFY_REG(OPAMPx->CSR, OPAMP_CSR_GAIN_2 | OPAMP_CSR_GAIN_1 | OPAMP_CSR_GAIN_0, OPAGain);
}

/**
  * @function  OPAMP_Get_PGAGain
  * @brief  Get OPAMP gain.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval Returned value can be one of the following values:
  *         @arg OPAMP_MODE_N1      
  *         @arg OPAMP_MODE_N2       
  *         @arg OPAMP_MODE_PGA     
  *         @arg OPAMP_MODE_FOLLOWER 
  */
uint32_t OPAMP_Get_PGAGain(OPAMP_TypeDef *OPAMPx)
{
  return (uint32_t)(READ_BIT(OPAMPx->CSR, OPAMP_CSR_GAIN_2 | OPAMP_CSR_GAIN_1 | OPAMP_CSR_GAIN_0));
}                                              
                                                   
/**                   
  * @function  OPAMP_Set_InputInverting
  * @brief  Set OPAMP inverting input connection.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  InputInverting:
  *         @arg OPAMP_INPUT_INVERT_IO0       
  *         @arg OPAMP_INPUT_INVERT_IO1       
  *         @arg OPAMP_INPUT_INVERT_CONNECT_NO 
  * @retval None
  */
void OPAMP_Set_InputInverting(OPAMP_TypeDef *OPAMPx, uint32_t InputInverting)
{
  MODIFY_REG(OPAMPx->CSR, (~(InputInverting >> 1)) & OPAMP_CSR_VMSEL_0, InputInverting);
}

/**
  * @function  OPAMP_Get_InputInverting
  * @brief  Get OPAMP inverting input connection.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval Returned value can be one of the following values:
  *         @arg OPAMP_INPUT_INVERT_IO0       
  *         @arg OPAMP_INPUT_INVERT_IO1       
  *         @arg OPAMP_INPUT_INVERT_CONNECT_NO 
  */
uint32_t OPAMP_Get_InputInverting(OPAMP_TypeDef *OPAMPx)
{
  uint32_t input_inverting = READ_BIT(OPAMPx->CSR, OPAMP_CSR_VMSEL);

  /* Manage cases 0x10 and 0x11 to return the same value: OPAMP inverting     */
  /* input not connected.                                                     */
  return (input_inverting & ~((input_inverting >> 1) & OPAMP_CSR_VMSEL_0));
}

/**                   
  * @function  OPAMP_Set_TrimmingMode
  * @brief  Set OPAMP trimming mode.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  TrimmingMode:
  *         @arg OPAMP_TRIMMING_FACTORY
  *         @arg OPAMP_TRIMMING_USER
  * @retval None
  */
void OPAMP_Set_TrimmingMode(OPAMP_TypeDef *OPAMPx, uint32_t TrimmingMode)
{
  MODIFY_REG(OPAMPx->CSR, OPAMP_CSR_USERTRIM, TrimmingMode);
}

/**
  * @function  OPAMP_Get_TrimmingMode
  * @brief  Get OPAMP trimming mode.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval Returned value can be one of the following values:
  *         @arg OPAMP_TRIMMING_FACTORY
  *         @arg OPAMP_TRIMMING_USER
  */
uint32_t OPAMP_Get_TrimmingMode(OPAMP_TypeDef *OPAMPx)
{
  return (uint32_t)(READ_BIT(OPAMPx->CSR, OPAMP_CSR_USERTRIM));
}

/**                   
  * @function  OPAMP_Set_CalibrationSelection
  * @brief  Set OPAMP offset to calibrate the selected transistors
  *         differential pair NMOS or PMOS.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  TransistorsDiffPair:
  *         @arg OPAMP_TRIMMING_NMOS            (1)
  *         @arg OPAMP_TRIMMING_PMOS            (1)
  *         @arg OPAMP_TRIMMING_NMOS_VREF_50PC_VDDA
  *         @arg OPAMP_TRIMMING_PMOS_VREF_3p3PC_VDDA
  *
  *         (1) Default parameters to be used for calibration
  *             using two trimming steps (one with each transistors differential
  *             pair NMOS and PMOS)
  * @retval None
  */
void OPAMP_Set_CalibrationSelection(OPAMP_TypeDef *OPAMPx, uint32_t TransistorsDiffPair)
{
  MODIFY_REG(OPAMPx->CSR, OPAMP_CSR_CALSEL, (TransistorsDiffPair & OPAMP_TRIMMING_SELECT_MASK));
}

/**
  * @function  OPAMP_Get_CalibrationSelection
  * @brief  Get OPAMP offset to calibrate the selected transistors
  *         differential pair NMOS or PMOS.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval Returned value can be one of the following values:
  *         @arg OPAMP_TRIMMING_NMOS            (1)
  *         @arg OPAMP_TRIMMING_PMOS            (1)
  *         @arg OPAMP_TRIMMING_NMOS_VREF_50PC_VDDA
  *         @arg OPAMP_TRIMMING_PMOS_VREF_3p3PC_VDDA
  *
  *         (1) Default parameters to be used for calibration
  *             using two trimming steps (one with each transistors differential
  *             pair NMOS and PMOS)
  */
uint32_t OPAMP_Get_CalibrationSelection(OPAMP_TypeDef *OPAMPx)
{
  uint32_t CalibrationSelection = (uint32_t)(READ_BIT(OPAMPx->CSR, OPAMP_CSR_CALSEL));

  return (CalibrationSelection |
          (((CalibrationSelection & OPAMP_CSR_CALSEL_1) == 0UL) ? OPAMP_CSR_TRIMOFFSETP : OPAMP_CSR_TRIMOFFSETN));
}

/**                   
  * @function  OPAMP_Set_TrimmingValue
  * @brief  Set OPAMP trimming factor for the selected transistors
  *         differential pair NMOS or PMOS, corresponding to the selected
  *         power mode.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  TransistorsDiffPair:
  *         @arg OPAMP_TRIMMING_NMOS            (1)
  *         @arg OPAMP_TRIMMING_PMOS            (1)
  *         @arg OPAMP_TRIMMING_NMOS_VREF_50PC_VDDA
  *         @arg OPAMP_TRIMMING_PMOS_VREF_3p3PC_VDDA
  *
  *         (1) Default parameters to be used for calibration
  *             using two trimming steps (one with each transistors differential
  *             pair NMOS and PMOS)
  * @param  TrimmingValue:  0x00...0x1F
  * @retval None
  */
void OPAMP_Set_TrimmingValue(OPAMP_TypeDef *OPAMPx, uint32_t TransistorsDiffPair,
                                              uint32_t TrimmingValue)
{
  MODIFY_REG(OPAMPx->CSR,
             (TransistorsDiffPair & OPAMP_TRIMMING_VALUE_MASK),
             TrimmingValue << ((TransistorsDiffPair == OPAMP_TRIMMING_NMOS) ? OPAMP_CSR_TRIMOFFSETN_Pos : OPAMP_CSR_TRIMOFFSETP_Pos));
}

/**
  * @function  OPAMP_Get_TrimmingValue
  * @brief  Get OPAMP trimming factor for the selected transistors
  *         differential pair NMOS or PMOS, corresponding to the selected
  *         power mode.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @param  TransistorsDiffPair:
  *         @arg OPAMP_TRIMMING_NMOS            (1)
  *         @arg OPAMP_TRIMMING_PMOS            (1)
  *         @arg OPAMP_TRIMMING_NMOS_VREF_50PC_VDDA
  *         @arg OPAMP_TRIMMING_PMOS_VREF_3p3PC_VDDA
  *
  *         (1) Default parameters to be used for calibration
  *             using two trimming steps (one with each transistors differential
  *             pair NMOS and PMOS)
  * @retval 0x0...0x1F
  */
uint32_t OPAMP_Get_TrimmingValue(OPAMP_TypeDef *OPAMPx, uint32_t TransistorsDiffPair)
{
  return (uint32_t)(READ_BIT(OPAMPx->CSR, (TransistorsDiffPair & OPAMP_TRIMMING_VALUE_MASK))
                    >> ((TransistorsDiffPair == OPAMP_TRIMMING_NMOS) ? OPAMP_CSR_TRIMOFFSETN_Pos : OPAMP_CSR_TRIMOFFSETP_Pos));
}

/**
  * @function  OPAMP_Enable
  * @brief  Enable OPAMP1 instance.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval None
  */
void OPAMP_Enable(OPAMP_TypeDef *OPAMPx)
{
  SET_BIT(OPAMPx->CSR, OPAMP_CSR_OPAEN);
}

/**
  * @function  OPAMP_Disable
  * @brief  Disable OPAMP instance.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval None
  */
void OPAMP_Disable(OPAMP_TypeDef *OPAMPx)
{
  CLEAR_BIT(OPAMPx->CSR, OPAMP_CSR_OPAEN);
}

/**
  * @function  OPAMP_Is_Enabled
  * @brief   Get OPAMP instance enable state
  *         (0: OPAMP is disabled, 1: OPAMP is enabled)
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval State of bit (1 or 0).
  */
uint32_t OPAMP_Is_Enabled(OPAMP_TypeDef *OPAMPx)
{
  return ((READ_BIT(OPAMPx->CSR, OPAMP_CSR_OPAEN) == (OPAMP_CSR_OPAEN)) ? 1UL : 0UL);
}

/**
  * @function  OPAMP_Lock
  * @brief  Lock OPAMP instance.
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval None
  */
void OPAMP_Lock(OPAMP_TypeDef *OPAMPx)
{
  SET_BIT(OPAMPx->CSR, OPAMP_CSR_LOCK);
}

/**
  * @function  OPAMP_Is_Locked
  * @brief  Get OPAMP lock state
  * @param  OPAMPx:
  *         @arg OPAMP1
  *         @arg OPAMP2
  *         @arg OPAMP3
  *         @arg OPAMP4
  * @retval State of bit (1 or 0).
  */
uint32_t OPAMP_Is_Locked(OPAMP_TypeDef *OPAMPx)
{
  return ((READ_BIT(OPAMPx->CSR, OPAMP_CSR_LOCK) == (OPAMP_CSR_LOCK)) ? 1UL : 0UL);
}

