/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_pwr.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#define  __rx32s12xx_PWR_C

/* Includes ------------------------------------------------------------------*/
#include "rx32s12xx_pwr.h"
#include "rx32s12xx_gpio.h"
#include "rx32s12xx_rcc.h"


/**
  * @function  PWR_Enable_BkUpAccess
  * @brief  Enable access to the backup domain
  * @param  None
  * @retval None
  */
void PWR_Enable_BkUpAccess(void)
{
  SET_BIT(PWR->CR1, PWR_CR1_DBP);
}

/**
  * @function  PWR_Disable_BkUpAccess
  * @brief  Disable access to the backup domain
  * @param  None
  * @retval None
  */
void PWR_Disable_BkUpAccess(void)
{
  CLEAR_BIT(PWR->CR1, PWR_CR1_DBP);
}

/**
  * @function  PWR_Get_BkUpAccess
  * @brief  Check if the backup domain is enabled
  * @param  None
  * @retval State of bit (1 or 0).
  */
uint32_t PWR_Get_BkUpAccess(void)
{
  return (READ_BIT(PWR->CR1, PWR_CR1_DBP) == (PWR_CR1_DBP));
}

/**
  * @function  PWR_Set_RegulModeDS
  * @brief  Set voltage Regulator mode during deep sleep mode
  * @param  RegulMode :
  *         @arg PWR_REGU_DSMODE_MAIN
  *         @arg PWR_REGU_DSMODE_LOW_POWER
  * @retval None
  */
void PWR_Set_RegulModeDS(uint32_t RegulMode)
{
  MODIFY_REG(PWR->CR1, PWR_CR1_LPMS, RegulMode);
}

/**
  * @function  PWR_Get_RegulModeDS
  * @brief  Get voltage Regulator mode during deep sleep mode
  * @param  None
  * @retval Returned value can be one of the following values:
  *         @arg PWR_REGU_DSMODE_MAIN
  *         @arg PWR_REGU_DSMODE_LOW_POWER
  */
uint32_t PWR_Get_RegulModeDS(void)
{
  return (uint32_t)(READ_BIT(PWR->CR1, PWR_CR1_LPMS));
}

/**
  * @function  PWR_Set_PowerMode
  * @brief  Set Power Down mode when CPU enters deepsleep
  * @param  PDMode :
  *         @arg PWR_MODE_STOP_MAINREGU
  *         @arg PWR_MODE_STOP_LPREGU
  *         @arg PWR_MODE_STANDBY
  * @retval None
  */
void PWR_Set_PowerMode(uint32_t PDMode)
{
  MODIFY_REG(PWR->CR1, (PWR_CR1_LPMS), PDMode);
}

/**
  * @function  PWR_Get_PowerMode
  * @brief  Get Power Down mode when CPU enters deepsleep
  * @param  None
  * @retval Returned value can be one of the following values:
  *         @arg PWR_MODE_STOP_MAINREGU
  *         @arg PWR_MODE_STOP_LPREGU
  *         @arg PWR_MODE_STANDBY
  */
uint32_t PWR_Get_PowerMode(void)
{
  return (uint32_t)(READ_BIT(PWR->CR1, (PWR_CR1_LPMS)));
}

/**
  * @function  PWR_Set_PVDLevel
  * @rmtoll CR    PLS       PWR_SetPVDLevel
  * @param  PVDLevel 
  *         @arg PWR_CR2_PLS_2V2
  *         @arg PWR_CR2_PLS_2V3
  *         @arg PWR_CR2_PLS_2V4
  *         @arg PWR_CR2_PLS_2V5
  *         @arg PWR_CR2_PLS_2V6
  *         @arg PWR_CR2_PLS_2V7
  *         @arg PWR_CR2_PLS_2V8
  *         @arg PWR_CR2_PLS_2V9
  * @retval None
  */
void PWR_Set_PVDLevel(uint32_t PVDLevel)
{
  MODIFY_REG(PWR->CR2, PWR_CR2_PLS, PVDLevel);
}

/**
  * @function  PWR_Get_PVDLevel
  * @brief  Get the voltage threshold detection
  * @param  None
  * @retval Returned value can be one of the following values:
  *         @arg PWR_CR2_PLS_2V2
  *         @arg PWR_CR2_PLS_2V3
  *         @arg PWR_CR2_PLS_2V4
  *         @arg PWR_CR2_PLS_2V5
  *         @arg PWR_CR2_PLS_2V6
  *         @arg PWR_CR2_PLS_2V7
  *         @arg PWR_CR2_PLS_2V8
  *         @arg PWR_CR2_PLS_2V9
  */
uint32_t PWR_Get_PVDLevel(void)
{
  return (uint32_t)(READ_BIT(PWR->CR1, PWR_CR2_PLS));
}

/**
  * @function  PWR_Enable_PVD
  * @brief  Enable Power Voltage Detector
  * @param  None
  * @retval None
  */
void PWR_Enable_PVD(void)
{
  SET_BIT(PWR->CR2, PWR_CR2_PVDE);
}

/**
  * @function  PWR_Disable_PVD
  * @brief  Disable Power Voltage Detector
  * @param  None
  * @retval None
  */
void PWR_Disable_PVD(void)
{
  CLEAR_BIT(PWR->CR1, PWR_CR2_PVDE);
}

/**
  * @function  PWR_Get_PVD
  * @brief  Check if Power Voltage Detector is enabled
  * @param  None
  * @retval State of bit (1 or 0).
  */
uint32_t PWR_Get_PVD(void)
{
  return (READ_BIT(PWR->CR1, PWR_CR2_PVDE) == (PWR_CR2_PVDE));
}

/**
  * @function  PWR_Enable_WakeUpPin
  * @brief  Enable the WakeUp PINx functionality
  * @param  WakeUpPin :
  *         @arg PWR_WAKEUP_PIN1
  * @retval None
  */
void PWR_Enable_WakeUpPin(uint32_t WakeUpPin)
{
  SET_BIT(PWR->CR3, WakeUpPin);
}

/**
  * @function  PWR_Disable_WakeUpPin
  * @brief  Disable the WakeUp PINx functionality
  * @param  WakeUpPin :
  *         @arg PWR_WAKEUP_PIN1
  * @retval None
  */
void PWR_Disable_WakeUpPin(uint32_t WakeUpPin)
{
  CLEAR_BIT(PWR->CR3, WakeUpPin);
}

/**
  * @function  PWR_Get_WakeUpPin
  * @brief  Check if the WakeUp PINx functionality is enabled
  * @param  WakeUpPin This parameter can be one of the following values:
  *         @arg PWR_WAKEUP_PIN1
  * @retval State of bit (1 or 0).
  */
//uint32_t PWR_Get_WakeUpPin(uint32_t WakeUpPin)
//{
//  return (READ_BIT(PWR->CSR, WakeUpPin) == (WakeUpPin));
//}

/**
  * @brief  Get Flag
  * @rmtoll CSR   WUF       PWR_Get_Flag
  * @param  Flag :
  *         @arg @ref PWR_CSR_WUF
  *         @arg @ref PWR_CSR_SBF
  *         @arg @ref PWR_CSR_PVDO
  * @retval State of bit (1 or 0).
  */
//uint32_t PWR_Get_Flag(uint32_t Flag)
//{
//  return (READ_BIT(PWR->CSR, Flag) == (Flag));
//}

/**
  * @brief  Clear Flag
  * @rmtoll CSR   WUF       PWR_Clear_Flag
  * @param  Flag :
  *         @arg @ref PWR_CR_CSBF
  *         @arg @ref PWR_CR_CWUF
  * @retval None.
  */
void PWR_Clear_Flag(uint32_t Flag)
{
  SET_BIT(PWR->CR1, Flag);
}


/**
  * @brief  Enter SLEEP Mode
  * @rmtoll PWR_Enter_SLEEPMode
  * @retval None.
  */
void PWR_Enter_SLEEPMode(uint8_t SLEEPEntry)
{
   CLEAR_BIT(SCB->SCR, ((uint32_t)SCB_SCR_SLEEPDEEP_Msk));

		if(SLEEPEntry==PWR_STOPEntry_WFI)
		{
			__WFI();
		}
		else
		{	
      __SEV();
			__WFE();
      __WFE();
		}
}

/**
  * @brief  Enter STANDBY Mode
  * @rmtoll PWR_Enter_STANDBYMode
  * @retval None.
  */
void PWR_Enter_STOPMode()
{  
   LPM_EnableDeepSleep();
  __WFI( );
}


/**
  * @brief  Enter STANDBY Mode
  * @rmtoll PWR_Enter_STANDBYMode
  * @retval None.
  */
void PWR_Enter_STANDBYMode( void )
{
  PWR_Disable_WakeUpPin(PWR_WAKEUP_PIN1);
  
  /* Clear all wake up Flag */
//  PWR_Clear_Flag(PWR_CR_CWUF);
	
  /* Enable wakeup pin */
  PWR_Enable_WakeUpPin(PWR_WAKEUP_PIN1);
  
  /* As default User push-button state is high level, need to clear all wake up Flag again */
//  PWR_Clear_Flag(PWR_CR_CWUF);

  /* Set STANDBY mode when CPU enters deepsleep */
  PWR_Set_PowerMode(PWR_MODE_STANDBY);
  
  /* Set SLEEPDEEP bit of Cortex System Control Register */
  SET_BIT(SCB->SCR, ((uint32_t)SCB_SCR_SLEEPDEEP_Msk));
  
  /* This option is used to ensure that store operations are completed */
#if defined ( __CC_ARM)
  __force_stores();
#endif
  /* Request Wait For Interrupt */
  __WFI();
}






