/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_rcc.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#include "rx32s12xx.h"
#include "rx32s12xx_rcc.h"
#include "rx32s12xx_flash.h"


/**
***********************************************************************************
*@Function RCC_ClockConfig
*@brief    Config SYSCLK
*@param  RCC_ClkInitStruct: pointer to a RCC_ClkInitTypeDef structure that contains
*         the configuration information for the specified CMU peripheral.
  * @param  SYSCLKSource:
	*         @arg RCC_CFGR_SW_HSI
	*         @arg RCC_CFGR_SW_HSE
  *         @arg RCC_CFGR_SW_PLL
  *         @arg RCC_CFGR_SW_LSI
  * @param  PLLSource:
  *         @ref RCC_CFGR_PLLSRC_HSIDIV2
  *         @ref RCC_CFGR_PLLSRC_HSIDIV1
  *         @ref RCC_CFGR_PLLSRC_HSEDIV1
  *         @ref RCC_CFGR_PLLSRC_HSEDIV2 
  * @param  PLLMUL:
  *         @arg RCC_CFGR_PLLMUL_2x 
  *         @arg RCC_CFGR_PLLMUL_3x 
	*         @arg RCC_CFGR_PLLMUL_4x 
	*         @arg RCC_CFGR_PLLMUL_5x 
	*         @arg RCC_CFGR_PLLMUL_6x 
	*         @arg RCC_CFGR_PLLMUL_7x 
	*         @arg RCC_CFGR_PLLMUL_8x 
	*         @arg RCC_CFGR_PLLMUL_9x 
	*         @arg RCC_CFGR_PLLMUL_10x
	*         @arg RCC_CFGR_PLLMUL_11x
	*         @arg RCC_CFGR_PLLMUL_12x
	*         @arg RCC_CFGR_PLLMUL_13x
	*         @arg RCC_CFGR_PLLMUL_14x
	*         @arg RCC_CFGR_PLLMUL_15x
	*         @arg RCC_CFGR_PLLMUL_16x
	*         @arg 
  * @param  LATENCY:
  *         @arg FLASH_Latency_0
	*         @arg FLASH_Latency_1
	*         @arg FLASH_Latency_2
  * @param  AHBCLKDivider:
	*         @arg RCC_CFGR_HPRE_DIV1  
	*         @arg RCC_CFGR_HPRE_DIV2  
	*         @arg RCC_CFGR_HPRE_DIV4  
	*         @arg RCC_CFGR_HPRE_DIV8  
	*         @arg RCC_CFGR_HPRE_DIV16 
	*         @arg RCC_CFGR_HPRE_DIV64 
	*         @arg RCC_CFGR_HPRE_DIV128
	*         @arg RCC_CFGR_HPRE_DIV256
	*         @arg RCC_CFGR_HPRE_DIV512
	* @param  APB1CLKDivider:
  *         @arg RCC_CFGR_PPRE1_DIV1 
	*         @arg RCC_CFGR_PPRE1_DIV2 
	*         @arg RCC_CFGR_PPRE1_DIV4 
	*         @arg RCC_CFGR_PPRE1_DIV8 
	*         @arg RCC_CFGR_PPRE1_DIV16
	* @param  APB2CLKDivider:
  *         @arg RCC_CFGR_PPRE2_DIV1 
	*         @arg RCC_CFGR_PPRE2_DIV2 
	*         @arg RCC_CFGR_PPRE2_DIV4 
	*         @arg RCC_CFGR_PPRE2_DIV8 
	*         @arg RCC_CFGR_PPRE2_DIV16
	* @param  ADCSource:
	*         @arg RCC_CFGR_ADCCLK_SEL_APB2
  *         @arg RCC_CFGR_ADCCLK_SEL_PLL2	
	* @param  ADCCLKDivider:
	*         @arg RCC_CFGR_ADCPRE_DIV2
  *         @arg RCC_CFGR_ADCPRE_DIV4
	*         @arg RCC_CFGR_ADCPRE_DIV5
	*         @arg RCC_CFGR_ADCPRE_DIV6
* @retval None
***********************************************************************************
  */
void RCC_ClockConfig(RCC_ClkInitTypeDef  *RCC_ClkInitStruct)
{
  switch(RCC_ClkInitStruct->SYSCLKSource)
  {    
    case RCC_CFGR_SW_HSI:
      RCC_Enable_HSI();
      while(RCC_Get_HSIRDY() != 1){};
      RCC_Set_SysClkSource(RCC_CFGR_SW_HSI);
      while(RCC_Get_SysClkSource() != RCC_CFGR_SWS_HSI){};
      break;
//    case RCC_CFGR_SW_HSE:	
//      RCC_Enable_HSE();
//      while(RCC_Get_HSERDY() != 1){};
//      RCC_Set_SysClkSource(RCC_CFGR_SW_HSE);
//      while(RCC_Get_SysClkSource() != RCC_CFGR_SWS_HSE){};
//      break;
    case RCC_CFGR_SW_LSI:	
      RCC_Enable_LSI();
      while(RCC_Get_LSIRDY() != 1){};
      RCC_Set_SysClkSource(RCC_CFGR_SW_LSI);  
      while(RCC_Get_SysClkSource() != RCC_CFGR_SWS_LSI){};        
      break;
    case RCC_CFGR_SW_PLL:	
//      if(RCC_ClkInitStruct->PLLSource == RCC_CFGR_PLLSRC_HSE)
//        {
//          RCC_Enable_HSE();
//          while(RCC_Get_HSERDY() != 1){};
//        }
//        else if(RCC_ClkInitStruct->PLLSource == RCC_CFGR_PLLSRC_HSI)
//        {
          RCC_Enable_HSI();
          while(RCC_Get_HSIRDY() != 1){};
//        }
      FLASH_Set_Latency(RCC_ClkInitStruct->LATENCY);  
//      RCC_Set_PLLMUL(RCC_ClkInitStruct->PLLMUL);
			RCC_Set_PLLMUL(RCC_CFGR_PLLMUL_12x);	//set PLLMUL
      RCC_Enable_PLL();
      while(RCC_Get_PLLRDY() != 1){};
      RCC_Set_SysClkSource(RCC_CFGR_SW_PLL); 
      while(RCC_Get_SysClkSource() != RCC_CFGR_SWS_PLL){};        
      break;    
    default:
      RCC_Enable_HSI();
      while(RCC_Get_HSIRDY() != 1){};
      RCC_Set_SysClkSource(RCC_CFGR_SW_HSI);
      while(RCC_Get_SysClkSource() != RCC_CFGR_SWS_HSI){};
      break;
  }
  
  RCC_Set_AHBPrescaler(RCC_ClkInitStruct->AHBCLKDivider);
  RCC_Set_APB1Prescaler(RCC_ClkInitStruct->APB1CLKDivider);
  RCC_Set_APB2Prescaler(RCC_ClkInitStruct->APB2CLKDivider);
	RCC_Set_ADCClkSource(RCC_ClkInitStruct->ADCSource);
  RCC_Set_ADCPrescaler(RCC_ClkInitStruct->ADCCLKDivider);
	
}

/**
  * @function  RCC_Enable_CSS
  * @brief  Enable CSS
  * @retval None
*/
//void RCC_Enable_CSS(void)
//{
//   SET_BIT(RCC->CR, RCC_CR_CSSON);
//}
/**
  * @function  RCC_Disable_CSS
  * @brief  Disable CSS
  * @retval None
*/
//void RCC_Disable_CSS(void)
//{
//   CLEAR_BIT(RCC->CR, RCC_CR_CSSON);
//}

/**
  * @function  RCC_Enable_HSEBYP
  * @brief  Enable HSE Bypass 
  * @retval None
*/
//void RCC_Enable_HSEBYP(void)
//{
//   SET_BIT(RCC->CR, RCC_CR_HSEBYP);
//}

/**
  * @function  RCC_Disable_HSEBYP
  * @brief  Disable HSE Bypass 
  * @retval None
*/
//void RCC_Disable_HSEBYP(void)
//{
//   CLEAR_BIT(RCC->CR, RCC_CR_HSEBYP);
//}

/**
  * @function  RCC_Enable_HSE
  * @brief  Enable HSE
  * @retval None
*/
//void RCC_Enable_HSE(void)
//{
//   SET_BIT(RCC->CR, RCC_CR_HSEON);
//}

/**
  * @function  RCC_Disable_HSE
  * @brief  Disable HSE
  * @retval None
*/
//void RCC_Disable_HSE(void)
//{
//   CLEAR_BIT(RCC->CR, RCC_CR_HSEON);
//}

/**
  * @function  RCC_Get_HSERDY
  * @brief  check if HSE ready
  * @param  None
  * @retval 1:HSE is ready, 0: HSE not ready
*/
//uint32_t RCC_Get_HSERDY(void)
//{
//  return (READ_BIT(RCC->CR, RCC_CR_HSERDY) == (RCC_CR_HSERDY));
//}


/**
  * @function  RCC_Enable_HSI
  * @brief  Enable HSI
  * @retval None
*/
void RCC_Enable_HSI(void)
{
   SET_BIT(RCC->CR, RCC_CR_HSION);
}

/**
  * @function  RCC_Disable_HSI
  * @brief  Disable HSI
  * @retval None
*/
void RCC_Disable_HSI(void)
{
   CLEAR_BIT(RCC->CR, RCC_CR_HSION);
}

/**
  * @function  uint32_t RCC_Get_HSIRDY(void)
  * @brief  check if HSI ready
  * @param  None
  * @retval 1:HSI is ready, 0: HSI not ready
*/
uint32_t RCC_Get_HSIRDY(void)
{
  return (READ_BIT(RCC->CR, RCC_CR_HSIRDY) == (RCC_CR_HSIRDY));
}

/**
  * @function  RCC_Enable_LSI
  * @brief  Enable LSI
  * @retval None
*/
void RCC_Enable_LSI(void)
{
   SET_BIT(RCC->CSR, RCC_CSR_LSION);
}

///**
//  * @function  RCC_Disable_LSI
//  * @brief  Disable LSI
//  * @retval None
//*/
void RCC_Disable_LSI(void)
{
   CLEAR_BIT(RCC->CSR, RCC_CSR_LSION);
}

/**
  * @function  RCC_Get_LSIRDY
  * @brief  check if LSI ready
  * @param  None
  * @retval 1:LSI is ready, 0: LSI not ready
*/
uint32_t RCC_Get_LSIRDY(void)
{
  return (READ_BIT(RCC->CSR, RCC_CSR_LSIRDY) == (RCC_CSR_LSIRDY));
}

/**
  * @function  RCC_Enable_PLL
  * @brief  Enable PLL
  * @retval None
*/
void RCC_Enable_PLL(void)
{
   SET_BIT(RCC->CR, RCC_CR_PLLON);
}

/**
  * @function  RCC_Disable_PLL
  * @brief  Disable PLL
  * @retval None
*/
void RCC_Disable_PLL(void)
{
   CLEAR_BIT(RCC->CR, RCC_CR_PLLON);
}

/**
  * @function  RCC_Get_PLLRDY
  * @brief  check if PLL ready
  * @param  None
  * @retval 1:PLL is ready, 0: PLL not ready
*/
uint32_t RCC_Get_PLLRDY(void)
{
  return (READ_BIT(RCC->CR, RCC_CR_PLLRDY) == (RCC_CR_PLLRDY));
}
/**
  * @function  RCC_Set_PLLMUL
  * @brief  config PLL mul for system clock
  * @param  Source:
            @arg RCC_CFGR_PLLSRC_HSI
            @arg RCC_CFGR_PLLSRC_HSE
            @arg 
            PLLMUL:
            @arg RCC_CFGR_PLLMUL_10x
            @arg RCC_CFGR_PLLMUL_12x
            @arg RCC_CFGR_PLLMUL_14x
            @arg RCC_CFGR_PLLMUL_15x
            @arg RCC_CFGR_PLLMUL_16x
            @arg RCC_CFGR_PLLMUL_17x
            @arg RCC_CFGR_PLLMUL_18x
            @arg RCC_CFGR_PLLMUL_19x
  * @retval None
*/
void RCC_Set_PLLMUL(uint32_t PLLMUL)
{
   MODIFY_REG(RCC->CFGR,RCC_CFGR_PLLMULL,PLLMUL);	
}
/**
  * @function  RCC_Set_SysClkSource
  * @brief  Set the system clock source
  * @param  Source:
            @arg RCC_CFGR_SW_HSI
            @arg RCC_CFGR_SW_HSE
            @arg RCC_CFGR_SW_PLL
            @arg RCC_CFGR_SW_LSI
  * @retval None
*/
void RCC_Set_SysClkSource(uint32_t Source)
{
  MODIFY_REG(RCC->CFGR, RCC_CFGR_SW, Source);
}

/**
  * @function  RCC_Get_SysClkSource
  * @brief  Get the system clock source
  * @param  None
  * @retval 
            @arg RCC_CFGR_SWS_HSI
            @arg RCC_CFGR_SWS_HSE
            @arg RCC_CFGR_SWS_PLL
            @arg RCC_CFGR_SWS_LSI
*/
uint32_t RCC_Get_SysClkSource(void)
{
  return (uint32_t)(READ_BIT(RCC->CFGR, RCC_CFGR_SWS));
}

/**
  * @function  RCC_Set_ADCClkSource
  * @brief  Set the ADC clock source
  * @param  Source:
            @arg RCC_CFGR_ADCCLK_SEL_SYSCLK
            @arg RCC_CFGR_ADCCLK_SEL_PLL2
  * @retval None
*/
void RCC_Set_ADCClkSource(uint32_t Source)
{
  MODIFY_REG(RCC->CFGR, RCC_CFGR_ADCCLK_SEL, Source);
}

/**
  * @function  RCC_Get_ADCClkSource
  * @brief  Get the ADC clock source
  * @param  None
  * @retval 
            @arg RCC_CFGR_ADCCLK_SEL_APB2
            @arg RCC_CFGR_ADCCLK_SEL_PLL2
*/
uint32_t RCC_Get_ADCClkSource(void)
{
  return (uint32_t)(READ_BIT(RCC->CFGR, RCC_CFGR_ADCCLK_SEL));
}

/**
  * @function  RCC_Set_AHBPrescaler
  * @brief  Set AHB prescaler
  * @param  Source:
            @arg RCC_CFGR_HPRE_DIV1
            @arg RCC_CFGR_HPRE_DIV2
            @arg RCC_CFGR_HPRE_DIV4
            @arg RCC_CFGR_HPRE_DIV8
            @arg RCC_CFGR_HPRE_DIV16
            @arg RCC_CFGR_HPRE_DIV64
            @arg RCC_CFGR_HPRE_DIV128
            @arg RCC_CFGR_HPRE_DIV256
            @arg RCC_CFGR_HPRE_DIV512
  * @retval None
*/
void RCC_Set_AHBPrescaler(uint32_t Prescaler)
{
  MODIFY_REG(RCC->CFGR, RCC_CFGR_HPRE, Prescaler);
}

/**
  * @function  RCC_Set_APB1Prescaler
  * @brief  Set APB1 prescaler
  * @param  Source:
            @arg RCC_CFGR_PPRE1_DIV1
            @arg RCC_CFGR_PPRE1_DIV2
            @arg RCC_CFGR_PPRE1_DIV4
            @arg RCC_CFGR_PPRE1_DIV8
            @arg RCC_CFGR_PPRE1_DIV16
  * @retval None
*/
void RCC_Set_APB1Prescaler(uint32_t Prescaler)
{
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PPRE1, Prescaler);
}

/**
  * @function  RCC_Set_APB2Prescaler
  * @brief  Set APB2 prescaler
  * @param  Source:
            @arg RCC_CFGR_PPRE2_DIV1
            @arg RCC_CFGR_PPRE2_DIV2
            @arg RCC_CFGR_PPRE2_DIV4
            @arg RCC_CFGR_PPRE2_DIV8
            @arg RCC_CFGR_PPRE2_DIV16
  * @retval None
*/
void RCC_Set_APB2Prescaler(uint32_t Prescaler)
{
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PPRE2, Prescaler);
}
/**
  * @function  RCC_Set_ADCPrescaler
  * @brief  Set ADC prescaler
  * @param  Source:
            @arg RCC_CFGR_ADCPRE_DIV2
            @arg RCC_CFGR_ADCPRE_DIV4
            @arg RCC_CFGR_ADCPRE_DIV5
            @arg RCC_CFGR_ADCPRE_DIV6
  * @retval None
*/
void RCC_Set_ADCPrescaler(uint32_t ADCxSource)
{
  MODIFY_REG(RCC->CFGR, RCC_CFGR_ADCPRE, ADCxSource);
}

/**
  * @function  RCC_Get_AHBPrescaler
  * @brief  Get AHB Prescaler
  * @param  None
  * @retval 
            @arg RCC_CFGR_HPRE_DIV1
            @arg RCC_CFGR_HPRE_DIV2
            @arg RCC_CFGR_HPRE_DIV4
            @arg RCC_CFGR_HPRE_DIV8
            @arg RCC_CFGR_HPRE_DIV16
            @arg RCC_CFGR_HPRE_DIV32
            @arg RCC_CFGR_HPRE_DIV64
            @arg RCC_CFGR_HPRE_DIV128
            @arg RCC_CFGR_HPRE_DIV256
            @arg RCC_CFGR_HPRE_DIV512
*/
uint32_t RCC_Get_AHBPrescaler(void)
{
  return (uint32_t)(READ_BIT(RCC->CFGR, RCC_CFGR_HPRE));
}

/**
  * @function  RCC_Get_APB1Prescaler
  * @brief  Get APB1 Prescaler
  * @param  None
  * @retval 
            @arg RCC_CFGR_PPRE1_DIV1
            @arg RCC_CFGR_PPRE1_DIV2
            @arg RCC_CFGR_PPRE1_DIV4
            @arg RCC_CFGR_PPRE1_DIV8
            @arg RCC_CFGR_PPRE1_DIV16
*/
uint32_t RCC_Get_APB1Prescaler(void)
{
  return (uint32_t)(READ_BIT(RCC->CFGR, RCC_CFGR_PPRE1));
}

/**
  * @function  RCC_Get_APB2Prescaler
  * @brief  Get APB2 Prescaler
  * @param  None
  * @retval 
            @arg RCC_CFGR_PPRE2_DIV1
            @arg RCC_CFGR_PPRE2_DIV2
            @arg RCC_CFGR_PPRE2_DIV4
            @arg RCC_CFGR_PPRE2_DIV8
            @arg RCC_CFGR_PPRE2_DIV16
*/
uint32_t RCC_Get_APB2Prescaler(void)
{
  return (uint32_t)(READ_BIT(RCC->CFGR, RCC_CFGR_PPRE2));
}
/**
  * @function  RCC_Get_ADCPrescaler
  * @brief  Get APB2 Prescaler
  * @param  None
  * @retval 
            @arg RCC_CFGR_ADCPRE_DIV2
            @arg RCC_CFGR_ADCPRE_DIV4
            @arg RCC_CFGR_ADCPRE_DIV5
            @arg RCC_CFGR_ADCPRE_DIV6
*/
uint32_t RCC_Get_ADCPrescaler(void)
{
  return (uint32_t)(READ_BIT(RCC->CFGR, RCC_CFGR_ADCPRE));
}

/**
  * @function  RCC_Select_MCO
  * @brief  MCO clock out select
  * @param  MCOxSource:
            @arg RCC_CFGR_MCO_NOCLK
            @arg RCC_CFGR_MCO_LSI
            @arg RCC_CFGR_MCO_FSYS
            @arg RCC_CFGR_MCO_HSI
            @arg RCC_CFGR_MCO_HSE
            @arg RCC_CFGR_MCO_PLLDIV2
  * @retval None
*/
void RCC_Select_MCO(uint32_t MCOxSource)
{
  MODIFY_REG(RCC->CFGR, RCC_CFGR_MCO_SEL, MCOxSource);
}

/**
  * @function  RCC_ConfigMCO
  * @brief  MCO clock out select
  * @param  MCOxSource:
            @arg RCC_CFGR_MCO_NOCLK
            @arg RCC_CFGR_MCO_LSI
            @arg RCC_CFGR_MCO_FSYS
            @arg RCC_CFGR_MCO_HSI
            @arg RCC_CFGR_MCO_HSE
            @arg RCC_CFGR_MCO_PLLDIV2
  * @param  MCOxPrescaler:
            @arg RCC_CFGR_MCO_DIV1
            @arg RCC_CFGR_MCO_DIV2
            @arg RCC_CFGR_MCO_DIV4
            @arg RCC_CFGR_MCO_DIV8
*/

void RCC_ConfigMCO(uint32_t MCOxSource, uint32_t MCOxPrescaler)
{
  MODIFY_REG(RCC->CFGR, RCC_CFGR_MCO_SEL | RCC_CFGR_MCO_PRE, MCOxSource | MCOxPrescaler);
}
/**
  * @function  RCC_Enable_APB1Clock
  * @brief  Enable APB1 periphal clock
  * @param  Periphal:
            @arg RCC_APB1ENR_TIM2EN  
            @arg RCC_APB1ENR_TIM3EN  
            @arg RCC_APB1ENR_TIM4EN  
            @arg RCC_APB1ENR_TIM6EN  
            @arg RCC_APB1ENR_TIM7EN  
            @arg RCC_APB1ENR_USART2EN
            @arg RCC_APB1ENR_USART3EN
            @arg RCC_APB1ENR_I2C1EN  
            @arg RCC_APB1ENR_CANEN   
            @arg RCC_APB1ENR_BKPEN   
            @arg RCC_APB1ENR_PWREN   
  * @retval None
*/
void RCC_Enable_APB1Clock(uint32_t Periphal)
{
    SET_BIT(RCC->APB1ENR, Periphal);
}

/**
  * @function  RCC_Disable_APB1Clock
  * @brief  Disable APB1 periphal clock
  * @param  Periphal:
            @arg RCC_APB1ENR_TIM2EN  
            @arg RCC_APB1ENR_TIM3EN  
            @arg RCC_APB1ENR_TIM4EN  
            @arg RCC_APB1ENR_TIM6EN  
            @arg RCC_APB1ENR_TIM7EN  
            @arg RCC_APB1ENR_USART2EN
            @arg RCC_APB1ENR_USART3EN
            @arg RCC_APB1ENR_I2C1EN  
            @arg RCC_APB1ENR_CANEN   
            @arg RCC_APB1ENR_BKPEN   
            @arg RCC_APB1ENR_PWREN   
  * @retval None
*/
void RCC_Disable_APB1Clock(uint32_t Periphal)
{
    CLEAR_BIT(RCC->APB1ENR, Periphal);
}

/**
  * @function  RCC_Enable_APB2Clock
  * @brief  Enable APB2 periphal clock
  * @param  Periphal:
            @arg RCC_APB2ENR_AFIOEN  
            @arg RCC_APB2ENR_ADC1EN  
            @arg RCC_APB2ENR_TIM1EN  
            @arg RCC_APB2ENR_SPI1EN  
            @arg RCC_APB2ENR_USART1EN
            @arg RCC_APB2ENR_OPAMPEN   
            @arg RCC_APB2ENR_CMPEN   
  * @retval None
*/
void RCC_Enable_APB2Clock(uint32_t Periphal)
{
    SET_BIT(RCC->APB2ENR, Periphal);
}

/**
  * @function  RCC_Disable_APB2Clock
  * @brief  Disable APB2 periphal clock
  * @param  Periphal:
            @arg RCC_APB2ENR_AFIOEN  
            @arg RCC_APB2ENR_ADC1EN  
            @arg RCC_APB2ENR_TIM1EN  
            @arg RCC_APB2ENR_SPI1EN  
            @arg RCC_APB2ENR_USART1EN
            @arg RCC_APB2ENR_OPAMPEN   
            @arg RCC_APB2ENR_CMPEN    
  * @retval None
*/
void RCC_Disable_APB2Clock(uint32_t Periphal)
{
  CLEAR_BIT(RCC->APB2ENR, Periphal);
}

/**
  * @function  RCC_Enable_AHBClock
  * @brief  Enable AHB periphal clock
  * @param  Periphal:
            @arg RCC_AHBENR_DMA1EN
            @arg RCC_AHBENR_SRAMEN
            @arg RCC_AHBENR_FLITEN
            @arg RCC_AHBENR_CRCEN 
            @arg RCC_AHBENR_MEEN  
            @arg RCC_AHBENR_IOPAEN
            @arg RCC_AHBENR_IOPBEN
            @arg RCC_AHBENR_IOPCEN
            @arg RCC_AHBENR_IOPDEN
  * @retval None
*/
void RCC_Enable_AHBClock(uint32_t Periphal)
{
    SET_BIT(RCC->AHBENR, Periphal);
}

/**
  * @function  RCC_Disable_AHBClock
  * @brief  Disable AHB periphal clock
  * @param  Periphal:
            @arg RCC_AHBENR_DMA1EN
            @arg RCC_AHBENR_SRAMEN
            @arg RCC_AHBENR_FLITEN
            @arg RCC_AHBENR_CRCEN 
            @arg RCC_AHBENR_MEEN  
            @arg RCC_AHBENR_IOPAEN
            @arg RCC_AHBENR_IOPBEN
            @arg RCC_AHBENR_IOPCEN
            @arg RCC_AHBENR_IOPDEN
  * @retval None
*/
void RCC_Disable_AHBClock(uint32_t Periphal)
{
    CLEAR_BIT(RCC->AHBENR, Periphal);
}

/**
  * @function  RCC_Set_RTCClockSource
  * @note Once the RTC clock source has been selected, it cannot be changed any more unless
  *       the Backup domain is reset. The BDRST bit can be used to reset them.
  * @brief  Set BDCR_RTCSEL       
  * @param  Source This parameter can be one of the following values:
  *         @arg RCC_BDCR_RTCSEL_NONE     
  *         @arg RCC_BDCR_RTCSEL_LSI      
  *         @arg RCC_BDCR_RTCSEL_HSEDIV128
  * @retval None
  */
//void RCC_Set_RTCClockSource(uint32_t Source)
//{
//  MODIFY_REG(RCC->BDCR, RCC_BDCR_RTCSEL, Source);
//}
/**
  * @function  RCC_Get_RTCClockSource
  * @brief  Get BDCR_RTCSEL  
  * @retval Returned value can be one of the following values:
  *         @arg RCC_BDCR_RTCSEL_NONE     
  *         @arg RCC_BDCR_RTCSEL_LSI      
  *         @arg RCC_BDCR_RTCSEL_HSEDIV128
  */
//uint32_t RCC_Get_RTCClockSource(void)
//{
//  return (uint32_t)(READ_BIT(RCC->BDCR, RCC_BDCR_RTCSEL));
//}

/**
  * @function  RCC_Enable_RTC
  * @brief  Enable_RTCEN
  * @retval None
*/
void RCC_Enable_RTC(void)
{
    SET_BIT(RCC->BDCR, RCC_BDCR_RTCEN);
}

/**
  * @function  RCC_Disable_RTC
  * @brief  Disable RTCEN
  * @retval None
*/
void RCC_Disable_RTC(void)
{
  CLEAR_BIT(RCC->BDCR, RCC_BDCR_RTCEN);
}

/**
  * @brief  Check if RTC has been enabled or not
  * @rmtoll BDCR         RTCEN         RCC_Get_RTC
  * @retval State of bit (1 or 0).
  */
uint32_t RCC_Get_RTC(void)
{
  return (READ_BIT(RCC->BDCR, RCC_BDCR_RTCEN) == (RCC_BDCR_RTCEN));
}

/**
  * @function  RCC_Enable_BDRST
  * @brief  Enable BDRST
  * @retval None
  */
void RCC_Enable_BDRST(void)
{
    SET_BIT(RCC->BDCR, RCC_BDCR_BDRST);
}

/**
  * @function  RCC_Disable_BDRST
  * @brief  Disable BDRST
  * @retval None
  */
void RCC_Disable_BDRST(void)
{
    CLEAR_BIT(RCC->BDCR, RCC_BDCR_BDRST);
}

/**
  * @function  RCC_Enable_AHBRST
  * @brief  Enable AHB periphal RESET
  * @param  Source This parameter can be one of the following values:
            @arg RCC_AHBRSTR_DMA1RST
            @arg RCC_AHBRSTR_CRCRST 
            @arg RCC_AHBRSTR_MERST  
            @arg RCC_AHBRSTR_IOPARST
            @arg RCC_AHBRSTR_IOPBRST
            @arg RCC_AHBRSTR_IOPCRST
            @arg RCC_AHBRSTR_IOPDRST
  * @retval None
  */
void RCC_Enable_AHBRST(uint32_t Periphal)
{
    SET_BIT(RCC->AHBRSTR, Periphal);
}

/**
  * @function  RCC_Disable_AHBRST
  * @brief  Disable AHB periphal RESET
  * @param  Source This parameter can be one of the following values:
            @arg RCC_AHBRSTR_DMA1RST
            @arg RCC_AHBRSTR_CRCRST 
            @arg RCC_AHBRSTR_MERST  
            @arg RCC_AHBRSTR_IOPARST
            @arg RCC_AHBRSTR_IOPBRST
            @arg RCC_AHBRSTR_IOPCRST
            @arg RCC_AHBRSTR_IOPDRST
  * @retval None
  */
void RCC_Disable_AHBRST(uint32_t Periphal)
{
    CLEAR_BIT(RCC->AHBRSTR, Periphal);
}
/**
  * @function  RCC_Enable_APB1RST
  * @brief  Enable APB1 periphal RESET
  * @param  Source This parameter can be one of the following values:
            @arg RCC_APB1RSTR_TIM2RST  
            @arg RCC_APB1RSTR_TIM3RST  
            @arg RCC_APB1RSTR_TIM4RST  
            @arg RCC_APB1RSTR_TIM6RST  
            @arg RCC_APB1RSTR_TIM7RST  
            @arg RCC_APB1RSTR_USART2RST
            @arg RCC_APB1RSTR_USART3RST
            @arg RCC_APB1RSTR_I2C1RST  
            @arg RCC_APB1RSTR_CANRST   
            @arg RCC_APB1RSTR_BKPRST   
            @arg RCC_APB1RSTR_PWRRST   
  * @retval None
  */
void RCC_Enable_APB1RST(uint32_t Periphal)
{
    SET_BIT(RCC->APB1RSTR, Periphal);
}

/**
  * @function  RCC_Disable_APB1RST
  * @brief  Disable APB1 periphal RESET
  * @param  Source This parameter can be one of the following values:
            @arg RCC_APB1RSTR_TIM2RST  
            @arg RCC_APB1RSTR_TIM3RST  
            @arg RCC_APB1RSTR_TIM4RST  
            @arg RCC_APB1RSTR_TIM6RST  
            @arg RCC_APB1RSTR_TIM7RST  
            @arg RCC_APB1RSTR_USART2RST
            @arg RCC_APB1RSTR_USART3RST
            @arg RCC_APB1RSTR_I2C1RST  
            @arg RCC_APB1RSTR_CANRST   
            @arg RCC_APB1RSTR_BKPRST   
            @arg RCC_APB1RSTR_PWRRST  
  * @retval None
  */
void RCC_Disable_APB1RST(uint32_t Periphal)
{
    CLEAR_BIT(RCC->APB1RSTR, Periphal);
}

/**
  * @function  RCC_Enable_APB2RST
  * @brief  Enable APB2 periphal RESET
  * @param  Source This parameter can be one of the following values:
  *         @arg RCC_APB2RSTR_AFIORST  
            @arg RCC_APB2RSTR_ADC1RST  
            @arg RCC_APB2RSTR_TIM1RST  
            @arg RCC_APB2RSTR_SPI1RST  
            @arg RCC_APB2RSTR_USART1RST
            @arg RCC_APB2RSTR_OPAMPRST   
            @arg RCC_APB2RSTR_CMPRST   
  * @retval None
  */
void RCC_Enable_APB2RST(uint32_t Periphal)
{
    SET_BIT(RCC->APB2RSTR, Periphal);
}

/**
  * @function  RCC_Disable_APB2RST
  * @brief  Disable APB2 periphal RESET
  * @param  Source This parameter can be one of the following values:
  *         @arg RCC_APB2RSTR_AFIORST  
            @arg RCC_APB2RSTR_ADC1RST  
            @arg RCC_APB2RSTR_TIM1RST  
            @arg RCC_APB2RSTR_SPI1RST  
            @arg RCC_APB2RSTR_USART1RST
            @arg RCC_APB2RSTR_OPAMPRST   
            @arg RCC_APB2RSTR_CMPRST  
  * @retval None
  */
void RCC_Disable_APB2RST(uint32_t Periphal)
{
    CLEAR_BIT(RCC->APB2RSTR, Periphal);
}

/**
  * @function  RCC_Get_RSTFlag
  * @brief  check if Reset flag
  * @param  Source This parameter can be one of the following values:
  *         @arg RCC_CSR_PINRSTF 
  *         @arg RCC_CSR_PORRSTF 
  *         @arg RCC_CSR_SFTRSTF 
  *         @arg RCC_CSR_IWDGRSTF
  *         @arg RCC_CSR_LPWRRSTF
  * @retval 1:reset, 0: no reset
*/
uint32_t RCC_Get_RSTFlag(uint32_t FLAG)
{
  return (READ_BIT(RCC->CSR, FLAG) == (FLAG));
}

/**
  * @function RCC_Clear_RSTFlags
  * @brief  Set RMVF bit to clear the reset flags.
  * @param  None
  * @retval None
  */
void RCC_Clear_RSTFlags(void)
{
  SET_BIT(RCC->CSR, RCC_CSR_RMVF);
}

/**
  * @function  RCC_Get_ITFlag
  * @brief  Get if Interrupt flag
  * @param  Source This parameter can be one of the following values:
  *         @arg RCC_CIR_LSIRDYF
  *         @arg RCC_CIR_HSIRDYF
  *         @arg RCC_CIR_HSERDYF
  *         @arg RCC_CIR_PLLRDYF
  *         @arg RCC_CIR_CSSF   
  * @retval 1:ready, 0: no ready
*/
uint32_t RCC_Get_ITFlag(uint32_t InterruptFLAG)
{
  return (READ_BIT(RCC->CIR, InterruptFLAG) == (InterruptFLAG));
}

/**
  * @function  RCC_Clear_ITFlags
  * @brief  Clear Interrupt flag
  * @param  Source This parameter can be one of the following values:
  *         @arg RCC_CIR_LSIRDYC
  *         @arg RCC_CIR_HSIRDYC
  *         @arg RCC_CIR_HSERDYC
  *         @arg RCC_CIR_PLLRDYC
  *         @arg RCC_CIR_CSSC   
  * @retval None
*/
void RCC_Clear_ITFlags(uint32_t InterruptFLAG)
{
  SET_BIT(RCC->CIR, InterruptFLAG);
}

/**
  * @function  RCC_Enable_IT
  * @brief  Enable Periphal Interrupt
  * @param  Source This parameter can be one of the following values:
  *         @arg RCC_CIR_LSIRDYIE
  *         @arg RCC_CIR_HSIRDYIE
  *         @arg RCC_CIR_HSERDYIE
  *         @arg RCC_CIR_PLLRDYIE 
  * @retval None
  */
void RCC_Enable_IT(uint32_t Periphal)
{
    SET_BIT(RCC->CIR, Periphal);
}

/**
  * @function  RCC_Disable_IT
  * @brief  Disable Periphal Interrupt
  * @param  Source This parameter can be one of the following values:
  *         @arg RCC_CIR_LSIRDYIE
  *         @arg RCC_CIR_HSIRDYIE
  *         @arg RCC_CIR_HSERDYIE
  *         @arg RCC_CIR_PLLRDYIE 
  * @retval None
  */
void RCC_Disable_IT(uint32_t Periphal)
{
    CLEAR_BIT(RCC->CIR, Periphal);
}


/**
  * @function  RCC_Get_HSEDET_DELTA
  * @brief  Get the HSEDET DELTA
  * @param  None
  * @retval DELTA of HSEDET
*/
//uint32_t RCC_Get_HSEDET_DELTA(void)
//{
//	return (uint32_t)(READ_BIT(RCC->HSEDET, RCC_HSEDET_DELTA));
//}

/**
  * @function  RCC_Enable_HSEDET_FAIL
  * @brief  Enable HSEDET FAIL
	* @param  Periphal:
  *         @arg RCC_HSEDET_FAIL_M0
  *         @arg RCC_HSEDET_FAIL_M1
  *         @arg RCC_HSEDET_FAIL_M2
  * @retval None
  */
//void RCC_Enable_HSEDET_FAIL(uint32_t Periphal)
//{
//    SET_BIT(RCC->HSEDET, Periphal);
//}

/**
  * @function  RCC_Disable_HSEDET_FAIL
  * @brief  Disable HSEDET FAIL
	* @param  Periphal:
  *         @arg RCC_HSEDET_FAIL_M0
  *         @arg RCC_HSEDET_FAIL_M1
  *         @arg RCC_HSEDET_FAIL_M2
  * @retval None
  */
//void RCC_Disable_HSEDET_FAIL(uint32_t Periphal)
//{
//    CLEAR_BIT(RCC->HSEDET, Periphal);
//}

/**
  * @function  RCC_Get_HSEDET_TCNT_UPEN
  * @brief  Get the status that whether the tracking count is update
  * @param  None
  * @retval 1:update, 0: no update
*/
//uint32_t RCC_Get_HSEDET_TCNT_UPEN(void)
//{
//	return (uint32_t)(READ_BIT(RCC->HSEDET, RCC_HSEDET_TCNT_UPEN)==(RCC_HSEDET_TCNT_UPEN));
//}

/**
  * @function  RCC_Set_HSEDET_FCNT_WIDTH
  * @brief  Set the HSEDET FCNT WIDTH
  * @param  Source:
            @arg RCC_HSEDET_FCNT_WIDTH_5bit 
            @arg RCC_HSEDET_FCNT_WIDTH_6bit 
            @arg RCC_HSEDET_FCNT_WIDTH_7bit 
            @arg RCC_HSEDET_FCNT_WIDTH_8bit 
            @arg RCC_HSEDET_FCNT_WIDTH_9bit 
            @arg RCC_HSEDET_FCNT_WIDTH_10bit
            @arg RCC_HSEDET_FCNT_WIDTH_11bit
  * @retval None
*/
//void RCC_Set_HSEDET_FCNT_WIDTH(uint32_t Periphal)
//{
//  MODIFY_REG(RCC->HSEDET, RCC_HSEDET_FCNT_WIDTH, Periphal);
//}


/**
  * @function  RCC_Get_HSEDET_TOUT
  * @brief  Get the HSEDET TOUT
  * @param  None
  * @retval TOUT of HSEDET
*/
//uint32_t RCC_Get_HSEDET_TOUT(void)
//{
//	return (uint32_t)(READ_BIT(RCC->HSEDET, RCC_HSEDET_TOUT)>>16);
//}

/**
  * @function  RCC_Get_PLLDET_DELTA
  * @brief  Get the PLLDET DELTA
  * @param  None
  * @retval DELTA of PLLDET
*/
uint32_t RCC_Get_PLLDET_DELTA(void)
{
	return (uint32_t)(READ_BIT(RCC->PLLDET, RCC_PLLDET_DELTA));
}


/**
  * @function  RCC_Enable_PLLDET_FAIL
  * @brief  Enable PLLDET FAIL
	* @param  Periphal:
  *         @arg RCC_PLLDET_FAIL_M0
  *         @arg RCC_PLLDET_FAIL_M1
  *         @arg RCC_PLLDET_FAIL_M2
  * @retval None
  */
void RCC_Enable_PLLDET_FAIL(uint32_t Periphal)
{
   SET_BIT(RCC->PLLDET, Periphal);
}

/**
  * @function  RCC_Disable_PLLDET_FAIL
  * @brief  Disable PLLDET FAIL
	* @param  Periphal:
  *         @arg RCC_PLLDET_FAIL_M0
  *         @arg RCC_PLLDET_FAIL_M1
  *         @arg RCC_PLLDET_FAIL_M2
  * @retval None
  */
void RCC_Disable_PLLDET_FAIL(uint32_t Periphal)
{
   CLEAR_BIT(RCC->PLLDET, Periphal);
}


/**
  * @function  RCC_Get_PLLDET_TCNT_UPEN
  * @brief  Get the status that whether the tracking count is update
  * @param  None
  * @retval 1:update, 0: no update
*/
uint32_t RCC_Get_PLLDET_TCNT_UPEN(void)
{
	return (uint32_t)(READ_BIT(RCC->PLLDET, RCC_PLLDET_TCNT_UPEN)==(RCC_PLLDET_TCNT_UPEN));
}



/**
  * @function  RCC_Set_PLLDET_FCNT_WIDTH
  * @brief  Set the PLLDET FCNT WIDTH
  * @param  Source:
            @arg RCC_PLLDET_FCNT_WIDTH_5bit 
            @arg RCC_PLLDET_FCNT_WIDTH_6bit 
            @arg RCC_PLLDET_FCNT_WIDTH_7bit 
            @arg RCC_PLLDET_FCNT_WIDTH_8bit 
            @arg RCC_PLLDET_FCNT_WIDTH_9bit 
            @arg RCC_PLLDET_FCNT_WIDTH_10bit
            @arg RCC_PLLDET_FCNT_WIDTH_11bit
  * @retval None
*/
void RCC_Set_PLLDET_FCNT_WIDTH(uint32_t Periphal)
{
  MODIFY_REG(RCC->PLLDET, RCC_PLLDET_FCNT_WIDTH, Periphal);
}


/**
  * @function  RCC_Get_PLLDET_TOUT
  * @brief  Get the PLLDET TOUT
  * @param  None
  * @retval TOUT of PLLDET
*/
uint32_t RCC_Get_PLLDET_TOUT(void)
{
	return (uint32_t)(READ_BIT(RCC->PLLDET, RCC_PLLDET_TOUT)>>16);
}

/**
***********************************************************************************
	*@Function RCC_Get_PLLClock
	*@brief    Get  PLLClock .
	*@param    none.
	*@retval   PLLClock
***********************************************************************************
  */
//uint32_t RCC_Get_PLLClock(void)
//{
//  uint32_t frequency = 0U;
////  uint32_t HSE_VALUE = 8000000;
//  uint32_t HSI_VALUE = 8000000;
//	frequency = (HSI_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL )>>18)+12) );
////	switch( READ_BIT( RCC->CFGR, RCC_CFGR_PLLSRC) )
////	{
////		case RCC_CFGR_PLLSRC_HSI:
////		default:
////			frequency = (HSI_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL )>>18)+2) );
////			break;
//////		case RCC_CFGR_PLLSRC_HSEDIV1:
//////			frequency = (HSE_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL_MASK )>>18)+2) );
//////			break;
////		case RCC_CFGR_PLLSRC_HSE:
////			frequency = (HSE_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL )>>18)+2) );
////			break;
//////		case RCC_CFGR_PLLSRC_HSEDIV2:
//////			frequency = (HSE_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL_MASK )>>18)+2) ) / 2 ;
//////			break;
////	}
//  return frequency;
//}


///**
//***********************************************************************************
//	*@Function RCC_Get_SystemClock
//	*@brief    Get  SystemClock .
//	*@param    none.
//	*@retval   SystemClock
//***********************************************************************************
//  */
uint32_t RCC_Get_SystemClock(void)
{
  uint32_t frequency = 0U;
//  uint32_t HSE_VALUE = 8000000;
  uint32_t HSI_VALUE = 8000000;
  switch( RCC_Get_SysClkSource() )
  {
    case RCC_CFGR_SWS_HSI:
    default:
      frequency = HSI_VALUE;
      break;
//    case RCC_CFGR_SWS_HSE:
//      frequency = HSE_VALUE;
//      break;
    case RCC_CFGR_SWS_PLL:
    {
      switch( READ_BIT( RCC->CFGR, (RCC_CFGR_PLLMULL)) )
//      {
//        case RCC_CFGR_PLLSRC_HSI:
//        default:
//          frequency = (HSI_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL )>>18)+12) );
//          break;
//        case RCC_CFGR_PLLSRC_HSE:
//          frequency = (HSE_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL )>>18)+12) );
//          break;
//        case RCC_CFGR_PLLSRC_HSIDIV2:
//          frequency = (HSI_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL_MASK )>>18)+2) ) / 2 ;
//          break;
//        case RCC_CFGR_PLLSRC_HSEDIV2:
//          frequency = (HSE_VALUE*((READ_BIT( RCC->CFGR, RCC_CFGR_PLLMULL_MASK )>>18)+2) ) / 2 ;
//          break;
//      }
      break;
    }
  }
  return frequency;
}


///**
//***********************************************************************************
//	*@Function RCC_Get_AHBClock
//	*@brief    Get  AHBClock .
//	*@param    none.
//	*@retval   AHBClock
//***********************************************************************************
//  */
//uint32_t RCC_Get_AHBClock(void)
//{
//  return ( RCC_Get_SystemClock() >> AHBPrescTable[ RCC_Get_AHBPrescaler() >> 4 ]);
//}


///**
//***********************************************************************************
//	*@Function RCC_Get_APB1Clock
//	*@brief    Get  APB1Clock .
//	*@param    none.
//	*@retval   APB1Clock
//***********************************************************************************
//  */
//uint32_t RCC_Get_APB1Clock(void)
//{
//  return ( RCC_Get_AHBClock() >> APBPrescTable[ RCC_Get_APB1Prescaler() >> 8 ] );
//}


///**
//***********************************************************************************
//	*@Function RCC_Get_APB2Clock
//	*@brief    Get  APB2Clock .
//	*@param    none.
//	*@retval   APB2Clock
//***********************************************************************************
//  */
//uint32_t RCC_Get_APB2Clock(void)
//{
//  return ( RCC_Get_AHBClock() >> APBPrescTable[ RCC_Get_APB2Prescaler() >> 11 ] );
//}


///**
//***********************************************************************************
//	*@Function RCC_Get_ADCClock
//	*@brief    Get  ADCClock .
//	*@param    none.
//	*@retval   ADCClock
//***********************************************************************************
//  */
//uint32_t RCC_Get_ADCClock(void)
//{
//	if( RCC_Get_ADCClkSource()==RCC_CFGR_ADCCLK_SEL_SYSCLK )
//	{
//		if( RCC_Get_ADCPrescaler() == RCC_CFGR_ADCPRE_DIV6 )
//		{
//			return ( RCC_Get_APB2Clock() / (( RCC_Get_ADCPrescaler() >> 14 ) *3 ) );
//		}
//		else if( RCC_Get_ADCPrescaler() == RCC_CFGR_ADCPRE_DIV8)
//		{
//			return ( RCC_Get_APB2Clock() >> ( RCC_Get_ADCPrescaler() >> 14  ) );
//		}
//		else
//		{
//			return ( RCC_Get_APB2Clock() >> (( RCC_Get_ADCPrescaler() >> 14 ) +1 ) );
//		}
//	}
//	else
//	{
//		if( RCC_Get_ADCPrescaler() == RCC_CFGR_ADCPRE_DIV6 )
//		{
//			return ( (RCC_Get_PLLClock()*2) / (( RCC_Get_ADCPrescaler() >> 14 ) *3 ) );
//		}
//		else if( RCC_Get_ADCPrescaler() == RCC_CFGR_ADCPRE_DIV8)
//		{
//			return ( (RCC_Get_PLLClock()*2) >> ( RCC_Get_ADCPrescaler() >> 14 )  );
//		}
//		else
//		{
//			return ( (RCC_Get_PLLClock()*2) >> (( RCC_Get_ADCPrescaler() >> 14 ) +1 ) );
//		}
//	}
//}

