/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2021, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_usart.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/

#include "rx32s12xx_usart.h"
#include "rx32s12xx_rcc.h"

#define USART1_PUT 1
#define USART2_PUT 0

int fputc(int ch, FILE *f)
{
#if defined(USART1_PUT)
	USART1->DR = ch & 0x1FFU;
  while((USART1->SR & USART_SR_TXE) == 0);
#endif
  
#if defined(USART2_PUT)
	USART2->DR = ch & 0x1FFU;
  while((USART2->SR & USART_SR_TXE) == 0);
#endif
  return ch;
}

/**
  * @brief  De-initialize USART registers (Registers restored to their default values).
  * @param  USARTx USART Instance

  */
void USART_DeInit(USART_TypeDef *USARTx)
{
  if (USARTx == USART1)
  {
		SET_BIT(RCC->APB2RSTR,RCC_APB2RSTR_USART1RST);
		CLEAR_BIT(RCC->APB2RSTR,RCC_APB2RSTR_USART1RST);
  }
  else if (USARTx == USART2)
  {
		SET_BIT(RCC->APB1RSTR,RCC_APB1RSTR_USART2RST);
		CLEAR_BIT(RCC->APB1RSTR,RCC_APB1RSTR_USART2RST);
  }
}

/**
  * @brief Set each @ref USART_InitTypeDef field to default value.
  * @param USART_InitStruct Pointer to a @ref USART_InitTypeDef structure
  *                         whose fields will be set to default values.
  * @retval None
  */
void USART_StructInit(USART_InitTypeDef *USART_InitStruct)
{
  /* Set USART_InitStruct fields to default values */
  USART_InitStruct->BaudRate            = 9600U;
  USART_InitStruct->DataWidth           = USART_DATAWIDTH_8B;
  USART_InitStruct->StopBits            = USART_STOPBITS_1;
  USART_InitStruct->Parity              = USART_PARITY_NONE ;
  USART_InitStruct->TransferDirection   = USART_DIRECTION_TX_RX;
  USART_InitStruct->HardwareFlowControl = USART_HWCONTROL_NONE;
}

/**
  * @brief  Initialize USART registers according to the specified
  *         parameters in USART_InitStruct.
  * @note   As some bits in USART configuration registers can only be written when the USART is disabled (USART_CR1_UE bit =0),
  *         USART IP should be in disabled state prior calling this function. Otherwise, ERROR result will be returned.
  * @note   Baud rate value stored in USART_InitStruct BaudRate field, should be valid (different from 0).
  * @param  USARTx USART Instance
  * @param  USART_InitStruct pointer to a USART_InitTypeDef structure
  *         that contains the configuration information for the specified USART peripheral.
  */
void USART_Init(USART_TypeDef *USARTx, USART_InitTypeDef *USART_InitStruct)
{
  uint32_t periphclk = 0U;
	
  RCC_ClocksTypeDef rcc_clocks = {0};
	
	/* Get SYSCLK frequency */
	rcc_clocks.SYSCLK_Frequency = 80000000;  //RCC_Get_SystemClock()
	/* HCLK clock frequency */
	rcc_clocks.HCLK_Frequency   = 80000000;  //RCC_Get_AHBClock()
	/* PCLK1 clock frequency */
	rcc_clocks.PCLK1_Frequency  = 40000000;  //RCC_Get_APB1Clock()
	/* PCLK2 clock frequency */
	rcc_clocks.PCLK2_Frequency  = 80000000;  //RCC_Get_APB2Clock()
	
  if (USART_Is_Enable(USARTx) == 0U)
  {
    MODIFY_REG(USARTx->CR1,
               (USART_CR1_M | USART_CR1_PCE | USART_CR1_PS |
                USART_CR1_TE | USART_CR1_RE),
               (USART_InitStruct->DataWidth | USART_InitStruct->Parity |
                USART_InitStruct->TransferDirection));

    USART_Set_StopBitsLength(USARTx, USART_InitStruct->StopBits);

    USART_Set_HWFlowCtrl(USARTx, USART_InitStruct->HardwareFlowControl);

    //RCC_GetSystemClocksFreq(&rcc_clocks);		
    if (USARTx == USART1)
    {
      periphclk = rcc_clocks.PCLK2_Frequency;
    }
    else if (USARTx == USART2)
    {
      periphclk = rcc_clocks.PCLK1_Frequency;
    }
#if defined(USART3)
    else if (USARTx == USART3)
    {
      periphclk = rcc_clocks.PCLK1_Frequency;
    }
#endif /* USART3 */
    else {;}

    if ((periphclk != 0U) && (USART_InitStruct->BaudRate != 0U))
    {
      USART_Set_BaudRate(USARTx,periphclk,USART_InitStruct->BaudRate);
    }
  }
 
}

/**
  * @brief  USART Enable
  * @rmtoll CR1          UE            USART_Enable
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR1, USART_CR1_UE);
}

/**
  * @brief  USART Disable (all USART prescalers and outputs are disabled)
  * @note   When USART is disabled, USART prescalers and outputs are stopped immediately,
  *         and current operations are discarded. The configuration of the USART is kept, but all the status
  *         flags, in the USARTx_SR are set to their default values.
  * @rmtoll CR1          UE            USART_Disable
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR1, USART_CR1_UE);
}

/**
  * @brief  Indicate if USART is enabled
  * @rmtoll CR1          UE            USART_Is_Enable
  * @param  USARTx USART Instance
  * @retval State of bit (1 or 0).
  */
uint32_t USART_Is_Enable(USART_TypeDef *USARTx)
{
  return (READ_BIT(USARTx->CR1, USART_CR1_UE) == (USART_CR1_UE));
}

/**
  * @brief  Receiver Enable (Receiver is enabled and begins searching for a start bit)
  * @rmtoll CR1          RE            USART_Enable_Rx
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable_Rx(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR1, USART_CR1_RE);
}

/**
  * @brief  Receiver Disable
  * @rmtoll CR1          RE            USART_Disable_Rx
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable_Rx(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR1, USART_CR1_RE);
}

/**
  * @brief  Transmitter Enable
  * @rmtoll CR1          TE            USART_Enable_Tx
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable_Tx(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR1, USART_CR1_TE);
}

/**
  * @brief  Transmitter Disable
  * @rmtoll CR1          TE            USART_Disable_Tx
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable_Tx(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR1, USART_CR1_TE);
}

/**
  * @brief  Configure simultaneously enabled/disabled states
  *         of Transmitter and Receiver
  * @rmtoll CR1          RE            USART_Set_TransferDirection\n
  *         CR1          TE            USART_Set_TransferDirection
  * @param  USARTx USART Instance
  * @param  TransferDirection This parameter can be one of the following values:
  *         @arg @ref USART_DIRECTION_NONE
  *         @arg @ref USART_DIRECTION_RX
  *         @arg @ref USART_DIRECTION_TX
  *         @arg @ref USART_DIRECTION_TX_RX
  * @retval None
  */
void USART_Set_TransferDirection(USART_TypeDef *USARTx, uint32_t TransferDirection)
{
  MODIFY_REG(USARTx->CR1, USART_CR1_RE | USART_CR1_TE, TransferDirection);
}

/**
  * @brief  Return enabled/disabled states of Transmitter and Receiver
  * @rmtoll CR1          RE            USART_GetTransferDirection\n
  *         CR1          TE            USART_GetTransferDirection
  * @param  USARTx USART Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref USART_DIRECTION_NONE
  *         @arg @ref USART_DIRECTION_RX
  *         @arg @ref USART_DIRECTION_TX
  *         @arg @ref USART_DIRECTION_TX_RX
  */
uint32_t USART_Get_TransferDirection(USART_TypeDef *USARTx)
{
  return (uint32_t)(READ_BIT(USARTx->CR1, USART_CR1_RE | USART_CR1_TE));
}

/**
  * @brief  Configure Parity (enabled/disabled and parity mode if enabled).
  * @note   This function selects if hardware parity control (generation and detection) is enabled or disabled.
  *         When the parity control is enabled (Odd or Even), computed parity bit is inserted at the MSB position
  *         (9th or 8th bit depending on data width) and parity is checked on the received data.
  * @rmtoll CR1          PS            USART_SetParity\n
  *         CR1          PCE           USART_SetParity
  * @param  USARTx USART Instance
  * @param  Parity This parameter can be one of the following values:
  *         @arg @ref USART_PARITY_NONE
  *         @arg @ref USART_PARITY_EVEN
  *         @arg @ref USART_PARITY_ODD
  * @retval None
  */
void USART_Set_Parity(USART_TypeDef *USARTx, uint32_t Parity)
{
  MODIFY_REG(USARTx->CR1, USART_CR1_PS | USART_CR1_PCE, Parity);
}

/**
  * @brief  Return Parity configuration (enabled/disabled and parity mode if enabled)
  * @rmtoll CR1          PS            USART_GetParity\n
  *         CR1          PCE           USART_GetParity
  * @param  USARTx USART Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref USART_PARITY_NONE
  *         @arg @ref USART_PARITY_EVEN
  *         @arg @ref USART_PARITY_ODD
  */
uint32_t USART_Get_Parity(USART_TypeDef *USARTx)
{
  return (uint32_t)(READ_BIT(USARTx->CR1, USART_CR1_PS | USART_CR1_PCE));
}

/**
  * @brief  Set Receiver Wake Up method from Mute mode.
  * @rmtoll CR1          WAKE          USART_SetWakeUpMethod
  * @param  USARTx USART Instance
  * @param  Method This parameter can be one of the following values:
  *         @arg @ref USART_WAKEUP_IDLELINE
  *         @arg @ref USART_WAKEUP_ADDRESSMARK
  * @retval None
  */
void USART_Set_WakeUpMethod(USART_TypeDef *USARTx, uint32_t Method)
{
  MODIFY_REG(USARTx->CR1, USART_CR1_WAKE, Method);
}

/**
  * @brief  Return Receiver Wake Up method from Mute mode
  * @rmtoll CR1          WAKE          USART_GetWakeUpMethod
  * @param  USARTx USART Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref USART_WAKEUP_IDLELINE
  *         @arg @ref USART_WAKEUP_ADDRESSMARK
  */
uint32_t USART_Get_WakeUpMethod(USART_TypeDef *USARTx)
{
  return (uint32_t)(READ_BIT(USARTx->CR1, USART_CR1_WAKE));
}

/**
  * @brief  Set Word length (i.e. nb of data bits, excluding start and stop bits)
  * @rmtoll CR1          M             USART_SetDataWidth
  * @param  USARTx USART Instance
  * @param  DataWidth This parameter can be one of the following values:
  *         @arg @ref USART_DATAWIDTH_8B
  *         @arg @ref USART_DATAWIDTH_9B
  * @retval None
  */
void USART_Set_DataWidth(USART_TypeDef *USARTx, uint32_t DataWidth)
{
  MODIFY_REG(USARTx->CR1, USART_CR1_M, DataWidth);
}

/**
  * @brief  Return Word length (i.e. nb of data bits, excluding start and stop bits)
  * @rmtoll CR1          M             USART_GetDataWidth
  * @param  USARTx USART Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref USART_DATAWIDTH_8B
  *         @arg @ref USART_DATAWIDTH_9B
  */
uint32_t USART_Get_DataWidth(USART_TypeDef *USARTx)
{
  return (uint32_t)(READ_BIT(USARTx->CR1, USART_CR1_M));
}

/**
  * @brief  Set the length of the stop bits
  * @rmtoll CR2          STOP          USART_SetStopBitsLength
  * @param  USARTx USART Instance
  * @param  StopBits This parameter can be one of the following values:
  *         @arg @ref USART_STOPBITS_0_5
  *         @arg @ref USART_STOPBITS_1
  *         @arg @ref USART_STOPBITS_1_5
  *         @arg @ref USART_STOPBITS_2
  * @retval None
  */
void USART_Set_StopBitsLength(USART_TypeDef *USARTx, uint32_t StopBits)
{
  MODIFY_REG(USARTx->CR2, USART_CR2_STOP, StopBits);
}

/**
  * @brief  Retrieve the length of the stop bits
  * @rmtoll CR2          STOP          USART_GetStopBitsLength
  * @param  USARTx USART Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref USART_STOPBITS_0_5
  *         @arg @ref USART_STOPBITS_1
  *         @arg @ref USART_STOPBITS_1_5
  *         @arg @ref USART_STOPBITS_2
  */
uint32_t USART_Get_StopBitsLength(USART_TypeDef *USARTx)
{
  return (uint32_t)(READ_BIT(USARTx->CR2, USART_CR2_STOP));
}

/**
  * @brief  Configure Character frame format (Datawidth, Parity control, Stop Bits)
  * @note   Call of this function is equivalent to following function call sequence :
  *         - Data Width configuration using @ref USART_SetDataWidth() function
  *         - Parity Control and mode configuration using @ref USART_SetParity() function
  *         - Stop bits configuration using @ref USART_SetStopBitsLength() function
  * @rmtoll CR1          PS            USART_Set_Character\n
  *         CR1          PCE           USART_Set_Character\n
  *         CR1          M             USART_Set_Character\n
  *         CR2          STOP          USART_Set_Character
  * @param  USARTx USART Instance
  * @param  DataWidth This parameter can be one of the following values:
  *         @arg @ref USART_DATAWIDTH_8B
  *         @arg @ref USART_DATAWIDTH_9B
  * @param  Parity This parameter can be one of the following values:
  *         @arg @ref USART_PARITY_NONE
  *         @arg @ref USART_PARITY_EVEN
  *         @arg @ref USART_PARITY_ODD
  * @param  StopBits This parameter can be one of the following values:
  *         @arg @ref USART_STOPBITS_0_5
  *         @arg @ref USART_STOPBITS_1
  *         @arg @ref USART_STOPBITS_1_5
  *         @arg @ref USART_STOPBITS_2
  * @retval None
  */
void USART_Set_Character(USART_TypeDef *USARTx, uint32_t DataWidth, uint32_t Parity, uint32_t StopBits)
{
  MODIFY_REG(USARTx->CR1, USART_CR1_PS | USART_CR1_PCE | USART_CR1_M, Parity | DataWidth);
  MODIFY_REG(USARTx->CR2, USART_CR2_STOP, StopBits);
}

/**
  * @brief  Set Address of the USART node.
  * @note   This is used in multiprocessor communication during Mute mode or Stop mode,
  *         for wake up with address mark detection.
  * @rmtoll CR2          ADD           USART_SetNodeAddress
  * @param  USARTx USART Instance
  * @param  NodeAddress 4 bit Address of the USART node.
  * @retval None
  */
void USART_Set_NodeAddress(USART_TypeDef *USARTx, uint32_t NodeAddress)
{
  MODIFY_REG(USARTx->CR2, USART_CR2_ADD, (NodeAddress & USART_CR2_ADD));
}

/**
  * @brief  Return 4 bit Address of the USART node as set in ADD field of CR2.
  * @note   only 4bits (b3-b0) of returned value are relevant (b31-b4 are not relevant)
  * @rmtoll CR2          ADD           USART_GetNodeAddress
  * @param  USARTx USART Instance
  * @retval Address of the USART node (Value between Min_Data=0 and Max_Data=255)
  */
uint32_t USART_Get_NodeAddress(USART_TypeDef *USARTx)
{
  return (uint32_t)(READ_BIT(USARTx->CR2, USART_CR2_ADD));
}

/**
  * @brief  Enable RTS HW Flow Control
  * @note   Macro @ref IS_UART_HWFLOW_INSTANCE(USARTx) can be used to check whether or not
  *         Hardware Flow control feature is supported by the USARTx instance.
  * @rmtoll CR3          RTSE          USART_EnableRTSHWFlowCtrl
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable_RTSHWFlowCtrl(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR3, USART_CR3_RTSE);
}

/**
  * @brief  Disable RTS HW Flow Control
  * @note   Macro @ref IS_UART_HWFLOW_INSTANCE(USARTx) can be used to check whether or not
  *         Hardware Flow control feature is supported by the USARTx instance.
  * @rmtoll CR3          RTSE          USART_DisableRTSHWFlowCtrl
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable_RTSHWFlowCtrl(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR3, USART_CR3_RTSE);
}

/**
  * @brief  Enable CTS HW Flow Control
  * @note   Macro @ref IS_UART_HWFLOW_INSTANCE(USARTx) can be used to check whether or not
  *         Hardware Flow control feature is supported by the USARTx instance.
  * @rmtoll CR3          CTSE          USART_EnableCTSHWFlowCtrl
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable_CTSHWFlowCtrl(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR3, USART_CR3_CTSE);
}

/**
  * @brief  Disable CTS HW Flow Control
  * @note   Macro @ref IS_UART_HWFLOW_INSTANCE(USARTx) can be used to check whether or not
  *         Hardware Flow control feature is supported by the USARTx instance.
  * @rmtoll CR3          CTSE          USART_DisableCTSHWFlowCtrl
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable_CTSHWFlowCtrl(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR3, USART_CR3_CTSE);
}

/**
  * @brief  Configure HW Flow Control mode (both CTS and RTS)
  * @note   Macro @ref IS_UART_HWFLOW_INSTANCE(USARTx) can be used to check whether or not
  *         Hardware Flow control feature is supported by the USARTx instance.
  * @rmtoll CR3          RTSE          USART_SetHWFlowCtrl\n
  *         CR3          CTSE          USART_SetHWFlowCtrl
  * @param  USARTx USART Instance
  * @param  HardwareFlowControl This parameter can be one of the following values:
  *         @arg @ref USART_HWCONTROL_NONE
  *         @arg @ref USART_HWCONTROL_RTS
  *         @arg @ref USART_HWCONTROL_CTS
  *         @arg @ref USART_HWCONTROL_RTS_CTS
  * @retval None
  */
void USART_Set_HWFlowCtrl(USART_TypeDef *USARTx, uint32_t HardwareFlowControl)
{
  MODIFY_REG(USARTx->CR3, USART_CR3_RTSE | USART_CR3_CTSE, HardwareFlowControl);
}

/**
  * @brief  Return HW Flow Control configuration (both CTS and RTS)
  * @note   Macro @ref IS_UART_HWFLOW_INSTANCE(USARTx) can be used to check whether or not
  *         Hardware Flow control feature is supported by the USARTx instance.
  * @rmtoll CR3          RTSE          USART_GetHWFlowCtrl\n
  *         CR3          CTSE          USART_GetHWFlowCtrl
  * @param  USARTx USART Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref USART_HWCONTROL_NONE
  *         @arg @ref USART_HWCONTROL_RTS
  *         @arg @ref USART_HWCONTROL_CTS
  *         @arg @ref USART_HWCONTROL_RTS_CTS
  */
uint32_t USART_Get_HWFlowCtrl(USART_TypeDef *USARTx)
{
  return (uint32_t)(READ_BIT(USARTx->CR3, USART_CR3_RTSE | USART_CR3_CTSE));
}

/**
  * @brief  Configure USART BRR register for achieving expected Baud Rate value.
  * @note   Compute and set USARTDIV value in BRR Register (full BRR content)
  *         according to used Peripheral Clock, Oversampling mode, and expected Baud Rate values
  * @note   Peripheral clock and Baud rate values provided as function parameters should be valid
  *         (Baud rate value != 0)
  * @rmtoll BRR          BRR           USART_SetBaudRate
  * @param  USARTx USART Instance
  * @param  PeriphClk Peripheral Clock
  * @param  BaudRate Baud Rate
  * @retval None
  */
void USART_Set_BaudRate(USART_TypeDef *USARTx, uint32_t PeriphClk, uint32_t BaudRate)
{
  USARTx->BRR = (uint16_t)(__USART_DIV_SAMPLING16(PeriphClk, BaudRate));
}

/**
  * @brief  Return current Baud Rate value, according to USARTDIV present in BRR register
  *         (full BRR content), and to used Peripheral Clock and Oversampling mode values
  * @note   In case of non-initialized or invalid value stored in BRR register, value 0 will be returned.
  * @rmtoll BRR          BRR           USART_GetBaudRate
  * @param  USARTx USART Instance
  * @param  PeriphClk Peripheral Clock
  * @retval Baud Rate
  */
uint32_t USART_Get_BaudRate(USART_TypeDef *USARTx, uint32_t PeriphClk)
{
  register uint32_t usartdiv = 0x0U;
  register uint32_t brrresult = 0x0U;

  usartdiv = USARTx->BRR;

  if ((usartdiv & 0xFFFFU) != 0U)
  {
    brrresult = PeriphClk / usartdiv;
  }
  return (brrresult);
}

/**
  * @brief  Enable Single Wire Half-Duplex mode
  * @note   Macro @ref IS_UART_HALFDUPLEX_INSTANCE(USARTx) can be used to check whether or not
  *         Half-Duplex mode is supported by the USARTx instance.
  * @rmtoll CR3          HDSEL         USART_EnableHalfDuplex
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable_HalfDuplex(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR3, USART_CR3_HDSEL);
}

/**
  * @brief  Disable Single Wire Half-Duplex mode
  * @note   Macro @ref IS_UART_HALFDUPLEX_INSTANCE(USARTx) can be used to check whether or not
  *         Half-Duplex mode is supported by the USARTx instance.
  * @rmtoll CR3          HDSEL         USART_DisableHalfDuplex
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable_HalfDuplex(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR3, USART_CR3_HDSEL);
}

/**
  * @brief  Indicate if Single Wire Half-Duplex mode is enabled
  * @note   Macro @ref IS_UART_HALFDUPLEX_INSTANCE(USARTx) can be used to check whether or not
  *         Half-Duplex mode is supported by the USARTx instance.
  * @rmtoll CR3          HDSEL         USART_Get_HalfDuplex
  * @param  USARTx USART Instance
  * @retval State of bit (1 or 0).
  */
uint32_t USART_Get_HalfDuplex(USART_TypeDef *USARTx)
{
  return (READ_BIT(USARTx->CR3, USART_CR3_HDSEL) == (USART_CR3_HDSEL));
}

/**
  * @brief  Set LIN Break Detection Length
  * @note   Macro @ref IS_UART_LIN_INSTANCE(USARTx) can be used to check whether or not
  *         LIN feature is supported by the USARTx instance.
  * @rmtoll CR2          LBDL          USART_SetLINBrkDetectionLen
  * @param  USARTx USART Instance
  * @param  LINBDLength This parameter can be one of the following values:
  *         @arg @ref USART_LINBREAK_DETECT_10B
  *         @arg @ref USART_LINBREAK_DETECT_11B
  * @retval None
  */
void USART_Set_LINBrkDetectionLen(USART_TypeDef *USARTx, uint32_t LINBDLength)
{
  MODIFY_REG(USARTx->CR2, USART_CR2_LBDL, LINBDLength);
}

/**
  * @brief  Return LIN Break Detection Length
  * @note   Macro @ref IS_UART_LIN_INSTANCE(USARTx) can be used to check whether or not
  *         LIN feature is supported by the USARTx instance.
  * @rmtoll CR2          LBDL          USART_GetLINBrkDetectionLen
  * @param  USARTx USART Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref USART_LINBREAK_DETECT_10B
  *         @arg @ref USART_LINBREAK_DETECT_11B
  */
uint32_t USART_Get_LINBrkDetectionLen(USART_TypeDef *USARTx)
{
  return (uint32_t)(READ_BIT(USARTx->CR2, USART_CR2_LBDL));
}

/**
  * @brief  Enable LIN mode
  * @note   Macro @ref IS_UART_LIN_INSTANCE(USARTx) can be used to check whether or not
  *         LIN feature is supported by the USARTx instance.
  * @rmtoll CR2          LINEN         USART_EnableLIN
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable_LIN(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR2, USART_CR2_LINEN);
}

/**
  * @brief  Disable LIN mode
  * @note   Macro @ref IS_UART_LIN_INSTANCE(USARTx) can be used to check whether or not
  *         LIN feature is supported by the USARTx instance.
  * @rmtoll CR2          LINEN         USART_DisableLIN
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable_LIN(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR2, USART_CR2_LINEN);
}

/**
  * @brief  Indicate if LIN mode is enabled
  * @note   Macro @ref IS_UART_LIN_INSTANCE(USARTx) can be used to check whether or not
  *         LIN feature is supported by the USARTx instance.
  * @rmtoll CR2          LINEN         USART_Get_LIN
  * @param  USARTx USART Instance
  * @retval State of bit (1 or 0).
  */
uint32_t USART_Get_LIN(USART_TypeDef *USARTx)
{
  return (READ_BIT(USARTx->CR2, USART_CR2_LINEN) == (USART_CR2_LINEN));
}


/**
***********************************************************************************
*@Function USART_Get_Flag
*@brief    Get USART Flag.
*@param  USARTx USART Instance
*@param1   Flag:
*          @arg  USART_FLAG_PE  
*          @arg  USART_FLAG_FE  
*          @arg  USART_FLAG_NE  
*          @arg  USART_FLAG_ORE 
*          @arg  USART_FLAG_IDLE
*          @arg  USART_FLAG_RXNE
*          @arg  USART_FLAG_TC  
*          @arg  USART_FLAG_TXE  
*          @arg  USART_FLAG_LBD 
*          @arg  USART_FLAG_CTS 
*          @arg  USART_FLAG_SBK 
*          @arg  USART_FLAG_RWU 
*@retval State of bit (1 or 0).
***********************************************************************************
  */
uint32_t USART_Get_Flag(USART_TypeDef *USARTx, uint32_t Flag)
{	
	if((Flag&0x10000000)==0x10000000)
	{
		return (READ_BIT(USARTx->SR, (Flag&0xFFFFFFF) ) == (Flag&0xFFFFFFF));
	}
	else if((Flag&0x20000000)==0x20000000)
	{
		return (READ_BIT(USARTx->CR1, (Flag&0x1FFFFFFF) ) == (Flag&0x1FFFFFFF));
	}
	return 0;
}

/**
***********************************************************************************
*@Function USART_Clear_Flag
*@brief    Clear USART Flag.
*@param  USARTx USART Instance
*@param1   Flag:
*          @arg  USART_FLAG_PE  
*          @arg  USART_FLAG_FE  
*          @arg  USART_FLAG_NE  
*          @arg  USART_FLAG_ORE 
*          @arg  USART_FLAG_IDLE
*          @arg  USART_FLAG_RXNE
*          @arg  USART_FLAG_TC  
*          @arg  USART_FLAG_TXE  
*          @arg  USART_FLAG_LBD 
*          @arg  USART_FLAG_CTS 
*          @arg  USART_FLAG_SBK 
*          @arg  USART_FLAG_RWU 
*@retval None.
***********************************************************************************
  */
void USART_Clear_Flag(USART_TypeDef *USARTx, uint32_t Flag)
{	
	if((Flag==USART_FLAG_PE)|(Flag==USART_FLAG_FE)|(Flag==USART_FLAG_NE)|(Flag==USART_FLAG_ORE)|(Flag==USART_FLAG_IDLE))
	{
		__IO uint32_t tmpreg;
	  tmpreg = USARTx->SR;
	  (void) tmpreg;
	  tmpreg = USARTx->DR;
	  (void) tmpreg;
	}
	else if((Flag&0x10000000)==0x10000000)
	{
		WRITE_REG(USARTx->SR, ~(Flag&0xFFFFFFF) );
	}
	else if((Flag&0x20000000)==0x20000000)
	{
		CLEAR_BIT(USARTx->CR1, (Flag&0x1FFFFFFF) );
	}
}


/**
***********************************************************************************
*@Function USART_Enable_IT
*@brief    Enable USART Iterrupt.
*@param  USARTx USART Instance
*@param1   IT:
*          @arg  USART_IT_IDLEIE
*          @arg  USART_IT_RXNEIE
*          @arg  USART_IT_TCIE  
*          @arg  USART_IT_TXEIE 
*          @arg  USART_IT_PEIE  
*          @arg  USART_IT_LBDIE 
*          @arg  USART_IT_EIE   
*          @arg  USART_IT_CTSIE 
*@retval None.
***********************************************************************************
  */
void USART_Enable_IT(USART_TypeDef *USARTx, uint32_t IT)
{
	if((IT&0x10000000)==0x10000000)
	{
		SET_BIT(USARTx->CR1, (IT&0xFFFFFFF) );
	}
	else if((IT&0x20000000)==0x20000000)
	{
		SET_BIT(USARTx->CR2, (IT&0x1FFFFFFF) );
	}
	else if((IT&0x40000000)==0x40000000)
	{
		SET_BIT(USARTx->CR3, (IT&0x3FFFFFFF) );
	}	
}

/**
***********************************************************************************
*@Function USART_Disable_IT
*@brief    Disable USART Iterrupt.
*@param  USARTx USART Instance
*@param1   IT:
*          @arg  USART_IT_IDLEIE
*          @arg  USART_IT_RXNEIE
*          @arg  USART_IT_TCIE  
*          @arg  USART_IT_TXEIE 
*          @arg  USART_IT_PEIE  
*          @arg  USART_IT_LBDIE 
*          @arg  USART_IT_EIE   
*          @arg  USART_IT_CTSIE 
*@retval None.
***********************************************************************************
  */
void USART_Disable_IT(USART_TypeDef *USARTx, uint32_t IT)
{	
	if((IT&0x10000000)==0x10000000)
	{
		CLEAR_BIT(USARTx->CR1, (IT&0xFFFFFFF) );
	}
	else if((IT&0x20000000)==0x20000000)
	{
		CLEAR_BIT(USARTx->CR2, (IT&0x1FFFFFFF) );
	}
	else if((IT&0x40000000)==0x40000000)
	{
		CLEAR_BIT(USARTx->CR3, (IT&0x3FFFFFFF) );
	}	
}

/**
***********************************************************************************
*@Function USART_Get_IT
*@brief    Get USART Iterrupt.
*@param  USARTx USART Instance
*@param1   IT:
*          @arg  USART_IT_IDLEIE
*          @arg  USART_IT_RXNEIE
*          @arg  USART_IT_TCIE  
*          @arg  USART_IT_TXEIE 
*          @arg  USART_IT_PEIE  
*          @arg  USART_IT_LBDIE 
*          @arg  USART_IT_EIE   
*          @arg  USART_IT_CTSIE 
*@retval State of bit (1 or 0).
***********************************************************************************
  */
uint32_t USART_Get_IT(USART_TypeDef *USARTx, uint32_t IT)
{	
	if((IT&0x10000000)==0x10000000)
	{
		return (READ_BIT(USARTx->CR1, (IT&0xFFFFFFF) ) == (IT&0xFFFFFFF));
	}
	else if((IT&0x20000000)==0x20000000)
	{
		return (READ_BIT(USARTx->CR2, (IT&0x1FFFFFFF) ) == (IT&0x1FFFFFFF));
	}
	else if((IT&0x40000000)==0x40000000)
	{
		return (READ_BIT(USARTx->CR3, (IT&0x3FFFFFFF) ) == (IT&0x3FFFFFFF));
	}
	return 0;
}

/**
  * @brief  Enable DMA Mode for reception
  * @rmtoll CR3          DMAR          USART_Enable_DMAReq_RX
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable_DMAReq_RX(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR3, USART_CR3_DMAR);
}

/**
  * @brief  Disable DMA Mode for reception
  * @rmtoll CR3          DMAR          USART_Disable_DMAReq_RX
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable_DMAReq_RX(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR3, USART_CR3_DMAR);
}

/**
  * @brief  Check if DMA Mode is enabled for reception
  * @rmtoll CR3          DMAR          USART_Get_DMAReq_RX
  * @param  USARTx USART Instance
  * @retval State of bit (1 or 0).
  */
uint32_t USART_Get_DMAReq_RX(USART_TypeDef *USARTx)
{
  return (READ_BIT(USARTx->CR3, USART_CR3_DMAR) == (USART_CR3_DMAR));
}

/**
  * @brief  Enable DMA Mode for transmission
  * @rmtoll CR3          DMAT          USART_Enable_DMAReq_TX
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enable_DMAReq_TX(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR3, USART_CR3_DMAT);
}

/**
  * @brief  Disable DMA Mode for transmission
  * @rmtoll CR3          DMAT          USART_Disable_DMAReq_TX
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Disable_DMAReq_TX(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR3, USART_CR3_DMAT);
}

/**
  * @brief  Check if DMA Mode is enabled for transmission
  * @rmtoll CR3          DMAT          USART_Get_DMAReq_TX
  * @param  USARTx USART Instance
  * @retval State of bit (1 or 0).
  */
uint32_t USART_Get_DMAReq_TX(USART_TypeDef *USARTx)
{
  return (READ_BIT(USARTx->CR3, USART_CR3_DMAT) == (USART_CR3_DMAT));
}

/**
  * @brief  Get the data register address used for DMA transfer
  * @rmtoll DR           DR            USART_Get_DMARegAddr
  * @note   Address of Data Register is valid for both Transmit and Receive transfers.
  * @param  USARTx USART Instance
  * @retval Address of data register
  */
uint32_t USART_Get_DMARegAddr(USART_TypeDef *USARTx)
{
  /* return address of DR register */
  return ((uint32_t) & (USARTx->DR));
}

/**
  * @brief  Read Receiver Data register (Receive Data value, 8 bits)
  * @rmtoll DR           DR            USART_ReceiveData8
  * @param  USARTx USART Instance
  * @retval Value between Min_Data=0x00 and Max_Data=0xFF
  */
uint8_t USART_Receive_Data8(USART_TypeDef *USARTx)
{
  return (uint8_t)(READ_BIT(USARTx->DR, USART_DR_DR));
}

/**
  * @brief  Read Receiver Data register (Receive Data value, 9 bits)
  * @rmtoll DR           DR            USART_ReceiveData9
  * @param  USARTx USART Instance
  * @retval Value between Min_Data=0x00 and Max_Data=0x1FF
  */
uint16_t USART_Receive_Data9(USART_TypeDef *USARTx)
{
  return (uint16_t)(READ_BIT(USARTx->DR, USART_DR_DR));
}

/**
  * @brief  Write in Transmitter Data Register (Transmit Data value, 8 bits)
  * @rmtoll DR           DR            USART_TransmitData8
  * @param  USARTx USART Instance
  * @param  Value between Min_Data=0x00 and Max_Data=0xFF
  * @retval None
  */
void USART_Transmit_Data8(USART_TypeDef *USARTx, uint8_t Value)
{
  USARTx->DR = Value;
}

/**
  * @brief  Write in Transmitter Data Register (Transmit Data value, 9 bits)
  * @rmtoll DR           DR            USART_TransmitData9
  * @param  USARTx USART Instance
  * @param  Value between Min_Data=0x00 and Max_Data=0x1FF
  * @retval None
  */
void USART_Transmit_Data9(USART_TypeDef *USARTx, uint16_t Value)
{
  USARTx->DR = Value & 0x1FFU;
}

/**
  * @brief  Request Break sending
  * @rmtoll CR1          SBK           USART_RequestBreakSending
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Request_BreakSending(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR1, USART_CR1_SBK);
}

/**
  * @brief  Put USART in Mute mode
  * @rmtoll CR1          RWU           USART_Enter_MuteMode
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Enter_MuteMode(USART_TypeDef *USARTx)
{
  SET_BIT(USARTx->CR1, USART_CR1_RWU);
}

/**
  * @brief  Put USART in Active mode
  * @rmtoll CR1          RWU           USART_Exit_MuteMode
  * @param  USARTx USART Instance
  * @retval None
  */
void USART_Exit_MuteMode(USART_TypeDef *USARTx)
{
  CLEAR_BIT(USARTx->CR1, USART_CR1_RWU);
}

