/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/

/*
*********************************************************************************************************
*                                              rx32s12xx
*                                           Library Function
*
*                                   Copyright 2019, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s12xx
* File         : rx32s12xx_tim.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s12xx
*********************************************************************************************************
*/
#define  __rx32s12xx_TIM_C

#include "rx32s12xx_tim.h"

static const uint8_t SHIFT_TAB_OISx[] =
{
  0U,            /* 0: OIS1 */
  1U,            /* 1: OIS1N */
  2U,            /* 2: OIS2 */
  3U,            /* 3: OIS2N */
  4U,            /* 4: OIS3 */
  5U,            /* 5: OIS3N */
  6U,            /* 6: OIS4 */
  8U,            /* 7: OIS5 */
  10U            /* 8: OIS6 */
};

const uint8_t OFFSET_TAB_CCMRx[] =
{
  0x00U,   /* 0: TIMx_CH1  */
  0x00U,   /* 1: TIMx_CH1N */
  0x00U,   /* 2: TIMx_CH2  */
  0x00U,   /* 3: TIMx_CH2N */
  0x04U,   /* 4: TIMx_CH3  */
  0x04U,   /* 5: TIMx_CH3N */
  0x04U,   /* 6: TIMx_CH4  */
  0x04U,   /* 7: TIMx_CH4N */
  0x38U,   /* 8: TIMx_CH5  */
  0x38U    /* 9: TIMx_CH6  */
};

 const uint8_t SHIFT_TAB_OCxx[] =
{
  0U,            /* 0: OC1M, OC1FE, OC1PE */
  0U,            /* 1: - NA */
  8U,            /* 2: OC2M, OC2FE, OC2PE */
  0U,            /* 3: - NA */
  0U,            /* 4: OC3M, OC3FE, OC3PE */
  0U,            /* 5: - NA */
  8U,            /* 6: OC4M, OC4FE, OC4PE */
  0U,            /* 7: - NA */
  0U,            /* 8: OC5M, OC5FE, OC5PE */
  8U             /* 9: OC6M, OC6FE, OC6PE */
};

 const uint8_t SHIFT_TAB_CCxP[] =
{
  0U,            /* 0: CC1P */
  2U,            /* 1: CC1NP */
  4U,            /* 2: CC2P */
  6U,            /* 3: CC2NP */
  8U,            /* 4: CC3P */
  10U,           /* 5: CC3NP */
  12U,           /* 6: CC4P */
  14U,           /* 7: CC5P */
  16U            /* 8: CC6P */
};

 const uint8_t SHIFT_TAB_ICxx[] =
{
  0U,            /* 0: CC1S, IC1PSC, IC1F */
  0U,            /* 1: - NA */
  8U,            /* 2: CC2S, IC2PSC, IC2F */
  0U,            /* 3: - NA */
  0U,            /* 4: CC3S, IC3PSC, IC3F */
  0U,            /* 5: - NA */
  8U,            /* 6: CC4S, IC4PSC, IC4F */
  0U,            /* 7: - NA */
  0U             /* 8: - NA */
};

/**
  * @brief  Deinitializes the TIMx peripheral registers to their default reset values.
  * @param  TIMx: where x can be 1-8 to select the TIM peripheral.
  * @retval None

  */
void TIM_DeInit(TIM_TypeDef* TIMx)
{
  TIMx->CR1    = 0x0000;
	TIMx->CR2    = 0x0000;
	TIMx->SMCR   = 0x0000;
	TIMx->DIER   = 0x0000;
	TIMx->SR     = 0x0000;
	TIMx->EGR    = 0x0000;
	TIMx->CCMR1  = 0x0000;
	TIMx->CCMR2  = 0x0000;
	TIMx->CCER   = 0x0000;
	TIMx->CNT    = 0x0000;
	TIMx->PSC    = 0x0000;
	TIMx->ARR    = 0x0000;
	TIMx->RCR    = 0x0000;
	TIMx->CCR1   = 0x0000;
	TIMx->CCR2   = 0x0000;
	TIMx->CCR3   = 0x0000;
	TIMx->CCR4   = 0x0000;
	TIMx->BDTR   = 0x0000;
	TIMx->DCR    = 0x0000;
	TIMx->DMAR   = 0x0000;
	TIMx->CCR5   = 0x0000;
	TIMx->CCMR3  = 0x0000;
	TIMx->CCR6   = 0x0000;
}

/**
  * @brief  Fills each TIM_TimeBaseInitStruct member with its default value.
  * @param  TIM_TimeBaseInitStruct : pointer to a ATIM_TimeBaseInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void TIM_TimeBaseStructInit(TIM_TimeBaseInitTypeDef* TIM_TimeBaseInitStruct)
{
  /* Set the default configuration */
  TIM_TimeBaseInitStruct->Period            = 0xFFFF;
  TIM_TimeBaseInitStruct->Prescaler         = 0x0000;
  TIM_TimeBaseInitStruct->ClockDivision     = 0x0000;
  TIM_TimeBaseInitStruct->CounterMode       = TIM_COUNTERMODE_UP;
  TIM_TimeBaseInitStruct->RepetitionCounter = 0x0000;
}

/**
  * @brief  Fills each TIM_OCInitStruct member with its default value.
  * @param  TIM_OCInitStruct: pointer to a ATIM_OCInitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void TIM_OCStructInit(TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  /* Set the default configuration */
  TIM_OCInitStruct->OCMode       = TIM_OCMODE_FROZEN;
  TIM_OCInitStruct->OutputState  = TIM_OutputState_Disable;
  TIM_OCInitStruct->OutputNState = TIM_OutputNState_Disable;
  TIM_OCInitStruct->Pulse        = 0x00000000;
  TIM_OCInitStruct->OCPolarity   = TIM_OCPolarity_High;
  TIM_OCInitStruct->OCNPolarity  = TIM_OCNPolarity_High;
  TIM_OCInitStruct->OCIdleState  = TIM_OCIdleState_Reset;
  TIM_OCInitStruct->OCNIdleState = TIM_OCNIdleState_Reset;
}

/**
  * @brief  Fills each TIM_ICInitStruct member with its default value.
  * @param  TIM_ICInitStruct: pointer to a ATIM_ICInitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void TIM_ICStructInit(TIM_ICInitTypeDef* TIM_ICInitStruct)
{
  /* Set the default configuration */
  TIM_ICInitStruct->Channel     = TIM_CHANNEL_CH1;
  TIM_ICInitStruct->ICPolarity  = TIM_IC_POLARITY_RISING;
  TIM_ICInitStruct->ICSelection = TIM_ACTIVEINPUT_DIRECTTI;
  TIM_ICInitStruct->ICPrescaler = TIM_ICPSC_DIV1;
  TIM_ICInitStruct->ICFilter    = 0x00;
}

/**
  * @brief  Fills each TIM_BDTRInitStruct member with its default value.
  * @param  TIM_BDTRInitStruct: pointer to a ATIM_BDTRInitTypeDef structure which
  *         will be initialized.
  * @retval None
  */
void TIM_BDTRStructInit(TIM_BDTRInitTypeDef* TIM_BDTRInitStruct)
{
  /* Set the default configuration */
  TIM_BDTRInitStruct->OSSRState       = TIM_OSSR_DISABLE;
  TIM_BDTRInitStruct->OSSIState       = TIM_OSSI_DISABLE;
  TIM_BDTRInitStruct->LOCKLevel       = TIM_BDTR_LOCK_OFF;
  TIM_BDTRInitStruct->DeadTime        = 0x00;
  TIM_BDTRInitStruct->BreakState      = TIM_Break_Enable;
  TIM_BDTRInitStruct->BreakPolarity   = TIM_BREAK_POLARITY_LOW;
  TIM_BDTRInitStruct->AutomaticOutput = TIM_AutomaticOutput_Disable;
}

/**
  * @function  TIM_TimeBaseInit
  * @brief  Initializes the TIMx Time Base Unit peripheral according to 
  *         the specified parameters in the TIM_TimeBaseInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  CounterModeo
  *         @arg @ref TIM_COUNTERMODE_UP
  *         @arg @ref TIM_COUNTERMODE_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP
  *         @arg @ref TIM_COUNTERMODE_CENTER_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP_DOWN
  * @param  ClockDivisiono
  *         @arg @ref TIM_CR1_CKD_1
  *         @arg @ref TIM_CR1_CKD_2
  *         @arg @ref TIM_CR1_CKD_4
  * @param  Period between Min_Data=0 and Max_Data=65535
  * @param  Prescaler between Min_Data=0 and Max_Data=65535
  * @param  RepetitionCounter between Min_Data=0 and Max_Data=255
  * @retval None
  */
void TIM_TimeBaseInit(TIM_TypeDef* TIMx, TIM_TimeBaseInitTypeDef* TIM_TimeBaseInitStruct)
{
  uint32_t tmpcr1 = 0;

  tmpcr1 = TIMx->CR1;

	/* Select the Counter Mode */
	tmpcr1 &= (uint32_t)(~((uint32_t)(TIM_CR1_DIR | TIM_CR1_CMS)));
	tmpcr1 |= (uint32_t)TIM_TimeBaseInitStruct->CounterMode;

	/* Set the clock division */
	tmpcr1 &= (uint32_t)(~((uint32_t)TIM_CR1_CKD));
	tmpcr1 |= (uint32_t)TIM_TimeBaseInitStruct->ClockDivision;

  TIMx->CR1 = tmpcr1;

  /* Set the Autoreload value */
  TIMx->ARR = TIM_TimeBaseInitStruct->Period ;
 
  /* Set the Prescaler value */
  TIMx->PSC = TIM_TimeBaseInitStruct->Prescaler;

  /* Set the Repetition Counter value */
  TIMx->RCR = TIM_TimeBaseInitStruct->RepetitionCounter;

  /* Generate an update event to reload the Prescaler and the Repetition counter
     values immediately */
  TIMx->EGR = 0x0001;           
}



/**
  * @function  TIM_OC1Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode: 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState:
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState:
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity:
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity:
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState:
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState:
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC1Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
 /* Disable the Channel 1: Reset the CC1E Bit */
  TIMx->CCER &= (uint32_t)(~(uint32_t)TIM_CCER_CC1E);
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR1 register value */
  tmpccmrx = TIMx->CCMR1;
    
  /* Reset the Output Compare Mode Bits */
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR1_OC1M));
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR1_CC1S));

  /* Select the Output Compare Mode */
  tmpccmrx |= TIM_OCInitStruct->OCMode;
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC1P));
  /* Set the Output Compare Polarity */
  tmpccer |= TIM_OCInitStruct->OCPolarity;
  
  /* Set the Output State */
  tmpccer |= TIM_OCInitStruct->OutputState;
      
	/* Reset the Output N Polarity level */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC1NP));
	/* Set the Output N Polarity */
	tmpccer |= TIM_OCInitStruct->OCNPolarity;
	
	/* Reset the Output N State */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC1NE));    
	/* Set the Output N State */
	tmpccer |= TIM_OCInitStruct->OutputNState;
	
	/* Reset the Output Compare and Output Compare N IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS1));
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS1N));
	
	/* Set the Output Idle state */
	tmpcr2 |= TIM_OCInitStruct->OCIdleState;
	/* Set the Output N Idle state */
	tmpcr2 |= TIM_OCInitStruct->OCNIdleState;

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR1 */
  TIMx->CCMR1 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR1 = TIM_OCInitStruct->Pulse; 
 
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC2Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode: 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState:
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState:
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity:
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity:
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState:
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState:
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC2Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
   /* Disable the Channel 2: Reset the CC2E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC2E));
  
  /* Get the TIMx CCER register value */  
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR1 register value */
  tmpccmrx = TIMx->CCMR1;
    
  /* Reset the Output Compare mode and Capture/Compare selection Bits */
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR1_OC2M));
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR1_CC2S));
  
  /* Select the Output Compare Mode */
  tmpccmrx |= (uint32_t)(TIM_OCInitStruct->OCMode << 8);
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC2P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 4);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 4);
    
	/* Reset the Output N Polarity level */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC2NP));
	/* Set the Output N Polarity */
	tmpccer |= (uint32_t)(TIM_OCInitStruct->OCNPolarity << 4);
	
	/* Reset the Output N State */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC2NE));    
	/* Set the Output N State */
	tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputNState << 4);
	
	/* Reset the Output Compare and Output Compare N IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS2));
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS2N));
	
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 2);
	/* Set the Output N Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCNIdleState << 2);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR1 */
  TIMx->CCMR1 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR2 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC3Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode: 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState:
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState:
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity:
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity:
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState:
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState:
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC3Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;

  /* Disable the Channel 3: Reset the CC3E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC3E));
  
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR2 register value */
  tmpccmrx = TIMx->CCMR2;
    
  /* Reset the Output Compare mode and Capture/Compare selection Bits */
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR2_OC3M));
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR2_CC3S));  
  /* Select the Output Compare Mode */
  tmpccmrx |= TIM_OCInitStruct->OCMode;
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC3P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 8);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 8);
    
	/* Reset the Output N Polarity level */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC3NP));
	/* Set the Output N Polarity */
	tmpccer |= (uint32_t)(TIM_OCInitStruct->OCNPolarity << 8);
	/* Reset the Output N State */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC3NE));
	
	/* Set the Output N State */
	tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputNState << 8);
	/* Reset the Output Compare and Output Compare N IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS3));
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS3N));
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 4);
	/* Set the Output N Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCNIdleState << 4);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR2 */
  TIMx->CCMR2 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR3 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC4Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode: 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState:
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState:
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity:
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity:
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState:
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState:
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC4Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
  /* Disable the Channel 4: Reset the CC4E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC4E));
  
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR2 register value */
  tmpccmrx = TIMx->CCMR2;
    
  /* Reset the Output Compare mode and Capture/Compare selection Bits */
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR2_OC4M));
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR2_CC4S));
  
  /* Select the Output Compare Mode */
  tmpccmrx |= (uint32_t)(TIM_OCInitStruct->OCMode << 8);
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC4P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 12);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 12);
    
	/* Reset the Output Compare IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS4));
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 6);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR2 */  
  TIMx->CCMR2 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR4 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC5Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode: 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState:
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState:
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity:
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity:
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState:
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset		
	* @param  OCNIdleState:
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC5Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
  /* Disable the Channel 5: Reset the CC5E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC5E));
  
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR2 register value */
  tmpccmrx = TIMx->CCMR3;
    
  /* Reset the Output Compare mode and Capture*/
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR3_OC5M));
  
  /* Select the Output Compare Mode */
  tmpccmrx |= (uint32_t)(TIM_OCInitStruct->OCMode);
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC5P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 14);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 14);
    
	/* Reset the Output Compare IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS5));
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 7);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR2 */  
  TIMx->CCMR3 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR5 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC6Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode: 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState:
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState:
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity:
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity:
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState:
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState:
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC6Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
  /* Disable the Channel 5: Reset the CC5E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC6E));
  
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR3 register value */
  tmpccmrx = TIMx->CCMR3;
    
  /* Reset the Output Compare mode and Capture*/
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR3_OC6M));
  
  /* Select the Output Compare Mode */
  tmpccmrx |= (uint32_t)(TIM_OCInitStruct->OCMode << 8);
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC6P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 16);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 16);
    
	/* Reset the Output Compare IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS6));
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 8);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR3 */  
  TIMx->CCMR3 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR6 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_ICInit
  * @brief  Initializes the TIM peripheral according to the specified
  *         parameters in the TIM_ICInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel 
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICPolarity
  *         @arg @ref TIM_IC_POLARITY_RISING
  *         @arg @ref TIM_IC_POLARITY_FALLING
  * @param  ICSelection
  *         @arg @ref TIM_ACTIVEINPUT_DIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_INDIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_TRC
  * @param  ICPrescaler
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @param  ICFilter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8	
  * @retval None
  */
void TIM_ICInit(TIM_TypeDef* TIMx, TIM_ICInitTypeDef* TIM_ICInitStruct)
{
  if (TIM_ICInitStruct->Channel == TIM_CHANNEL_CH1)
  {
    /* TI1 Configuration */
    TIM_Set_TI1(TIMx, TIM_ICInitStruct);
  }
  else if (TIM_ICInitStruct->Channel == TIM_CHANNEL_CH2)
  {
    /* TI2 Configuration */
    TIM_Set_TI2(TIMx, TIM_ICInitStruct);
  }
  else if (TIM_ICInitStruct->Channel == TIM_CHANNEL_CH3)
  {
    /* TI3 Configuration */
    TIM_Set_TI3(TIMx,  TIM_ICInitStruct);
  }
  else if (TIM_ICInitStruct->Channel == TIM_CHANNEL_CH4)
  {
    /* TI4 Configuration */
    TIM_Set_TI4(TIMx, TIM_ICInitStruct);
  }
}

/**
  * @function  TIM_BDTRInit
  * @brief  Configures the: Break feature, dead time, Lock level, the OSSI,
  *         the OSSR State and the AOE(automatic output enable).
  * @param  TIMx:
            @arg TIM Instance
  * @param  OSSRState 
  *         @arg @ref TIM_OSSR_ENABLE
  *         @arg @ref TIM_OSSR_DISABLE
  * @param  OSSIState
  *         @arg @ref TIM_OSSI_ENABLE
  *         @arg @ref TIM_OSSI_DISABLE
  * @param  LOCKLevel
  *         @arg @ref TIM_BDTR_LOCK_OFF
  *         @arg @ref TIM_BDTR_LOCK_1
  *         @arg @ref TIM_BDTR_LOCK_2
  *         @arg @ref TIM_BDTR_LOCK_3
  * @param  DeadTime between 0x00 and 0xFF
  * @param  BreakState
  *         @arg @ref TIM_Break_Enable
  *         @arg @ref TIM_Break_Disable	
  * @param  BreakPolarity
  *         @arg @ref TIM_BREAK_POLARITY_HIGH
  *         @arg @ref TIM_BREAK_POLARITY_LOW	
  * @param  CMP1_BreakState
  *         @arg @ref TIM_CMP1_Break_Enable
  *         @arg @ref TIM_CMP1_Break_Disable	
  * @param  CMP1_BreakPolarity
  *         @arg @ref TIM_CMP1OEP_POLARITY_HIGH
  *         @arg @ref TIM_CMP1OEP_POLARITY_LOW	
  * @param  CMP2_BreakState
  *         @arg @ref TIM_CMP2_Break_Enable
  *         @arg @ref TIM_CMP2_Break_Disable	
  * @param  CMP2_BreakPolarity
  *         @arg @ref TIM_CMP2OEP_POLARITY_HIGH
  *         @arg @ref TIM_CMP2OEP_POLARITY_LOW
  * @param  CMP3_BreakState
  *         @arg @ref TIM_CMP3_Break_Enable
  *         @arg @ref TIM_CMP3_Break_Disable	
  * @param  CMP3_BreakPolarity
  *         @arg @ref TIM_CMP3OEP_POLARITY_HIGH
  *         @arg @ref TIM_CMP3OEP_POLARITY_LOW
  * @param  AutomaticOutput
  *         @arg @ref TIM_AutomaticOutput_Enable
  *         @arg @ref TIM_AutomaticOutput_Disable	
  * @retval None
  */
void TIM_BDTRInit(TIM_TypeDef* TIMx, TIM_BDTRInitTypeDef *TIM_BDTRInitStruct)
{
  /* Set the Lock level, the Break enable Bit and the Ploarity, the OSSR State,
     the OSSI State, the dead time value and the Automatic Output Enable Bit */
  TIMx->BDTR = (uint32_t)TIM_BDTRInitStruct->OSSRState | TIM_BDTRInitStruct->OSSIState |
             TIM_BDTRInitStruct->LOCKLevel | TIM_BDTRInitStruct->DeadTime |
             TIM_BDTRInitStruct->BreakState | TIM_BDTRInitStruct->BreakPolarity |
             TIM_BDTRInitStruct->AutomaticOutput | TIM_BDTRInitStruct->CMP1_BreakPolarity |
             TIM_BDTRInitStruct->CMP1_BreakState | TIM_BDTRInitStruct->CMP2_BreakPolarity |
             TIM_BDTRInitStruct->CMP2_BreakState | TIM_BDTRInitStruct->CMP3_BreakPolarity |
             TIM_BDTRInitStruct->CMP3_BreakState | TIM_BDTRInitStruct->HDFE | TIM_BDTRInitStruct->PVD;
}

/**
  * @function  TIM_ENCODERInit
  * @brief  Initializes the encoder interface of the TIMx 
  * @param  TIMx:
            @arg TIM Instance
  * @param  EncoderMode 
  *         @arg @ref TIM_ENCODERMODE_X2_TI1
  *         @arg @ref TIM_ENCODERMODE_X2_TI2
  *         @arg @ref TIM_ENCODERMODE_X4_TI12
  * @param  IC1Polarity
  *         @arg @ref TIM_IC_POLARITY_RISING
  *         @arg @ref TIM_IC_POLARITY_FALLING
  * @param  IC1Selection
  *         @arg @ref TIM_ACTIVEINPUT_DIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_INDIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_TRC
  * @param  IC1Prescaler
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @param  IC1Filter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8	
  * @param  IC2Polarity
  *         @arg @ref TIM_IC_POLARITY_RISING
  *         @arg @ref TIM_IC_POLARITY_FALLING
  * @param  IC2Selection
  *         @arg @ref TIM_ACTIVEINPUT_DIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_INDIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_TRC
  * @param  IC2Prescaler
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @param  IC2Filter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8		
  * @retval None
  */
	
void TIM_ENCODERInit(TIM_TypeDef *TIMx, TIM_ENCODERInitTypeDef *TIM_ENCODERInitStruct)
{
  /* Configure TI1 */
	TIM_Set_IC_ActiveInput(TIMx,TIM_CHANNEL_CH1,TIM_ENCODERInitStruct->IC1Selection);
	TIM_Set_IC_Filter(TIMx,TIM_CHANNEL_CH1,TIM_ENCODERInitStruct->IC1Filter);	
	TIM_Set_IC_Prescaler(TIMx,TIM_CHANNEL_CH1,TIM_ENCODERInitStruct->IC1Prescaler);	
	
	
  /* Configure TI2 */
	TIM_Set_IC_ActiveInput(TIMx,TIM_CHANNEL_CH2,TIM_ENCODERInitStruct->IC2Selection);
	TIM_Set_IC_Filter(TIMx,TIM_CHANNEL_CH2,TIM_ENCODERInitStruct->IC2Filter);	
	TIM_Set_IC_Prescaler(TIMx,TIM_CHANNEL_CH2,TIM_ENCODERInitStruct->IC2Prescaler);	

  /* Set TI1 and TI2 polarity and enable TI1 and TI2 */
	TIM_Set_IC_Polarity(TIMx,TIM_CHANNEL_CH1,TIM_ENCODERInitStruct->IC1Polarity);		
	TIM_Set_IC_Polarity(TIMx,TIM_CHANNEL_CH2,TIM_ENCODERInitStruct->IC2Polarity);		
  TIM_Enable_CC_Channel(TIMx,TIM_CHANNEL_CH1);
  TIM_Enable_CC_Channel(TIMx,TIM_CHANNEL_CH2);	
	
  /* Set encoder mode */
  TIM_Set_EncoderMode(TIMx, TIM_ENCODERInitStruct->EncoderMode);

}

/**
  * @function  TIM_HALLSENSORInit
  * @brief  Initializes the encoder interface of the TIMx 
  * @param  TIMx:
            @arg TIM Instance
  * @param  IC1Polarity
  *         @arg @ref TIM_IC_POLARITY_RISING
  *         @arg @ref TIM_IC_POLARITY_FALLING
  * @param  IC1Prescaler
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @param  IC1Filter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8	
  * @param  CommutationDelay between Min_Data = 0x0000 and Max_Data = 0xFFFF
  * @retval None
  */
	
 void TIM_HALLSENSORInit(TIM_TypeDef *TIMx, TIM_HALLSENSORInitTypeDef *TIM_HALLSENSORInitStruct)
{
  uint32_t tmpcr2;
  uint32_t tmpccmr1;
  uint32_t tmpccer;
  uint32_t tmpsmcr;
	
	/* Disable the CC1 and CC2: Reset the CC1E and CC2E Bits */
  TIMx->CCER &= (uint32_t)~(TIM_CCER_CC1E | TIM_CCER_CC2E);

  /* Get the TIMx CR2 register value */
  tmpcr2 = TIMx->CR2;

  /* Get the TIMx CCMR1 register value */
  tmpccmr1 = TIMx->CCMR1;

  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;

  /* Get the TIMx SMCR register value */
  tmpsmcr = TIMx->SMCR;

  /* Connect TIMx_CH1, CH2 and CH3 pins to the TI1 input */
  tmpcr2 |= TIM_CR2_TI1S;

  /* OC2REF signal is used as trigger output (TRGO) */
  tmpcr2 |= TIM_CR2_MMS_TRGO_OC2REF;

  /* Configure the slave mode controller */
  tmpsmcr &= (uint32_t)~(TIM_SMCR_TS | TIM_SMCR_SMS);
  tmpsmcr |= TIM_SMCR_TS_TI1F_ED;
  tmpsmcr |= TIM_SMCR_SMS_RESET;

  /* Configure input channel 1 */
  tmpccmr1 &= (uint32_t)~(TIM_CCMR1_CC1S | TIM_CCMR1_IC1F  | TIM_CCMR1_IC1PSC);
  tmpccmr1 |= (uint32_t)(TIM_ACTIVEINPUT_TRC >> 16U);
  tmpccmr1 |= (uint32_t)(TIM_HALLSENSORInitStruct->IC1Filter >> 16U);
  tmpccmr1 |= (uint32_t)(TIM_HALLSENSORInitStruct->IC1Prescaler >> 16U);

  /* Configure input channel 2 */
  tmpccmr1 &= (uint32_t)~(TIM_CCMR1_OC2M | TIM_CCMR1_OC2FE  | TIM_CCMR1_OC2PE  | TIM_CCMR1_OC2CE);
  tmpccmr1 |= (uint32_t)(TIM_OCMODE_PWM2 << 8U);

  /* Set Channel 1 polarity and enable Channel 1 and Channel2 */
  tmpccer &= (uint32_t)~(TIM_CCER_CC1P | TIM_CCER_CC1NP | TIM_CCER_CC2P | TIM_CCER_CC2NP);
  tmpccer |= (uint32_t)(TIM_HALLSENSORInitStruct->IC1Polarity);
  tmpccer |= (uint32_t)(TIM_CCER_CC1E | TIM_CCER_CC2E);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;

  /* Write to TIMx SMCR */
  TIMx->SMCR = tmpsmcr;

  /* Write to TIMx CCMR1 */
  TIMx->CCMR1 = tmpccmr1;

  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;

  /* Write to TIMx CCR2 */
  TIM_Set_OC_CompareCH2(TIMx, TIM_HALLSENSORInitStruct->CommutationDelay);
	
}



/**
  * @function  TIM_Enable_CEN
  * @brief  Enable counter of the selected TIMx
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CEN(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_CEN);
}

/**
  * @function  TIM_Disable_CEN
  * @brief  Disable counter of the selected TIMx
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CEN(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR1, TIM_CR1_CEN);
}

/**
  * @function  TIM_Get_CENStatus
  * @brief  Indicates whether the timer counter is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0:CEN counter is disalbe, 1:CEN counter is enable
  */
 uint32_t TIM_Get_CENStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->CR1, TIM_CR1_CEN) == (TIM_CR1_CEN));
}

/**
  * @function  TIM_Enable_UpdateEvent
  * @brief  Enable update event generation
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_UpdateEvent(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR1, TIM_CR1_UDIS);
}

/**
  * @function  TIM_Disable_UpdateEvent
  * @brief  Disable update event generation
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_UpdateEvent(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_UDIS);
}

/**
  * @function  TIM_Get_UpdateEventStatus
  * @brief  Indicates whether update event generation is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0:UpdateEvent is disalbe, 1:UpdateEvent is enable
  */
 uint32_t TIM_Get_UpdateEventStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->CR1, TIM_CR1_UDIS) == TIM_CR1_UDIS);
}

/**
  * @function  TIM_Set_UpdateSource
  * @brief  Set update event sorce
  * @param  TIMx:
            @arg TIM Instance
  * @param  UpdateSource This parameter can be one of the following values:
  *         @arg @ref TIM_UPDATESOURCE_REGULAR
  *         @arg @ref TIM_UPDATESOURCE_COUNTER
  * @retval None
  */
 void TIM_Set_UpdateSource(TIM_TypeDef *TIMx,uint32_t UpdateSource)
{
  MODIFY_REG(TIMx->CR1, TIM_CR1_URS, UpdateSource);
}

/**
  * @function  TIM_Get_UpdateSource
  * @brief  Get actual event update source
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_UPDATESOURCE_REGULAR
  *         @arg @ref TIM_UPDATESOURCE_COUNTER
  */
 uint32_t TIM_Get_UpdateSource(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1,TIM_CR1_URS));
}

/**
  * @function  TIM_Set_OnePulseMode
  * @brief  Set one pulse mode
  * @param  TIMx:
            @arg TIM Instance
  * @param  OnePulseMode This parameter can be one of the following values:
  *         @arg @ref TIM_ONEPULSEMODE_SINGLE
  *         @arg @ref TIM_ONEPULSEMODE_REPETITIVE
  * @retval None
  */
 void TIM_Set_OnePulseMode(TIM_TypeDef *TIMx,uint32_t OnePulseMode)
{
  MODIFY_REG(TIMx->CR1,TIM_CR1_OPM, OnePulseMode);
}
 
/**
  * @function  TIM_Get_OnePulseMode
  * @brief  Get one pulse mode
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_ONEPULSEMODE_SINGLE
  *         @arg @ref TIM_ONEPULSEMODE_REPETITIVE
  */
 uint32_t TIM_Get_OnePulseMode(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1, TIM_CR1_OPM));
}

/**
  * @function  TIM_Set_CounterMode
  * @brief  Set TIM8 counter counting mode
  * @param  TIMx:
            @arg TIM Instance
  * @param  CounterMode This parameter can be one of the following values:
  *         @arg @ref TIM_COUNTERMODE_UP
  *         @arg @ref TIM_COUNTERMODE_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP
  *         @arg @ref TIM_COUNTERMODE_CENTER_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP_DOWN
  * @retval None
  */
 void TIM_Set_CounterMode(TIM_TypeDef *TIMx,uint32_t CounterMode)
{
  MODIFY_REG(TIMx->CR1, TIM_CR1_DIR|TIM_CR1_CMS, CounterMode);
}

/**
  * @function  TIM_Get_CounterMode
  * @brief  Get actual counter mode
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_COUNTERMODE_UP
  *         @arg @ref TIM_COUNTERMODE_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP
  *         @arg @ref TIM_COUNTERMODE_CENTER_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP_DOWN
  */
 uint32_t TIM_Get_CounterMode(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1, TIM_CR1_DIR|TIM_CR1_CMS));
}

/**
  * @function  TIM_Get_CounterDirection
  * @brief  Get the current direction of the counter
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_COUNTERMODE_UP
  *         @arg @ref TIM_COUNTERMODE_DOWN
  */
 uint32_t TIM_Get_CounterDirection(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1, TIM_CR1_DIR));
}

/**
  * @function  TIM_Enable_ARRPreload
  * @brief  Enable auto-reload (ARR) preload
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_ARRPreload(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_ARPE);
}


/**
  * @function  TIM_Disable_ARRPreload
  * @brief  Disable auto-reload (ARR) preload
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_ARRPreload(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR1, TIM_CR1_ARPE);
}

/**
  * @function  TIM_Get_ARRPreloadStatus
  * @brief  Indicates whether auto-reload (ARR) preload is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0:auto-reload (ARR) preload is disalbe, 1:auto-reload (ARR) preload is enable
  */
 uint32_t TIM_Get_ARRPreloadStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->CR1, TIM_CR1_ARPE) == TIM_CR1_ARPE);
}

/**
  * @function  TIM_Set_ClockDivision
  * @brief  Set the division ratio between the timer clock  and the sampling clock used by the dead-time generators (when supported) and the digital filters.
  * @param  TIMx:
            @arg TIM Instance
  * @param  ClockDivision This parameter can be one of the following values:
  *         @arg @ref TIM_CR1_CKD_1
  *         @arg @ref TIM_CR1_CKD_2
  *         @arg @ref TIM_CR1_CKD_4
  * @retval None
  */
 void TIM_Set_ClockDivision(TIM_TypeDef *TIMx,uint32_t ClockDivision)
{
  MODIFY_REG(TIMx->CR1, TIM_CR1_CKD, ClockDivision);
}

/**
  * @function  TIM_Get_ClockDivision
  * @brief  Get the actual division ratio between the timer clock  and the sampling clock used by the dead-time generators (when supported) and the digital filters.
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_CR1_CKD_1
  *         @arg @ref TIM_CR1_CKD_2
  *         @arg @ref TIM_CR1_CKD_4
  */
 uint32_t TIM_Get_ClockDivision(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1, TIM_CR1_CKD));
}

/**
  * @function  TIM_Enable_CC_Preload
  * @brief  Enable  the capture/compare control bits (CCxE, CCxNE and OCxM) preload
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CC_Preload(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR2, TIM_CR2_CCPC);
}

/**
  * @function  TIM_Disable_CC_Preload
  * @brief  Disable  the capture/compare control bits (CCxE, CCxNE and OCxM) preload
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CC_Preload(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR2,TIM_CR2_CCPC);
}

/**
  * @function  TIM_Get_CC_Preload
  * @brief  Indicates whether the capture/compare control bits (CCxE, CCxNE and OCxM) preload.
  * @param  TIMx:
            @arg TIM Instance
  * @retval State of bit (1 or 0)
  */
 uint32_t TIM_Get_CC_Preload(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR2, TIM_CR2_CCPC) == (TIM_CR2_CCPC));
}

/**
  * @function  TIM_Set_CC_UpdateSource
  * @brief  Set the updated source of the capture/compare control bits (CCxE, CCxNE and OCxM)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CCUpdateSource This parameter can be one of the following values:
  *         @arg @ref TIM_CCUPDATESOURCE_COMG_ONLY
  *         @arg @ref TIM_CCUPDATESOURCE_COMG_AND_TRGI
  * @retval None
  */
 void TIM_Set_CC_UpdateSource(TIM_TypeDef *TIMx,uint32_t CCUpdateSource)
{
  MODIFY_REG(TIMx->CR2,TIM_CR2_CCUS, CCUpdateSource);
}

/**
  * @function  TIM_Get_CC_UpdateSource
  * @brief  Get the updated source of the capture/compare control
  * @param  TIMx:
            @arg TIM Instance
  * @retval 
  *         @arg @ref TIM_CCUPDATESOURCE_COMG_ONLY
  *         @arg @ref TIM_CCUPDATESOURCE_COMG_AND_TRGI
  */
 uint32_t TIM_Get_CC_UpdateSource(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR2, TIM_CR2_CCUS));
}

/**
  * @function  TIM_Set_CC_DMASource
  * @brief  Set the trigger of the capture/compare DMA request
  * @param  TIMx:
            @arg TIM Instance
  * @param  CC DMA Selection Source This parameter can be one of the following values:
  *         @arg @ref TIM_CCDMASOURCE_CC_EVENT
  *         @arg @ref TIM_CCDMASOURCE_UPDATE_EVENT
  * @retval None
  */
 void TIM_Set_CC_DMASource(TIM_TypeDef *TIMx,uint32_t CCDMASource)
{
  MODIFY_REG(TIMx->CR2,TIM_CR2_CCDS, CCDMASource);
}

/**
  * @function  TIM_Get_CC_DMASource
  * @brief  Get the the trigger of the capture/compare DMA request
  * @param  TIMx:
            @arg TIM Instance
  * @retval 
  *         @arg @ref TIM_CCDMASOURCE_CC_EVENT
  *         @arg @ref TIM_CCDMASOURCE_UPDATE_EVENT
  */
 uint32_t TIM_Get_CC_DMASource(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR2, TIM_CR2_CCDS));
}

/**
  * @function  TIM_Set_TriggerOutput
  * @brief  Set the trigger output (TRGO) used for timer synchronization
  * @param  TIMx:
            @arg TIM Instance
  * @param  TimerSynchronization This parameter can be one of the following values:
  *         @arg @ref TIM_CR2_MMS_TRGO_RESET
  *         @arg @ref TIM_CR2_MMS_TRGO_ENABLE
  *         @arg @ref TIM_CR2_MMS_TRGO_UPDATE
  *         @arg @ref TIM_CR2_MMS_TRGO_CC1IF
  *         @arg @ref TIM_CR2_MMS_TRGO_OC1REF
  *         @arg @ref TIM_CR2_MMS_TRGO_OC2REF
  *         @arg @ref TIM_CR2_MMS_TRGO_OC3REF
  *         @arg @ref TIM_CR2_MMS_TRGO_OC4REF
  * @retval None
  */
 void TIM_Set_TriggerOutput(TIM_TypeDef *TIMx, uint32_t TriggerInput)
{
  MODIFY_REG(TIMx->CR2, TIM_CR2_MMS, TriggerInput);
}

/**
  * @function  TIM_Enable_IC_XORCombination
  * @brief  Connect the TIM_CH1, CH2 and CH3 pins  to the TI1 input (XOR combination)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_IC_XORCombination(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR2, (TIM_CR2_TI1S));
}

/**
  * @function  TIM_Disable_IC_XORCombination
  * @brief  Disconnect the TIM_CH1, CH2 and CH3 pins  from the TI1 input
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_IC_XORCombination(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR2, TIM_CR2_TI1S);
}

/**
  * @function  TIM_Set_IC_XORCombination
  * @brief  Indicates whether the TIM_CH1, CH2 and CH3 pins are connectected to the TI1 input.
  * @param  TIMx:
            @arg TIM Instance
  * @retval State of bit (1 or 0).
  */
 uint32_t TIM_Get_IC_XORCombination(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR2, TIM_CR2_TI1S) == (TIM_CR2_TI1S));
}

/**
  * @brief  Set the trigger output 2 (TRGO2) used for ADC synchronization .
  * @note Macro IS_TIM_TRGO2_INSTANCE(TIMx) can be used to check
  *       whether or not a timer instance can be used for ADC synchronization.
  * @rmtoll CR2          MMS2          LL_TIM_SetTriggerOutput2
  * @param  TIMx Timer Instance
  * @param  ADCSynchronization This parameter can be one of the following values:
  *         @arg @ref TIM_TRGO2_RESET
  *         @arg @ref TIM_TRGO2_ENABLE
  *         @arg @ref TIM_TRGO2_UPDATE
  *         @arg @ref TIM_TRGO2_CC1F
  *         @arg @ref TIM_TRGO2_OC1
  *         @arg @ref TIM_TRGO2_OC2
  *         @arg @ref TIM_TRGO2_OC3
  *         @arg @ref TIM_TRGO2_OC4
  *         @arg @ref TIM_TRGO2_OC5
  *         @arg @ref TIM_TRGO2_OC6
  *         @arg @ref TIM_TRGO2_OC4_RISINGFALLING
  *         @arg @ref TIM_TRGO2_OC6_RISINGFALLING
  *         @arg @ref TIM_TRGO2_OC4_RISING_OC6_RISING
  *         @arg @ref TIM_TRGO2_OC4_RISING_OC6_FALLING
  *         @arg @ref TIM_TRGO2_OC5_RISING_OC6_RISING
  *         @arg @ref TIM_TRGO2_OC5_RISING_OC6_FALLING
  * @retval None
  */
void TIM_Set_TriggerOutput2(TIM_TypeDef *TIMx, uint32_t ADCSynchronization)
{
  MODIFY_REG(TIMx->CR2, TIM_CR2_MMS2, ADCSynchronization);
}

/**
  * @function  TIM_Set_OC_IdleState
  * @brief  Set the IDLE state of an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @param  IdleState This parameter can be one of the following values:
  *         @arg @ref TIM_OCIDLESTATE_LOW
  *         @arg @ref TIM_OCIDLESTATE_HIGH
  * @retval None
  */
 void TIM_Set_OC_IdleState(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t IdleState)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  MODIFY_REG(TIMx->CR2, (TIM_CR2_OIS1<< SHIFT_TAB_OISx[iChannel]),  IdleState << SHIFT_TAB_OISx[iChannel]);
}

/**
  * @function  TIM_Get_OC_IdleState
  * @brief  Get the IDLE state of an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_OCIDLESTATE_LOW
  *         @arg @ref TIM_OCIDLESTATE_HIGH
  */
 uint32_t TIM_Get_OC_IdleState(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  return (READ_BIT(TIMx->CR2, (TIM_CR2_OIS1<< SHIFT_TAB_OISx[iChannel])) >> SHIFT_TAB_OISx[iChannel]);
}

/**
  * @function  TIM_Set_SlaveMode
  * @brief  Set the synchronization mode of a slave timer
  * @param  TIMx:
            @arg TIM Instance
  * @param  SlaveMode This parameter can be one of the following values:
  *         @arg @ref TIM_SMCR_SMS_DISABLED
  *         @arg @ref TIM_SMCR_SMS_ENCODER1
  *         @arg @ref TIM_SMCR_SMS_ENCODER2
  *         @arg @ref TIM_SMCR_SMS_ENCODER3
  *         @arg @ref TIM_SMCR_SMS_RESET
  *         @arg @ref TIM_SMCR_SMS_GATEDRESET
  *         @arg @ref TIM_SMCR_SMS_TRIGGER
  *         @arg @ref TIM_SMCR_SMS_EXTCLOCK1
  *         @arg @ref TIM_SMCR_SMS_COMBINED_RESETTRIGGER
  *         @arg @ref TIM_SMCR_SMS_COMBINED_GATEDRESET
  * @retval None
  */
 void TIM_Set_SlaveMode(TIM_TypeDef *TIMx, uint32_t SlaveMode)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_SMS, SlaveMode);
}

/**
  * @function  TIM_Set_TriggerInput
  * @brief  Set the selects the trigger input to be used to synchronize the counter
  * @param  TIMx:
            @arg TIM Instance
  * @param  TriggerInput This parameter can be one of the following values:
  *         @arg @ref TIM_SMCR_TS_ITR0
  *         @arg @ref TIM_SMCR_TS_ITR1
  *         @arg @ref TIM_SMCR_TS_ITR2
  *         @arg @ref TIM_SMCR_TS_ITR3
  *         @arg @ref TIM_SMCR_TS_TI1F_ED
  *         @arg @ref TIM_SMCR_TS_TI1FP1
  *         @arg @ref TIM_SMCR_TS_TI2FP2
  *         @arg @ref TIM_SMCR_TS_ETRF
  *         @arg @ref TIM_SMCR_TS_ITR4  
  *         @arg @ref TIM_SMCR_TS_ITR5  
  *         @arg @ref TIM_SMCR_TS_ITR6 
  *         @arg @ref TIM_SMCR_TS_ITR7   
  *         @arg @ref TIM_SMCR_TS_ITR8 
  *         @arg @ref TIM_SMCR_TS_ITR9 
  *         @arg @ref TIM_SMCR_TS_ITR10
  * @retval None
  */
 void TIM_Set_TriggerInput(TIM_TypeDef *TIMx, uint32_t TriggerInput)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_TS, TriggerInput);
}

/**
  * @function  TIM_Enable_MasterSlaveMode
  * @brief  Enable the Master/Slave mode
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_MasterSlaveMode(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->SMCR, TIM_SMCR_MSM);
}

/**
  * @function  TIM_Disable_MasterSlaveMode
  * @brief  Disable the Master/Slave mode
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_MasterSlaveMode(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->SMCR, TIM_SMCR_MSM);
}

/**
  * @function  TIM_Disable_MasterSlaveMode
  * @brief  Whether or not a timer instance can operate as a slave timer
  * @param  TIMx:
            @arg TIM Instance
  * @retval State of bit (1 or 0).
  */
 uint32_t TIM_Get_MasterSlaveModeStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->SMCR, TIM_SMCR_MSM) == TIM_SMCR_MSM);
}

/**
  * @function  TIM_Set_ETR
  * @brief  Configure the external trigger (ETR) input
  * @param  TIMx:
            @arg TIM Instance
  * @param  ETRPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_ETR_POLARITY_NONINVERTED
  *         @arg @ref TIM_ETR_POLARITY_INVERTED
  * @param  ETRPrescaler This parameter can be one of the following values:
  *         @arg @ref TIM_SMCR_ETPS_DIV1
  *         @arg @ref TIM_SMCR_ETPS_DIV2
  *         @arg @ref TIM_SMCR_ETPS_DIV4
  *         @arg @ref TIM_SMCR_ETPS_DIV8
  * @param  ETRFilter This parameter can be one of the following values:
  *         @arg @ref TIM_SMCR_ETF_FDIV1
  *         @arg @ref TIM_SMCR_ETF_FDIV1_N2
  *         @arg @ref TIM_SMCR_ETF_FDIV1_N4
  *         @arg @ref TIM_SMCR_ETF_FDIV1_N8
  *         @arg @ref TIM_SMCR_ETF_FDIV2_N6
  *         @arg @ref TIM_SMCR_ETF_FDIV2_N8
  *         @arg @ref TIM_SMCR_ETF_FDIV4_N6
  *         @arg @ref TIM_SMCR_ETF_FDIV4_N8
  *         @arg @ref TIM_SMCR_ETF_FDIV8_N6
  *         @arg @ref TIM_SMCR_ETF_FDIV8_N8
  *         @arg @ref TIM_SMCR_ETF_FDIV16_N5
  *         @arg @ref TIM_SMCR_ETF_FDIV16_N6
  *         @arg @ref TIM_SMCR_ETF_FDIV16_N8
  *         @arg @ref TIM_SMCR_ETF_FDIV32_N5
  *         @arg @ref TIM_SMCR_ETF_FDIV32_N6
  *         @arg @ref TIM_SMCR_ETF_FDIV32_N8
  * @retval None
  */
 void TIM_Set_ETR(TIM_TypeDef *TIMx,uint32_t ETRPolarity, uint32_t ETRPrescaler,uint32_t ETRFilter)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_ETP | TIM_SMCR_ETPS | TIM_SMCR_ETF, ETRPolarity | ETRPrescaler | ETRFilter);
}


/**
  * @brief  Select the external trigger (ETR) input source.
  * @note Macro IS_TIM_ETRSEL_INSTANCE(TIMx) can be used to check whether or
  *       not a timer instance supports ETR source selection.
  * @rmtoll AF1          ETRSEL        TIM_SetETRSource
  * @param  TIMx Timer instance
  * @param  ETRSource This parameter can be one of the following values:
  *
  *         TIM1: any combination of ETR_RMP where
  *
  *            @arg @ref TIM_TIM1_ETRSOURCE_GPIO
  *            @arg @ref TIM_TIM1_ETRSOURCE_COMP1
  *            @arg @ref TIM_TIM1_ETRSOURCE_COMP2
  *            @arg @ref TIM_TIM1_ETRSOURCE_COMP3
  *            @arg @ref TIM_TIM1_ETRSOURCE_COMP4
  *            @arg @ref TIM_TIM1_ETRSOURCE_COMP5       (*)
  *            @arg @ref TIM_TIM1_ETRSOURCE_COMP6       (*)
  *            @arg @ref TIM_TIM1_ETRSOURCE_COMP7       (*)
  *            @arg @ref TIM_TIM1_ETRSOURCE_ADC1_AWD1
  *            @arg @ref TIM_TIM1_ETRSOURCE_ADC1_AWD2
  *            @arg @ref TIM_TIM1_ETRSOURCE_ADC1_AWD3
  *            @arg @ref TIM_TIM1_ETRSOURCE_ADC4_AWD1   (*)
  *            @arg @ref TIM_TIM1_ETRSOURCE_ADC4_AWD2   (*)
  *            @arg @ref TIM_TIM1_ETRSOURCE_ADC4_AWD3   (*)
  *
  *         TIM2: any combination of ETR_RMP where
  *
  *            @arg @ref TIM_TIM2_ETRSOURCE_GPIO
  *            @arg @ref TIM_TIM2_ETRSOURCE_COMP1
  *            @arg @ref TIM_TIM2_ETRSOURCE_COMP2
  *            @arg @ref TIM_TIM2_ETRSOURCE_COMP3
  *            @arg @ref TIM_TIM2_ETRSOURCE_COMP4
  *            @arg @ref TIM_TIM2_ETRSOURCE_COMP5       (*)
  *            @arg @ref TIM_TIM2_ETRSOURCE_COMP6       (*)
  *            @arg @ref TIM_TIM2_ETRSOURCE_COMP7       (*)
  *            @arg @ref TIM_TIM2_ETRSOURCE_TIM3_ETR
  *            @arg @ref TIM_TIM2_ETRSOURCE_TIM4_ETR
  *            @arg @ref TIM_TIM2_ETRSOURCE_TIM5_ETR    (*)
  *            @arg @ref TIM_TIM2_ETRSOURCE_LSE
  *
  *         TIM3: any combination of ETR_RMP where
  *
  *            @arg @ref TIM_TIM3_ETRSOURCE_GPIO
  *            @arg @ref TIM_TIM3_ETRSOURCE_COMP1
  *            @arg @ref TIM_TIM3_ETRSOURCE_COMP2
  *            @arg @ref TIM_TIM3_ETRSOURCE_COMP3
  *            @arg @ref TIM_TIM3_ETRSOURCE_COMP4
  *            @arg @ref TIM_TIM3_ETRSOURCE_COMP5       (*)
  *            @arg @ref TIM_TIM3_ETRSOURCE_COMP6       (*)
  *            @arg @ref TIM_TIM3_ETRSOURCE_COMP7       (*)
  *            @arg @ref TIM_TIM3_ETRSOURCE_TIM2_ETR
  *            @arg @ref TIM_TIM3_ETRSOURCE_TIM4_ETR
  *            @arg @ref TIM_TIM3_ETRSOURCE_ADC2_AWD1
  *            @arg @ref TIM_TIM3_ETRSOURCE_ADC2_AWD2
  *            @arg @ref TIM_TIM3_ETRSOURCE_ADC2_AWD3
  *
  *         TIM4: any combination of ETR_RMP where
  *
  *            @arg @ref TIM_TIM4_ETRSOURCE_GPIO
  *            @arg @ref TIM_TIM4_ETRSOURCE_COMP1
  *            @arg @ref TIM_TIM4_ETRSOURCE_COMP2
  *            @arg @ref TIM_TIM4_ETRSOURCE_COMP3
  *            @arg @ref TIM_TIM4_ETRSOURCE_COMP4
  *            @arg @ref TIM_TIM4_ETRSOURCE_COMP5       (*)
  *            @arg @ref TIM_TIM4_ETRSOURCE_COMP6       (*)
  *            @arg @ref TIM_TIM4_ETRSOURCE_COMP7       (*)
  *            @arg @ref TIM_TIM4_ETRSOURCE_TIM3_ETR
  *            @arg @ref TIM_TIM4_ETRSOURCE_TIM5_ETR    (*)
  *
  *         TIM5: any combination of ETR_RMP where       (**)
  *
  *            @arg @ref TIM_TIM5_ETRSOURCE_GPIO        (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_COMP1       (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_COMP2       (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_COMP3       (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_COMP4       (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_COMP5       (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_COMP6       (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_COMP7       (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_TIM2_ETR    (*)
  *            @arg @ref TIM_TIM5_ETRSOURCE_TIM3_ETR    (*)
  *
  *         TIM8: any combination of ETR_RMP where
  *
  *            . . ETR_RMP can be one of the following values
  *            @arg @ref TIM_TIM8_ETRSOURCE_GPIO
  *            @arg @ref TIM_TIM8_ETRSOURCE_COMP1
  *            @arg @ref TIM_TIM8_ETRSOURCE_COMP2
  *            @arg @ref TIM_TIM8_ETRSOURCE_COMP3
  *            @arg @ref TIM_TIM8_ETRSOURCE_COMP4
  *            @arg @ref TIM_TIM8_ETRSOURCE_COMP5       (*)
  *            @arg @ref TIM_TIM8_ETRSOURCE_COMP6       (*)
  *            @arg @ref TIM_TIM8_ETRSOURCE_COMP7       (*)
  *            @arg @ref TIM_TIM8_ETRSOURCE_ADC2_AWD1
  *            @arg @ref TIM_TIM8_ETRSOURCE_ADC2_AWD2
  *            @arg @ref TIM_TIM8_ETRSOURCE_ADC2_AWD3
  *            @arg @ref TIM_TIM8_ETRSOURCE_ADC3_AWD1   (*)
  *            @arg @ref TIM_TIM8_ETRSOURCE_ADC3_AWD2   (*)
  *            @arg @ref TIM_TIM8_ETRSOURCE_ADC3_AWD3   (*)
  *
  *         (*)  Value not defined in all devices. \n
  *         (**) Register not available in all devices.
  * @retval None
  */
void TIM_Set_ETRSource(TIM_TypeDef *TIMx, uint32_t ETRSource)
{
  MODIFY_REG(TIMx->AF1, TIMx_AF1_ETRSEL, ETRSource);
}

/**
  * @brief  Enable SMS preload.
  * @note Macro IS_TIM_SMS_PRELOAD_INSTANCE(TIMx) can be used to check
  *       whether or not a timer instance supports the preload of SMS field in SMCR register.
  * @rmtoll SMCR         SMSPE           LL_TIM_EnableSMSPreload
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_SMSPreload(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->SMCR, TIM_SMCR_SMSPE);
}

/**
  * @brief  Disable SMS preload.
  * @note Macro IS_TIM_SMS_PRELOAD_INSTANCE(TIMx) can be used to check
  *       whether or not a timer instance supports the preload of SMS field in SMCR register.
  * @rmtoll SMCR         SMSPE           LL_TIM_DisableSMSPreload
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_SMSPreload(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->SMCR, TIM_SMCR_SMSPE);
}

/**
  * @brief  Indicate whether  SMS preload is enabled.
  * @note Macro IS_TIM_SMS_PRELOAD_INSTANCE(TIMx) can be used to check
  *       whether or not a timer instance supports the preload of SMS field in SMCR register.
  * @rmtoll SMCR         SMSPE           LL_TIM_IsEnabledSMSPreload
  * @param  TIMx Timer instance
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_SMSPreload(TIM_TypeDef *TIMx)
{
  return ((READ_BIT(TIMx->SMCR, TIM_SMCR_SMSPE) == (TIM_SMCR_SMSPE)) ? 1UL : 0UL);
}

/**
  * @brief  Set the preload source of SMS.
  * @note Macro IS_TIM_SMS_PRELOAD_INSTANCE(TIMx) can be used to check
  *       whether or not a timer instance supports the preload of SMS field in SMCR register.
  * @rmtoll SMCR         SMSPS        LL_TIM_SetSMSPreloadSource\n
  * @param  TIMx Timer instance
  * @param  PreloadSource This parameter can be one of the following values:
  *         @arg @ref TIM_SMSPS_TIMUPDATE
  *         @arg @ref TIM_SMSPS_INDEX
  * @retval None
  */
void TIM_Set_SMSPreloadSource(TIM_TypeDef *TIMx, uint32_t PreloadSource)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_SMSPS, PreloadSource);
}

/**
  * @brief  Get the preload source of SMS.
  * @note Macro IS_TIM_SMS_PRELOAD_INSTANCE(TIMx) can be used to check
  *       whether or not a timer instance supports the preload of SMS field in SMCR register.
  * @rmtoll SMCR         SMSPS        LL_TIM_GetSMSPreloadSource\n
  * @param  TIMx Timer instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_SMSPS_TIMUPDATE
  *         @arg @ref TIM_SMSPS_INDEX
  */
uint32_t TIM_Get_SMSPreloadSource(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->SMCR, TIM_SMCR_SMSPS));
}


/**
  * @function  TIM_Enable_ExternalClock
  * @brief  Enable external clock mode 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_ExternalClock(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->SMCR, TIM_SMCR_ECE);
}

/**
  * @function  TIM_Disable_ExternalClock
  * @brief  Disable external clock mode 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_ExternalClock(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->SMCR, TIM_SMCR_ECE);
}

/**
  * @function  TIM_Get_ExternalClockStatus
  * @brief  Indicate whether external clock mode 2 is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0:disable external clock,1:enable external clock
  */
 uint32_t TIM_Get_ExternalClockStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->SMCR, TIM_SMCR_ECE) == TIM_SMCR_ECE);
}

/**
  * @function  TIM_Set_ClockSource
  * @brief  Set the clock source of the counter clock
  * @param  TIMx:
            @arg TIM Instance
  * @param  ClockSource This parameter can be one of the following values:
  *         @arg @ref TIM_CLOCKSOURCE_INTERNAL
  *         @arg @ref TIM_CLOCKSOURCE_EXT_MODE1
  *         @arg @ref TIM_CLOCKSOURCE_EXT_MODE2
  * @retval None
  */
 void TIM_Set_ClockSource(TIM_TypeDef *TIMx,uint32_t ClockSource)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_SMS| TIM_SMCR_ECE, ClockSource);
}

/**
  * @function  TIM_Set_EncoderMode
  * @brief  Set the encoder interface mode
  * @param  TIMx:
            @arg TIM Instance
  * @param  EncoderMode This parameter can be one of the following values:
  *         @arg @ref TIM_ENCODERMODE_X2_TI1
  *         @arg @ref TIM_ENCODERMODE_X2_TI2
  *         @arg @ref TIM_ENCODERMODE_X4_TI12
  * @retval None
  */
 void TIM_Set_EncoderMode(TIM_TypeDef *TIMx,uint32_t EncoderMode)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_SMS, EncoderMode);
}

/**
  * @function  TIM_Enable_IT
  * @brief  TIM enable IT
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_IT:This parameter can be one of the following values: 
  *         @arg @ref TIM_DIER_UIE
  *         @arg @ref TIM_DIER_CC1IE 
  *         @arg @ref TIM_DIER_CC2IE       
  *         @arg @ref TIM_DIER_CC3IE        
  *         @arg @ref TIM_DIER_CC4IE       
  *         @arg @ref TIM_DIER_COMIE      
  *         @arg @ref TIM_DIER_TIE     
  *         @arg @ref TIM_DIER_BIE  
  *         @arg @ref TIM_DIER_CC5IE  
  *         @arg @ref TIM_DIER_CC6IE 
  *         @arg @ref TIM_DIER_IDXIE 
  *         @arg @ref TIM_DIER_DIRIE 
  *         @arg @ref TIM_DIER_IERRIE
  *         @arg @ref TIM_DIER_TERRIE
  * @retval None
  */
void TIM_Enable_IT(TIM_TypeDef* TIMx, uint32_t TIM_IT)
{  
  SET_BIT(TIMx->DIER, TIM_IT);
}

/**
  * @function  TIM_Disable_IT
  * @brief  TIM disable IT
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_IT:This parameter can be one of the following values: 
  *         @arg @ref TIM_DIER_UIE
  *         @arg @ref TIM_DIER_CC1IE 
  *         @arg @ref TIM_DIER_CC2IE       
  *         @arg @ref TIM_DIER_CC3IE        
  *         @arg @ref TIM_DIER_CC4IE       
  *         @arg @ref TIM_DIER_COMIE      
  *         @arg @ref TIM_DIER_TIE     
  *         @arg @ref TIM_DIER_BIE 
  *         @arg @ref TIM_DIER_CC5IE   
  *         @arg @ref TIM_DIER_CC6IE 
  *         @arg @ref TIM_DIER_IDXIE 
  *         @arg @ref TIM_DIER_DIRIE 
  *         @arg @ref TIM_DIER_IERRIE
  *         @arg @ref TIM_DIER_TERRIE
  * @retval None
  */
void TIM_Disable_IT(TIM_TypeDef* TIMx, uint32_t TIM_IT)
{  
  CLEAR_BIT(TIMx->DIER, TIM_IT);
}

/**
  * @function  TIM_Enable_DMA
  * @brief  TIM enable DMA
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_DMA:This parameter can be one of the following values: 
  *         @arg @ref TIM_DIER_UDE  
  *         @arg @ref TIM_DIER_CC1DE  
  *         @arg @ref TIM_DIER_CC2DE  
  *         @arg @ref TIM_DIER_CC3DE  
  *         @arg @ref TIM_DIER_CC4DE  
  *         @arg @ref TIM_DIER_COMDE  
  *         @arg @ref TIM_DIER_TDE   
  *         @arg @ref TIM_DIER_CC5DE   
  *         @arg @ref TIM_DIER_CC6DE  
  * @retval None
  */
void TIM_Enable_DMA(TIM_TypeDef* TIMx, uint32_t TIM_DMA)
{  
  SET_BIT(TIMx->DIER, TIM_DMA);
}

/**
  * @function  TIM_Disable_DMA
  * @brief  TIM disable DMA
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_DMA:This parameter can be one of the following values: 
  *         @arg @ref TIM_DIER_UDE  
  *         @arg @ref TIM_DIER_CC1DE  
  *         @arg @ref TIM_DIER_CC2DE  
  *         @arg @ref TIM_DIER_CC3DE  
  *         @arg @ref TIM_DIER_CC4DE  
  *         @arg @ref TIM_DIER_COMDE  
  *         @arg @ref TIM_DIER_TDE   
  *         @arg @ref TIM_DIER_CC5DE   
  *         @arg @ref TIM_DIER_CC6DE  
  * @retval None
  */
void TIM_Disable_DMA(TIM_TypeDef* TIMx, uint32_t TIM_DMA)
{  
  CLEAR_BIT(TIMx->DIER , TIM_DMA);
}

/**
  * @function  TIM_Get_Flag
  * @brief  TIM_ClearITPendingBit
  * @param  TIMx:
            @arg TIM Instance
  * @param  Flag:This parameter can be one of the following values:
  *         @arg @ref TIM_SR_UIF
  *         @arg @ref TIM_SR_CC1IF
  *         @arg @ref TIM_SR_CC2IF
  *         @arg @ref TIM_SR_CC3IF
  *         @arg @ref TIM_SR_CC4IF
  *         @arg @ref TIM_SR_COMIF
  *         @arg @ref TIM_SR_TIF
  *         @arg @ref TIM_SR_BIF
  *         @arg @ref TIM_SR_B2IF
  *         @arg @ref TIM_SR_CC1OF
  *         @arg @ref TIM_SR_CC2OF
  *         @arg @ref TIM_SR_CC3OF
  *         @arg @ref TIM_SR_CC4OF
  *         @arg @ref TIM_SR_SBIF
  *         @arg @ref TIM_SR_CC5IF
  *         @arg @ref TIM_SR_CC6IF
  *         @arg @ref TIM_SR_IDXF 
  *         @arg @ref TIM_SR_DIRF 
  *         @arg @ref TIM_SR_IERRF
  *         @arg @ref TIM_SR_TERRF
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_Flag(TIM_TypeDef* TIMx, uint32_t Flag)
{
  return (READ_BIT(TIMx->SR, Flag) == (Flag));
}

/**
  * @function  TIM_Clear_Flag
  * @brief  Clear IT flag
  * @param  TIMx:
            @arg TIM Instance
  * @param  Flag:This parameter can be one of the following values:
  *         @arg @ref TIM_SR_UIF
  *         @arg @ref TIM_SR_CC1IF
  *         @arg @ref TIM_SR_CC2IF
  *         @arg @ref TIM_SR_CC3IF
  *         @arg @ref TIM_SR_CC4IF
  *         @arg @ref TIM_SR_COMIF
  *         @arg @ref TIM_SR_TIF
  *         @arg @ref TIM_SR_BIF
  *         @arg @ref TIM_SR_B2IF
  *         @arg @ref TIM_SR_CC1OF
  *         @arg @ref TIM_SR_CC2OF
  *         @arg @ref TIM_SR_CC3OF
  *         @arg @ref TIM_SR_CC4OF
  *         @arg @ref TIM_SR_SBIF
  *         @arg @ref TIM_SR_CC5IF
  *         @arg @ref TIM_SR_CC6IF
  *         @arg @ref TIM_SR_IDXF 
  *         @arg @ref TIM_SR_DIRF 
  *         @arg @ref TIM_SR_IERRF
  *         @arg @ref TIM_SR_TERRF
  * @retval None
  */
void TIM_Clear_Flag(TIM_TypeDef* TIMx, uint32_t IT_Flag)
{
  CLEAR_BIT(TIMx->SR, IT_Flag);
}

/**
  * @function  TIM_Set_GenerateEvent
  * @brief  Generate event
  * @param  TIMx:
            @arg TIM Instance
  * @param  Event:This parameter can be one of the following values:
  *         @arg @ref TIM_EGR_UG 
  *         @arg @ref TIM_EGR_CC1G 
  *         @arg @ref TIM_EGR_CC2G 
  *         @arg @ref TIM_EGR_CC3G
  *         @arg @ref TIM_EGR_CC4G 
  *         @arg @ref TIM_EGR_COMG 
  *         @arg @ref TIM_EGR_TG 
  *         @arg @ref TIM_EGR_BG 
  *         @arg @ref TIM_EGR_CC5G 
  *         @arg @ref TIM_EGR_CC6G 
  * @retval None
  */
 void TIM_Set_GenerateEvent(TIM_TypeDef *TIMx, uint32_t Event)
{
  SET_BIT(TIMx->EGR, Event);
}

/**
  * @function  TIM_Set_OC_Mode
  * @brief  Define the behavior of the output reference signal OCxREF from which
  *         OCx and OCxN (when relevant) are derived.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @param  Mode This parameter can be one of the following values:
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  *         @arg @ref TIM_OCMODE_RETRIG_OPM1
  *         @arg @ref TIM_OCMODE_RETRIG_OPM2
  *         @arg @ref TIM_OCMODE_PULSE_ON_COMPARE
  *         @arg @ref TIM_OCMODE_DIRECTION_OUTPUT 
  *         @arg @ref TIM_OCMODE_COMBINED_PWM1
  *         @arg @ref TIM_OCMODE_COMBINED_PWM2
  *         @arg @ref TIM_OCMODE_ASSYMETRIC_PWM1
  *         @arg @ref TIM_OCMODE_ASSYMETRIC_PWM2
  * @retval None
  */
 void TIM_Set_OC_Mode(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t Mode)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register __IO uint32_t *pReg = (__IO uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_OC1M  | TIM_CCMR1_CC1S) << SHIFT_TAB_OCxx[iChannel]),  Mode << SHIFT_TAB_OCxx[iChannel]);
}

/**
  * @function  TIM_Set_OC_Polarity
  * @brief  Set the polarity of an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @param  Polarity This parameter can be one of the following values:
  *         @arg @ref TIM_OCPOLARITY_HIGH
  *         @arg @ref TIM_OCPOLARITY_LOW
  * @retval None
  */
 void TIM_Set_OC_Polarity(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t Polarity)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  MODIFY_REG(TIMx->CCER, (TIM_CCER_CC1P << SHIFT_TAB_CCxP[iChannel]),  Polarity << SHIFT_TAB_CCxP[iChannel]);
}

/**
  * @function  TIM_Get_OC_Polarity
  * @brief  Get the polarity of an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_OCPOLARITY_HIGH
  *         @arg @ref TIM_OCPOLARITY_LOW
  */
 uint32_t TIM_Get_OC_Polarity(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  return (READ_BIT(TIMx->CCER, (TIM_CCER_CC1P<< SHIFT_TAB_CCxP[iChannel])) >> SHIFT_TAB_CCxP[iChannel]);
}

/**
  * @function  TIM_Enable_OC_Fast
  * @brief  Enable fast mode for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Enable_OC_Fast(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  SET_BIT(*pReg, (TIM_CCMR1_OC1FE << SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Disable_OC_Fast
  * @brief  Disable fast mode for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Disable_OC_Fast(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  CLEAR_BIT(*pReg, (TIM_CCMR1_OC1FE<< SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Get_OC_FastStatus
  * @brief  Indicates whether fast mode is enabled for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval 0: Output Compare  fast disable, 1:Output Compare  fast enable
  */
 uint32_t TIM_Get_OC_FastStatus(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  register uint32_t bitfield = TIM_CCMR1_OC1FE<< SHIFT_TAB_OCxx[iChannel];
  return (READ_BIT(*pReg, bitfield) == bitfield);
}

/**
  * @function  TIM_Enable_OC_Preload
  * @brief  Enable compare register (TIM_CCRx) preload for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Enable_OC_Preload(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  SET_BIT(*pReg, (TIM_CCMR1_OC1PE<< SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Disable_OC_Preload
  * @brief  Disable compare register (TIM_CCRx) preload for the output channel.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Disable_OC_Preload(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  CLEAR_BIT(*pReg, (TIM_CCMR1_OC1PE<< SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Get_OC_PreloadStatus
  * @brief  Indicates whether compare register (TIM_CCRx) preload is enabled for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval 0: Output Compare  preload  disable, 1:Output Compare  preload enable
  */
 uint32_t TIM_Get_OC_PreloadStatus(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  register uint32_t bitfield = TIM_CCMR1_OC1PE<< SHIFT_TAB_OCxx[iChannel];
  return (READ_BIT(*pReg, bitfield) == bitfield);
}

/**
  * @function  TIM_Enable_OC_Clear
  * @brief  Enable clearing the output channel on an external event
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Enable_OC_Clear(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  SET_BIT(*pReg, (TIM_CCMR1_OC1CE<< SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Disable_OC_Clear
  * @brief  Disable clearing the output channel on an external event
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Disable_OC_Clear(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  CLEAR_BIT(*pReg, (TIM_CCMR1_OC1CE << SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Get_OC_ClearStatus
  * @brief  Indicates clearing the output channel on an external event is enabled for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval 0: Output Compare  clear  disable, 1:Output Compare  clear enable
  */
 uint32_t TIM_Get_OC_ClearStatus(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  register uint32_t bitfield = TIM_CCMR1_OC1CE << SHIFT_TAB_OCxx[iChannel];
  return (READ_BIT(*pReg, bitfield) == bitfield);
}

/**
  * @function  TIM_Enable_CC_Channel
  * @brief  Enable capture/compare channels.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channels This parameter can be a combination of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Enable_CC_Channel(TIM_TypeDef *TIMx,uint32_t Channels)
{
  SET_BIT(TIMx->CCER, Channels);
}

/**
  * @function  TIM_Disable_CC_Channel
  * @brief  Disable capture/compare channels.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channels This parameter can be a combination of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Disable_CC_Channel(TIM_TypeDef *TIMx,uint32_t Channels)
{
  CLEAR_BIT(TIMx->CCER, Channels);
}

/**
  * @function  TIM_Get_CC_ChannelStatus
  * @brief  Indicate whether channel(s) is(are) enabled.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channels This parameter can be a combination of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval 0:channel is disalbe, 1:channel is enable
  */
 uint32_t TIM_Get_CC_ChannelStatus(TIM_TypeDef *TIMx,uint32_t Channels)
{
  return (READ_BIT(TIMx->CCER, Channels) == (Channels));
}

/**
  * @function  TIM_Set_OC_CompareCH1
  * @brief  Set compare value for output channel 1 (TIM_CCR1)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH1(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR1, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH2
  * @brief  Set compare value for output channel 2 (TIM_CCR2)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH2(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR2, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH3
  * @brief  Set compare value for output channel 3 (TIM_CCR3)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH3(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR3, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH4
  * @brief  Set compare value for output channel 4 (TIM_CCR4)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH4(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR4, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH5
  * @brief  Set compare value for output channel 5 (TIM_CCR5)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH5(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR5, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH6
  * @brief  Set compare value for output channel 6 (TIM_CCR6)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH6(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR6, CompareValue);
}

/**
  * @function  TIM_Get_OC_CompareCH1
  * @brief  Get compare value (TIM_CCR1) set for  output channel 1
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH1(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR1));
}

/**
  * @function  TIM_Get_OC_CompareCH2
  * @brief  Get compare value (TIM_CCR2) set for  output channel 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH2(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR2));
}

/**
  * @function  TIM_Get_OC_CompareCH3
  * @brief  Get compare value (TIM_CCR3) set for  output channel 3
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH3(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR3));
}

/**
  * @function  TIM_Get_OC_CompareCH4
  * @brief  Get compare value (TIM_CCR4) set for  output channel 4
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH4(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR4));
}

/**
  * @function  TIM_Get_OC_CompareCH5
  * @brief  Get compare value (TIM_CCR5) set for  output channel 5
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH5(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR5));
}

/**
  * @function  TIM_Get_OC_CompareCH6
  * @brief  Get compare value (TIM_CCR6) set for  output channel 6
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH6(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR6));
}


/**
  * @brief  Select on which reference signal the OC5REF is combined to.
  * @note Macro IS_TIM_COMBINED3PHASEPWM_INSTANCE(TIMx) can be used to check
  *       whether or not a timer instance supports the combined 3-phase PWM mode.
  * @rmtoll CCR5         GC5C3          LL_TIM_SetCH5CombinedChannels\n
  *         CCR5         GC5C2          LL_TIM_SetCH5CombinedChannels\n
  *         CCR5         GC5C1          LL_TIM_SetCH5CombinedChannels
  * @param  TIMx Timer instance
  * @param  GroupCH5 This parameter can be a combination of the following values:
  *         @arg @ref TIM_GROUPCH5_NONE
  *         @arg @ref TIM_GROUPCH5_OC1REFC
  *         @arg @ref TIM_GROUPCH5_OC2REFC
  *         @arg @ref TIM_GROUPCH5_OC3REFC
  * @retval None
  */
void TIM_Set_CH5CombinedChannels(TIM_TypeDef *TIMx, uint32_t GroupCH5)
{
  MODIFY_REG(TIMx->CCR5, (TIM_CCR5_GC5C3 | TIM_CCR5_GC5C2 | TIM_CCR5_GC5C1), GroupCH5);
}

/**
  * @brief  Set the pulse on compare pulse width prescaler.
  * @note Macro IS_TIM_PULSEONCOMPARE_INSTANCE(TIMx) can be used to check
  *       whether or not the pulse on compare feature is supported by the timer
  *       instance.
  * @rmtoll ECR          PWPRSC           LL_TIM_OC_SetPulseWidthPrescaler
  * @param  TIMx Timer instance
  * @param  PulseWidthPrescaler This parameter can be one of the following values:
  *         @arg @ref TIM_PWPRSC_X1
  *         @arg @ref TIM_PWPRSC_X2
  *         @arg @ref TIM_PWPRSC_X4
  *         @arg @ref TIM_PWPRSC_X8
  *         @arg @ref TIM_PWPRSC_X16
  *         @arg @ref TIM_PWPRSC_X32
  *         @arg @ref TIM_PWPRSC_X64
  *         @arg @ref TIM_PWPRSC_X128
  * @retval None
  */
void TIM_OC_Set_PulseWidthPrescaler(TIM_TypeDef *TIMx, uint32_t PulseWidthPrescaler)
{
  MODIFY_REG(TIMx->ECR, TIM_ECR_PWPRSC, PulseWidthPrescaler);
}

/**
  * @brief  Get the pulse on compare pulse width prescaler.
  * @note Macro IS_TIM_PULSEONCOMPARE_INSTANCE(TIMx) can be used to check
  *       whether or not the pulse on compare feature is supported by the timer
  *       instance.
  * @rmtoll ECR          PWPRSC           LL_TIM_OC_GetPulseWidthPrescaler
  * @param  TIMx Timer instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_PWPRSC_X1
  *         @arg @ref TIM_PWPRSC_X2
  *         @arg @ref TIM_PWPRSC_X4
  *         @arg @ref TIM_PWPRSC_X8
  *         @arg @ref TIM_PWPRSC_X16
  *         @arg @ref TIM_PWPRSC_X32
  *         @arg @ref TIM_PWPRSC_X64
  *         @arg @ref TIM_PWPRSC_X128
  */
uint32_t TIM_OC_Get_PulseWidthPrescaler(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->ECR, TIM_ECR_PWPRSC));
}

/**
  * @brief  Set the pulse on compare pulse width duration.
  * @note Macro IS_TIM_PULSEONCOMPARE_INSTANCE(TIMx) can be used to check
  *       whether or not the pulse on compare feature is supported by the timer
  *       instance.
  * @rmtoll ECR          PW           LL_TIM_OC_SetPulseWidth
  * @param  TIMx Timer instance
  * @param  PulseWidth This parameter can be between Min_Data=0 and Max_Data=255
  * @retval None
  */
void TIM_OC_Set_PulseWidth(TIM_TypeDef *TIMx, uint32_t PulseWidth)
{
  MODIFY_REG(TIMx->ECR, TIM_ECR_PW, PulseWidth << (16U) );
}

/**
  * @brief  Get the pulse on compare pulse width duration.
  * @note Macro IS_TIM_PULSEONCOMPARE_INSTANCE(TIMx) can be used to check
  *       whether or not the pulse on compare feature is supported by the timer
  *       instance.
  * @rmtoll ECR          PW           LL_TIM_OC_GetPulseWidth
  * @param  TIMx Timer instance
  * @retval Returned value can be between Min_Data=0 and Max_Data=255:
  */
uint32_t TIM_OC_Get_PulseWidth(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->ECR, TIM_ECR_PW));
}

/**
  * @function  TIM_Set_IC
  * @brief  Configure input channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  Configuration This parameter must be a combination of all the following values:
  *         @arg @ref TIM_ACTIVEINPUT_DIRECTTI or @ref TIM_ACTIVEINPUT_INDIRECTTI or @ref TIM_ACTIVEINPUT_TRC
  *         @arg @ref TIM_ICPSC_DIV1 or ... or @ref TIM_ICPSC_DIV8
  *         @arg @ref TIM_IC_FILTER_FDIV1 or ... or @ref TIM_IC_FILTER_FDIV32_N8
  *         @arg @ref TIM_IC_POLARITY_RISING or @ref TIM_IC_POLARITY_FALLING
  * @retval None
  */
void TIM_Set_IC(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t Configuration)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_IC1F | TIM_CCMR1_IC1PSC | TIM_CCMR1_CC1S) << SHIFT_TAB_ICxx[iChannel]),
             ((Configuration >> 16U) & (TIM_CCMR1_IC1F | TIM_CCMR1_IC1PSC | TIM_CCMR1_CC1S))  << SHIFT_TAB_ICxx[iChannel]);
  MODIFY_REG(TIMx->CCER, ((TIM_CCER_CC1NP | TIM_CCER_CC1P) << SHIFT_TAB_CCxP[iChannel]),
             (Configuration & (TIM_CCER_CC1NP | TIM_CCER_CC1P)) << SHIFT_TAB_CCxP[iChannel]);
}

/**
  * @function  TIM_Set_IC_ActiveInput
  * @brief  Set the active input
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICActiveInput This parameter can be one of the following values:
  *         @arg @ref TIM_ACTIVEINPUT_DIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_INDIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_TRC
  * @retval None
  */
void TIM_Set_IC_ActiveInput(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t ICActiveInput)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_CC1S) << SHIFT_TAB_ICxx[iChannel]), (ICActiveInput >> 16U) << SHIFT_TAB_ICxx[iChannel]);
}

/**
  * @function  TIM_Get_IC_ActiveInput
  * @brief  Get the current active input
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_ACTIVEINPUT_DIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_INDIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_TRC
  */
uint32_t TIM_Get_IC_ActiveInput(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  return ((READ_BIT(*pReg, ((TIM_CCMR1_CC1S) << SHIFT_TAB_ICxx[iChannel])) >> SHIFT_TAB_ICxx[iChannel]) << 16U);
}

/**
  * @function  TIM_Set_IC_Prescaler
  * @brief  Set the prescaler of input channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICPrescaler This parameter can be one of the following values:
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @retval None
  */
void TIM_Set_IC_Prescaler(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t ICPrescaler)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_IC1PSC) << SHIFT_TAB_ICxx[iChannel]), (ICPrescaler >> 16U) << SHIFT_TAB_ICxx[iChannel]);
}

/**
  * @function  TIM_Get_IC_Prescaler
  * @brief  Get the current prescaler value acting on an  input channel.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  */
uint32_t TIM_Get_IC_Prescaler(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  return ((READ_BIT(*pReg, ((TIM_CCMR1_IC1PSC) << SHIFT_TAB_ICxx[iChannel])) >> SHIFT_TAB_ICxx[iChannel]) << 16U);
}

/**
  * @function  TIM_Set_IC_Filter
  * @brief  Set the input filter duration
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICFilter This parameter can be one of the following values:
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @retval None
  */
 void TIM_Set_IC_Filter(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t ICFilter)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_IC1F) << SHIFT_TAB_ICxx[iChannel]), (ICFilter >> 16U) << SHIFT_TAB_ICxx[iChannel]);
}

/**
  * @function  TIM_Get_IC_Filter
  * @brief  Get the input filter duration
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8
  */
 uint32_t TIM_Get_IC_Filter(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  return ((READ_BIT(*pReg, ((TIM_CCMR1_IC1F) << SHIFT_TAB_ICxx[iChannel])) >> SHIFT_TAB_ICxx[iChannel]) << 16U);
}

/**
  * @function  TIM_Set_IC_Polarity
  * @brief  Set the input channel polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_IC_POLARITY_RISING
  *         @arg @ref TIM_IC_POLARITY_FALLING
  * @retval None
  */
 void TIM_Set_IC_Polarity(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t ICPolarity)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  MODIFY_REG(TIMx->CCER, ((TIM_CCER_CC1NP|TIM_CCER_CC1P) << SHIFT_TAB_CCxP[iChannel]),
             ICPolarity << SHIFT_TAB_CCxP[iChannel]);
}

/**
  * @function  TIM_Get_IC_Polarity
  * @brief  Get the current input channel polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_IC_POLARITY_RISING
  *         @arg @ref TIM_IC_POLARITY_FALLING
  */
 uint32_t TIM_Get_IC_Polarity(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  return (READ_BIT(TIMx->CCER, ((TIM_CCER_CC1NP|TIM_CCER_CC1P)  << SHIFT_TAB_CCxP[iChannel])) >>
          SHIFT_TAB_CCxP[iChannel]);
}


/**
  * @function  TIM_Get_IC_CaptureCH1
  * @brief  Get captured value for input channel 1
  * @param  TIMx:
            @arg TIM Instance
  * @retval CapturedValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_IC_CaptureCH1(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR1));
}

/**
  * @function  TIM_Get_IC_CaptureCH2
  * @brief  Get captured value for input channel 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval CapturedValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_IC_CaptureCH2(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR2));
}

/**
  * @function  TIM_Get_IC_CaptureCH3
  * @brief  Get captured value for input channel 3
  * @param  TIMx:
            @arg TIM Instance
  * @retval CapturedValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_IC_CaptureCH3(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR3));
}

/**
  * @function  TIM_Get_IC_CaptureCH4
  * @brief  Get captured value for input channel 4
  * @param  TIMx:
            @arg TIM Instance
  * @retval CapturedValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_IC_CaptureCH4(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR4));
}

/**
  * @function  TIM_Set_OC_DeadTime
  * @brief  Set the dead-time delay (delay inserted between the rising edge of the OCxREF signal and the rising edge if the Ocx and OCxN signals).
  * @param  TIMx:
            @arg TIM Instance
  * @param  DeadTime between Min_Data=0 and Max_Data=255
  * @retval None
  */
 void TIM_Set_OC_DeadTime(TIM_TypeDef *TIMx,uint32_t DeadTime)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_DTG, DeadTime);
}


/**
  * @function  TIM_Set_CC_LockLevel
  * @brief  Set the lock level to freeze the configuration of several capture/compare parameters
  * @param  TIMx:
            @arg TIM Instance
  * @param  LockLevel This parameter can be one of the following values:
  *         @arg @ref TIM_BDTR_LOCK_OFF
  *         @arg @ref TIM_BDTR_LOCK_1
  *         @arg @ref TIM_BDTR_LOCK_2
  *         @arg @ref TIM_BDTR_LOCK_3
  * @retval None
  */
 void TIM_Set_CC_LockLevel(TIM_TypeDef *TIMx,uint32_t LockLevel)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_LOCK, LockLevel);
}

/**
  * @function  TIM_Set_OffStates
  * @brief  Select the outputs off state (enabled v.s. disabled) in Idle and Run modes
  * @param  TIMx:
            @arg TIM Instance
  * @param  OffStateIdle This parameter can be one of the following values:
  *         @arg @ref TIM_OSSI_DISABLE
  *         @arg @ref TIM_OSSI_ENABLE
  * @param  OffStateRun This parameter can be one of the following values:
  *         @arg @ref TIM_OSSR_DISABLE
  *         @arg @ref TIM_OSSR_ENABLE
  * @retval None
  */
 void TIM_Set_OffStates(TIM_TypeDef *TIMx,uint32_t OffStateIdle, uint32_t OffStateRun)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_OSSI |TIM_BDTR_OSSR, OffStateIdle | OffStateRun);
}

/**
  * @function  TIM_Enable_BRK
  * @brief  Enable the break function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_BRK(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_BKE);
}

/**
  * @function  TIM_Disable_BRK
  * @brief  Disable the break function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_BRK(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_BKE);
}

/**
  * @brief  Configure the break input.
  * @note Macro IS_TIM_BREAK_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides a break input.
  * @note Bidirectional mode is only supported by advanced timer instances.
  *       Macro IS_TIM_ADVANCED_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance is an advanced-control timer.
  * @note In bidirectional mode (BKBID bit set), the Break input is configured both
  *        in input mode and in open drain output mode. Any active Break event will
  *        assert a low logic level on the Break input to indicate an internal break
  *        event to external devices.
  * @note When bidirectional mode isn't supported, BreakAFMode must be set to
  *       TIM_BREAK_AFMODE_INPUT.
  * @rmtoll BDTR         BKP           TIM_ConfigBRK\n
  *         BDTR         BKF           TIM_ConfigBRK\n
  *         BDTR         BKBID         TIM_ConfigBRK
  * @param  TIMx Timer instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_POLARITY_LOW
  *         @arg @ref TIM_BREAK_POLARITY_HIGH
  * @param  BreakFilter This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_FILTER_FDIV1
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N2
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N4
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV2_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV2_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV4_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV4_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV8_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV8_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N5
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N5
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N8
  * @param  BreakAFMode This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_AFMODE_INPUT
  *         @arg @ref TIM_BREAK_AFMODE_BIDIRECTIONAL
  * @retval None
  */
void TIM_Set_BRK(TIM_TypeDef *TIMx, uint32_t BreakPolarity, uint32_t BreakFilter, uint32_t BreakAFMode)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_BKP | TIM_BDTR_BKF | TIM_BDTR_BKBID, BreakPolarity | BreakFilter | BreakAFMode);
}

/**
  * @brief  Disarm the break input (when it operates in bidirectional mode).
  * @note  The break input can be disarmed only when it is configured in
  *        bidirectional mode and when when MOE is reset.
  * @note  Purpose is to be able to have the input voltage back to high-state,
  *        whatever the time constant on the output .
  * @rmtoll BDTR         BKDSRM        LL_TIM_DisarmBRK
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disarm_BRK(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_BKDSRM);
}

/**
  * @brief  Re-arm the break input (when it operates in bidirectional mode).
  * @note  The Break input is automatically armed as soon as MOE bit is set.
  * @rmtoll BDTR         BKDSRM        LL_TIM_ReArmBRK
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_ReArm_BRK(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_BKDSRM);
}



/**
  * @function  TIM_Set_BreakPolarity
  * @brief  Configure the break input
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_POLARITY_LOW
  *         @arg @ref TIM_BREAK_POLARITY_HIGH
  * @retval None
  */
 void TIM_Set_BreakPolarity(TIM_TypeDef *TIMx,uint32_t BreakPolarity)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_BKP, BreakPolarity);
}


/**
  * @function  TIM_Enable_AOE
  * @brief  Enable automatic output (MOE can be set by software or automatically when a break input is active)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_AOE(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR,TIM_BDTR_AOE);
}

/**
  * @function  TIM_Disable_AOE
  * @brief  Disable automatic output (MOE can be set only by software)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_AOE(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_AOE);
}

/**
  * @function  TIM_Get_AOEStatus
  * @brief  Indicate whether automatic output is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0: disable automatic output,1:enable automatic output
  */
 uint32_t TIM_Get_AOEStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->BDTR, TIM_BDTR_AOE) == TIM_BDTR_AOE);
}

/**
  * @function  TIM_Enable_MOE
  * @brief  Enable the outputs (set the MOE bit in TIM_BDTR register)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_MOE(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_MOE);
}

/**
  * @function  TIM_Disable_MOE
  * @brief  Disable the outputs (reset the MOE bit in TIM_BDTR register)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_MOE(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_MOE);
}

/**
  * @function  TIM_Get_MOEStatus
  * @brief  Indicates whether outputs are enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0: disable outputs,1:enable outputs
  */
 uint32_t TIM_Get_MOEStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->BDTR, TIM_BDTR_MOE) == TIM_BDTR_MOE);
}

/**
  * @brief  Enable the signals connected to the designated timer break input.
  * @note Macro IS_TIM_BREAKSOURCE_INSTANCE(TIMx) can be used to check whether
  *       or not a timer instance allows for break input selection.
  * @rmtoll AF1          BKINE         TIM_EnableBreakInputSource\n
  *         AF1          BKCMP1E       TIM_EnableBreakInputSource\n
  *         AF1          BKCMP2E       TIM_EnableBreakInputSource\n
  *         AF1          BKCMP3E       TIM_EnableBreakInputSource\n
  *         AF1          BKCMP4E       TIM_EnableBreakInputSource\n
  *         AF1          BKCMP5E       TIM_EnableBreakInputSource\n
  *         AF1          BKCMP6E       TIM_EnableBreakInputSource\n
  *         AF1          BKCMP7E       TIM_EnableBreakInputSource\n
  *         AF2          BK2NE         TIM_EnableBreakInputSource\n
  *         AF2          BK2CMP1E      TIM_EnableBreakInputSource\n
  *         AF2          BK2CMP2E      TIM_EnableBreakInputSource\n
  *         AF2          BK2CMP3E      TIM_EnableBreakInputSource\n
  *         AF2          BK2CMP4E      TIM_EnableBreakInputSource\n
  *         AF2          BK2CMP5E      TIM_EnableBreakInputSource\n
  *         AF2          BK2CMP6E      TIM_EnableBreakInputSource\n
  *         AF2          BK2CMP7E      TIM_EnableBreakInputSource
  * @param  TIMx Timer instance
  * @param  BreakInput This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_INPUT_BKIN
  *         @arg @ref TIM_BREAK_INPUT_BKIN2
  * @param  Source This parameter can be one of the following values:
  *         @arg @ref TIM_BKIN_SOURCE_BKIN
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP1
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP2
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP3
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP4
  *
  *         (*)  Value not defined in all devices.
  * @retval None
  */
void TIMX_Enable_BreakInputSource(TIM_TypeDef *TIMx, uint32_t BreakInput, uint32_t Source)
{
  __IO uint32_t *pReg = (__IO uint32_t *)((uint32_t)((uint32_t)(&TIMx->AF1) + BreakInput));
  SET_BIT(*pReg, Source);
}

/**
  * @brief  Disable the signals connected to the designated timer break input.
  * @note Macro IS_TIM_BREAKSOURCE_INSTANCE(TIMx) can be used to check whether
  *       or not a timer instance allows for break input selection.
  * @rmtoll AF1          BKINE         TIM_DisableBreakInputSource\n
  *         AF1          BKCMP1E       TIM_DisableBreakInputSource\n
  *         AF1          BKCMP2E       TIM_DisableBreakInputSource\n
  *         AF1          BKCMP3E       TIM_DisableBreakInputSource\n
  *         AF1          BKCMP4E       TIM_DisableBreakInputSource\n
  *         AF1          BKCMP5E       TIM_DisableBreakInputSource\n
  *         AF1          BKCMP6E       TIM_DisableBreakInputSource\n
  *         AF1          BKCMP7E       TIM_DisableBreakInputSource\n
  *         AF2          BKINE         TIM_DisableBreakInputSource\n
  *         AF2          BKCMP1E       TIM_DisableBreakInputSource\n
  *         AF2          BKCMP2E       TIM_DisableBreakInputSource\n
  *         AF2          BKCMP3E       TIM_DisableBreakInputSource\n
  *         AF2          BKCMP4E       TIM_DisableBreakInputSource\n
  *         AF2          BKCMP5E       TIM_DisableBreakInputSource\n
  *         AF2          BKCMP6E       TIM_DisableBreakInputSource\n
  *         AF2          BKCMP7E       TIM_DisableBreakInputSource
  * @param  TIMx Timer instance
  * @param  BreakInput This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_INPUT_BKIN
  *         @arg @ref TIM_BREAK_INPUT_BKIN2
  * @param  Source This parameter can be one of the following values:
  *         @arg @ref TIM_BKIN_SOURCE_BKIN
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP1
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP2
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP3
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP4
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP5 (*)
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP6 (*)
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP7 (*)
  *
  *         (*)  Value not defined in all devices.
  * @retval None
  */
void TIMX_Disable_BreakInputSource(TIM_TypeDef *TIMx, uint32_t BreakInput, uint32_t Source)
{
  __IO uint32_t *pReg = (__IO uint32_t *)((uint32_t)((uint32_t)(&TIMx->AF1) + BreakInput));
  CLEAR_BIT(*pReg, Source);
}

/**
  * @brief  Set the polarity of the break signal for the timer break input.
  * @note Macro IS_TIM_BREAKSOURCE_INSTANCE(TIMx) can be used to check whether
  *       or not a timer instance allows for break input selection.
  * @rmtoll AF1          BKINP         TIM_SetBreakInputSourcePolarity\n
  *         AF1          BKCMP1P       TIM_SetBreakInputSourcePolarity\n
  *         AF1          BKCMP2P       TIM_SetBreakInputSourcePolarity\n
  *         AF1          BKCMP3P       TIM_SetBreakInputSourcePolarity\n
  *         AF1          BKCMP4P       TIM_SetBreakInputSourcePolarity\n
  *         AF2          BK2INP        TIM_SetBreakInputSourcePolarity\n
  *         AF2          BK2CMP1P      TIM_SetBreakInputSourcePolarity\n
  *         AF2          BK2CMP2P      TIM_SetBreakInputSourcePolarity\n
  *         AF2          BK2CMP3P      TIM_SetBreakInputSourcePolarity\n
  *         AF2          BK2CMP4P      TIM_SetBreakInputSourcePolarity
  * @param  TIMx Timer instance
  * @param  BreakInput This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_INPUT_BKIN
  *         @arg @ref TIM_BREAK_INPUT_BKIN2
  * @param  Source This parameter can be one of the following values:
  *         @arg @ref TIM_BKIN_SOURCE_BKIN
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP1
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP2
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP3
  *         @arg @ref TIM_BKIN_SOURCE_BKCOMP4
  * @param  Polarity This parameter can be one of the following values:
  *         @arg @ref TIM_BKIN_POLARITY_NoInvert
  *         @arg @ref TIM_BKIN_POLARITY_Invert
  * @retval None
  */
void TIMX_Set_BreakInputSourcePolarity(TIM_TypeDef *TIMx, uint32_t BreakInput, uint32_t Source, uint32_t Polarity)
{
  __IO uint32_t *pReg = (__IO uint32_t *)((uint32_t)((uint32_t)(&TIMx->AF1) + BreakInput));
  MODIFY_REG(*pReg, (TIMx_AF1_BKINP << TIM_POSITION_BRK_SOURCE), (Polarity << TIM_POSITION_BRK_SOURCE));
}

/**
  * @brief  Enable asymmetrical deadtime.
  * @note Macro IS_TIM_DEADTIME_ASYMMETRICAL_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides asymmetrical deadtime.
  * @rmtoll DTR2          DTAE          LL_TIM_EnableAsymmetricalDeadTime
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_AsymmetricalDeadTime(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->DTR2, TIM_DTR2_DTAE);
}

/**
  * @brief  Disable asymmetrical dead-time.
  * @note Macro IS_TIM_DEADTIME_ASYMMETRICAL_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides asymmetrical deadtime.
  * @rmtoll DTR2          DTAE          LL_TIM_DisableAsymmetricalDeadTime
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_AsymmetricalDeadTime(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->DTR2, TIM_DTR2_DTAE);
}

/**
  * @brief  Indicates whether asymmetrical deadtime is activated.
  * @note Macro IS_TIM_DEADTIME_ASYMMETRICAL_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides asymmetrical deadtime.
  * @rmtoll DTR2          DTAE          LL_TIM_IsEnabledAsymmetricalDeadTime
  * @param  TIMx Timer instance
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_AsymmetricalDeadTime(TIM_TypeDef *TIMx)
{
  return ((READ_BIT(TIMx->DTR2, TIM_DTR2_DTAE) == (TIM_DTR2_DTAE)) ? 1UL : 0UL);
}

/**
  * @brief  Set the falling edge dead-time delay (delay inserted between the falling edge of the OCxREF signal and the
  *         rising edge of OCxN signals).
  * @note Macro IS_TIM_DEADTIME_ASYMMETRICAL_INSTANCE(TIMx) can be used to check whether or not
  *       asymmetrical dead-time insertion feature is supported by a timer instance.
  * @note Helper macro @ref __LL_TIM_CALC_DEADTIME can be used to calculate the DeadTime parameter
  * @note This bit-field can not be modified as long as LOCK level 1, 2 or 3 has been programmed
  *       (LOCK bits in TIMx_BDTR register).
  * @rmtoll DTR2         DTGF           LL_TIM_SetFallingDeadTime
  * @param  TIMx Timer instance
  * @param  DeadTime between Min_Data=0 and Max_Data=255
  * @retval None
  */
void TIM_Set_FallingDeadTime(TIM_TypeDef *TIMx, uint32_t DeadTime)
{
  MODIFY_REG(TIMx->DTR2, TIM_DTR2_DTGF, DeadTime);
}

/**
  * @brief  Get the falling edge dead-time delay (delay inserted between the falling edge of the OCxREF signal and
  *         the rising edge of OCxN signals).
  * @note Macro IS_TIM_DEADTIME_ASYMMETRICAL_INSTANCE(TIMx) can be used to check whether or not
  *       asymmetrical dead-time insertion feature is supported by a timer instance.
  * @note This bit-field can not be modified as long as LOCK level 1, 2 or 3 has been programmed
  *       (LOCK bits in TIMx_BDTR register).
  * @rmtoll DTR2          DTGF           LL_TIM_GetFallingDeadTime
  * @param  TIMx Timer instance
  * @retval Returned value can be between Min_Data=0 and Max_Data=255:
  */
uint32_t TIM_Get_FallingDeadTime(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->DTR2, TIM_DTR2_DTGF));
}

/**
  * @brief  Enable deadtime preload.
  * @note Macro IS_TIM_BREAK_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides deadtime preload.
  * @rmtoll DTR2          DTPE          LL_TIM_EnableDeadTimePreload
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_DeadTimePreload(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->DTR2, TIM_DTR2_DTPE);
}

/**
  * @brief  Disable dead-time preload.
  * @note Macro IS_TIM_BREAK_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides deadtime preload.
  * @rmtoll DTR2          DTPE          LL_TIM_DisableDeadTimePreload
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_DeadTimePreload(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->DTR2, TIM_DTR2_DTPE);
}

/**
  * @brief  Indicates whether deadtime preload is activated.
  * @note Macro IS_TIM_BREAK_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides deadtime preload.
  * @rmtoll DTR2          DTPE          LL_TIM_IsEnabledDeadTimePreload
  * @param  TIMx Timer instance
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_DeadTimePreload(TIM_TypeDef *TIMx)
{
  return ((READ_BIT(TIMx->DTR2, TIM_DTR2_DTPE) == (TIM_DTR2_DTPE)) ? 1UL : 0UL);
}

/**
  * @function  TIM_Enable_CMP1Break
  * @brief  Enable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CMP1Break(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_CMP1_OE);
}

/**
  * @function  TIM_Disable_CMP1Break
  * @brief  Disable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CMP1Break(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_CMP1_OE);
}

/**
  * @function  TIM_Set_CMP1BreakPolarity
  * @brief  Configure CMP1 out polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_CMP1OEP_POLARITY_LOW 
  *         @arg @ref TIM_CMP1OEP_POLARITY_HIGH
  * @retval None
  */
 void TIM_Set_CMP1BreakPolarity(TIM_TypeDef *TIMx,uint32_t BreakPolarity)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_CMP1_OEP, BreakPolarity);
}

/**
  * @function  TIM_Enable_CMP2Break
  * @brief  Enable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CMP2Break(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_CMP2_OE);
}

/**
  * @function  TIM_Disable_CMP2Break
  * @brief  Disable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CMP2Break(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_CMP2_OE);
}

/**
  * @function  TIM_Set_CMP2BreakPolarity
  * @brief  Configure CMP2 out polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_CMP2OEP_POLARITY_LOW
  *         @arg @ref TIM_CMP2OEP_POLARITY_HIGH
  * @retval None
  */
 void TIM_Set_CMP2BreakPolarity(TIM_TypeDef *TIMx,uint32_t BreakPolarity)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_CMP2_OEP, BreakPolarity);
}

/**
  * @function  TIM_Enable_CMP3Break
  * @brief  Enable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CMP3Break(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_CMP3_OE);
}

/**
  * @function  TIM_Disable_CMP3Break
  * @brief  Disable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CMP3Break(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_CMP3_OE);
}

/**
  * @function  TIM_Set_CMP3BreakPolarity
  * @brief  Configure CMP3 out polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_CMP3OEP_POLARITY_LOW
  *         @arg @ref TIM_CMP3OEP_POLARITY_HIGH
  * @retval None
  */
 void TIM_Set_CMP3BreakPolarity(TIM_TypeDef *TIMx,uint32_t BreakPolarity)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_CMP3_OEP, BreakPolarity);
}

/**
  * @function  TIM_Enable_HardFaultBreak
  * @brief  Enable hardfault output of break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
void TIM_Enable_HardFaultBreak( TIM_TypeDef *TIMx )
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_Hardfault_OE);
}

/**
  * @function  TIM_Disable_HardFaultBreak
  * @brief  Disnable hardfault output of break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
void TIM_Disable_HardFaultBreak( TIM_TypeDef *TIMx )
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_Hardfault_OE);
}

/**
  * @function  TIM_Enable_PVDBreak
  * @brief  Enable PVD of break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
void TIM_Enable_PVDBreak( TIM_TypeDef *TIMx )
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_PVD_OE);
}

/**
  * @function  TIM_Disable_PVDBreak
  * @brief  Disnable PVD of break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
void TIM_Disable_PVDBreak( TIM_TypeDef *TIMx )
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_PVD_OE);
}

/**
  * @function  TIM_Set_BreakFilter
  * @brief  Set the break source filter duration.
  * @param  TIMx:
            @arg TIM Instance
  * @param  ICFilter This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_FILTER_FDIV1    
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N2 
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N4 
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N8 
  *         @arg @ref TIM_BREAK_FILTER_FDIV2_N6 
  *         @arg @ref TIM_BREAK_FILTER_FDIV2_N8 
  *         @arg @ref TIM_BREAK_FILTER_FDIV4_N6 
  *         @arg @ref TIM_BREAK_FILTER_FDIV4_N8 
  *         @arg @ref TIM_BREAK_FILTER_FDIV8_N6 
  *         @arg @ref TIM_BREAK_FILTER_FDIV8_N8 
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N5
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N5
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N8
  * @retval None
  */
 void TIM_Set_BreakFilter(TIM_TypeDef *TIMx, uint32_t BreakFilter)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_BKF, BreakFilter);
}

/**
  * @function  TIM_Set_DMABurst
  * @brief  Configures the timer DMA burst feature
  * @param  TIMx:
            @arg TIM Instance
  * @param  DMABurstBaseAddress This parameter can be one of the following values:
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CR1
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CR2
  *         @arg @ref TIM_DCR_DBA_BASEADDR_SMCR
  *         @arg @ref TIM_DCR_DBA_BASEADDR_DIER
  *         @arg @ref TIM_DCR_DBA_BASEADDR_SR
  *         @arg @ref TIM_DCR_DBA_BASEADDR_EGR
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCMR1
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCMR2
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCER
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CNT
  *         @arg @ref TIM_DCR_DBA_BASEADDR_PSC
  *         @arg @ref TIM_DCR_DBA_BASEADDR_ARR
  *         @arg @ref TIM_DCR_DBA_BASEADDR_RCR
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR1
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR2
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR3
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR4
  *         @arg @ref TIM_DCR_DBA_BASEADDR_BDTR
  *         @arg @ref TIM_DCR_DBA_BASEADDR_DCR
  *         @arg @ref TIM_DCR_DBA_BASEADDR_DMAR
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR5
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCMR3
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR6
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR1N
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR2N
  *         @arg @ref TIM_DCR_DBA_BASEADDR_CCR3N
  * @param  DMABurstLength This parameter can be one of the following values:
  *         @arg @ref TIM_DCR_LENGTH_1
  *         @arg @ref TIM_DCR_LENGTH_2
  *         @arg @ref TIM_DCR_LENGTH_3
  *         @arg @ref TIM_DCR_LENGTH_4
  *         @arg @ref TIM_DCR_LENGTH_5
  *         @arg @ref TIM_DCR_LENGTH_6
  *         @arg @ref TIM_DCR_LENGTH_7
  *         @arg @ref TIM_DCR_LENGTH_8
  *         @arg @ref TIM_DCR_LENGTH_9
  *         @arg @ref TIM_DCR_LENGTH_10
  *         @arg @ref TIM_DCR_LENGTH_11
  *         @arg @ref TIM_DCR_LENGTH_12
  *         @arg @ref TIM_DCR_LENGTH_13
  *         @arg @ref TIM_DCR_LENGTH_14
  *         @arg @ref TIM_DCR_LENGTH_15
  *         @arg @ref TIM_DCR_LENGTH_16
  *         @arg @ref TIM_DCR_LENGTH_17
  *         @arg @ref TIM_DCR_LENGTH_18
  * @retval None
  */
 void TIM_Set_DMABurst(TIM_TypeDef *TIMx, uint32_t DMABurstBaseAddress, uint32_t DMABurstLength)
{
  MODIFY_REG(TIMx->DCR, (TIM_DCR_DBL | TIM_DCR_DBA), (DMABurstBaseAddress | DMABurstLength));
}

/**
  * @brief  Enable encoder index.
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR         IE           LL_TIM_EnableEncoderIndex
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_EncoderIndex(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->ECR, TIM_ECR_IE);
}

/**
  * @brief  Disable encoder index.
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR         IE           LL_TIM_DisableEncoderIndex
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_EncoderIndex(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->ECR, TIM_ECR_IE);
}

/**
  * @brief  Indicate whether encoder index is enabled.
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR         IE           LL_TIM_IsEnabledEncoderIndex
  * @param  TIMx Timer instance
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_EncoderIndex(TIM_TypeDef *TIMx)
{
  return ((READ_BIT(TIMx->ECR, TIM_ECR_IE) == (TIM_ECR_IE)) ? 1U : 0U);
}

/**
  * @brief  Set index direction
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR          IDIR           TIM_SetIndexDirection
  * @param  TIMx Timer instance
  * @param  IndexDirection This parameter can be one of the following values:
  *         @arg @ref TIM_INDEX_UP_DOWN
  *         @arg @ref TIM_INDEX_UP
  *         @arg @ref TIM_INDEX_DOWN
  * @retval None
  */
void TIM_Set_IndexDirection(TIM_TypeDef *TIMx, uint32_t IndexDirection)
{
  MODIFY_REG(TIMx->ECR, TIM_ECR_IDIR, IndexDirection);
}

/**
  * @brief  Get actual index direction
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR          IDIR           TIM_GetIndexDirection
  * @param  TIMx Timer instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_INDEX_UP_DOWN
  *         @arg @ref TIM_INDEX_UP
  *         @arg @ref TIM_INDEX_DOWN
  */
uint32_t TIM_Get_IndexDirection(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->ECR, TIM_ECR_IDIR));
}

/**
  * @brief  Enable first index.
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR          FIDX          TIM_EnableFirstIndex
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_FirstIndex(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->ECR, TIM_ECR_FIDX);
}

/**
  * @brief  Disable first index.
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR          FIDX          TIM_DisableFirstIndex
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_FirstIndex(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->ECR, TIM_ECR_FIDX);
}

/**
  * @brief  Indicates whether first index is enabled.
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR          FIDX          TIM_IsEnabledFirstIndex
  * @param  TIMx Timer instance
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_FirstIndex(TIM_TypeDef *TIMx)
{
  return ((READ_BIT(TIMx->ECR, TIM_ECR_FIDX) == (TIM_ECR_FIDX)) ? 1UL : 0UL);
}

/**
  * @brief  Set index positioning
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR          IPOS           TIM_SetIndexPositionning
  * @param  TIMx Timer instance
  * @param  IndexPositionning This parameter can be one of the following values:
  *         @arg @ref TIM_INDEX_POSITION_DOWN_DOWN
  *         @arg @ref TIM_INDEX_POSITION_DOWN_UP
  *         @arg @ref TIM_INDEX_POSITION_UP_DOWN
  *         @arg @ref TIM_INDEX_POSITION_UP_UP
  *         @arg @ref TIM_INDEX_POSITION_DOWN
  *         @arg @ref TIM_INDEX_POSITION_UP
  * @retval None
  */
void TIM_Set_IndexPositionning(TIM_TypeDef *TIMx, uint32_t IndexPositionning)
{
  MODIFY_REG(TIMx->ECR, TIM_ECR_IPOS, IndexPositionning);
}

/**
  * @brief  Get actual index positioning
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR          IPOS           TIM_GetIndexPositionning
  * @param  TIMx Timer instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_INDEX_POSITION_DOWN_DOWN
  *         @arg @ref TIM_INDEX_POSITION_DOWN_UP
  *         @arg @ref TIM_INDEX_POSITION_UP_DOWN
  *         @arg @ref TIM_INDEX_POSITION_UP_UP
  *         @arg @ref TIM_INDEX_POSITION_DOWN
  *         @arg @ref TIM_INDEX_POSITION_UP
  */
uint32_t TIM_Get_IndexPositionning(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->ECR, TIM_ECR_IPOS));
}

/**
  * @brief  Configure encoder index.
  * @note Macro IS_TIM_INDEX_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides an index input.
  * @rmtoll ECR          IDIR          TIM_ConfigIDX\n
  *         ECR          FIDX          TIM_ConfigIDX\n
  *         ECR          IPOS          TIM_ConfigIDX
  * @param  TIMx Timer instance
  * @param  Configuration This parameter must be a combination of all the following values:
  *         @arg @ref TIM_INDEX_UP or @ref TIM_INDEX_DOWN or @ref TIM_INDEX_UP_DOWN
  *         @arg @ref TIM_INDEX_ALL or @ref TIM_INDEX_FIRST_ONLY
  *         @arg @ref TIM_INDEX_POSITION_DOWN_DOWN or ... or @ref TIM_INDEX_POSITION_UP
  * @retval None
  */
void TIM_Set_IDX(TIM_TypeDef *TIMx, uint32_t Configuration)
{
  MODIFY_REG(TIMx->ECR, TIM_ECR_IDIR | TIM_ECR_FIDX | TIM_ECR_IPOS, Configuration);
}

/**
  * @brief  Remap TIM inputs (input channel, internal/external triggers).
  * @note Macro IS_TIM_REMAP_INSTANCE(TIMx) can be used to check whether or not
  *       a some timer inputs can be remapped.
  * @rmtoll TIM1_TISEL    TI1SEL      TIM_SetRemap\n
  *         TIM2_TISEL    TI1SEL      TIM_SetRemap\n
  *         TIM2_TISEL    TI2SEL      TIM_SetRemap\n
  *         TIM2_TISEL    TI3SEL      TIM_SetRemap\n
  *         TIM2_TISEL    TI4SEL      TIM_SetRemap\n
  *         TIM3_TISEL    TI1SEL      TIM_SetRemap\n
  *         TIM3_TISEL    TI2SEL      TIM_SetRemap\n
  *         TIM3_TISEL    TI3SEL      TIM_SetRemap\n
  *         TIM4_TISEL    TI1SEL      TIM_SetRemap\n
  *         TIM4_TISEL    TI2SEL      TIM_SetRemap\n
  *         TIM4_TISEL    TI3SEL      TIM_SetRemap\n
  *         TIM4_TISEL    TI4SEL      TIM_SetRemap\n
  *         TIM5_TISEL    TI1SEL      TIM_SetRemap\n
  *         TIM5_TISEL    TI2SEL      TIM_SetRemap\n
  *         TIM8_TISEL    TI1SEL      TIM_SetRemap\n
  *         TIM15_TISEL   TI1SEL      TIM_SetRemap\n
  *         TIM15_TISEL   TI2SEL      TIM_SetRemap\n
  *         TIM16_TISEL   TI1SEL      TIM_SetRemap\n
  *         TIM17_TISEL   TI1SEL      TIM_SetRemap\n
  *         TIM20_TISEL   TI1SEL      TIM_SetRemap
  * @param  TIMx Timer instance
  * @param  Remap Remap param depends on the TIMx. Description available only
  *         in CHM version of the User Manual (not in .pdf).
  *         Otherwise see Reference Manual description of TISEL registers.
  *
  *         Below description summarizes "Timer Instance" and "Remap" param combinations:
  *
  *         TIM1: one of the following values
  *
  *            @arg @ref TIM_TIM1_TI1_RMP_GPIO
  *            @arg @ref TIM_TIM1_TI1_RMP_COMP1
  *            @arg @ref TIM_TIM1_TI1_RMP_COMP2
  *            @arg @ref TIM_TIM1_TI1_RMP_COMP3
  *            @arg @ref TIM_TIM1_TI1_RMP_COMP4
  *
  *         TIM2: any combination of TI1_RMP, TI2_RMP, TI3_RMP and TI4_RMP where
  *
  *            . . TI1_RMP can be one of the following values
  *            @arg @ref TIM_TIM2_TI1_RMP_GPIO
  *            @arg @ref TIM_TIM2_TI1_RMP_COMP1
  *            @arg @ref TIM_TIM2_TI1_RMP_COMP2
  *            @arg @ref TIM_TIM2_TI1_RMP_COMP3
  *            @arg @ref TIM_TIM2_TI1_RMP_COMP4
  *
  *            . . TI2_RMP can be one of the following values
  *            @arg @ref TIM_TIM2_TI2_RMP_GPIO
  *            @arg @ref TIM_TIM2_TI2_RMP_COMP1
  *            @arg @ref TIM_TIM2_TI2_RMP_COMP2
  *            @arg @ref TIM_TIM2_TI2_RMP_COMP3
  *            @arg @ref TIM_TIM2_TI2_RMP_COMP4
  *
  *            . . TI3_RMP can be one of the following values
  *            @arg @ref TIM_TIM2_TI3_RMP_GPIO
  *            @arg @ref TIM_TIM2_TI3_RMP_COMP4
  *
  *            . . TI4_RMP can be one of the following values
  *            @arg @ref TIM_TIM2_TI4_RMP_GPIO
  *            @arg @ref TIM_TIM2_TI4_RMP_COMP1
  *            @arg @ref TIM_TIM2_TI4_RMP_COMP2
  *
  *         TIM3: any combination of TI1_RMP and TI2_RMP where
  *
  *            . . TI1_RMP can be one of the following values
  *            @arg @ref TIM_TIM3_TI1_RMP_GPIO
  *            @arg @ref TIM_TIM3_TI1_RMP_COMP1
  *            @arg @ref TIM_TIM3_TI1_RMP_COMP2
  *            @arg @ref TIM_TIM3_TI1_RMP_COMP3
  *            @arg @ref TIM_TIM3_TI1_RMP_COMP4
  *
  *            . . TI2_RMP can be one of the following values
  *            @arg @ref TIM_TIM3_TI2_RMP_GPIO
  *            @arg @ref TIM_TIM3_TI2_RMP_COMP1
  *            @arg @ref TIM_TIM3_TI2_RMP_COMP2
  *            @arg @ref TIM_TIM3_TI2_RMP_COMP3
  *            @arg @ref TIM_TIM3_TI2_RMP_COMP4
  *
  *            . . TI3_RMP can be one of the following values
  *            @arg @ref TIM_TIM3_TI3_RMP_GPIO
  *            @arg @ref TIM_TIM3_TI3_RMP_COMP3
  *
  *         TIM4: any combination of TI1_RMP, TI2_RMP, TI3_RMP and TI4_RMP where
  *
  *            . . TI1_RMP can be one of the following values
  *            @arg @ref TIM_TIM4_TI1_RMP_GPIO
  *            @arg @ref TIM_TIM4_TI1_RMP_COMP1
  *            @arg @ref TIM_TIM4_TI1_RMP_COMP2
  *            @arg @ref TIM_TIM4_TI1_RMP_COMP3
  *            @arg @ref TIM_TIM4_TI1_RMP_COMP4
  *
  *            . . TI2_RMP can be one of the following values
  *            @arg @ref TIM_TIM4_TI2_RMP_GPIO
  *            @arg @ref TIM_TIM4_TI2_RMP_COMP1
  *            @arg @ref TIM_TIM4_TI2_RMP_COMP2
  *            @arg @ref TIM_TIM4_TI2_RMP_COMP3
  *            @arg @ref TIM_TIM4_TI2_RMP_COMP4
  *
  *            . . TI3_RMP can be one of the following values
  *            @arg @ref TIM_TIM4_TI3_RMP_GPIO
  *
  *            . . TI4_RMP can be one of the following values
  *            @arg @ref TIM_TIM4_TI4_RMP_GPIO
  *
  *         TIM5: any combination of TI1_RMP and TI2_RMP where (**)
  *
  *            . . TI1_RMP can be one of the following values
  *            @arg @ref TIM_TIM5_TI1_RMP_GPIO   
  *            @arg @ref TIM_TIM5_TI1_RMP_LSI    
  *            @arg @ref TIM_TIM5_TI1_RMP_LSE    
  *            @arg @ref TIM_TIM5_TI1_RMP_RTC_WK 
  *            @arg @ref TIM_TIM5_TI1_RMP_COMP1  
  *            @arg @ref TIM_TIM5_TI1_RMP_COMP2  
  *            @arg @ref TIM_TIM5_TI1_RMP_COMP3  
  *            @arg @ref TIM_TIM5_TI1_RMP_COMP4   
  *
  *            . . TI2_RMP can be one of the following values
  *            @arg @ref TIM_TIM5_TI2_RMP_GPIO  
  *            @arg @ref TIM_TIM5_TI2_RMP_COMP1 
  *            @arg @ref TIM_TIM5_TI2_RMP_COMP2 
  *            @arg @ref TIM_TIM5_TI2_RMP_COMP3 
  *            @arg @ref TIM_TIM5_TI2_RMP_COMP4 
  *
  *         TIM8: one of the following values
  *
  *            @arg @ref TIM_TIM8_TI1_RMP_GPIO
  *            @arg @ref TIM_TIM8_TI1_RMP_COMP1
  *            @arg @ref TIM_TIM8_TI1_RMP_COMP2
  *            @arg @ref TIM_TIM8_TI1_RMP_COMP3
  *            @arg @ref TIM_TIM8_TI1_RMP_COMP4
  *
  *         TIM15: any combination of TI1_RMP and TI2_RMP where
  *
  *            . . TI1_RMP can be one of the following values
  *            @arg @ref TIM_TIM15_TI1_RMP_GPIO
  *            @arg @ref TIM_TIM15_TI1_RMP_LSE
  *            @arg @ref TIM_TIM15_TI1_RMP_COMP1
  *            @arg @ref TIM_TIM15_TI1_RMP_COMP2
  *
  *            . . TI2_RMP can be one of the following values
  *            @arg @ref TIM_TIM15_TI2_RMP_GPIO
  *            @arg @ref TIM_TIM15_TI2_RMP_COMP2
  *            @arg @ref TIM_TIM15_TI2_RMP_COMP3
  *
  *         TIM16: one of the following values
  *
  *            @arg @ref TIM_TIM16_TI1_RMP_GPIO
  *            @arg @ref TIM_TIM16_TI1_RMP_MCO
  *            @arg @ref TIM_TIM16_TI1_RMP_HSE_32
  *            @arg @ref TIM_TIM16_TI1_RMP_RTC_WK
  *            @arg @ref TIM_TIM16_TI1_RMP_LSE
  *            @arg @ref TIM_TIM16_TI1_RMP_LSI
  *
  *         TIM17: one of the following values
  *
  *            @arg @ref TIM_TIM17_TI1_RMP_GPIO
  *            @arg @ref TIM_TIM17_TI1_RMP_MCO
  *            @arg @ref TIM_TIM17_TI1_RMP_HSE_32
  *            @arg @ref TIM_TIM17_TI1_RMP_RTC_WK
  *            @arg @ref TIM_TIM17_TI1_RMP_LSE
  *            @arg @ref TIM_TIM17_TI1_RMP_LSI
  *
  *
  * @retval None
  */
void TIM_Set_Remap(TIM_TypeDef *TIMx, uint32_t Remap)
{
  MODIFY_REG(TIMx->TISEL, (TIM_TISEL_TI1SEL | TIM_TISEL_TI2SEL | TIM_TISEL_TI3SEL | TIM_TISEL_TI4SEL), Remap);
}

/**
  * @brief  Enable request for HSE/32 clock used for TISEL remap.
  * @note Only TIM16 and TIM17 support HSE/32 remap
  * @rmtoll OR         HSE32EN           LL_TIM_EnableHSE32
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_HSE32(TIM_TypeDef *TIMx)
{
//  SET_BIT(TIMx->OR, TIM_OR_HSE32EN);
}

/**
  * @brief  Disable request for HSE/32 clock used for TISEL remap.
  * @note Only TIM16 and TIM17 support HSE/32 remap
  * @rmtoll OR         HSE32EN           LL_TIM_DisableHSE32
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_HSE32(TIM_TypeDef *TIMx)
{
//  CLEAR_BIT(TIMx->OR, TIM_OR_HSE32EN);
}

/**
  * @brief  Indicate whether request for HSE/32 clock is enabled.
  * @note Only TIM16 and TIM17 support HSE/32 remap
  * @rmtoll OR         HSE32EN           LL_TIM_IsEnabledHSE32
  * @param  TIMx Timer instance
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_HSE32(TIM_TypeDef *TIMx)
{
//  return ((READ_BIT(TIMx->OR, TIM_OR_HSE32EN) == (TIM_OR_HSE32EN)) ? 1UL : 0UL);
}

/**
  * @brief  Set the OCREF clear input source
  * @note The OCxREF signal of a given channel can be cleared when a high level is applied on the OCREF_CLR_INPUT
  * @note This function can only be used in Output compare and PWM modes.
  * @note Macro IS_TIM_OCCS_INSTANCE(TIMx) can be used to check whether
  *       or not a timer instance can configure OCREF clear input source.
  * @rmtoll SMCR          OCCS                TIM_SetOCRefClearInputSource
  * @rmtoll AF2           OCRSEL              TIM_SetOCRefClearInputSource
  * @param  TIMx Timer instance
  * @param  OCRefClearInputSource This parameter can be one of the following values:
  *         @arg @ref TIM_OCREF_CLR_INT_ETR
  *         @arg @ref TIM_OCREF_CLR_INT_COMP1
  *         @arg @ref TIM_OCREF_CLR_INT_COMP2
  *         @arg @ref TIM_OCREF_CLR_INT_COMP3
  *         @arg @ref TIM_OCREF_CLR_INT_COMP4
  *         @arg @ref TIM_OCREF_CLR_INT_COMP5 (*)
  *         @arg @ref TIM_OCREF_CLR_INT_COMP6 (*)
  *         @arg @ref TIM_OCREF_CLR_INT_COMP7 (*)
  *
  *         (*)  Value not defined in all devices. \n
  * @retval None
  */
void TIM_Set_OCRefClearInputSource(TIM_TypeDef *TIMx, uint32_t OCRefClearInputSource)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_OCCS,
             ((OCRefClearInputSource & OCREF_CLEAR_SELECT_Msk) >> OCREF_CLEAR_SELECT_Pos) << (3U) );
  MODIFY_REG(TIMx->AF2, TIM1_AF2_OCRSEL, OCRefClearInputSource);
}


/**
  * @function  TIM_Set_CounterValue
  * @brief  Set the counter value
  * @param  TIMx:
            @arg TIM Instance
  * @param  Counter Counter value (between Min_Data=0 and Max_Data=0xFFFF)
  * @retval None
  */
 void TIM_Set_CounterValue(TIM_TypeDef *TIMx,uint32_t Counter)
{
  WRITE_REG(TIMx->CNT, Counter);
}

/**
  * @function  TIM_Clear_CounterValue
  * @brief  Clear the counter value
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Clear_CounterValue(TIM_TypeDef *TIMx)
{
  CLEAR_REG(TIMx->CNT);
}

/**
  * @function  TIM_Get_CounterValue
  * @brief  Get the counter value
  * @param  TIMx:
            @arg TIM Instance
  * @retval Counter value (between Min_Data=0 and Max_Data=0xFFFF)
  */
 uint32_t TIM_Get_CounterValue(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CNT));
}

/**
  * @function  TIM_Set_Prescaler
  * @brief  Set TIMx the prescaler value
  * @param  TIMx:
            @arg TIM Instance
  * @param  Prescaler between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_Prescaler(TIM_TypeDef *TIMx,uint32_t Prescaler)
{
  WRITE_REG(TIMx->PSC, Prescaler);
}

/**
  * @function  TIM_Get_Prescaler
  * @brief  Get the prescaler value
  * @param  TIMx:
            @arg TIM Instance
  * @retval  Prescaler value between Min_Data=0 and Max_Data=65535
  */
 uint32_t TIM_Get_Prescaler(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->PSC));
}

/**
  * @function  TIM_Set_AutoReload
  * @brief  Set TIMx the auto-reload value
  * @param  TIMx:
            @arg TIM Instance
  * @param  AutoReload between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_AutoReload(TIM_TypeDef *TIMx,uint32_t AutoReload)
{
  WRITE_REG(TIMx->ARR, AutoReload);
}

/**
  * @function  TIM_Get_AutoReload
  * @brief  Get the prescaler value
  * @param  TIMx:
            @arg TIM Instance
  * @retval  Auto-reload value value between Min_Data=0 and Max_Data=65535
  */
 uint32_t TIM_Get_AutoReload(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->ARR));
}

/**
  * @function  TIM_Set_RepetitionCounter
  * @brief  Set the repetition counter value
  * @param  TIMx:
            @arg TIM Instance
  * @param  RepetitionCounter value between Min_Data=0 and Max_Data=255
  * @retval None
  */
 void TIM_Set_RepetitionCounter(TIM_TypeDef *TIMx,uint32_t RepetitionCounter)
{
  WRITE_REG(TIMx->RCR, RepetitionCounter);
}

/**
  * @function  TIM_Get_RepetitionCounter
  * @brief  Get the repetition counter value
  * @param  TIMx:
            @arg TIM Instance
  * @retval RepetitionCounter value between Min_Data=0 and Max_Data=255
  */
 uint32_t TIM_Get_RepetitionCounter(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->RCR));
}


/**
  * @brief  Force a continuous copy of the update interrupt flag (UIF) into the timer counter register (bit 31).
  * @note This allows both the counter value and a potential roll-over condition signalled by the UIFCPY flag to be read
  *       in an atomic way.
  * @rmtoll CR1          UIFREMAP      LL_TIM_EnableUIFRemap
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_UIFRemap(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_UIFREMAP);
}

/**
  * @brief  Disable update interrupt flag (UIF) remapping.
  * @rmtoll CR1          UIFREMAP      LL_TIM_DisableUIFRemap
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_UIFRemap(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR1, TIM_CR1_UIFREMAP);
}

/**
  * @brief  Indicate whether update interrupt flag (UIF) copy is set.
  * @param  Counter Counter value
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_UIFCPY(uint32_t Counter)
{
  return (((Counter & TIM_CNT_UIFCPY) == (TIM_CNT_UIFCPY)) ? 1UL : 0UL);
}

/**
  * @brief  Enable dithering.
  * @note Macro IS_TIM_DITHERING_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides dithering.
  * @rmtoll CR1          DITHEN          LL_TIM_EnableDithering
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_Dithering(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_DITHEN);
}

/**
  * @brief  Disable dithering.
  * @note Macro IS_TIM_DITHERING_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides dithering.
  * @rmtoll CR1          DITHEN          LL_TIM_DisableDithering
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_Dithering(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR1, TIM_CR1_DITHEN);
}

/**
  * @brief  Indicates whether dithering is activated.
  * @note Macro IS_TIM_DITHERING_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides dithering.
  * @rmtoll CR1          DITHEN          LL_TIM_IsEnabledDithering
  * @param  TIMx Timer instance
  * @retval State of bit (1 or 0).
  */
uint32_t TIM_Get_Dithering(TIM_TypeDef *TIMx)
{
  return ((READ_BIT(TIMx->CR1, TIM_CR1_DITHEN) == (TIM_CR1_DITHEN)) ? 1UL : 0UL);
}

/***
  * @function  TIM_Set_TI1
  * @brief  Configure the TI1 as Input
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_ICPolarity : The Input Polarity
  *   This parameter can be one of the following values:
  *     @arg TIM_IC_POLARITY_RISING
  *     @arg TIM_IC_POLARITY_FALLING
  * @param  TIM_ICSelection: specifies the input to be used
  *   This parameter can be one of the following values:
  *     @arg TIM_ACTIVEINPUT_DIRECTTI: TIM Input 1 is selected to be connected to IC1
  *     @arg TIM_ACTIVEINPUT_INDIRECTTI: TIM Input 1 is selected to be connected to IC2
  *     @arg TIM_ACTIVEINPUT_TRC: TIM Input 1 is selected to be connected to TRC
  * @param  TIM_ICFilter: Specifies the Input Capture Filter
  *     @arg @ref TIM_IC_FILTER_FDIV1
  *     @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *     @arg @ref TIM_IC_FILTER_FDIV1_N4
  *     @arg @ref TIM_IC_FILTER_FDIV1_N8
  *     @arg @ref TIM_IC_FILTER_FDIV2_N6
  *     @arg @ref TIM_IC_FILTER_FDIV2_N8
  *     @arg @ref TIM_IC_FILTER_FDIV4_N6
  *     @arg @ref TIM_IC_FILTER_FDIV4_N8
  *     @arg @ref TIM_IC_FILTER_FDIV8_N6
  *     @arg @ref TIM_IC_FILTER_FDIV8_N8
  *     @arg @ref TIM_IC_FILTER_FDIV16_N5
  *     @arg @ref TIM_IC_FILTER_FDIV16_N6
  *     @arg @ref TIM_IC_FILTER_FDIV16_N8
  *     @arg @ref TIM_IC_FILTER_FDIV32_N5
  *     @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *     @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @param  ICPrescaler This parameter can be one of the following values:
  *     @arg @ref TIM_ICPSC_DIV1
  *     @arg @ref TIM_ICPSC_DIV2
  *     @arg @ref TIM_ICPSC_DIV4
  *     @arg @ref TIM_ICPSC_DIV8
  * @retval None
  */
 void TIM_Set_TI1(TIM_TypeDef* TIMx, TIM_ICInitTypeDef* TIM_ICInitStruct)
{
  /* Disable the Channel 1: Reset the CC1E Bit */
  TIMx->CCER &= ~((uint16_t)TIM_CCER_CC1E); 
  
  MODIFY_REG(TIMx->CCMR1,
             (TIM_CCMR1_CC1S | TIM_CCMR1_IC1F | TIM_CCMR1_IC1PSC),
             (TIM_ICInitStruct->ICSelection | TIM_ICInitStruct->ICFilter | TIM_ICInitStruct->ICPrescaler) >> 16U);

  /* Select the Polarity and set the CC1E Bit */
  MODIFY_REG(TIMx->CCER,
             (TIM_CCER_CC1P | TIM_CCER_CC1NP),
             (TIM_ICInitStruct->ICPolarity | TIM_CCER_CC1E));
             
}

/***
  * @function  TIM_Set_TI2
  * @brief  Configure the TI2 as Input
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_ICPolarity : The Input Polarity
  *   This parameter can be one of the following values:
  *     @arg TIM_IC_POLARITY_RISING
  *     @arg TIM_IC_POLARITY_FALLING
  * @param  TIM_ICSelection: specifies the input to be used
  *   This parameter can be one of the following values:
  *     @arg TIM_ACTIVEINPUT_DIRECTTI: TIM Input 2 is selected to be connected to IC2
  *     @arg TIM_ACTIVEINPUT_INDIRECTTI: TIM Input 2 is selected to be connected to IC1
  *     @arg TIM_ACTIVEINPUT_TRC: TIM Input 2 is selected to be connected to TRC
  * @param  TIM_ICFilter: Specifies the Input Capture Filter
  *     @arg @ref TIM_IC_FILTER_FDIV1
  *     @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *     @arg @ref TIM_IC_FILTER_FDIV1_N4
  *     @arg @ref TIM_IC_FILTER_FDIV1_N8
  *     @arg @ref TIM_IC_FILTER_FDIV2_N6
  *     @arg @ref TIM_IC_FILTER_FDIV2_N8
  *     @arg @ref TIM_IC_FILTER_FDIV4_N6
  *     @arg @ref TIM_IC_FILTER_FDIV4_N8
  *     @arg @ref TIM_IC_FILTER_FDIV8_N6
  *     @arg @ref TIM_IC_FILTER_FDIV8_N8
  *     @arg @ref TIM_IC_FILTER_FDIV16_N5
  *     @arg @ref TIM_IC_FILTER_FDIV16_N6
  *     @arg @ref TIM_IC_FILTER_FDIV16_N8
  *     @arg @ref TIM_IC_FILTER_FDIV32_N5
  *     @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *     @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @param  ICPrescaler This parameter can be one of the following values:
  *     @arg @ref TIM_ICPSC_DIV1
  *     @arg @ref TIM_ICPSC_DIV2
  *     @arg @ref TIM_ICPSC_DIV4
  *     @arg @ref TIM_ICPSC_DIV8
  * @retval None
  */
 void TIM_Set_TI2(TIM_TypeDef* TIMx, TIM_ICInitTypeDef* TIM_ICInitStruct)
{
  /* Disable the Channel 2: Reset the CC2E Bit */
  TIMx->CCER &= (uint32_t)~TIM_CCER_CC2E;

  /* Select the Input and set the filter and the prescaler value */
  MODIFY_REG(TIMx->CCMR1,
             (TIM_CCMR1_CC2S | TIM_CCMR1_IC2F | TIM_CCMR1_IC2PSC),
             (TIM_ICInitStruct->ICSelection | TIM_ICInitStruct->ICFilter | TIM_ICInitStruct->ICPrescaler) >> 8U);

  /* Select the Polarity and set the CC2E Bit */
  MODIFY_REG(TIMx->CCER,
             (TIM_CCER_CC2P | TIM_CCER_CC2NP),
             ((TIM_ICInitStruct->ICPolarity << 4U) | TIM_CCER_CC2E));
}

/***
  * @function  TIM_Set_TI3
  * @brief  Configure the TI3 as Input
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_ICPolarity : The Input Polarity
  *   This parameter can be one of the following values:
  *     @arg TIM_IC_POLARITY_RISING
  *     @arg TIM_IC_POLARITY_FALLING
  * @param  TIM_ICSelection: specifies the input to be used
  *   This parameter can be one of the following values:
  *     @arg TIM_ACTIVEINPUT_DIRECTTI: TIM Input 3 is selected to be connected to IC3
  *     @arg TIM_ACTIVEINPUT_INDIRECTTI: TIM Input 3 is selected to be connected to IC4
  *     @arg TIM_ACTIVEINPUT_TRC: TIM Input 3 is selected to be connected to TRC
  * @param  TIM_ICFilter: Specifies the Input Capture Filter
  *     @arg @ref TIM_IC_FILTER_FDIV1
  *     @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *     @arg @ref TIM_IC_FILTER_FDIV1_N4
  *     @arg @ref TIM_IC_FILTER_FDIV1_N8
  *     @arg @ref TIM_IC_FILTER_FDIV2_N6
  *     @arg @ref TIM_IC_FILTER_FDIV2_N8
  *     @arg @ref TIM_IC_FILTER_FDIV4_N6
  *     @arg @ref TIM_IC_FILTER_FDIV4_N8
  *     @arg @ref TIM_IC_FILTER_FDIV8_N6
  *     @arg @ref TIM_IC_FILTER_FDIV8_N8
  *     @arg @ref TIM_IC_FILTER_FDIV16_N5
  *     @arg @ref TIM_IC_FILTER_FDIV16_N6
  *     @arg @ref TIM_IC_FILTER_FDIV16_N8
  *     @arg @ref TIM_IC_FILTER_FDIV32_N5
  *     @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *     @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @param  ICPrescaler This parameter can be one of the following values:
  *     @arg @ref TIM_ICPSC_DIV1
  *     @arg @ref TIM_ICPSC_DIV2
  *     @arg @ref TIM_ICPSC_DIV4
  *     @arg @ref TIM_ICPSC_DIV8
  * @retval None
  */
 void TIM_Set_TI3(TIM_TypeDef* TIMx, TIM_ICInitTypeDef* TIM_ICInitStruct)
{
  /* Disable the Channel 3: Reset the CC3E Bit */
  TIMx->CCER &= (uint32_t)~TIM_CCER_CC3E;

  /* Select the Input and set the filter and the prescaler value */
  MODIFY_REG(TIMx->CCMR2,
             (TIM_CCMR2_CC3S | TIM_CCMR2_IC3F | TIM_CCMR2_IC3PSC),
             (TIM_ICInitStruct->ICSelection | TIM_ICInitStruct->ICFilter | TIM_ICInitStruct->ICPrescaler) >> 16U);

  /* Select the Polarity and set the CC3E Bit */
  MODIFY_REG(TIMx->CCER,
             (TIM_CCER_CC3P | TIM_CCER_CC3NP),
             ((TIM_ICInitStruct->ICPolarity << 8U) | TIM_CCER_CC3E));
}

/***
  * @function  TIM_Set_TI4
  * @brief  Configure the TI4 as Input
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_ICPolarity : The Input Polarity.
  *   This parameter can be one of the following values:
  *     @arg TIM_IC_POLARITY_RISING
  *     @arg TIM_IC_POLARITY_FALLING
  * @param  TIM_ICSelection: specifies the input to be used
  *   This parameter can be one of the following values:
  *     @arg TIM_ACTIVEINPUT_DIRECTTI: TIM Input 4 is selected to be connected to IC4
  *     @arg TIM_ACTIVEINPUT_INDIRECTTI: TIM Input 4 is selected to be connected to IC3
  *     @arg TIM_ACTIVEINPUT_TRC: TIM Input 4 is selected to be connected to TRC
  * @param  TIM_ICFilter: Specifies the Input Capture Filter
  *     @arg @ref TIM_IC_FILTER_FDIV1
  *     @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *     @arg @ref TIM_IC_FILTER_FDIV1_N4
  *     @arg @ref TIM_IC_FILTER_FDIV1_N8
  *     @arg @ref TIM_IC_FILTER_FDIV2_N6
  *     @arg @ref TIM_IC_FILTER_FDIV2_N8
  *     @arg @ref TIM_IC_FILTER_FDIV4_N6
  *     @arg @ref TIM_IC_FILTER_FDIV4_N8
  *     @arg @ref TIM_IC_FILTER_FDIV8_N6
  *     @arg @ref TIM_IC_FILTER_FDIV8_N8
  *     @arg @ref TIM_IC_FILTER_FDIV16_N5
  *     @arg @ref TIM_IC_FILTER_FDIV16_N6
  *     @arg @ref TIM_IC_FILTER_FDIV16_N8
  *     @arg @ref TIM_IC_FILTER_FDIV32_N5
  *     @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *     @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @param  ICPrescaler This parameter can be one of the following values:
  *     @arg @ref TIM_ICPSC_DIV1
  *     @arg @ref TIM_ICPSC_DIV2
  *     @arg @ref TIM_ICPSC_DIV4
  *     @arg @ref TIM_ICPSC_DIV8
  * @retval None
  */
 void TIM_Set_TI4(TIM_TypeDef* TIMx, TIM_ICInitTypeDef* TIM_ICInitStruct)
{
  /* Disable the Channel 4: Reset the CC4E Bit */
  TIMx->CCER &= (uint32_t)~TIM_CCER_CC4E;

  /* Select the Input and set the filter and the prescaler value */
  MODIFY_REG(TIMx->CCMR2,
             (TIM_CCMR2_CC4S | TIM_CCMR2_IC4F | TIM_CCMR2_IC4PSC),
             (TIM_ICInitStruct->ICSelection | TIM_ICInitStruct->ICFilter | TIM_ICInitStruct->ICPrescaler) >> 8U);

  /* Select the Polarity and set the CC2E Bit */
  MODIFY_REG(TIMx->CCER,
             (TIM_CCER_CC4P),
             ((TIM_ICInitStruct->ICPolarity << 12U) | TIM_CCER_CC4E));
}

/**
  * @function  TIM_Enable_BreakInputSource
  * @brief  Enable the signals connected to the designated timer break input.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Event:This parameter can be one of the following values:
  *         @arg @ref TIM_AF_BKINE   
  *         @arg @ref TIM_AF_BKCMP1E 
  *         @arg @ref TIM_AF_BKCMP2E 
  *         @arg @ref TIM_AF_BKCMP3E
  *         @arg @ref TIM_AF_BKCMP4E 
  *         @arg @ref TIM_AF_BKCMP5E 
  * @retval None
  */
 void TIM_Enable_BreakInputSource(TIM_TypeDef *TIMx, uint32_t Source)
{
  SET_BIT(TIMx->AF1, Source);
}
/**
  * @function  TIM_Disable_BreakInputSource
  * @brief  Disable the signals connected to the designated timer break input.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Event:This parameter can be one of the following values:
  *         @arg @ref TIM_AF_BKINE   
  *         @arg @ref TIM_AF_BKCMP1E 
  *         @arg @ref TIM_AF_BKCMP2E 
  *         @arg @ref TIM_AF_BKCMP3E
  *         @arg @ref TIM_AF_BKCMP4E 
  *         @arg @ref TIM_AF_BKCMP5E 
  * @retval None
  */
 void TIM_Disable_BreakInputSource(TIM_TypeDef *TIMx, uint32_t Source)
{
  CLEAR_BIT(TIMx->AF1, Source);
}

/**
  * @function  TIM_Set_BreakInputSourcePolarity
  * @brief  Set the polarity of the break signal for the timer break input.
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakInput This parameter can be one of the following values:
  *         @arg @ref TIM_AF_BKINP   
  *         @arg @ref TIM_AF_BKCMP1P 
  *         @arg @ref TIM_AF_BKCMP2P 
  *         @arg @ref TIM_AF_BKCMP3P
  *         @arg @ref TIM_AF_BKCMP4P 
  *         @arg @ref TIM_AF_BKCMP5P 
  * @param  Polarity This parameter can be one of the following values:
  *         @arg @ref TIM_BKCMP_POLARITY_NoInvert
  *         @arg @ref TIM_BKCMP_POLARITY_Invert
  * @retval None
  */
 void TIM_Set_BreakInputSourcePolarity(TIM_TypeDef *TIMx,  uint32_t bkcmpP,uint32_t Polarity)
{
  uint32_t bkpp = (Polarity << (POSITION_VAL(bkcmpP)));
  MODIFY_REG(TIMx->AF1, bkcmpP,bkpp );
}

/**
  * @brief  Configure the break input.
  * @note Macro @ref IS_TIM_BREAK_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides a break input.
  * @note Bidirectional mode is only supported by advanced timer instances.
  *       Macro @ref IS_TIM_ADVANCED_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance is an advanced-control timer.
  * @note In bidirectional mode (BKBID bit set), the Break input is configured both
  *        in input mode and in open drain output mode. Any active Break event will
  *        assert a low logic level on the Break input to indicate an internal break
  *        event to external devices.
  * @note When bidirectional mode isn't supported, BreakAFMode must be set to
  *       TIM_BREAK_AFMODE_INPUT.
  * @rmtoll BDTR         BKP           TIM_ConfigBRK\n
  *         BDTR         BKF           TIM_ConfigBRK\n
  *         BDTR         BKBID         TIM_ConfigBRK
  * @param  TIMx Timer instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_POLARITY_LOW
  *         @arg @ref TIM_BREAK_POLARITY_HIGH
  * @param  BreakFilter This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_FILTER_FDIV1
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N2
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N4
  *         @arg @ref TIM_BREAK_FILTER_FDIV1_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV2_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV2_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV4_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV4_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV8_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV8_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N5
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV16_N8
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N5
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N6
  *         @arg @ref TIM_BREAK_FILTER_FDIV32_N8
  * @param  BreakAFMode This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_AFMODE_INPUT
  *         @arg @ref TIM_BREAK_AFMODE_BIDIRECTIONAL
  * @retval None
  */
void TIM_ConfigBRK(TIM_TypeDef *TIMx, uint32_t BreakPolarity, uint32_t BreakFilter,
                                      uint32_t BreakAFMode)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_BKP | TIM_BDTR_BKF | TIM_BDTR_BKBID, BreakPolarity | BreakFilter | BreakAFMode);
}

void TIM_DisarmBRK(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_BKDSRM);

}
void TIM_ReArmBRK(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_BKDSRM);
}

uint8_t TIM_Read_ReArmBRK(TIM_TypeDef *TIMx)
{

  return (READ_BIT(TIMx->BDTR, TIM_BDTR_BKDSRM) == (TIM_BDTR_BKDSRM));
}
/**
  * @brief  Enable the break 2 function.
  * @note Macro IS_TIM_BKIN2_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides a second break input.
  * @rmtoll BDTR         BK2E          TIM_EnableBRK2
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Enable_BRK2(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_BK2E);
}

/**
  * @brief  Disable the break  2 function.
  * @note Macro IS_TIM_BKIN2_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides a second break input.
  * @rmtoll BDTR         BK2E          TIM_DisableBRK2
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disable_BRK2(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_BK2E);
}
/**
  * @function  TIM_Set_BreakPolarity
  * @brief  Configure the break input
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK2_POLARITY_LOW
  *         @arg @ref TIM_BREAK2_POLARITY_HIGH
  * @retval None
  */
 void TIM_Set_Break2Polarity(TIM_TypeDef *TIMx,uint32_t BreakPolarity)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_BK2P, BreakPolarity);
}
/**
  * @brief  Configure the break 2 input.
  * @note Macro IS_TIM_BKIN2_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance provides a second break input.
  * @note Bidirectional mode is only supported by advanced timer instances.
  *       Macro IS_TIM_ADVANCED_INSTANCE(TIMx) can be used to check whether or not
  *       a timer instance is an advanced-control timer.
  * @note In bidirectional mode (BK2BID bit set), the Break 2 input is configured both
  *        in input mode and in open drain output mode. Any active Break event will
  *        assert a low logic level on the Break 2 input to indicate an internal break
  *        event to external devices.
  * @note When bidirectional mode isn't supported, Break2AFMode must be set to
  *       TIM_BREAK2_AFMODE_INPUT.
  * @rmtoll BDTR         BK2P          TIM_ConfigBRK2\n
  *         BDTR         BK2F          TIM_ConfigBRK2\n
  *         BDTR         BK2BID        TIM_ConfigBRK2
  * @param  TIMx Timer instance
  * @param  Break2Polarity This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK2_POLARITY_LOW
  *         @arg @ref TIM_BREAK2_POLARITY_HIGH
  * @param  Break2Filter This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK2_FILTER_FDIV1
  *         @arg @ref TIM_BREAK2_FILTER_FDIV1_N2
  *         @arg @ref TIM_BREAK2_FILTER_FDIV1_N4
  *         @arg @ref TIM_BREAK2_FILTER_FDIV1_N8
  *         @arg @ref TIM_BREAK2_FILTER_FDIV2_N6
  *         @arg @ref TIM_BREAK2_FILTER_FDIV2_N8
  *         @arg @ref TIM_BREAK2_FILTER_FDIV4_N6
  *         @arg @ref TIM_BREAK2_FILTER_FDIV4_N8
  *         @arg @ref TIM_BREAK2_FILTER_FDIV8_N6
  *         @arg @ref TIM_BREAK2_FILTER_FDIV8_N8
  *         @arg @ref TIM_BREAK2_FILTER_FDIV16_N5
  *         @arg @ref TIM_BREAK2_FILTER_FDIV16_N6
  *         @arg @ref TIM_BREAK2_FILTER_FDIV16_N8
  *         @arg @ref TIM_BREAK2_FILTER_FDIV32_N5
  *         @arg @ref TIM_BREAK2_FILTER_FDIV32_N6
  *         @arg @ref TIM_BREAK2_FILTER_FDIV32_N8
  * @param  Break2AFMode This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK2_AFMODE_INPUT
  *         @arg @ref TIM_BREAK2_AFMODE_BIDIRECTIONAL
  * @retval None
  */
void TIM_Set_BRK2(TIM_TypeDef *TIMx, uint32_t Break2Polarity, uint32_t Break2Filter,
                                       uint32_t Break2AFMode)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_BK2P | TIM_BDTR_BK2F | TIM_BDTR_BK2BID, Break2Polarity | Break2Filter | Break2AFMode);
}

/**
  * @brief  Disarm the break 2 input (when it operates in bidirectional mode).
  * @note  The break 2 input can be disarmed only when it is configured in
  *        bidirectional mode and when when MOE is reset.
  * @note  Purpose is to be able to have the input voltage back to high-state,
  *        whatever the time constant on the output.
  * @rmtoll BDTR         BK2DSRM       TIM_DisarmBRK2
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_Disarm_BRK2(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_BK2DSRM);
}

/**
  * @brief  Re-arm the break 2 input (when it operates in bidirectional mode).
  * @note  The Break 2 input is automatically armed as soon as MOE bit is set.
  * @rmtoll BDTR         BK2DSRM       TIM_ReArmBRK2
  * @param  TIMx Timer instance
  * @retval None
  */
void TIM_ReArm_BRK2(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_BK2DSRM);
}


/**
  * @function  TIM_Set_OC_CompareCH1N
  * @brief  Set compare value for output channel 1N (TIM_CCR1N).
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH1N(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR1N, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH2N
  * @brief  Set compare value for output channel 2N (TIM_CCR2N).
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH2N(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR2N, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH3N
  * @brief  Set compare value for output channel 3N (TIM_CCR3N).
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH3N(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR3N, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH4N
  * @brief  Set compare value for output channel 3N (TIM_CCR4N).
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH4N(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR4N, CompareValue);
}

/**
  * @function  TIM_Get_OC_CompareCH1N
  * @brief  Get compare value (TIM_CCR1N) set for  output channel 1N.
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH1N(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR1N));
}

/**
  * @function  TIM_Get_OC_CompareCH2N
  * @brief  Get compare value (TIM_CCR2N) set for  output channel 2N.
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH2N(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR2N));
}

/**
  * @function  TIM_Get_OC_CompareCH3N
  * @brief  Get compare value (TIM_CCR3N) set for  output channel 3N.
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH3N(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR3N));
}

/**
  * @function  TIM_Get_OC_CompareCH4N
  * @brief  Get compare value (TIM_CCR4N) set for  output channel 4N.
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH4N(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR4N));
}

/**
  * @function  TIM_Enable_CCR1N
  * @brief  Enable the CCR1N function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CCR1N(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CCR1N, TIM_CCR1N_EN);
}

void TIM_Enable_AutomaticOutput(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_AOE);
}	
	
void TIM_Enable_AllOutputs(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_MOE);
}

void TIM_GenerateEvent_UPDATE(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->EGR, TIM_EGR_UG);
}

void TIM_Clear_Flag_UPDATE(TIM_TypeDef *TIMx)
{
  WRITE_REG(TIMx->SR, ~(TIM_SR_UIF));
}

void TIM_Enable_Counter(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_CEN);
}	
	
/**
  * @function  TIM_Disable_CCR1N
  * @brief  Disable the CCR1N function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CCR1N(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CCR1N, TIM_CCR1N_EN);
}

/**
  * @function  TIM_Enable_CCR2N
  * @brief  Enable the CCR2N function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CCR2N(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CCR2N, TIM_CCR2N_EN);
}

/**
  * @function  TIM_Disable_CCR2N
  * @brief  Disable the CCR2N function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CCR2N(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CCR2N, TIM_CCR2N_EN);
}

/**
  * @function  TIM_Enable_CCR3N
  * @brief  Enable the CCR3N function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CCR3N(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CCR3N, TIM_CCR3N_EN);
}

/**
  * @function  TIM_Disable_CCR3N
  * @brief  Disable the CCR3N function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CCR3N(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CCR3N, TIM_CCR3N_EN);
}

/**
  * @function  TIM_Enable_CCR4N
  * @brief  Enable the CCR4N function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CCR4N(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CCR4N, TIM_CCR4N_EN);
}

/**
  * @function  TIM_Disable_CCR4N
  * @brief  Disable the CCR4N function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CCR4N(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CCR4N, TIM_CCR4N_EN);
}

/**
  * @function  TIM_Enable_Break_Control
  * @brief    
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
void TIM_Enable_Break_Control(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BIC, TIM_BICE_EN);
}
void TIM_Disable_Break_Control(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BIC, TIM_BICE_EN);
}
void TIM_Set_CH1Break_Invalid(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BIC, TIM_BICS_CH1_EN);
}
void TIM_Set_CH2Break_Invalid(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BIC, TIM_BICS_CH2_EN);
}
void TIM_Set_CH3Break_Invalid(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BIC, TIM_BICS_CH3_EN);
}
void TIM_Set_CH4Break_Invalid(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BIC, TIM_BICS_CH4_EN);
}
void TIM_Set_CH5Break_Invalid(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BIC, TIM_BICS_CH5_EN);
}
void TIM_Set_CH6Break_Invalid(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BIC, TIM_BICS_CH6_EN);
}

void TIM_Set_CH1Break_Valid(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BIC, TIM_BICS_CH1_EN);
}
void TIM_Set_CH2Break_Valid(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BIC, TIM_BICS_CH2_EN);
}
void TIM_Set_CH3Break_Valid(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BIC, TIM_BICS_CH3_EN);
}
void TIM_Set_CH4Break_Valid(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BIC, TIM_BICS_CH4_EN);
}
void TIM_Set_CH5Break_Valid(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BIC, TIM_BICS_CH5_EN);
}
void TIM_Set_CH6Break_Valid(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BIC, TIM_BICS_CH6_EN);
}

